﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Cosmos.Hardware2 {

    public class PCIBus {
        //TODO: Change to Dictionary<UInt32, string> when the IL2CPU bug is fixed
        public class DeviceID {
            UInt32 key;

            public UInt32 Key {
                get { return key; }
                set { key = value; }
            }

            String value;

            public String Value {
                get { return value; }
                set { this.value = value; }
            }

            public DeviceID(UInt32 pkey, String pvalue) {
                key = pkey;
                value = pvalue;
            }
        }

        //Dont make static. We dont want all the strings loaded in RAM
        // all the time.
        public class DeviceIDs {
            protected TempDictionary<String> mVendors = new TempDictionary<String>();
            protected TempDictionary<String> mDevices = new TempDictionary<String>();

            public DeviceIDs()
            {
                // Current QEMU hardware
                mVendors.Add(0x8086, "Intel Corporation");
                // 1237  440FX - 82441FX PMC [Natoma]
                // 7000  82371SB PIIX3 ISA [Natoma/Triton II]
                // 7010  82371SB PIIX3 IDE [Natoma/Triton II]
                // 7113  82371AB/EB/MB PIIX4 ACPI
                //    Need to check sub attr - maybe we have this:
                //    15ad 1976  Virtual Machine Chipset
                mVendors.Add(0x1013, "Cirrus Logic");
                // 00b8  GD 5446
                mVendors.Add(0x10EC, "Realtek Semiconductor");
                //8139  RTL-8029(AS)
                //0357 000a  TTP-Monitoring Card V2.0
                //1025 005a  TravelMate 290
                //1025 8920  ALN-325
                //1025 8921  ALN-325
                //103c 006a  NX9500
                //1043 1045  L8400B or L3C/S notebook
                //1043 8109  P5P800-MX Mainboard
                //1071 8160  MIM2000
                //10bd 0320  EP-320X-R
                //10ec 8139  RT8139
                //10f7 8338  Panasonic CF-Y5 laptop
                //1113 ec01  FNC-0107TX
                //1186 1300  DFE-538TX
                //1186 1320  SN5200
                //1186 8139  DRN-32TX
                //11f6 8139  FN22-3(A) LinxPRO Ethernet Adapter
                //1259 2500  AT-2500TX
                //1259 2503  AT-2500TX/ACPI
                //1429 d010  ND010
                //1432 9130  EN-9130TX
                //1436 8139  RT8139
                //144d c00c  P30/P35 notebook
                //1458 e000  GA-7VM400M/7VT600 Motherboard
                //1462 788c  865PE Neo2-V Mainboard
                //146c 1439  FE-1439TX
                //1489 6001  GF100TXRII
                //1489 6002  GF100TXRA
                //149c 139a  LFE-8139ATX
                //149c 8139  LFE-8139TX
                //14cb 0200  LNR-100 Family 10/100 Base-TX Ethernet
                //1565 2300  P4TSV Onboard LAN (RTL8100B)
                //1695 9001  Onboard RTL8101L 10/100 MBit
                //1799 5000  F5D5000 PCI Card/Desktop Network PCI Card
                //1904 8139  RTL8139D Fast Ethernet Adapter
                //2646 0001  EtheRx
                //8e2e 7000  KF-230TX
                //8e2e 7100  KF-230TX/2
                //a0a0 0007  ALN-325C

                mVendors.Add(0x1274, "Ensoniq");

                //Found on ESX Server:
                mVendors.Add(0x15AD, "VMware Inc.");
                mVendors.Add(0x1000, "LSI Logic 53C810 Device");
                mVendors.Add(0x1022, "Advanced Micro Devices");

                #region PCIHelper

                #region Fill vendors

                //lines of this region are generated with PCIHelper

                // Source:  http://www.pcidatabase.com/vendors.php?sort=id


                //For now it is too slow to add all vendors
                /*
                mVendors.Add(0x0033, "Paradyne Corp.");
                mVendors.Add(0x003D, "master");
                mVendors.Add(0x0070, "Hauppauge Computer Works Inc.");
                mVendors.Add(0x0100, "Ncipher Corp. Ltd");
                mVendors.Add(0x0123, "General Dynamics");
                mVendors.Add(0x0315, "SK - Electronics Co., Ltd.");
                mVendors.Add(0x0A89, "BREA Technologies Inc.");
                mVendors.Add(0x0E11, "Compaq Computer Corp.");
                mVendors.Add(0x1000, "LSI Logic 53C810 Device");
                mVendors.Add(0x1001, "Kolter Electronic - Germany");
                mVendors.Add(0x1002, "ATI");
                mVendors.Add(0x1003, "ULSI");
                mVendors.Add(0x1004, "VLSI Technology");
                mVendors.Add(0x1006, "Reply Group");
                mVendors.Add(0x1007, "Netframe Systems Inc.");
                mVendors.Add(0x1008, "Epson");
                mVendors.Add(0x100A, "Phoenix Technologies Ltd.");
                mVendors.Add(0x100B, "National Semiconductors");
                mVendors.Add(0x100C, "Tseng Labs");
                mVendors.Add(0x100D, "AST Research");
                mVendors.Add(0x100E, "Weitek");
                mVendors.Add(0x1010, "Video Logic Ltd.");
                mVendors.Add(0x1011, "Digital Equipment Corporation");
                mVendors.Add(0x1012, "Micronics Computers Inc.");
                mVendors.Add(0x1013, "Cirrus Logic");
                mVendors.Add(0x1014, "International Business Machines Corp.");
                mVendors.Add(0x1016, "Fujitsu ICL Computers");
                mVendors.Add(0x1017, "Spea Software AG");
                mVendors.Add(0x1018, "Unisys Systems");
                mVendors.Add(0x1019, "Elitegroup Computer System");
                mVendors.Add(0x101A, "NCR Corporation");
                mVendors.Add(0x101B, "Vitesse Semiconductor");
                mVendors.Add(0x101E, "American Megatrends Inc.");
                mVendors.Add(0x101F, "PictureTel Corp.");
                mVendors.Add(0x1020, "Hitachi Computer Electronics");
                mVendors.Add(0x1021, "Oki Electric Industry");
                mVendors.Add(0x1022, "Advanced Micro Devices");
                mVendors.Add(0x1023, "TRIDENT MICRO");
                mVendors.Add(0x1025, "Acer Incorporated");
                mVendors.Add(0x1028, "Dell Computer Corporation");
                mVendors.Add(0x102A, "LSI Logic Headland Division");
                mVendors.Add(0x102B, "Matrox Electronic Systems Ltd.");
                mVendors.Add(0x102C, "Asiliant (Chips And Technologies)");
                mVendors.Add(0x102D, "Wyse Technologies");
                mVendors.Add(0x102E, "Olivetti Advanced Technology");
                mVendors.Add(0x102F, "Toshiba America");
                mVendors.Add(0x1030, "TMC Research");
                mVendors.Add(0x1031, "miro Computer Products AG");
                mVendors.Add(0x1033, "NEC Electronics");
                mVendors.Add(0x1034, "Burndy Corporation");
                mVendors.Add(0x1036, "Future Domain");
                mVendors.Add(0x1037, "Hitachi Micro Systems Inc");
                mVendors.Add(0x1038, "AMP Incorporated");
                mVendors.Add(0x1039, "Silicon Integrated Systems");
                mVendors.Add(0x103A, "Seiko Epson Corporation");
                mVendors.Add(0x103B, "Tatung Corp. Of America");
                mVendors.Add(0x103C, "Hewlett-Packard Company");
                mVendors.Add(0x103E, "Solliday Engineering");
                mVendors.Add(0x103F, "Logic Modeling");
                mVendors.Add(0x1041, "Computrend");
                mVendors.Add(0x1043, "Asustek Computer Inc.");
                mVendors.Add(0x1044, "Distributed Processing Tech");
                mVendors.Add(0x1045, "OPTi Inc.");
                mVendors.Add(0x1046, "IPC Corporation LTD");
                mVendors.Add(0x1047, "Genoa Systems Corp.");
                mVendors.Add(0x1048, "ELSA GmbH");
                mVendors.Add(0x1049, "Fountain Technology");
                mVendors.Add(0x104A, "STMicroelectronics");
                mVendors.Add(0x104B, "Mylex / Buslogic");
                mVendors.Add(0x104C, "Texas Instruments");
                mVendors.Add(0x104D, "Sony Corporation");
                mVendors.Add(0x104E, "Oak Technology");
                mVendors.Add(0x104F, "Co-Time Computer Ltd.");
                mVendors.Add(0x1050, "Winbond Electronics Corp.");
                mVendors.Add(0x1051, "Anigma Corp.");
                mVendors.Add(0x1053, "Young Micro Systems");
                mVendors.Add(0x1054, "Hitachi Ltd");
                mVendors.Add(0x1055, "Standard Microsystems Corp.");
                mVendors.Add(0x1056, "ICL");
                mVendors.Add(0x1057, "Motorola");
                mVendors.Add(0x1058, "Electronics & Telecommunication Res");
                mVendors.Add(0x1059, "Kontron Canada");
                mVendors.Add(0x105A, "Promise Technology");
                mVendors.Add(0x105B, "Foxconn International Inc.");
                mVendors.Add(0x105C, "Wipro Infotech Limited");
                mVendors.Add(0x105D, "Number Nine Visual Technology");
                mVendors.Add(0x105E, "Vtech Engineering Canada Ltd.");
                mVendors.Add(0x105F, "Infotronic America Inc.");
                mVendors.Add(0x1060, "United Microelectronics");
                mVendors.Add(0x1061, "8x8 Inc.");
                mVendors.Add(0x1062, "Maspar Computer Corp.");
                mVendors.Add(0x1063, "Ocean Office Automation");
                mVendors.Add(0x1064, "Alcatel Cit");
                mVendors.Add(0x1065, "Texas Microsystems");
                mVendors.Add(0x1066, "Picopower Technology");
                mVendors.Add(0x1067, "Mitsubishi Electronics");
                mVendors.Add(0x1068, "Diversified Technology");
                mVendors.Add(0x106A, "Aten Research Inc.");
                mVendors.Add(0x106B, "Apple Computer Inc.");
                mVendors.Add(0x106C, "Hyundai Electronics America");
                mVendors.Add(0x106D, "Sequent Computer Systems");
                mVendors.Add(0x106E, "DFI Inc.");
                mVendors.Add(0x106F, "City Gate Development LTD");
                mVendors.Add(0x1070, "Daewoo Telecom Ltd.");
                mVendors.Add(0x1071, "Mitac");
                mVendors.Add(0x1072, "GIT Co. Ltd.");
                mVendors.Add(0x1073, "Yamaha Corporation");
                mVendors.Add(0x1074, "Nexgen Microsystems");
                mVendors.Add(0x1075, "Advanced Integration Research");
                mVendors.Add(0x1077, "QLogic Corporation");
                mVendors.Add(0x1078, "Cyrix Corporation");
                mVendors.Add(0x1079, "I-Bus");
                mVendors.Add(0x107A, "Networth");
                mVendors.Add(0x107B, "Gateway 2000");
                mVendors.Add(0x107C, "Goldstar Co. Ltd.");
                mVendors.Add(0x107D, "Leadtek Research");
                mVendors.Add(0x107E, "Testernec");
                mVendors.Add(0x107F, "Data Technology Corporation");
                mVendors.Add(0x1080, "Cypress Semiconductor");
                mVendors.Add(0x1081, "Radius Inc.");
                mVendors.Add(0x1082, "EFA Corporation Of America");
                mVendors.Add(0x1083, "Forex Computer Corporation");
                mVendors.Add(0x1084, "Parador");
                mVendors.Add(0x1085, "Tulip Computers Int'l BV");
                mVendors.Add(0x1086, "J. Bond Computer Systems");
                mVendors.Add(0x1087, "Cache Computer");
                mVendors.Add(0x1088, "Microcomputer Systems (M) Son");
                mVendors.Add(0x1089, "Data General Corporation");
                mVendors.Add(0x108A, "SBS Operations");
                mVendors.Add(0x108C, "Oakleigh Systems Inc.");
                mVendors.Add(0x108D, "Olicom");
                mVendors.Add(0x108E, "Sun Microsystems");
                mVendors.Add(0x108F, "Systemsoft Corporation");
                mVendors.Add(0x1090, "Encore Computer Corporation");
                mVendors.Add(0x1091, "Intergraph Corporation");
                mVendors.Add(0x1092, "Diamond Computer Systems");
                mVendors.Add(0x1093, "National Instruments");
                mVendors.Add(0x1094, "First Int'l Computers");
                mVendors.Add(0x1095, "Silicon Image, Inc.");
                mVendors.Add(0x1096, "Alacron");
                mVendors.Add(0x1097, "Appian Graphics");
                mVendors.Add(0x1098, "Quantum Designs Ltd.");
                mVendors.Add(0x1099, "Samsung Electronics Co. Ltd.");
                mVendors.Add(0x109A, "Packard Bell");
                mVendors.Add(0x109B, "Gemlight Computer Ltd.");
                mVendors.Add(0x109C, "Megachips Corporation");
                mVendors.Add(0x109D, "Zida Technologies Ltd.");
                mVendors.Add(0x109E, "Brooktree Corporation");
                mVendors.Add(0x109F, "Trigem Computer Inc.");
                mVendors.Add(0x10A0, "Meidensha Corporation");
                mVendors.Add(0x10A1, "Juko Electronics Inc. Ltd.");
                mVendors.Add(0x10A2, "Quantum Corporation");
                mVendors.Add(0x10A3, "Everex Systems Inc.");
                mVendors.Add(0x10A4, "Globe Manufacturing Sales");
                mVendors.Add(0x10A5, "Racal Interlan");
                mVendors.Add(0x10A8, "Sierra Semiconductor");
                mVendors.Add(0x10A9, "Silicon Graphics");
                mVendors.Add(0x10AB, "Digicom");
                mVendors.Add(0x10AC, "Honeywell IASD");
                mVendors.Add(0x10AD, "Winbond Systems Labs");
                mVendors.Add(0x10AE, "Cornerstone Technology");
                mVendors.Add(0x10AF, "Micro Computer Systems Inc.");
                mVendors.Add(0x10B0, "CardExpert Technology");
                mVendors.Add(0x10B1, "Cabletron Systems Inc.");
                mVendors.Add(0x10B2, "Raytheon Company");
                mVendors.Add(0x10B3, "Databook Inc.");
                mVendors.Add(0x10B4, "STB Systems");
                mVendors.Add(0x10B5, "PLX Technology Inc.");
                mVendors.Add(0x10B6, "Madge Networks");
                mVendors.Add(0x10B7, "3Com Corporation");
                mVendors.Add(0x10B8, "Standard Microsystems Corporation");
                mVendors.Add(0x10B9, "Ali Corporation");
                mVendors.Add(0x10BA, "Mitsubishi Electronics Corp.");
                mVendors.Add(0x10BB, "Dapha Electronics Corporation");
                mVendors.Add(0x10BC, "Advanced Logic Research Inc.");
                mVendors.Add(0x10BD, "Surecom Technology");
                mVendors.Add(0x10BE, "Tsenglabs International Corp.");
                mVendors.Add(0x10BF, "MOST Corp.");
                mVendors.Add(0x10C0, "Boca Research Inc.");
                mVendors.Add(0x10C1, "ICM Corp. Ltd.");
                mVendors.Add(0x10C2, "Auspex Systems Inc.");
                mVendors.Add(0x10C3, "Samsung Semiconductors");
                mVendors.Add(0x10C4, "Award Software Int'l Inc.");
                mVendors.Add(0x10C5, "Xerox Corporation");
                mVendors.Add(0x10C6, "Rambus Inc.");
                mVendors.Add(0x10C8, "Neomagic Corporation");
                mVendors.Add(0x10C9, "Dataexpert Corporation");
                mVendors.Add(0x10CA, "Fujitsu Siemens");
                mVendors.Add(0x10CB, "Omron Corporation");
                mVendors.Add(0x10CD, "Advanced System Products");
                mVendors.Add(0x10CF, "Fujitsu Ltd.");
                mVendors.Add(0x10D1, "Future+ Systems");
                mVendors.Add(0x10D2, "Molex Incorporated");
                mVendors.Add(0x10D3, "Jabil Circuit Inc.");
                mVendors.Add(0x10D4, "Hualon Microelectronics");
                mVendors.Add(0x10D5, "Autologic Inc.");
                mVendors.Add(0x10D6, "Cetia");
                mVendors.Add(0x10D7, "BCM Advanced Research");
                mVendors.Add(0x10D8, "Advanced Peripherals Labs");
                mVendors.Add(0x10D9, "Macronix International Co. Ltd.");
                mVendors.Add(0x10DB, "Rohm Research");
                mVendors.Add(0x10DC, "CERN-European Lab. for Particle Physics");
                mVendors.Add(0x10DD, "Evans & Sutherland");
                mVendors.Add(0x10DE, "NVIDIA");
                mVendors.Add(0x10DF, "Emulex Corporation");
                mVendors.Add(0x10E1, "Tekram Technology Corp. Ltd.");
                mVendors.Add(0x10E2, "Aptix Corporation");
                mVendors.Add(0x10E3, "Tundra Semiconductor Corp.");
                mVendors.Add(0x10E4, "Tandem Computers");
                mVendors.Add(0x10E5, "Micro Industries Corporation");
                mVendors.Add(0x10E6, "Gainbery Computer Products Inc.");
                mVendors.Add(0x10E7, "Vadem");
                mVendors.Add(0x10E8, "Applied Micro Circuits Corp.");
                mVendors.Add(0x10E9, "Alps Electronic Corp. Ltd.");
                mVendors.Add(0x10EA, "Tvia, Inc.");
                mVendors.Add(0x10EB, "Artist Graphics");
                mVendors.Add(0x10EC, "Realtek Semiconductor");
                mVendors.Add(0x10ED, "Ascii Corporation");
                mVendors.Add(0x10EE, "Xilinx Corporation");
                mVendors.Add(0x10EF, "Racore Computer Products");
                mVendors.Add(0x10F0, "Curtiss-Wright Controls Embedded Computing");
                mVendors.Add(0x10F1, "Tyan Computer");
                mVendors.Add(0x10F2, "Achme Computer Inc. - GONE !!!!");
                mVendors.Add(0x10F3, "Alaris Inc.");
                mVendors.Add(0x10F4, "S-Mos Systems");
                mVendors.Add(0x10F5, "NKK Corporation");
                mVendors.Add(0x10F6, "Creative Electronic Systems SA");
                mVendors.Add(0x10F7, "Matsushita Electric Industrial Corp.");
                mVendors.Add(0x10F8, "Altos India Ltd.");
                mVendors.Add(0x10F9, "PC Direct");
                mVendors.Add(0x10FA, "Truevision");
                mVendors.Add(0x10FB, "Thesys Microelectronic's");
                mVendors.Add(0x10FC, "I-O Data Device Inc.");
                mVendors.Add(0x10FD, "Soyo Technology Corp. Ltd.");
                mVendors.Add(0x10FE, "Fast Electronic GmbH");
                mVendors.Add(0x10FF, "Ncube");
                mVendors.Add(0x1100, "Jazz Multimedia");
                mVendors.Add(0x1101, "Initio Corporation");
                mVendors.Add(0x1102, "Creative Technology LTD.");
                mVendors.Add(0x1103, "Triones Technologies Inc. (HighPoint)");
                mVendors.Add(0x1104, "Rasterops");
                mVendors.Add(0x1105, "Sigma Designs Inc.");
                mVendors.Add(0x1106, "VIA Technology");
                mVendors.Add(0x1107, "Stratus Computer");
                mVendors.Add(0x1108, "Proteon Inc.");
                mVendors.Add(0x1109, "Adaptec/Cogent Data Technologies");
                mVendors.Add(0x110A, "Siemens Nixdorf AG");
                mVendors.Add(0x110B, "Chromatic Research Inc");
                mVendors.Add(0x110C, "Mini-Max Technology Inc.");
                mVendors.Add(0x110D, "ZNYX Corporation");
                mVendors.Add(0x110E, "CPU Technology");
                mVendors.Add(0x110F, "Ross Technology");
                mVendors.Add(0x1112, "Osicom Technologies Inc.");
                mVendors.Add(0x1113, "Accton Technology Corporation");
                mVendors.Add(0x1114, "Atmel Corp.");
                mVendors.Add(0x1116, "Data Translation, Inc.");
                mVendors.Add(0x1117, "Datacube Inc.");
                mVendors.Add(0x1118, "Berg Electronics");
                mVendors.Add(0x1119, "ICP vortex Computersysteme GmbH");
                mVendors.Add(0x111A, "Efficent Networks");
                mVendors.Add(0x111C, "Tricord Systems Inc.");
                mVendors.Add(0x111D, "Integrated Device Technology Inc.");
                mVendors.Add(0x111F, "Precision Digital Images");
                mVendors.Add(0x1120, "EMC Corp.");
                mVendors.Add(0x1121, "Zilog");
                mVendors.Add(0x1123, "Excellent Design Inc.");
                mVendors.Add(0x1124, "Leutron Vision AG");
                mVendors.Add(0x1125, "Eurocore/Vigra");
                mVendors.Add(0x1127, "FORE Systems");
                mVendors.Add(0x1129, "Firmworks");
                mVendors.Add(0x112A, "Hermes Electronics Co. Ltd.");
                mVendors.Add(0x112C, "Zenith Data Systems");
                mVendors.Add(0x112D, "Ravicad");
                mVendors.Add(0x112E, "Infomedia");
                mVendors.Add(0x1130, "Computervision");
                mVendors.Add(0x1131, "Philips Semiconductors");
                mVendors.Add(0x1132, "Mitel Corp.");
                mVendors.Add(0x1133, "Eicon Networks Corporation");
                mVendors.Add(0x1134, "Mercury Computer Systems Inc.");
                mVendors.Add(0x1135, "Fuji Xerox Co Ltd");
                mVendors.Add(0x1136, "Momentum Data Systems");
                mVendors.Add(0x1137, "Cisco Systems Inc");
                mVendors.Add(0x1138, "Ziatech Corporation");
                mVendors.Add(0x1139, "Dynamic Pictures Inc");
                mVendors.Add(0x113A, "FWB Inc");
                mVendors.Add(0x113B, "Network Computing Devices");
                mVendors.Add(0x113C, "Cyclone Microsystems Inc.");
                mVendors.Add(0x113D, "Leading Edge Products Inc");
                mVendors.Add(0x113E, "Sanyo Electric Co");
                mVendors.Add(0x113F, "Equinox Systems");
                mVendors.Add(0x1140, "Intervoice Inc");
                mVendors.Add(0x1141, "Crest Microsystem Inc");
                mVendors.Add(0x1142, "Alliance Semiconductor");
                mVendors.Add(0x1143, "Netpower Inc");
                mVendors.Add(0x1144, "Cincinnati Milacron");
                mVendors.Add(0x1145, "Workbit Corp");
                mVendors.Add(0x1146, "Force Computers");
                mVendors.Add(0x1147, "Interface Corp");
                mVendors.Add(0x1148, "Marvell Semiconductor Germany GmbH");
                mVendors.Add(0x1149, "Win System Corporation");
                mVendors.Add(0x114A, "VMIC");
                mVendors.Add(0x114B, "Canopus corporation");
                mVendors.Add(0x114C, "Annabooks");
                mVendors.Add(0x114D, "IC Corporation");
                mVendors.Add(0x114E, "Nikon Systems Inc");
                mVendors.Add(0x114F, "Digi International");
                mVendors.Add(0x1150, "Thinking Machines Corporation");
                mVendors.Add(0x1151, "JAE Electronics Inc.");
                mVendors.Add(0x1153, "Land Win Electronic Corp");
                mVendors.Add(0x1154, "Melco Inc");
                mVendors.Add(0x1155, "Pine Technology Ltd");
                mVendors.Add(0x1156, "Periscope Engineering");
                mVendors.Add(0x1157, "Avsys Corporation");
                mVendors.Add(0x1158, "Voarx R&D Inc");
                mVendors.Add(0x1159, "Mutech");
                mVendors.Add(0x115A, "Harlequin Ltd");
                mVendors.Add(0x115B, "Parallax Graphics");
                mVendors.Add(0x115C, "Photron Ltd.");
                mVendors.Add(0x115D, "Xircom");
                mVendors.Add(0x115E, "Peer Protocols Inc");
                mVendors.Add(0x115F, "Maxtor Corporation");
                mVendors.Add(0x1160, "Megasoft Inc");
                mVendors.Add(0x1161, "PFU Ltd");
                mVendors.Add(0x1162, "OA Laboratory Co Ltd");
                mVendors.Add(0x1163, "Rendition Inc");
                mVendors.Add(0x1164, "Advanced Peripherals Tech");
                mVendors.Add(0x1165, "Imagraph Corporation");
                mVendors.Add(0x1166, "Broadcom / ServerWorks");
                mVendors.Add(0x1167, "Mutoh Industries Inc");
                mVendors.Add(0x1168, "Thine Electronics Inc");
                mVendors.Add(0x1169, "Centre f/Dev. of Adv. Computing");
                mVendors.Add(0x116A, "Polaris Communications");
                mVendors.Add(0x116B, "Connectware Inc");
                mVendors.Add(0x116C, "Intelligent Resources");
                mVendors.Add(0x116E, "Electronics for Imaging");
                mVendors.Add(0x1170, "Inventec Corporation");
                mVendors.Add(0x1172, "Altera Corporation");
                mVendors.Add(0x1173, "Adobe Systems");
                mVendors.Add(0x1174, "Bridgeport Machines");
                mVendors.Add(0x1175, "Mitron Computer Inc.");
                mVendors.Add(0x1176, "SBE");
                mVendors.Add(0x1177, "Silicon Engineering");
                mVendors.Add(0x1178, "Alfa Inc");
                mVendors.Add(0x1179, "Toshiba America Info Systems");
                mVendors.Add(0x117A, "A-Trend Technology");
                mVendors.Add(0x117B, "LG (Lucky Goldstar) Electronics Inc.");
                mVendors.Add(0x117C, "Atto Technology");
                mVendors.Add(0x117D, "Becton & Dickinson");
                mVendors.Add(0x117E, "T/R Systems");
                mVendors.Add(0x117F, "Integrated Circuit Systems");
                mVendors.Add(0x1180, "Ricoh Company, Ltd.");
                mVendors.Add(0x1183, "Fujikura Ltd");
                mVendors.Add(0x1184, "Forks Inc");
                mVendors.Add(0x1185, "Dataworld");
                mVendors.Add(0x1186, "D-Link System Inc");
                mVendors.Add(0x1187, "Advanced Technology Laboratories");
                mVendors.Add(0x1188, "Shima Seiki Manufacturing Ltd.");
                mVendors.Add(0x1189, "Matsushita Electronics");
                mVendors.Add(0x118A, "Hilevel Technology");
                mVendors.Add(0x118B, "Hypertec Pty Ltd");
                mVendors.Add(0x118C, "Corollary Inc");
                mVendors.Add(0x118D, "BitFlow Inc");
                mVendors.Add(0x118E, "Hermstedt AG");
                mVendors.Add(0x118F, "Green Logic");
                mVendors.Add(0x1190, "Tripace");
                mVendors.Add(0x1191, "Acard Technology Corp.");
                mVendors.Add(0x1192, "Densan Co. Ltd");
                mVendors.Add(0x1194, "Toucan Technology");
                mVendors.Add(0x1195, "Ratoc System Inc");
                mVendors.Add(0x1196, "Hytec Electronics Ltd");
                mVendors.Add(0x1197, "Gage Applied Technologies");
                mVendors.Add(0x1198, "Lambda Systems Inc");
                mVendors.Add(0x1199, "Attachmate Corp.");
                mVendors.Add(0x119A, "Mind/Share Inc.");
                mVendors.Add(0x119B, "Omega Micro Inc.");
                mVendors.Add(0x119C, "Information Technology Inst.");
                mVendors.Add(0x119D, "Bug Sapporo Japan");
                mVendors.Add(0x119E, "Fujitsu Microelectronics Ltd.");
                mVendors.Add(0x119F, "Bull Hn Information Systems");
                mVendors.Add(0x11A1, "Hamamatsu Photonics K.K.");
                mVendors.Add(0x11A2, "Sierra Research and Technology");
                mVendors.Add(0x11A3, "Deuretzbacher GmbH & Co. Eng. KG");
                mVendors.Add(0x11A4, "Barco");
                mVendors.Add(0x11A5, "MicroUnity Systems Engineering Inc.");
                mVendors.Add(0x11A6, "Pure Data");
                mVendors.Add(0x11A7, "Power Computing Corp.");
                mVendors.Add(0x11A8, "Systech Corp.");
                mVendors.Add(0x11A9, "InnoSys Inc.");
                mVendors.Add(0x11AA, "Actel");
                mVendors.Add(0x11AB, "Marvell Semiconductor");
                mVendors.Add(0x11AC, "Canon Information Systems");
                mVendors.Add(0x11AD, "Lite-On Technology Corp.");
                mVendors.Add(0x11AE, "Scitex Corporation Ltd");
                mVendors.Add(0x11AF, "Avid Technology Inc.");
                mVendors.Add(0x11B0, "Quicklogic Corp");
                mVendors.Add(0x11B1, "Apricot Computers");
                mVendors.Add(0x11B2, "Eastman Kodak");
                mVendors.Add(0x11B3, "Barr Systems Inc.");
                mVendors.Add(0x11B4, "Leitch Technology International");
                mVendors.Add(0x11B5, "Radstone Technology Ltd.");
                mVendors.Add(0x11B6, "United Video Corp");
                mVendors.Add(0x11B7, "Motorola");
                mVendors.Add(0x11B8, "Xpoint Technologies Inc");
                mVendors.Add(0x11B9, "Pathlight Technology Inc.");
                mVendors.Add(0x11BA, "Videotron Corp");
                mVendors.Add(0x11BB, "Pyramid Technology");
                mVendors.Add(0x11BC, "Network Peripherals Inc");
                mVendors.Add(0x11BD, "Pinnacle Systems Inc.");
                mVendors.Add(0x11BE, "International Microcircuits Inc");
                mVendors.Add(0x11BF, "Astrodesign Inc.");
                mVendors.Add(0x11C1, "Agere Systems");
                mVendors.Add(0x11C2, "Sand Microelectronics");
                mVendors.Add(0x11C4, "Document Technologies Ind.");
                mVendors.Add(0x11C5, "Shiva Corporatin");
                mVendors.Add(0x11C6, "Dainippon Screen Mfg. Co");
                mVendors.Add(0x11C7, "D.C.M. Data Systems");
                mVendors.Add(0x11C8, "Dolphin Interconnect Solutions");
                mVendors.Add(0x11C9, "MAGMA");
                mVendors.Add(0x11CA, "LSI Systems Inc");
                mVendors.Add(0x11CB, "Specialix International Ltd.");
                mVendors.Add(0x11CC, "Michels & Kleberhoff Computer GmbH");
                mVendors.Add(0x11CD, "HAL Computer Systems Inc.");
                mVendors.Add(0x11CE, "Primary Rate Inc");
                mVendors.Add(0x11CF, "Pioneer Electronic Corporation");
                mVendors.Add(0x11D0, "BAE SYSTEMS - Manassas");
                mVendors.Add(0x11D1, "AuraVision Corporation");
                mVendors.Add(0x11D2, "Intercom Inc.");
                mVendors.Add(0x11D3, "Trancell Systems Inc");
                mVendors.Add(0x11D4, "Analog Devices, Inc.");
                mVendors.Add(0x11D5, "Tahoma Technology");
                mVendors.Add(0x11D6, "Tekelec Technologies");
                mVendors.Add(0x11D7, "TRENTON Technology, Inc.");
                mVendors.Add(0x11D8, "Image Technologies Development");
                mVendors.Add(0x11D9, "Tec Corporation");
                mVendors.Add(0x11DA, "Novell");
                mVendors.Add(0x11DB, "Sega Enterprises Ltd");
                mVendors.Add(0x11DC, "Questra Corp");
                mVendors.Add(0x11DD, "Crosfield Electronics Ltd");
                mVendors.Add(0x11DE, "Zoran Corporation");
                mVendors.Add(0x11E1, "Gec Plessey Semi Inc");
                mVendors.Add(0x11E2, "Samsung Information Systems America");
                mVendors.Add(0x11E3, "Quicklogic Corp");
                mVendors.Add(0x11E4, "Second Wave Inc");
                mVendors.Add(0x11E5, "IIX Consulting");
                mVendors.Add(0x11E6, "Mitsui-Zosen System Research");
                mVendors.Add(0x11E8, "Digital Processing Systems Inc");
                mVendors.Add(0x11E9, "Highwater Designs Ltd");
                mVendors.Add(0x11EA, "Elsag Bailey");
                mVendors.Add(0x11EB, "Formation, Inc");
                mVendors.Add(0x11EC, "Coreco Inc");
                mVendors.Add(0x11ED, "Mediamatics");
                mVendors.Add(0x11EE, "Dome Imaging Systems Inc");
                mVendors.Add(0x11EF, "Nicolet Technologies BV");
                mVendors.Add(0x11F0, "Compu-Shack GmbH");
                mVendors.Add(0x11F2, "Picture Tel Japan KK");
                mVendors.Add(0x11F3, "Keithley Metrabyte");
                mVendors.Add(0x11F4, "Kinetic Systems Corporation");
                mVendors.Add(0x11F5, "Computing Devices Intl");
                mVendors.Add(0x11F6, "Powermatic Data Systems Ltd");
                mVendors.Add(0x11F7, "Scientific Atlanta");
                mVendors.Add(0x11F8, "PMC-Sierra Inc.");
                mVendors.Add(0x11F9, "I-Cube Inc");
                mVendors.Add(0x11FA, "Kasan Electronics Co Ltd");
                mVendors.Add(0x11FB, "Datel Inc");
                mVendors.Add(0x11FD, "High Street Consultants");
                mVendors.Add(0x11FE, "Comtrol Corp");
                mVendors.Add(0x11FF, "Scion Corp");
                mVendors.Add(0x1200, "CSS Corp");
                mVendors.Add(0x1201, "Vista Controls Corp");
                mVendors.Add(0x1202, "Network General Corp");
                mVendors.Add(0x1203, "Bayer Corporation Agfa Div");
                mVendors.Add(0x1204, "Lattice Semiconductor Corp");
                mVendors.Add(0x1205, "Array Corp");
                mVendors.Add(0x1206, "Amdahl Corp");
                mVendors.Add(0x1208, "Parsytec GmbH");
                mVendors.Add(0x1209, "Sci Systems Inc");
                mVendors.Add(0x120A, "Synaptel");
                mVendors.Add(0x120B, "Adaptive Solutions");
                mVendors.Add(0x120D, "Compression Labs Inc.");
                mVendors.Add(0x120E, "Cyclades Corporation");
                mVendors.Add(0x120F, "Essential Communications");
                mVendors.Add(0x1210, "Hyperparallel Technologies");
                mVendors.Add(0x1211, "Braintech Inc");
                mVendors.Add(0x1213, "Applied Intelligent Systems Inc");
                mVendors.Add(0x1214, "Performance Technologies Inc");
                mVendors.Add(0x1215, "Interware Co Ltd");
                mVendors.Add(0x1216, "Purup-Eskofot A/S");
                mVendors.Add(0x1217, "O2Micro Inc");
                mVendors.Add(0x1218, "Hybricon Corp");
                mVendors.Add(0x1219, "First Virtual Corp");
                mVendors.Add(0x121A, "3dfx Interactive Inc");
                mVendors.Add(0x121B, "Advanced Telecommunications Modules");
                mVendors.Add(0x121C, "Nippon Texa Co Ltd");
                mVendors.Add(0x121D, "Lippert Automationstechnik GmbH");
                mVendors.Add(0x121E, "CSPI");
                mVendors.Add(0x121F, "Arcus Technology Inc");
                mVendors.Add(0x1220, "Ariel Corporation");
                mVendors.Add(0x1221, "Contec Microelectronics Europe BV");
                mVendors.Add(0x1222, "Ancor Communications Inc");
                mVendors.Add(0x1223, "Emerson Network Power, Embedded Computing");
                mVendors.Add(0x1224, "Interactive Images");
                mVendors.Add(0x1225, "Power I/O Inc.");
                mVendors.Add(0x1227, "Tech-Source");
                mVendors.Add(0x1228, "Norsk Elektro Optikk A/S");
                mVendors.Add(0x1229, "Data Kinesis Inc.");
                mVendors.Add(0x122A, "Integrated Telecom");
                mVendors.Add(0x122B, "LG Industrial Systems Co. Ltd.");
                mVendors.Add(0x122C, "sci-worx GmbH");
                mVendors.Add(0x122D, "Aztech System Ltd");
                mVendors.Add(0x122E, "Xyratex");
                mVendors.Add(0x122F, "Andrew Corp.");
                mVendors.Add(0x1230, "Fishcamp Engineering");
                mVendors.Add(0x1231, "Woodward McCoach Inc.");
                mVendors.Add(0x1233, "Bus-Tech Inc.");
                mVendors.Add(0x1234, "Technical Corp");
                mVendors.Add(0x1236, "Sigma Designs, Inc");
                mVendors.Add(0x1237, "Alta Technology Corp.");
                mVendors.Add(0x1238, "Adtran");
                mVendors.Add(0x1239, "The 3DO Company");
                mVendors.Add(0x123A, "Visicom Laboratories Inc.");
                mVendors.Add(0x123B, "Seeq Technology Inc.");
                mVendors.Add(0x123C, "Century Systems Inc.");
                mVendors.Add(0x123D, "Engineering Design Team Inc.");
                mVendors.Add(0x123F, "C-Cube Microsystems");
                mVendors.Add(0x1240, "Marathon Technologies Corp.");
                mVendors.Add(0x1241, "DSC Communications");
                mVendors.Add(0x1242, "JNI Corporation");
                mVendors.Add(0x1243, "Delphax");
                mVendors.Add(0x1244, "AVM AUDIOVISUELLES MKTG & Computer GmbH");
                mVendors.Add(0x1245, "APD S.A.");
                mVendors.Add(0x1246, "Dipix Technologies Inc");
                mVendors.Add(0x1247, "Xylon Research Inc.");
                mVendors.Add(0x1248, "Central Data Corp.");
                mVendors.Add(0x1249, "Samsung Electronics Co. Ltd.");
                mVendors.Add(0x124A, "AEG Electrocom GmbH");
                mVendors.Add(0x124C, "Solitron Technologies Inc.");
                mVendors.Add(0x124D, "Stallion Technologies");
                mVendors.Add(0x124E, "Cylink");
                mVendors.Add(0x124F, "Infortrend Technology Inc");
                mVendors.Add(0x1250, "Hitachi Microcomputer System Ltd.");
                mVendors.Add(0x1251, "VLSI Solution OY");
                mVendors.Add(0x1253, "Guzik Technical Enterprises");
                mVendors.Add(0x1254, "Linear Systems Ltd.");
                mVendors.Add(0x1255, "Optibase Ltd.");
                mVendors.Add(0x1256, "Perceptive Solutions Inc.");
                mVendors.Add(0x1257, "Vertex Networks Inc.");
                mVendors.Add(0x1258, "Gilbarco Inc.");
                mVendors.Add(0x1259, "Allied Telesyn International");
                mVendors.Add(0x125A, "ABB Power Systems");
                mVendors.Add(0x125B, "Asix Electronics Corp.");
                mVendors.Add(0x125C, "Aurora Technologies Inc.");
                mVendors.Add(0x125D, "ESS Technology");
                mVendors.Add(0x125E, "Specialvideo Engineering SRL");
                mVendors.Add(0x125F, "Concurrent Technologies Inc.");
                mVendors.Add(0x1260, "Intersil Corporation");
                mVendors.Add(0x1261, "Matsushita-Kotobuki Electronics Indu");
                mVendors.Add(0x1262, "ES Computer Co. Ltd.");
                mVendors.Add(0x1263, "Sonic Solutions");
                mVendors.Add(0x1264, "Aval Nagasaki Corp.");
                mVendors.Add(0x1265, "Casio Computer Co. Ltd.");
                mVendors.Add(0x1266, "Microdyne Corp.");
                mVendors.Add(0x1267, "S.A. Telecommunications");
                mVendors.Add(0x1268, "Tektronix");
                mVendors.Add(0x1269, "Thomson-CSF/TTM");
                mVendors.Add(0x126A, "Lexmark International Inc.");
                mVendors.Add(0x126B, "Adax Inc.");
                mVendors.Add(0x126C, "Nortel Networks Corp.");
                mVendors.Add(0x126D, "Splash Technology Inc.");
                mVendors.Add(0x126E, "Sumitomo Metal Industries Ltd.");
                mVendors.Add(0x126F, "Silicon Motion");
                mVendors.Add(0x1270, "Olympus Optical Co. Ltd.");
                mVendors.Add(0x1271, "GW Instruments");
                mVendors.Add(0x1272, "Telematics International");
                mVendors.Add(0x1273, "Hughes Network Systems");
                mVendors.Add(0x1274, "Ensoniq");
                mVendors.Add(0x1275, "Network Appliance");
                mVendors.Add(0x1276, "Switched Network Technologies Inc.");
                mVendors.Add(0x1277, "Comstream");
                mVendors.Add(0x1278, "Transtech Parallel Systems");
                mVendors.Add(0x1279, "Transmeta Corp.");
                mVendors.Add(0x127B, "Pixera Corp");
                mVendors.Add(0x127C, "Crosspoint Solutions Inc.");
                mVendors.Add(0x127D, "Vela Research LP");
                mVendors.Add(0x127E, "Winnov L.P.");
                mVendors.Add(0x127F, "Fujifilm");
                mVendors.Add(0x1280, "Photoscript Group Ltd.");
                mVendors.Add(0x1281, "Yokogawa Electronic Corp.");
                mVendors.Add(0x1282, "Davicom Semiconductor Inc.");
                mVendors.Add(0x1283, "chandresh.j.mehta");
                mVendors.Add(0x1285, "Platform Technologies Inc.");
                mVendors.Add(0x1286, "MAZeT GmbH");
                mVendors.Add(0x1287, "LuxSonor Inc.");
                mVendors.Add(0x1288, "Timestep Corp.");
                mVendors.Add(0x1289, "AVC Technology Inc.");
                mVendors.Add(0x128A, "Asante Technologies Inc.");
                mVendors.Add(0x128B, "Transwitch Corp.");
                mVendors.Add(0x128C, "Retix Corp.");
                mVendors.Add(0x128D, "G2 Networks Inc.");
                mVendors.Add(0x128F, "Tateno Dennou Inc.");
                mVendors.Add(0x1290, "Sord Computer Corp.");
                mVendors.Add(0x1291, "NCS Computer Italia");
                mVendors.Add(0x1292, "Tritech Microelectronics Intl PTE");
                mVendors.Add(0x1293, "Media Reality Technology");
                mVendors.Add(0x1294, "Rhetorex Inc.");
                mVendors.Add(0x1295, "Imagenation Corp.");
                mVendors.Add(0x1296, "Kofax Image Products");
                mVendors.Add(0x1297, "Shuttle Computer");
                mVendors.Add(0x1298, "Spellcaster Telecommunications Inc.");
                mVendors.Add(0x1299, "Knowledge Technology Laboratories");
                mVendors.Add(0x129A, "VMETRO Inc.");
                mVendors.Add(0x129B, "Image Access");
                mVendors.Add(0x129D, "CompCore Multimedia Inc.");
                mVendors.Add(0x129E, "Victor Co. of Japan Ltd.");
                mVendors.Add(0x129F, "OEC Medical Systems Inc.");
                mVendors.Add(0x12A0, "Allen Bradley Co.");
                mVendors.Add(0x12A1, "Simpact Inc");
                mVendors.Add(0x12A2, "NewGen Systems Corp.");
                mVendors.Add(0x12A3, "Lucent Technologies AMR");
                mVendors.Add(0x12A4, "NTT Electronics Technology Co.");
                mVendors.Add(0x12A5, "Vision Dynamics Ltd.");
                mVendors.Add(0x12A6, "Scalable Networks Inc.");
                mVendors.Add(0x12A7, "AMO GmbH");
                mVendors.Add(0x12A8, "News Datacom");
                mVendors.Add(0x12A9, "Xiotech Corp.");
                mVendors.Add(0x12AA, "SDL Communications Inc.");
                mVendors.Add(0x12AB, "Yuan Yuan Enterprise Co. Ltd.");
                mVendors.Add(0x12AC, "MeasureX Corp.");
                mVendors.Add(0x12AD, "Multidata GmbH");
                mVendors.Add(0x12AE, "Alteon Networks Inc.");
                mVendors.Add(0x12AF, "TDK USA Corp.");
                mVendors.Add(0x12B0, "Jorge Scientific Corp.");
                mVendors.Add(0x12B1, "GammaLink");
                mVendors.Add(0x12B2, "General Signal Networks");
                mVendors.Add(0x12B3, "Inter-Face Co. Ltd.");
                mVendors.Add(0x12B4, "Future Tel Inc.");
                mVendors.Add(0x12B5, "Granite Systems Inc.");
                mVendors.Add(0x12B7, "Acumen");
                mVendors.Add(0x12B8, "Korg");
                mVendors.Add(0x12B9, "3Com Corporation");
                mVendors.Add(0x12BA, "Bittware, Inc");
                mVendors.Add(0x12BB, "Nippon Unisoft Corp.");
                mVendors.Add(0x12BC, "Array Microsystems");
                mVendors.Add(0x12BD, "Computerm Corp.");
                mVendors.Add(0x12BF, "Fujifilm Microdevices");
                mVendors.Add(0x12C0, "Infimed");
                mVendors.Add(0x12C1, "GMM Research Corp.");
                mVendors.Add(0x12C2, "Mentec Ltd.");
                mVendors.Add(0x12C3, "Holtek Microelectronics Inc.");
                mVendors.Add(0x12C4, "Connect Tech Inc.");
                mVendors.Add(0x12C5, "Picture Elements Inc.");
                mVendors.Add(0x12C6, "Mitani Corp.");
                mVendors.Add(0x12C7, "Dialogic Corp.");
                mVendors.Add(0x12C8, "G Force Co. Ltd.");
                mVendors.Add(0x12C9, "Gigi Operations");
                mVendors.Add(0x12CA, "Integrated Computing Engines, Inc.");
                mVendors.Add(0x12CB, "Antex Electronics Corp.");
                mVendors.Add(0x12CC, "Pluto Technologies International");
                mVendors.Add(0x12CD, "Aims Lab");
                mVendors.Add(0x12CE, "Netspeed Inc.");
                mVendors.Add(0x12CF, "Prophet Systems Inc.");
                mVendors.Add(0x12D0, "GDE Systems Inc.");
                mVendors.Add(0x12D1, "PsiTech");
                mVendors.Add(0x12D3, "Vingmed Sound A/S");
                mVendors.Add(0x12D4, "Ulticom, Inc.");
                mVendors.Add(0x12D5, "Equator Technologies");
                mVendors.Add(0x12D6, "Analogic Corp.");
                mVendors.Add(0x12D7, "Biotronic SRL");
                mVendors.Add(0x12D8, "Pericom Semiconductor");
                mVendors.Add(0x12D9, "Aculab Plc.");
                mVendors.Add(0x12DA, "TrueTime");
                mVendors.Add(0x12DB, "Annapolis Micro Systems Inc.");
                mVendors.Add(0x12DC, "Symicron Computer Communication Ltd.");
                mVendors.Add(0x12DD, "Management Graphics Inc.");
                mVendors.Add(0x12DE, "Rainbow Technologies");
                mVendors.Add(0x12DF, "SBS Technologies Inc.");
                mVendors.Add(0x12E0, "Chase Research PLC");
                mVendors.Add(0x12E1, "Nintendo Co. Ltd.");
                mVendors.Add(0x12E2, "Datum Inc. Bancomm-Timing Division");
                mVendors.Add(0x12E3, "Imation Corp. - Medical Imaging Syst");
                mVendors.Add(0x12E4, "Brooktrout Technology Inc.");
                mVendors.Add(0x12E6, "Cirel Systems");
                mVendors.Add(0x12E7, "Sebring Systems Inc");
                mVendors.Add(0x12E8, "CRISC Corp.");
                mVendors.Add(0x12E9, "GE Spacenet");
                mVendors.Add(0x12EB, "Aureal Semiconductor");
                mVendors.Add(0x12EC, "3A International Inc.");
                mVendors.Add(0x12ED, "Optivision Inc.");
                mVendors.Add(0x12EE, "Orange Micro, Inc.");
                mVendors.Add(0x12EF, "Vienna Systems");
                mVendors.Add(0x12F0, "Pentek");
                mVendors.Add(0x12F1, "Sorenson Vision Inc.");
                mVendors.Add(0x12F2, "Gammagraphx Inc.");
                mVendors.Add(0x12F4, "Megatel");
                mVendors.Add(0x12F5, "Forks");
                mVendors.Add(0x12F7, "Cognex");
                mVendors.Add(0x12F8, "Electronic-Design GmbH");
                mVendors.Add(0x12F9, "FourFold Technologies");
                mVendors.Add(0x12FB, "Spectrum Signal Processing");
                mVendors.Add(0x12FC, "Capital Equipment Corp");
                mVendors.Add(0x12FE, "esd Electronic System Design GmbH");
                mVendors.Add(0x1303, "Innovative Integration");
                mVendors.Add(0x1304, "Juniper Networks Inc.");
                mVendors.Add(0x1307, "ComputerBoards");
                mVendors.Add(0x1308, "Jato Technologies Inc.");
                mVendors.Add(0x130A, "Mitsubishi Electric Microcomputer");
                mVendors.Add(0x130B, "Colorgraphic Communications Corp");
                mVendors.Add(0x130F, "Advanet Inc.");
                mVendors.Add(0x1310, "Gespac");
                mVendors.Add(0x1312, "Robotic Vision Systems Incorporated");
                mVendors.Add(0x1313, "Yaskawa Electric Co.");
                mVendors.Add(0x1316, "Teradyne Inc.");
                mVendors.Add(0x1317, "ADMtek Inc");
                mVendors.Add(0x1318, "Packet Engines, Inc.");
                mVendors.Add(0x1319, "Forte Media");
                mVendors.Add(0x131F, "SIIG");
                mVendors.Add(0x1325, "Salix Technologies Inc");
                mVendors.Add(0x1326, "Seachange International");
                mVendors.Add(0x1328, "CIFELLI SYSTEMS CORPORATION");
                mVendors.Add(0x1331, "RadiSys Corporation");
                mVendors.Add(0x1332, "VMetro");
                mVendors.Add(0x1335, "Videomail Inc.");
                mVendors.Add(0x133D, "Prisa Networks");
                mVendors.Add(0x133F, "SCM Microsystems");
                mVendors.Add(0x1342, "Promax Systems Inc");
                mVendors.Add(0x1344, "Micron Technology, Inc.");
                mVendors.Add(0x1347, "Spectracom Corporation");
                mVendors.Add(0x134A, "DTC Technology Corp.");
                mVendors.Add(0x134B, "ARK Research Corp.");
                mVendors.Add(0x134C, "Chori Joho System Co. Ltd");
                mVendors.Add(0x134D, "PCTEL Inc.");
                mVendors.Add(0x135A, "Brain Boxes Limited");
                mVendors.Add(0x135B, "Giganet Inc.");
                mVendors.Add(0x135C, "Quatech Inc");
                mVendors.Add(0x135D, "ABB Network Partner AB");
                mVendors.Add(0x135E, "Sealevel Systems Inc.");
                mVendors.Add(0x135F, "I-Data International A-S");
                mVendors.Add(0x1360, "Meinberg Funkuhren GmbH & Co. KG");
                mVendors.Add(0x1361, "Soliton Systems K.K.");
                mVendors.Add(0x1363, "Phoenix Technologies Ltd");
                mVendors.Add(0x1365, "Hypercope Corp.");
                mVendors.Add(0x1366, "Teijin Seiki Co. Ltd.");
                mVendors.Add(0x1367, "Hitachi Zosen Corporation");
                mVendors.Add(0x1368, "Skyware Corporation");
                mVendors.Add(0x1369, "Digigram");
                mVendors.Add(0x136B, "Kawasaki Steel Corporation");
                mVendors.Add(0x136C, "Adtek System Science Co Ltd");
                mVendors.Add(0x1375, "Boeing - Sunnyvale");
                mVendors.Add(0x137A, "Mark Of The Unicorn Inc");
                mVendors.Add(0x137B, "PPT Vision");
                mVendors.Add(0x137C, "Iwatsu Electric Co Ltd");
                mVendors.Add(0x137D, "Dynachip Corporation");
                mVendors.Add(0x137E, "Patriot Scientific Corp.");
                mVendors.Add(0x1380, "Sanritz Automation Co LTC");
                mVendors.Add(0x1381, "Brains Co. Ltd");
                mVendors.Add(0x1382, "Marian - Electronic & Software");
                mVendors.Add(0x1384, "Stellar Semiconductor Inc");
                mVendors.Add(0x1385, "Netgear");
                mVendors.Add(0x1387, "Curtiss-Wright Controls Embedded Computing");
                mVendors.Add(0x1388, "Hitachi Information Technology Co Ltd");
                mVendors.Add(0x1389, "Applicom International");
                mVendors.Add(0x138B, "Tokimec Inc");
                mVendors.Add(0x138E, "Basler GMBH");
                mVendors.Add(0x138F, "Patapsco Designs Inc");
                mVendors.Add(0x1390, "Concept Development Inc.");
                mVendors.Add(0x1393, "Moxa Technologies Co Ltd");
                mVendors.Add(0x1394, "Level One Communications");
                mVendors.Add(0x1395, "Ambicom Inc");
                mVendors.Add(0x1396, "Cipher Systems Inc");
                mVendors.Add(0x1397, "Cologne Chip Designs GmbH");
                mVendors.Add(0x1398, "Clarion Co. Ltd");
                mVendors.Add(0x139A, "Alacritech Inc");
                mVendors.Add(0x139D, "Xstreams PLC/ EPL Limited");
                mVendors.Add(0x139E, "Echostar Data Networks");
                mVendors.Add(0x13A0, "Crystal Group Inc");
                mVendors.Add(0x13A1, "Kawasaki Heavy Industries Ltd");
                mVendors.Add(0x13A3, "HI-FN Inc.");
                mVendors.Add(0x13A4, "Rascom Inc");
                mVendors.Add(0x13A7, "amc330");
                mVendors.Add(0x13A8, "Exar Corp.");
                mVendors.Add(0x13A9, "Siemens Medical Solutions");
                mVendors.Add(0x13AA, "Nortel Networks - BWA Division");
                mVendors.Add(0x13AF, "T.Sqware");
                mVendors.Add(0x13B1, "Tamura Corporation");
                mVendors.Add(0x13B4, "Wellbean Co Inc");
                mVendors.Add(0x13B5, "ARM Ltd");
                mVendors.Add(0x13B6, "DLoG GMBH");
                mVendors.Add(0x13B8, "Nokia Telecommunications OY");
                mVendors.Add(0x13BD, "Sharp Corporation");
                mVendors.Add(0x13BF, "Sharewave Inc");
                mVendors.Add(0x13C0, "Microgate Corp.");
                mVendors.Add(0x13C1, "3ware Inc.");
                mVendors.Add(0x13C2, "Technotrend Systemtechnik GMBH");
                mVendors.Add(0x13C3, "Janz Computer AG");
                mVendors.Add(0x13C7, "Blue Chip Technology Ltd");
                mVendors.Add(0x13CC, "Metheus Corporation");
                mVendors.Add(0x13CF, "Studio Audio & Video Ltd");
                mVendors.Add(0x13D0, "B2C2 Inc");
                mVendors.Add(0x13D1, "AboCom Systems, Inc");
                mVendors.Add(0x13D4, "Graphics Microsystems Inc");
                mVendors.Add(0x13D6, "K.I. Technology Co Ltd");
                mVendors.Add(0x13D7, "Toshiba Engineering Corporation");
                mVendors.Add(0x13D8, "Phobos Corporation");
                mVendors.Add(0x13D9, "Apex Inc");
                mVendors.Add(0x13DC, "Netboost Corporation");
                mVendors.Add(0x13DE, "ABB Robotics Products AB");
                mVendors.Add(0x13DF, "E-Tech Inc.");
                mVendors.Add(0x13E0, "GVC Corporation");
                mVendors.Add(0x13E3, "Nest Inc");
                mVendors.Add(0x13E4, "Calculex Inc");
                mVendors.Add(0x13E5, "Telesoft Design Ltd");
                mVendors.Add(0x13E9, "Intraserver Technology Inc");
                mVendors.Add(0x13EA, "Dallas Semiconductor");
                mVendors.Add(0x13F0, "IC Plus Corporation");
                mVendors.Add(0x13F1, "OCE - Industries S.A.");
                mVendors.Add(0x13F4, "Troika Networks Inc");
                mVendors.Add(0x13F6, "C-Media Electronics Inc.");
                mVendors.Add(0x13F9, "NTT Advanced Technology Corp.");
                mVendors.Add(0x13FA, "Pentland Systems Ltd.");
                mVendors.Add(0x13FB, "Aydin Corp");
                mVendors.Add(0x13FD, "Micro Science Inc");
                mVendors.Add(0x13FE, "Advantech Co., Ltd.");
                mVendors.Add(0x13FF, "Silicon Spice Inc.");
                mVendors.Add(0x1400, "ArtX Inc");
                mVendors.Add(0x1402, "Meilhaus Electronic GmbH Germany");
                mVendors.Add(0x1404, "Fundamental Software Inc");
                mVendors.Add(0x1406, "Oce Print Logics Technologies S.A.");
                mVendors.Add(0x1407, "Lava Computer MFG Inc.");
                mVendors.Add(0x1408, "Aloka Co. Ltd");
                mVendors.Add(0x1409, "SUNIX Co., Ltd.");
                mVendors.Add(0x140A, "DSP Research Inc");
                mVendors.Add(0x140B, "Ramix Inc");
                mVendors.Add(0x140D, "Matsushita Electric Works Ltd");
                mVendors.Add(0x140F, "Salient Systems Corp");
                mVendors.Add(0x1412, "IC Ensemble, Inc.");
                mVendors.Add(0x1413, "Addonics");
                mVendors.Add(0x1415, "Oxford Semiconductor Ltd");
                mVendors.Add(0x1418, "Kyushu Electronics Systems Inc");
                mVendors.Add(0x1419, "Excel Switching Corp");
                mVendors.Add(0x141B, "Zoom Telephonics Inc");
                mVendors.Add(0x141E, "Fanuc Co. Ltd");
                mVendors.Add(0x141F, "Visiontech Ltd");
                mVendors.Add(0x1420, "Psion Dacom PLC");
                mVendors.Add(0x1425, "ASIC Designers Inc");
                mVendors.Add(0x1428, "Edec Co Ltd");
                mVendors.Add(0x1429, "Unex Technology Corp.");
                mVendors.Add(0x142A, "Kingmax Technology Inc");
                mVendors.Add(0x142B, "Radiolan");
                mVendors.Add(0x142C, "Minton Optic Industry Co Ltd");
                mVendors.Add(0x142D, "Pixstream Inc");
                mVendors.Add(0x1430, "ITT Aerospace/Communications Division");
                mVendors.Add(0x1433, "Eltec Elektronik AG");
                mVendors.Add(0x1435, "RTD Embedded Technologies, Inc.");
                mVendors.Add(0x1436, "CIS Technology Inc");
                mVendors.Add(0x1437, "Nissin Inc Co");
                mVendors.Add(0x1438, "Atmel-Dream");
                mVendors.Add(0x143F, "Lightwell Co Ltd - Zax Division");
                mVendors.Add(0x1441, "Agie SA.");
                mVendors.Add(0x1443, "Unibrain S.A.");
                mVendors.Add(0x1445, "Logical Co Ltd");
                mVendors.Add(0x1446, "Graphin Co. Ltd");
                mVendors.Add(0x1447, "Aim GMBH");
                mVendors.Add(0x1448, "Alesis Studio");
                mVendors.Add(0x144A, "ADLINK Technology Inc");
                mVendors.Add(0x144B, "Loronix Information Systems, Inc.");
                mVendors.Add(0x144D, "sanyo");
                mVendors.Add(0x1450, "Octave Communications Ind.");
                mVendors.Add(0x1451, "SP3D Chip Design GMBH");
                mVendors.Add(0x1453, "Mycom Inc");
                mVendors.Add(0x1458, "Giga-Byte Technologies");
                mVendors.Add(0x145C, "Cryptek");
                mVendors.Add(0x145F, "Baldor Electric Company");
                mVendors.Add(0x1460, "Dynarc Inc");
                mVendors.Add(0x1462, "Micro-Star International Co Ltd");
                mVendors.Add(0x1463, "Fast Corporation");
                mVendors.Add(0x1464, "Interactive Circuits & Systems Ltd");
                mVendors.Add(0x1468, "Ambit Microsystems Corp.");
                mVendors.Add(0x1469, "Cleveland Motion Controls");
                mVendors.Add(0x146C, "Ruby Tech Corp.");
                mVendors.Add(0x146D, "Tachyon Inc.");
                mVendors.Add(0x146E, "WMS Gaming");
                mVendors.Add(0x1471, "Integrated Telecom Express Inc");
                mVendors.Add(0x1473, "Zapex Technologies Inc");
                mVendors.Add(0x1474, "Doug Carson & Associates");
                mVendors.Add(0x1477, "Net Insight");
                mVendors.Add(0x1478, "Diatrend Corporation");
                mVendors.Add(0x147B, "Abit Computer Corp.");
                mVendors.Add(0x147F, "Nihon Unisys Ltd.");
                mVendors.Add(0x1482, "Isytec - Integrierte Systemtechnik Gmbh");
                mVendors.Add(0x1483, "Labway Coporation");
                mVendors.Add(0x1485, "Erma - Electronic GMBH");
                mVendors.Add(0x1489, "KYE Systems Corporation");
                mVendors.Add(0x148A, "Opto 22");
                mVendors.Add(0x148B, "Innomedialogic Inc.");
                mVendors.Add(0x148C, "C.P. Technology Co. Ltd");
                mVendors.Add(0x148D, "Digicom Systems Inc.");
                mVendors.Add(0x148E, "OSI Plus Corporation");
                mVendors.Add(0x148F, "Plant Equipment Inc.");
                mVendors.Add(0x1490, "TC Labs Pty Ltd.");
                mVendors.Add(0x1493, "Maker Communications");
                mVendors.Add(0x1495, "Tokai Communications Industry Co. Ltd");
                mVendors.Add(0x1496, "Joytech Computer Co. Ltd.");
                mVendors.Add(0x1497, "SMA Technologie AG");
                mVendors.Add(0x1498, "Tews Technologies");
                mVendors.Add(0x1499, "Micro-Technology Co Ltd");
                mVendors.Add(0x149A, "Andor Technology Ltd");
                mVendors.Add(0x149B, "Seiko Instruments Inc");
                mVendors.Add(0x149E, "Mapletree Networks Inc.");
                mVendors.Add(0x149F, "Lectron Co Ltd");
                mVendors.Add(0x14A0, "Softing GMBH");
                mVendors.Add(0x14A2, "Millennium Engineering Inc");
                mVendors.Add(0x14A4, "GVC/BCM Advanced Research");
                mVendors.Add(0x14A9, "Hivertec Inc.");
                mVendors.Add(0x14AB, "Mentor Graphics Corp.");
                mVendors.Add(0x14B1, "Nextcom K.K.");
                mVendors.Add(0x14B3, "Xpeed Inc.");
                mVendors.Add(0x14B4, "Philips Business Electronics B.V.");
                mVendors.Add(0x14B5, "Creamware GmbH");
                mVendors.Add(0x14B6, "Quantum Data Corp.");
                mVendors.Add(0x14B7, "Proxim Inc.");
                mVendors.Add(0x14B9, "Aironet Wireless Communication");
                mVendors.Add(0x14BA, "Internix Inc.");
                mVendors.Add(0x14BB, "Semtech Corporation");
                mVendors.Add(0x14BE, "L3 Communications");
                mVendors.Add(0x14C0, "Compal Electronics, Inc.");
                mVendors.Add(0x14C1, "Myricom Inc.");
                mVendors.Add(0x14C2, "DTK Computer");
                mVendors.Add(0x14C4, "Iwasaki Information Systems Co Ltd");
                mVendors.Add(0x14C5, "ABB Automation Products AB");
                mVendors.Add(0x14C6, "Data Race Inc");
                mVendors.Add(0x14C7, "Modular Technology Ltd.");
                mVendors.Add(0x14C8, "Turbocomm Tech Inc");
                mVendors.Add(0x14C9, "Odin Telesystems Inc");
                mVendors.Add(0x14CB, "Billionton Systems Inc./Cadmus Micro Inc");
                mVendors.Add(0x14CD, "Universal Scientific Ind.");
                mVendors.Add(0x14CF, "TEK Microsystems Inc.");
                mVendors.Add(0x14D4, "Panacom Technology Corporation");
                mVendors.Add(0x14D5, "Nitsuko Corporation");
                mVendors.Add(0x14D6, "Accusys Inc");
                mVendors.Add(0x14D7, "Hirakawa Hewtech Corp");
                mVendors.Add(0x14D8, "Hopf Elektronik GMBH");
                mVendors.Add(0x14D9, "Alpha Processor Inc");
                mVendors.Add(0x14DB, "Avlab Technology Inc.");
                mVendors.Add(0x14DC, "Amplicon Liveline Limited");
                mVendors.Add(0x14DD, "Imodl Inc.");
                mVendors.Add(0x14DE, "Applied Integration Corporation");
                mVendors.Add(0x14E3, "Amtelco");
                mVendors.Add(0x14E4, "Broadcom Corporation");
                mVendors.Add(0x14EA, "Planex Communications, Inc.");
                mVendors.Add(0x14EB, "Seiko Epson Corporation");
                mVendors.Add(0x14EC, "Acqiris");
                mVendors.Add(0x14ED, "Datakinetics Ltd");
                mVendors.Add(0x14EF, "Carry Computer Eng. Co Ltd");
                mVendors.Add(0x14F1, "Conexant Systems, Inc.");
                mVendors.Add(0x14F2, "Mobility Electronics, Inc.");
                mVendors.Add(0x14F4, "Tokyo Electronic Industry Co. Ltd.");
                mVendors.Add(0x14F5, "Sopac Ltd");
                mVendors.Add(0x14F6, "Coyote Technologies LLC");
                mVendors.Add(0x14F7, "Wolf Technology Inc");
                mVendors.Add(0x14F8, "Audiocodes Inc");
                mVendors.Add(0x14F9, "AG Communications");
                mVendors.Add(0x14FB, "Transas Marine (UK) Ltd");
                mVendors.Add(0x14FC, "Quadrics Ltd");
                mVendors.Add(0x14FD, "Silex Technology Inc.");
                mVendors.Add(0x14FE, "Archtek Telecom Corp.");
                mVendors.Add(0x14FF, "Twinhead International Corp.");
                mVendors.Add(0x1501, "Banksoft Canada Ltd");
                mVendors.Add(0x1502, "Mitsubishi Electric Logistics Support Co");
                mVendors.Add(0x1503, "Kawasaki LSI USA Inc");
                mVendors.Add(0x1504, "Kaiser Electronics");
                mVendors.Add(0x1506, "Chameleon Systems Inc");
                mVendors.Add(0x1507, "Htec Ltd.");
                mVendors.Add(0x1509, "First International Computer Inc");
                mVendors.Add(0x150B, "Yamashita Systems Corp");
                mVendors.Add(0x150C, "Kyopal Co Ltd");
                mVendors.Add(0x150D, "Warpspped Inc");
                mVendors.Add(0x150E, "C-Port Corporation");
                mVendors.Add(0x150F, "Intec GMBH");
                mVendors.Add(0x1510, "Behavior Tech Computer Corp");
                mVendors.Add(0x1511, "Centillium Technology Corp");
                mVendors.Add(0x1512, "Rosun Technologies Inc");
                mVendors.Add(0x1513, "Raychem");
                mVendors.Add(0x1514, "TFL LAN Inc");
                mVendors.Add(0x1515, "ICS Advent");
                mVendors.Add(0x1516, "Myson Technology Inc");
                mVendors.Add(0x1517, "Echotek Corporation");
                mVendors.Add(0x1518, "Kontron Modular Computers GmbH (PEP Modular Computers GMBH)");
                mVendors.Add(0x1519, "Telefon Aktiebolaget LM Ericsson");
                mVendors.Add(0x151A, "Globetek Inc.");
                mVendors.Add(0x151B, "Combox Ltd");
                mVendors.Add(0x151C, "Digital Audio Labs Inc");
                mVendors.Add(0x151D, "Fujitsu Computer Products Of America");
                mVendors.Add(0x151E, "Matrix Corp.");
                mVendors.Add(0x151F, "Topic Semiconductor Corp");
                mVendors.Add(0x1520, "Chaplet System Inc");
                mVendors.Add(0x1521, "Bell Corporation");
                mVendors.Add(0x1522, "Mainpine Limited");
                mVendors.Add(0x1523, "Music Semiconductors");
                mVendors.Add(0x1524, "ENE Technology Inc");
                mVendors.Add(0x1525, "Impact Technologies");
                mVendors.Add(0x1526, "ISS Inc");
                mVendors.Add(0x1527, "Solectron");
                mVendors.Add(0x1528, "Acksys");
                mVendors.Add(0x1529, "American Microsystems Inc");
                mVendors.Add(0x152A, "Quickturn Design Systems");
                mVendors.Add(0x152B, "Flytech Technology Co Ltd");
                mVendors.Add(0x152C, "Macraigor Systems LLC");
                mVendors.Add(0x152D, "Quanta Computer Inc");
                mVendors.Add(0x152E, "Melec Inc");
                mVendors.Add(0x152F, "Philips - Crypto");
                mVendors.Add(0x1532, "Echelon Corporation");
                mVendors.Add(0x1533, "Baltimore");
                mVendors.Add(0x1534, "Road Corporation");
                mVendors.Add(0x1535, "Evergreen Technologies Inc");
                mVendors.Add(0x1537, "Datalex Communcations");
                mVendors.Add(0x1538, "Aralion Inc.");
                mVendors.Add(0x1539, "Atelier Informatiques et Electronique Et");
                mVendors.Add(0x153A, "ONO Sokki");
                mVendors.Add(0x153B, "Terratec Electronic GMBH");
                mVendors.Add(0x153C, "Antal Electronic");
                mVendors.Add(0x153D, "Filanet Corporation");
                mVendors.Add(0x153E, "Techwell Inc");
                mVendors.Add(0x153F, "MIPS Technologies, Inc");
                mVendors.Add(0x1540, "Provideo Multimedia Co Ltd");
                mVendors.Add(0x1541, "Telocity Inc.");
                mVendors.Add(0x1542, "Vivid Technology Inc");
                mVendors.Add(0x1543, "Silicon Laboratories");
                mVendors.Add(0x1544, "DCM Technologies Ltd.");
                mVendors.Add(0x1545, "VisionTek");
                mVendors.Add(0x1546, "IOI Technology Corp.");
                mVendors.Add(0x1547, "Mitutoyo Corporation");
                mVendors.Add(0x1548, "Jet Propulsion Laboratory");
                mVendors.Add(0x1549, "Interconnect Systems Solutions");
                mVendors.Add(0x154A, "Max Technologies Inc.");
                mVendors.Add(0x154B, "Computex Co Ltd");
                mVendors.Add(0x154C, "Visual Technology Inc.");
                mVendors.Add(0x154D, "PAN International Industrial Corp");
                mVendors.Add(0x154E, "Servotest Ltd");
                mVendors.Add(0x154F, "Stratabeam Technology");
                mVendors.Add(0x1550, "Open Network Co Ltd");
                mVendors.Add(0x1551, "Smart Electronic Development GMBH");
                mVendors.Add(0x1553, "Chicony Electronics Co Ltd");
                mVendors.Add(0x1554, "Prolink Microsystems Corp.");
                mVendors.Add(0x1555, "Gesytec GmbH");
                mVendors.Add(0x1556, "PLD Applications");
                mVendors.Add(0x1557, "Mediastar Co. Ltd");
                mVendors.Add(0x1558, "Clevo/Kapok Computer");
                mVendors.Add(0x1559, "SI Logic Ltd");
                mVendors.Add(0x155A, "Innomedia Inc");
                mVendors.Add(0x155B, "Protac International Corp");
                mVendors.Add(0x155C, "s");
                mVendors.Add(0x155D, "MAC System Co Ltd");
                mVendors.Add(0x155E, "KUKA Roboter GmbH");
                mVendors.Add(0x155F, "Perle Systems Limited");
                mVendors.Add(0x1560, "Terayon Communications Systems");
                mVendors.Add(0x1561, "Viewgraphics Inc");
                mVendors.Add(0x1562, "Symbol Technologies, Inc.");
                mVendors.Add(0x1563, "A-Trend Technology Co Ltd");
                mVendors.Add(0x1564, "Yamakatsu Electronics Industry Co Ltd");
                mVendors.Add(0x1565, "Biostar Microtech Intl Corp");
                mVendors.Add(0x1566, "Ardent Technologies Inc");
                mVendors.Add(0x1567, "Jungsoft");
                mVendors.Add(0x1568, "DDK Electronics Inc");
                mVendors.Add(0x1569, "Palit Microsystems Inc");
                mVendors.Add(0x156A, "Avtec Systems Inc");
                mVendors.Add(0x156B, "S2io Inc");
                mVendors.Add(0x156C, "Vidac Electronics GMBH");
                mVendors.Add(0x156D, "Alpha-Top Corp");
                mVendors.Add(0x156E, "Alfa Inc.");
                mVendors.Add(0x156F, "M-Systems Flash Disk Pioneers Ltd");
                mVendors.Add(0x1570, "Lecroy Corporation");
                mVendors.Add(0x1571, "Contemporary Controls");
                mVendors.Add(0x1572, "Otis Elevator Company");
                mVendors.Add(0x1573, "Lattice - Vantis");
                mVendors.Add(0x1574, "Fairchild Semiconductor");
                mVendors.Add(0x1575, "Voltaire Advanced Data Security Ltd");
                mVendors.Add(0x1576, "Viewcast Com");
                mVendors.Add(0x1578, "Hitt");
                mVendors.Add(0x1579, "Dual Technology Corporation");
                mVendors.Add(0x157A, "Japan Elecronics Ind. Inc");
                mVendors.Add(0x157B, "Star Multimedia Corp.");
                mVendors.Add(0x157C, "Eurosoft (UK)");
                mVendors.Add(0x157D, "Gemflex Networks");
                mVendors.Add(0x157E, "Transition Networks");
                mVendors.Add(0x157F, "PX Instruments Technology Ltd");
                mVendors.Add(0x1580, "Primex Aerospace Co.");
                mVendors.Add(0x1581, "SEH Computertechnik GMBH");
                mVendors.Add(0x1582, "Cytec Corporation");
                mVendors.Add(0x1583, "Inet Technologies Inc");
                mVendors.Add(0x1584, "Uniwill Computer Corp.");
                mVendors.Add(0x1585, "Marconi Commerce Systems SRL");
                mVendors.Add(0x1586, "Lancast Inc");
                mVendors.Add(0x1587, "Konica Corporation");
                mVendors.Add(0x1588, "Solidum Systems Corp");
                mVendors.Add(0x1589, "Atlantek Microsystems Pty Ltd");
                mVendors.Add(0x158A, "Digalog Systems Inc");
                mVendors.Add(0x158B, "Allied Data Technologies");
                mVendors.Add(0x158C, "Hitachi Semiconductor & Devices Sales Co");
                mVendors.Add(0x158D, "Point Multimedia Systems");
                mVendors.Add(0x158E, "Lara Technology Inc");
                mVendors.Add(0x158F, "Ditect Coop");
                mVendors.Add(0x1590, "3pardata Inc.");
                mVendors.Add(0x1591, "ARN");
                mVendors.Add(0x1592, "Syba Tech Ltd.");
                mVendors.Add(0x1593, "Bops Inc");
                mVendors.Add(0x1594, "Netgame Ltd");
                mVendors.Add(0x1595, "Diva Systems Corp.");
                mVendors.Add(0x1596, "Folsom Research Inc");
                mVendors.Add(0x1597, "Memec Design Services");
                mVendors.Add(0x1598, "Granite Microsystems");
                mVendors.Add(0x1599, "Delta Electronics Inc");
                mVendors.Add(0x159A, "General Instrument");
                mVendors.Add(0x159B, "Faraday Technology Corp");
                mVendors.Add(0x159C, "Stratus Computer Systems");
                mVendors.Add(0x159D, "Ningbo Harrison Electronics Co Ltd");
                mVendors.Add(0x159E, "A-Max Technology Co Ltd");
                mVendors.Add(0x159F, "Galea Network Security");
                mVendors.Add(0x15A0, "Compumaster SRL");
                mVendors.Add(0x15A1, "Geocast Network Systems Inc");
                mVendors.Add(0x15A2, "Catalyst Enterprises Inc");
                mVendors.Add(0x15A3, "Italtel");
                mVendors.Add(0x15A4, "X-Net OY");
                mVendors.Add(0x15A5, "Toyota MACS Inc");
                mVendors.Add(0x15A6, "Sunlight Ultrasound Technologies Ltd");
                mVendors.Add(0x15A7, "SSE Telecom Inc");
                mVendors.Add(0x15A8, "Shanghai Communications Technologies Cen");
                mVendors.Add(0x15AA, "Moreton Bay");
                mVendors.Add(0x15AB, "Bluesteel Networks Inc");
                mVendors.Add(0x15AC, "North Atlantic Instruments");
                mVendors.Add(0x15AD, "VMware Inc.");
                mVendors.Add(0x15AE, "Amersham Pharmacia Biotech");
                mVendors.Add(0x15B0, "Zoltrix International Limited");
                mVendors.Add(0x15B1, "Source Technology Inc");
                mVendors.Add(0x15B2, "Mosaid Technologies Inc.");
                mVendors.Add(0x15B3, "Mellanox Technology");
                mVendors.Add(0x15B4, "CCI/Triad");
                mVendors.Add(0x15B5, "Cimetrics Inc");
                mVendors.Add(0x15B6, "Texas Memory Systems Inc");
                mVendors.Add(0x15B7, "Sandisk Corp.");
                mVendors.Add(0x15B8, "Addi-Data GMBH");
                mVendors.Add(0x15B9, "Maestro Digital Communications");
                mVendors.Add(0x15BA, "Impacct Technology Corp");
                mVendors.Add(0x15BB, "Portwell Inc");
                mVendors.Add(0x15BC, "Agilent Technologies");
                mVendors.Add(0x15BD, "DFI Inc.");
                mVendors.Add(0x15BE, "Sola Electronics");
                mVendors.Add(0x15BF, "High Tech Computer Corp (HTC)");
                mVendors.Add(0x15C0, "BVM Limited");
                mVendors.Add(0x15C1, "Quantel");
                mVendors.Add(0x15C2, "Newer Technology Inc");
                mVendors.Add(0x15C3, "Taiwan Mycomp Co Ltd");
                mVendors.Add(0x15C4, "EVSX Inc");
                mVendors.Add(0x15C5, "Procomp Informatics Ltd");
                mVendors.Add(0x15C6, "Technical University Of Budapest");
                mVendors.Add(0x15C7, "Tateyama System Laboratory Co Ltd");
                mVendors.Add(0x15C8, "Penta Media Co. Ltd");
                mVendors.Add(0x15C9, "Serome Technology Inc");
                mVendors.Add(0x15CA, "Bitboys OY");
                mVendors.Add(0x15CB, "AG Electronics Ltd");
                mVendors.Add(0x15CC, "Hotrail Inc.");
                mVendors.Add(0x15CD, "Dreamtech Co Ltd");
                mVendors.Add(0x15CE, "Genrad Inc.");
                mVendors.Add(0x15CF, "Hilscher GMBH");
                mVendors.Add(0x15D1, "Infineon Technologies AG");
                mVendors.Add(0x15D2, "FIC (First International Computer Inc)");
                mVendors.Add(0x15D3, "NDS Technologies Israel Ltd");
                mVendors.Add(0x15D4, "Iwill Corporation");
                mVendors.Add(0x15D5, "Tatung Co.");
                mVendors.Add(0x15D6, "Entridia Corporation");
                mVendors.Add(0x15D7, "Rockwell-Collins Inc");
                mVendors.Add(0x15D8, "Cybernetics Technology Co Ltd");
                mVendors.Add(0x15D9, "Super Micro Computer Inc");
                mVendors.Add(0x15DA, "Cyberfirm Inc.");
                mVendors.Add(0x15DB, "Applied Computing Systems Inc.");
                mVendors.Add(0x15DC, "Litronic Inc.");
                mVendors.Add(0x15DD, "Sigmatel Inc.");
                mVendors.Add(0x15DE, "Malleable Technologies Inc");
                mVendors.Add(0x15E0, "Cacheflow Inc");
                mVendors.Add(0x15E1, "Voice Technologies Group");
                mVendors.Add(0x15E2, "Quicknet Technologies Inc");
                mVendors.Add(0x15E3, "Networth Technologies Inc");
                mVendors.Add(0x15E4, "VSN Systemen BV");
                mVendors.Add(0x15E5, "Valley Technologies Inc");
                mVendors.Add(0x15E6, "Agere Inc.");
                mVendors.Add(0x15E7, "GET Engineering Corp.");
                mVendors.Add(0x15E8, "National Datacomm Corp.");
                mVendors.Add(0x15E9, "Pacific Digital Corp.");
                mVendors.Add(0x15EA, "Tokyo Denshi Sekei K.K.");
                mVendors.Add(0x15EB, "Drsearch GMBH");
                mVendors.Add(0x15EC, "Beckhoff GMBH");
                mVendors.Add(0x15ED, "Macrolink Inc");
                mVendors.Add(0x15EE, "IN Win Development Inc.");
                mVendors.Add(0x15EF, "Intelligent Paradigm Inc");
                mVendors.Add(0x15F0, "B-Tree Systems Inc");
                mVendors.Add(0x15F1, "Times N Systems Inc");
                mVendors.Add(0x15F2, "Diagnostic Instruments Inc");
                mVendors.Add(0x15F3, "Digitmedia Corp.");
                mVendors.Add(0x15F4, "Valuesoft");
                mVendors.Add(0x15F5, "Power Micro Research");
                mVendors.Add(0x15F6, "Extreme Packet Device Inc");
                mVendors.Add(0x15F7, "Banctec");
                mVendors.Add(0x15F8, "Koga Electronics Co");
                mVendors.Add(0x15F9, "Zenith Electronics Corporation");
                mVendors.Add(0x15FA, "Axzam Corporation");
                mVendors.Add(0x15FB, "Zilog Inc.");
                mVendors.Add(0x15FC, "Techsan Electronics Co Ltd");
                mVendors.Add(0x15FD, "N-Cubed.Net");
                mVendors.Add(0x15FE, "Kinpo Electronics Inc");
                mVendors.Add(0x15FF, "Fastpoint Technologies Inc.");
                mVendors.Add(0x1600, "Northrop Grumman - Canada Ltd");
                mVendors.Add(0x1601, "Tenta Technology");
                mVendors.Add(0x1602, "Prosys-TEC Inc.");
                mVendors.Add(0x1603, "Nokia Wireless Business Communications");
                mVendors.Add(0x1604, "Central System Research Co Ltd");
                mVendors.Add(0x1605, "Pairgain Technologies");
                mVendors.Add(0x1606, "Europop AG");
                mVendors.Add(0x1607, "Lava Semiconductor Manufacturing Inc.");
                mVendors.Add(0x1608, "Automated Wagering International");
                mVendors.Add(0x1609, "Sciemetric Instruments Inc");
                mVendors.Add(0x160A, "Kollmorgen Servotronix");
                mVendors.Add(0x160B, "Onkyo Corp.");
                mVendors.Add(0x160C, "Oregon Micro Systems Inc.");
                mVendors.Add(0x160D, "Aaeon Electronics Inc");
                mVendors.Add(0x160E, "CML Emergency Services");
                mVendors.Add(0x160F, "ITEC Co Ltd");
                mVendors.Add(0x1610, "Tottori Sanyo Electric Co Ltd");
                mVendors.Add(0x1611, "Bel Fuse Inc.");
                mVendors.Add(0x1612, "Telesynergy Research Inc.");
                mVendors.Add(0x1613, "System Craft Inc.");
                mVendors.Add(0x1614, "Jace Tech Inc.");
                mVendors.Add(0x1615, "Equus Computer Systems Inc");
                mVendors.Add(0x1616, "Iotech Inc.");
                mVendors.Add(0x1617, "Rapidstream Inc");
                mVendors.Add(0x1618, "Esec SA");
                mVendors.Add(0x1619, "FarSite Communications Limited");
                mVendors.Add(0x161B, "Mobilian Israel Ltd");
                mVendors.Add(0x161C, "Berkshire Products");
                mVendors.Add(0x161D, "Gatec");
                mVendors.Add(0x161E, "Kyoei Sangyo Co Ltd");
                mVendors.Add(0x161F, "Arima Computer Corporation");
                mVendors.Add(0x1620, "Sigmacom Co Ltd");
                mVendors.Add(0x1621, "Lynx Studio Technology Inc");
                mVendors.Add(0x1622, "Nokia Home Communications");
                mVendors.Add(0x1623, "KRF Tech Ltd");
                mVendors.Add(0x1624, "CE Infosys GMBH");
                mVendors.Add(0x1625, "Warp Nine Engineering");
                mVendors.Add(0x1626, "TDK Semiconductor Corp.");
                mVendors.Add(0x1627, "BCom Electronics Inc");
                mVendors.Add(0x1629, "Kongsberg Spacetec a.s.");
                mVendors.Add(0x162A, "Sejin Computerland Co Ltd");
                mVendors.Add(0x162B, "Shanghai Bell Company Limited");
                mVendors.Add(0x162C, "C&H Technologies Inc");
                mVendors.Add(0x162D, "Reprosoft Co Ltd");
                mVendors.Add(0x162E, "Margi Systems Inc");
                mVendors.Add(0x162F, "Rohde & Schwarz GMBH & Co KG");
                mVendors.Add(0x1630, "Sky Computers Inc");
                mVendors.Add(0x1631, "NEC Computer International");
                mVendors.Add(0x1632, "Verisys Inc");
                mVendors.Add(0x1633, "Adac Corporation");
                mVendors.Add(0x1634, "Visionglobal Network Corp.");
                mVendors.Add(0x1635, "Decros");
                mVendors.Add(0x1636, "Jean Company Ltd");
                mVendors.Add(0x1637, "NSI");
                mVendors.Add(0x1638, "Eumitcom Technology Inc");
                mVendors.Add(0x163A, "Air Prime Inc");
                mVendors.Add(0x163B, "Glotrex Co Ltd");
                mVendors.Add(0x163C, "intel");
                mVendors.Add(0x163D, "Heidelberg Digital LLC");
                mVendors.Add(0x163E, "3dpower");
                mVendors.Add(0x163F, "Renishaw PLC");
                mVendors.Add(0x1640, "Intelliworxx Inc");
                mVendors.Add(0x1641, "MKNet Corporation");
                mVendors.Add(0x1642, "Bitland");
                mVendors.Add(0x1643, "Hajime Industries Ltd");
                mVendors.Add(0x1644, "Western Avionics Ltd");
                mVendors.Add(0x1645, "Quick-Serv. Computer Co. Ltd");
                mVendors.Add(0x1646, "Nippon Systemware Co Ltd");
                mVendors.Add(0x1647, "Hertz Systemtechnik GMBH");
                mVendors.Add(0x1648, "MeltDown Systems LLC");
                mVendors.Add(0x1649, "Jupiter Systems");
                mVendors.Add(0x164A, "Aiwa Co. Ltd");
                mVendors.Add(0x164C, "Department Of Defense");
                mVendors.Add(0x164D, "Ishoni Networks");
                mVendors.Add(0x164E, "Micrel Inc.");
                mVendors.Add(0x164F, "Datavoice (Pty) Ltd.");
                mVendors.Add(0x1650, "Admore Technology Inc.");
                mVendors.Add(0x1651, "Chaparral Network Storage");
                mVendors.Add(0x1652, "Spectrum Digital Inc.");
                mVendors.Add(0x1653, "Nature Worldwide Technology Corp");
                mVendors.Add(0x1654, "Sonicwall Inc");
                mVendors.Add(0x1655, "Dazzle Multimedia Inc.");
                mVendors.Add(0x1656, "Insyde Software Corp");
                mVendors.Add(0x1657, "Brocade Communications Systems");
                mVendors.Add(0x1658, "Med Associates Inc.");
                mVendors.Add(0x1659, "Shiba Denshi Systems Inc.");
                mVendors.Add(0x165A, "Epix Inc.");
                mVendors.Add(0x165B, "Real-Time Digital Inc.");
                mVendors.Add(0x165C, "Kondo Kagaku");
                mVendors.Add(0x165D, "Hsing Tech. Enterprise Co. Ltd.");
                mVendors.Add(0x165E, "Hyunju Computer Co. Ltd.");
                mVendors.Add(0x165F, "Comartsystem Korea");
                mVendors.Add(0x1660, "Network Security Technologies Inc. (Net");
                mVendors.Add(0x1661, "Worldspace Corp.");
                mVendors.Add(0x1662, "Int Labs");
                mVendors.Add(0x1663, "Elmec Inc. Ltd.");
                mVendors.Add(0x1664, "Fastfame Technology Co. Ltd.");
                mVendors.Add(0x1665, "Edax Inc.");
                mVendors.Add(0x1666, "Norpak Corporation");
                mVendors.Add(0x1667, "CoSystems Inc.");
                mVendors.Add(0x1668, "Actiontec Electronics Inc.");
                mVendors.Add(0x166A, "Komatsu Ltd.");
                mVendors.Add(0x166B, "Supernet Inc.");
                mVendors.Add(0x166C, "Shade Ltd.");
                mVendors.Add(0x166D, "Sibyte Inc.");
                mVendors.Add(0x166E, "Schneider Automation Inc.");
                mVendors.Add(0x166F, "Televox Software Inc.");
                mVendors.Add(0x1670, "Rearden Steel");
                mVendors.Add(0x1671, "Atan Technology Inc.");
                mVendors.Add(0x1672, "Unitec Co. Ltd.");
                mVendors.Add(0x1673, "pctel");
                mVendors.Add(0x1675, "Square Wave Technology");
                mVendors.Add(0x1676, "Emachines Inc.");
                mVendors.Add(0x1677, "Bernecker + Rainer");
                mVendors.Add(0x1678, "INH Semiconductor");
                mVendors.Add(0x1679, "Tokyo Electron Device Ltd.");
                mVendors.Add(0x167F, "iba AG");
                mVendors.Add(0x1680, "Dunti Corp.");
                mVendors.Add(0x1681, "Hercules");
                mVendors.Add(0x1682, "PINE Technology, Ltd.");
                mVendors.Add(0x1688, "CastleNet Technology Inc.");
                mVendors.Add(0x168A, "Utimaco Safeware AG");
                mVendors.Add(0x168B, "Circut Assembly Corp.");
                mVendors.Add(0x168C, "Atheros Communications Inc.");
                mVendors.Add(0x168D, "NMI Electronics Ltd.");
                mVendors.Add(0x168E, "Hyundai MultiCAV Computer Co. Ltd.");
                mVendors.Add(0x168F, "KDS Innotech Corp.");
                mVendors.Add(0x1690, "NetContinuum, Inc.");
                mVendors.Add(0x1693, "FERMA");
                mVendors.Add(0x1695, "EPoX Computer Co., Ltd.");
                mVendors.Add(0x16AE, "SafeNet Inc.");
                mVendors.Add(0x16B3, "CNF Mobile Solutions");
                mVendors.Add(0x16B8, "Sonnet Technologies, Inc.");
                mVendors.Add(0x16CA, "Cenatek Inc.");
                mVendors.Add(0x16CB, "Minolta Co. Ltd.");
                mVendors.Add(0x16CC, "Inari Inc.");
                mVendors.Add(0x16D0, "Systemax");
                mVendors.Add(0x16E0, "Third Millenium Test Solutions, Inc.");
                mVendors.Add(0x16E5, "Intellon Corporation");
                mVendors.Add(0x16EC, "U.S. Robotics");
                mVendors.Add(0x16F0, "TLA Inc.");
                mVendors.Add(0x16F1, "Adicti Corp.");
                mVendors.Add(0x16F3, "Jetway Information Co., Ltd");
                mVendors.Add(0x16F6, "VideoTele.com Inc.");
                mVendors.Add(0x1700, "Antara LLC");
                mVendors.Add(0x1701, "Interactive Computer Products Inc.");
                mVendors.Add(0x1702, "Internet Machines Corp.");
                mVendors.Add(0x1703, "Desana Systems");
                mVendors.Add(0x1704, "Clearwater Networks");
                mVendors.Add(0x1705, "Digital First");
                mVendors.Add(0x1706, "Pacific Broadband Communications");
                mVendors.Add(0x1707, "Cogency Semiconductor Inc.");
                mVendors.Add(0x1708, "Harris Corp.");
                mVendors.Add(0x1709, "Zarlink Semiconductor");
                mVendors.Add(0x170A, "Alpine Electronics Inc.");
                mVendors.Add(0x170B, "NetOctave Inc.");
                mVendors.Add(0x170C, "YottaYotta Inc.");
                mVendors.Add(0x170D, "SensoMotoric Instruments GmbH");
                mVendors.Add(0x170E, "San Valley Systems, Inc.");
                mVendors.Add(0x170F, "Cyberdyne Inc.");
                mVendors.Add(0x1710, "Pelago Nutworks");
                mVendors.Add(0x1711, "MyName Technologies, Inc.");
                mVendors.Add(0x1712, "NICE Systems Inc.");
                mVendors.Add(0x1713, "TOPCON Corp.");
                mVendors.Add(0x1725, "Vitesse Semiconductor");
                mVendors.Add(0x1734, "Fujitsu-Siemens Computers GmbH");
                mVendors.Add(0x1737, "LinkSys");
                mVendors.Add(0x173B, "Altima Communications Inc.");
                mVendors.Add(0x1743, "Peppercon AG");
                mVendors.Add(0x174B, "PC Partner Limited");
                mVendors.Add(0x1752, "Global Brands Manufacture Ltd.");
                mVendors.Add(0x1753, "TeraRecon, Inc.");
                mVendors.Add(0x1755, "Alchemy Semiconductor Inc.");
                mVendors.Add(0x176A, "General Dynamics Canada");
                mVendors.Add(0x1789, "Ennyah Technologies Corp");
                mVendors.Add(0x1793, "Unitech Electronics Co., Ltd");
                mVendors.Add(0x17A7, "Start Network Technology Co., Ltd.");
                mVendors.Add(0x17AA, "Legend Ltd. (Beijing)");
                mVendors.Add(0x17AB, "Phillips Components");
                mVendors.Add(0x17AF, "Hightech Information Systems, Ltd.");
                mVendors.Add(0x17BE, "Philips Semiconductors");
                mVendors.Add(0x17C0, "Wistron Corp.");
                mVendors.Add(0x17C4, "Movita");
                mVendors.Add(0x17CC, "NetChip");
                mVendors.Add(0x17D5, "Neterion Inc.");
                mVendors.Add(0x17E9, "DH electronics GmbH");
                mVendors.Add(0x17EE, "Connect Components, Ltd.");
                mVendors.Add(0x1813, "Ambient Technologies Inc.");
                mVendors.Add(0x1814, "Ralink Technology, Corp");
                mVendors.Add(0x1815, "devolo AG");
                mVendors.Add(0x1820, "InfiniCon Systems, Inc.");
                mVendors.Add(0x1824, "Avocent");
                mVendors.Add(0x1860, "Primagraphics Ltd.");
                mVendors.Add(0x186C, "Humusoft S.R.O");
                mVendors.Add(0x1887, "Elan Digital Systems Ltd");
                mVendors.Add(0x1888, "Varisys Limited");
                mVendors.Add(0x188D, "Millogic Ltd.");
                mVendors.Add(0x1890, "Egenera, Inc.");
                mVendors.Add(0x18BC, "Info-Tek Corp.");
                mVendors.Add(0x18C9, "ARVOO Engineering BV");
                mVendors.Add(0x18CA, "XGI Technology Inc");
                mVendors.Add(0x18F1, "Spectrum Systementwicklung Microelectronic GmbH");
                mVendors.Add(0x18F4, "Napatech A/S");
                mVendors.Add(0x18F7, "Commtech, Inc.");
                mVendors.Add(0x18FB, "Resilience Corporation");
                mVendors.Add(0x1905, "WIS Technology, Inc.");
                mVendors.Add(0x1910, "Seaway Networks");
                mVendors.Add(0x1971, "AGEIA Technologies, Inc.");
                mVendors.Add(0x19A8, "DAQDATA GmbH");
                mVendors.Add(0x19AC, "Kasten Chase Applied Research");
                mVendors.Add(0x19E2, "Vector Informatik GmbH");
                mVendors.Add(0x1A08, "Linux Networx");
                mVendors.Add(0x1A42, "Imaginant");
                mVendors.Add(0x1B13, "Jaton Corporation USA");
                mVendors.Add(0x1DE1, "Tekram");
                mVendors.Add(0x1FCF, "Miranda Technologies Ltd.");
                mVendors.Add(0x2001, "Temporal Research Ltd");
                mVendors.Add(0x2646, "Kingston Technology Co.");
                mVendors.Add(0x270F, "ChainTech Computer Co. Ltd.");
                mVendors.Add(0x2EC1, "Zenic Inc");
                mVendors.Add(0x3388, "Hint Corp.");
                mVendors.Add(0x3411, "Quantum Designs (H.K.) Inc.");
                mVendors.Add(0x3513, "ARCOM Control Systems Ltd.");
                mVendors.Add(0x38EF, "4links");
                mVendors.Add(0x3D3D, "3Dlabs, Inc. Ltd");
                mVendors.Add(0x4005, "Avance Logic Inc.");
                mVendors.Add(0x4144, "Alpha Data");
                mVendors.Add(0x416C, "Aladdin Knowledge Systems");
                mVendors.Add(0x4680, "UMAX Computer Corp.");
                mVendors.Add(0x4843, "Hercules Computer Technology");
                mVendors.Add(0x4943, "Growth Networks");
                mVendors.Add(0x4954, "Integral Technologies");
                mVendors.Add(0x4978, "Axil Computer Inc.");
                mVendors.Add(0x4C48, "Lung Hwa Electronics");
                mVendors.Add(0x4C53, "SBS-OR Industrial Computers");
                mVendors.Add(0x4CA1, "Seanix Technology Inc");
                mVendors.Add(0x4D51, "Mediaq Inc.");
                mVendors.Add(0x4D54, "Microtechnica Co Ltd");
                mVendors.Add(0x4DDC, "ILC Data Device Corp.");
                mVendors.Add(0x5053, "TBS/Voyetra Technologies");
                mVendors.Add(0x5136, "S S Technologies");
                mVendors.Add(0x5143, "Qualcomm Inc.");
                mVendors.Add(0x5333, "S3 Graphics Co., Ltd");
                mVendors.Add(0x544C, "Teralogic Inc");
                mVendors.Add(0x5555, "Genroco Inc.");
                mVendors.Add(0x6409, "Logitec Corp.");
                mVendors.Add(0x6666, "Decision Computer International Co.");
                mVendors.Add(0x7604, "O.N. Electric Co. Ltd.");
                mVendors.Add(0x8086, "Intel Corporation");
                mVendors.Add(0x8866, "T-Square Design Inc.");
                mVendors.Add(0x8888, "Silicon Magic");
                mVendors.Add(0x8E0E, "Computone Corporation");
                mVendors.Add(0x9004, "Adaptec Inc");
                mVendors.Add(0x9005, "Adaptec Inc");
                mVendors.Add(0x919A, "Gigapixel Corp");
                mVendors.Add(0x9412, "Holtek");
                mVendors.Add(0x9699, "Omni Media Technology Inc.");
                mVendors.Add(0x9902, "StarGen, Inc.");
                mVendors.Add(0xA0A0, "Aopen Inc.");
                mVendors.Add(0xA0F1, "Unisys Corporation");
                mVendors.Add(0xA200, "NEC Corp.");
                mVendors.Add(0xA259, "Hewlett Packard");
                mVendors.Add(0xA304, "Sony");
                mVendors.Add(0xA727, "3com Corporation");
                mVendors.Add(0xAA42, "Scitex Digital Video");
                mVendors.Add(0xAC1E, "Digital Receiver Technology Inc");
                mVendors.Add(0xB1B3, "Shiva Europe Ltd.");
                mVendors.Add(0xB894, "Brown & Sharpe Mfg. Co.");
                mVendors.Add(0xBEEF, "Mindstream Computing");
                mVendors.Add(0xC001, "TSI Telsys");
                mVendors.Add(0xC0A9, "Micron/Crucial Technology");
                mVendors.Add(0xC0DE, "Motorola");
                mVendors.Add(0xC0FE, "Motion Engineering Inc.");
                mVendors.Add(0xC622, "Hudson Soft Co Ltd");
                mVendors.Add(0xCA50, "Varian Australia Pty. Ltd.");
                mVendors.Add(0xCAFE, "Chrysalis-ITS");
                mVendors.Add(0xCCCC, "Catapult Communications");
                mVendors.Add(0xD4D4, "Curtiss-Wright Controls Embedded Computing");
                mVendors.Add(0xDC93, "Dawicontrol");
                mVendors.Add(0xDEAD, "Indigita Corporation");
                mVendors.Add(0xDEAF, "Middle Digital, Inc");
                mVendors.Add(0xE159, "Tiger Jet Network Inc");
                mVendors.Add(0xE4BF, "EKF Elektronik GMBH");
                mVendors.Add(0xEA01, "Eagle Technology");
                mVendors.Add(0xEABB, "Aashima Technology B.V.");
                mVendors.Add(0xEACE, "Endace Measurement Systems Ltd.");
                mVendors.Add(0xECC0, "Echo Digital Audio Corporation");
                mVendors.Add(0xEDD8, "ARK Logic, Inc");
                mVendors.Add(0xF5F5, "F5 Networks Inc.");
                mVendors.Add(0xFA57, "Interagon AS");
                */

                #endregion Fill vendors

                #region Fill devices

                
                mDevices.Add(0x80861237,@"82440LX/EX");
                mDevices.Add(0x80867000,@"82371SB");
                mDevices.Add(0x80867010,@"82371SB");
                mDevices.Add(0x80867113,@"82371AB/EB/MB");
                mDevices.Add(0x101300B8,@"CL-GD5446");
                mDevices.Add(0x10222000,@"Am79C970/1/2/3/5/6");
                mDevices.Add(0x10EC8029,@"RTL8029(as)pci ethernet nic");
                mDevices.Add(0x12745000,@"ES1373");

                //For now it is too slow to add all vendors

                //mDevices.Add(0x003D00D1,@"mx98715/25");
                //mDevices.Add(0x0E110001,@"PCI to EISA Bridge");
                //mDevices.Add(0x0E110002,@"ISA Bridge");
                //mDevices.Add(0x0E11000F,@"CPQB1A9");
                //mDevices.Add(0x0E110012,@"686P7");
                //mDevices.Add(0x0E110049,@"NC7132");
                //mDevices.Add(0x0E11004A,@"NC6136");
                //mDevices.Add(0x0E1100C0,@"Adaptec AIC-7899G");
                //mDevices.Add(0x0E110508,@"Neteligent 4/16 TR");
                //mDevices.Add(0x0E111000,@"Triflex Model 1000");
                //mDevices.Add(0x0E112000,@"Triflex Model 2000");
                //mDevices.Add(0x0E113032,@"QVision 1280/p v0");
                //mDevices.Add(0x0E113033,@"QVision 1280/p v1");
                //mDevices.Add(0x0E113034,@"QVision 1280/p v2");
                //mDevices.Add(0x0E114000,@"Triflex Model 4000");
                //mDevices.Add(0x0E116010,@"Model 6010");
                //mDevices.Add(0x0E117020,@"USB Controller");
                //mDevices.Add(0x0E11A0EC,@"Original Compaq fibre Channel HBA");
                //mDevices.Add(0x0E11A0F0,@"Advanced System Management Controller");
                //mDevices.Add(0x0E11A0F3,@"Triflex PCI to ISA PnP Bridge");
                //mDevices.Add(0x0E11A0F7,@"PCI Hotplug Controller");
                //mDevices.Add(0x0E11A0F8,@"ZFMicro");
                //mDevices.Add(0x0E11A0FC,@"HPFC-5166A");
                //mDevices.Add(0x0E11AE29,@"MIS-L");
                //mDevices.Add(0x0E11AE2A,@"MPC");
                //mDevices.Add(0x0E11AE2B,@"MIS-E");
                //mDevices.Add(0x0E11AE31,@"System Management Controller");
                //mDevices.Add(0x0E11AE32,@"Netelligent 10/100 TX PCI UTP TLAN 2.3");
                //mDevices.Add(0x0E11AE33,@"Triflex");
                //mDevices.Add(0x0E11AE34,@"Netelligent 10 T PCI UTP TLAN 2.3");
                //mDevices.Add(0x0E11AE35,@"Integrated NetFlex 3/P TLAN 2.3");
                //mDevices.Add(0x0E11AE40,@"Dual Port Netelligent 10/100 TX PCI TLAN");
                //mDevices.Add(0x0E11AE43,@"Integrated Netelligent 10/100 TX PCI");
                //mDevices.Add(0x0E11AE69,@"CETUS-L");
                //mDevices.Add(0x0E11AE6C,@"DRACO");
                //mDevices.Add(0x0E11AE6D,@"NorthStar");
                //mDevices.Add(0x0E11B011,@"Dual Port Netelligent 10/100 TX");
                //mDevices.Add(0x0E11B012,@"Netelligent 10 T/2");
                //mDevices.Add(0x0E11B01E,@"NC3120");
                //mDevices.Add(0x0E11B01F,@"NC3122");
                //mDevices.Add(0x0E11B02F,@"NC1120");
                //mDevices.Add(0x0E11B030,@"Netelligent WS 1500");
                //mDevices.Add(0x0E11B04A,@"699209-003");
                //mDevices.Add(0x0E11B0C6,@"NC3161");
                //mDevices.Add(0x0E11B0C7,@"NC3160");
                //mDevices.Add(0x0E11B0D7,@"NC3121 rev. A & B");
                //mDevices.Add(0x0E11B0DD,@"NC3131");
                //mDevices.Add(0x0E11B0DE,@"NC3132");
                //mDevices.Add(0x0E11B0DF,@"NC6132");
                //mDevices.Add(0x0E11B0E0,@"NC6133");
                //mDevices.Add(0x0E11B0E1,@"NC3133");
                //mDevices.Add(0x0E11B123,@"NC6134");
                //mDevices.Add(0x0E11B134,@"NC3163");
                //mDevices.Add(0x0E11B13C,@"NC3162");
                //mDevices.Add(0x0E11B144,@"NC3123");
                //mDevices.Add(0x0E11B163,@"NC3134");
                //mDevices.Add(0x0E11B164,@"NC3135");
                //mDevices.Add(0x0E11B178,@"CISSB");
                //mDevices.Add(0x0E11B196,@"0230");
                //mDevices.Add(0x0E11B1A4,@"NC7131");
                //mDevices.Add(0x0E11B203,@"iLo");
                //mDevices.Add(0x0E11B204,@"iLo");
                //mDevices.Add(0x0E11F095,@"FCA2101");
                //mDevices.Add(0x0E11F130,@"ThunderLAN 1.0 NetFlex-3/P");
                //mDevices.Add(0x0E11F150,@"ThunderLAN 2.3 NetFlex-3/P with BNC");
                //mDevices.Add(0x0E11F700,@"LP7000 Compaq/Emulex Fibre Channel HBA");
                //mDevices.Add(0x0E11F800,@"LP8000 Compaq/Emulex Fibre Channel HBA");
                //mDevices.Add(0x10000001,@"LSI53C810");
                //mDevices.Add(0x10000002,@"53C820");
                //mDevices.Add(0x10000003,@"LSI53C1010-33");
                //mDevices.Add(0x10000004,@"53C815");
                //mDevices.Add(0x10000005,@"53C810AP");
                //mDevices.Add(0x10000006,@"LSI53C860/860E");
                //mDevices.Add(0x1000000A,@"LSI53C1510");
                //mDevices.Add(0x1000000B,@"LSI53C896/897");
                //mDevices.Add(0x1000000C,@"LSI53C895");
                //mDevices.Add(0x1000000D,@"53C885");
                //mDevices.Add(0x1000000F,@"53C875/875E/876/876E");
                //mDevices.Add(0x10000010,@"LSI53C1510");
                //mDevices.Add(0x10000012,@"LSI53C895A");
                //mDevices.Add(0x10000013,@"LSI53C875A");
                //mDevices.Add(0x10000020,@"LSI53C1010-33");
                //mDevices.Add(0x10000021,@"LSI53C1000/1000R/1010R/1010-66");
                //mDevices.Add(0x10000030,@"LSI53C1020/1030");
                //mDevices.Add(0x10000031,@"LSI53C1030ZC");
                //mDevices.Add(0x10000032,@"LSI53C1035");
                //mDevices.Add(0x10000035,@"LSI53C1035");
                //mDevices.Add(0x10000040,@"LSI53C1035");
                //mDevices.Add(0x10000050,@"LSISAS1064");
                //mDevices.Add(0x10000054,@"LSISAS1068");
                //mDevices.Add(0x10000056,@"LSISAS1064ET");
                //mDevices.Add(0x10000058,@"LSISAS1068E");
                //mDevices.Add(0x10000062,@"LSISAS1078");
                //mDevices.Add(0x1000008F,@"53C810");
                //mDevices.Add(0x10000621,@"LSIFC909");
                //mDevices.Add(0x10000622,@"LSIFC929");
                //mDevices.Add(0x10000623,@"LSIFC929");
                //mDevices.Add(0x10000624,@"LSIFC919");
                //mDevices.Add(0x10000625,@"LSIFC919");
                //mDevices.Add(0x10000626,@"LSIFC929X");
                //mDevices.Add(0x10000628,@"LSIFC919X");
                //mDevices.Add(0x10000630,@"LSIFC920");
                //mDevices.Add(0x10000640,@"LSIFC949X");
                //mDevices.Add(0x10000642,@"LSIFC939X");
                //mDevices.Add(0x10000646,@"LSIFC949ES");
                //mDevices.Add(0x10000701,@"53C885");
                //mDevices.Add(0x10000702,@"Yellowfin G-NIC");
                //mDevices.Add(0x10000901,@"61C102");
                //mDevices.Add(0x10001000,@"63C815");
                //mDevices.Add(0x10001001,@"53C895");
                //mDevices.Add(0x10001010,@"LSI 53C102000");
                //mDevices.Add(0x10010010,@"ispLSI1032E");
                //mDevices.Add(0x10010011,@"ispLSI1032E");
                //mDevices.Add(0x10010012,@"ispLSI1032E");
                //mDevices.Add(0x10010013,@"ispLSI1032E");
                //mDevices.Add(0x10010014,@"ispLSI1032E");
                //mDevices.Add(0x10010015,@"ispLSI1032E");
                //mDevices.Add(0x10010016,@"ispLSI1032E");
                //mDevices.Add(0x10010017,@"ispLSI1032E");
                //mDevices.Add(0x10010020,@"ispLSI1032E");
                //mDevices.Add(0x10020B12,@"ati 1900");
                //mDevices.Add(0x10023150,@"M24 1P");
                //mDevices.Add(0x10023154,@"M24 1T");
                //mDevices.Add(0x10023E50,@"RV380");
                //mDevices.Add(0x10023E54,@"RV380");
                //mDevices.Add(0x10024136,@"A3");
                //mDevices.Add(0x10024137,@"RS200");
                //mDevices.Add(0x10024144,@"R300");
                //mDevices.Add(0x10024145,@"R300");
                //mDevices.Add(0x10024146,@"R300");
                //mDevices.Add(0x10024147,@"ATI Fire GL Z1  Video Accelerator");
                //mDevices.Add(0x10024148,@"R350");
                //mDevices.Add(0x10024150,@"rv350");
                //mDevices.Add(0x10024151,@"RV350");
                //mDevices.Add(0x10024152,@"RV360");
                //mDevices.Add(0x10024153,@"RV350");
                //mDevices.Add(0x10024158,@"200E17af");
                //mDevices.Add(0x10024164,@"Radeon 9500 Series, secondary");
                //mDevices.Add(0x10024167,@"ATI Fire GL Z1 4P SECONDARY Video");
                //mDevices.Add(0x10024168,@"R350");
                //mDevices.Add(0x10024170,@"RV350");
                //mDevices.Add(0x10024171,@"RV350");
                //mDevices.Add(0x10024172,@"RV360");
                //mDevices.Add(0x10024173,@"RV350");
                //mDevices.Add(0x10024242,@"R200AIW");
                //mDevices.Add(0x10024243,@"Lucent OHCI IEEE1394 Host Controller");
                //mDevices.Add(0x10024336,@"rs200");
                //mDevices.Add(0x10024337,@"RS200M");
                //mDevices.Add(0x10024341,@"AD1981");
                //mDevices.Add(0x10024342,@"IXP 150");
                //mDevices.Add(0x10024345,@"IXP 150");
                //mDevices.Add(0x10024347,@"IXP 150");
                //mDevices.Add(0x10024348,@"IXP 150");
                //mDevices.Add(0x10024349,@"IXP 1xx/2xx");
                //mDevices.Add(0x1002434C,@"IXP 150");
                //mDevices.Add(0x10024353,@"IXP 150");
                //mDevices.Add(0x10024354,@"215r2qzua21");
                //mDevices.Add(0x10024358,@"216l0sas25");
                //mDevices.Add(0x10024369,@"IXP 3xx");
                //mDevices.Add(0x1002436E,@"IXP 3xx");
                //mDevices.Add(0x10024370,@"IXP_AUDIO_400");
                //mDevices.Add(0x10024372,@"x200");
                //mDevices.Add(0x10024376,@"SB4xx");
                //mDevices.Add(0x10024378,@"AC97");
                //mDevices.Add(0x10024379,@"SB4xx");
                //mDevices.Add(0x1002437A,@"SB4xx");
                //mDevices.Add(0x1002437B,@"OEM Intel Corporation&");
                //mDevices.Add(0x10024381,@"ATI ?");
                //mDevices.Add(0x10024385,@"ATI RD600/RS600");
                //mDevices.Add(0x10024386,@"690G");
                //mDevices.Add(0x1002438C,@"RD600/RS600");
                //mDevices.Add(0x10024390,@"SB7xx");
                //mDevices.Add(0x10024391,@"ATI SB700");
                //mDevices.Add(0x10024392,@"ATI SB700");
                //mDevices.Add(0x10024393,@"ATI ?");
                //mDevices.Add(0x1002439C,@"SB7xx");
                //mDevices.Add(0x10024554,@"Mach64 ET");
                //mDevices.Add(0x10024654,@"Mach64 VT");
                //mDevices.Add(0x10024742,@"(GT-C2U2)");
                //mDevices.Add(0x10024744,@"Rage 3D Pro AGP 2x");
                //mDevices.Add(0x10024747,@"Rage 3D Pro");
                //mDevices.Add(0x10024749,@"RAGE PRO TURBO AGP 2X");
                //mDevices.Add(0x1002474C,@"Rage XC PCI-66");
                //mDevices.Add(0x1002474D,@"Rage XL AGP 2x");
                //mDevices.Add(0x1002474E,@"Rage XC AGP 2x");
                //mDevices.Add(0x1002474F,@"Rage XL PCI-66");
                //mDevices.Add(0x10024750,@"ATI 3D Rage Pro 215GP");
                //mDevices.Add(0x10024751,@"Rage 3D Pro PCI");
                //mDevices.Add(0x10024752,@"Rage XL PCI");
                //mDevices.Add(0x10024753,@"Rage XC PCI");
                //mDevices.Add(0x10024754,@"Mach 64 VT");
                //mDevices.Add(0x10024755,@"Rage 3D II+pci");
                //mDevices.Add(0x10024756,@"Rage 3D IIC AGP");
                //mDevices.Add(0x10024757,@"3D 11C AGP");
                //mDevices.Add(0x10024758,@"210888GX");
                //mDevices.Add(0x10024759,@"215r2qzua21");
                //mDevices.Add(0x1002475A,@"215r2qua12");
                //mDevices.Add(0x10024966,@"RV250");
                //mDevices.Add(0x10024967,@"RV250");
                //mDevices.Add(0x1002496E,@"RV250");
                //mDevices.Add(0x1002496F,@"RV250");
                //mDevices.Add(0x10024C42,@"B10E0E11");
                //mDevices.Add(0x10024C44,@"Rage 3D LT Pro AGP");
                //mDevices.Add(0x10024C45,@"Rage Mobility M3 AGP");
                //mDevices.Add(0x10024C46,@"Mobility M3 AGP");
                //mDevices.Add(0x10024C47,@"ati rage pro");
                //mDevices.Add(0x10024C49,@"123");
                //mDevices.Add(0x10024C4D,@"01541014");
                //mDevices.Add(0x10024C4E,@"216lo sasa25");
                //mDevices.Add(0x10024C50,@"VEN_1002&DEV_4C50&SUBSYS_4C501002&REV_DC");
                //mDevices.Add(0x10024C51,@"113");
                //mDevices.Add(0x10024C52,@"1241243");
                //mDevices.Add(0x10024C53,@"216L0SASA25");
                //mDevices.Add(0x10024C54,@"Mach 64 LT");
                //mDevices.Add(0x10024C57,@"fdds");
                //mDevices.Add(0x10024C58,@"FireGL Mobility");
                //mDevices.Add(0x10024C59,@"Mobility 6");
                //mDevices.Add(0x10024C5A,@"Radeon Mobility M6 LZ");
                //mDevices.Add(0x10024C64,@"Radeon Mobility M9-GL");
                //mDevices.Add(0x10024C66,@"ATI MOBILITY RADEON 9000 (Microsoft Corporation -");
                //mDevices.Add(0x10024D46,@"ATI mobility128");
                //mDevices.Add(0x10024D4C,@"Rage Mobility 128 AGP");
                //mDevices.Add(0x10024D52,@"ATI Theater 550 Pro");
                //mDevices.Add(0x10024E44,@"R300");
                //mDevices.Add(0x10024E45,@"R300");
                //mDevices.Add(0x10024E47,@"ATI Fire GL X1/Z1 Video Accelerator");
                //mDevices.Add(0x10024E48,@"R360");
                //mDevices.Add(0x10024E49,@"R350");
                //mDevices.Add(0x10024E4A,@"R360");
                //mDevices.Add(0x10024E4B,@"FGL9800XT");
                //mDevices.Add(0x10024E50,@"RV360M10");
                //mDevices.Add(0x10024E54,@"RV350-WS");
                //mDevices.Add(0x10024E64,@"R300");
                //mDevices.Add(0x10024E65,@"R300");
                //mDevices.Add(0x10024E67,@"Fire GL X1/Z1 AGP [Pro] Secondary Video");
                //mDevices.Add(0x10024E68,@"R350");
                //mDevices.Add(0x10024E69,@"R350");
                //mDevices.Add(0x10024E6A,@"R350");
                //mDevices.Add(0x10024E6B,@"FGL9800XT");
                //mDevices.Add(0x10025041,@"gt");
                //mDevices.Add(0x10025042,@"Rage 128 Pro PB AGP 2x");
                //mDevices.Add(0x10025043,@"1231324445");
                //mDevices.Add(0x10025044,@"rv100");
                //mDevices.Add(0x10025045,@"Rage 128 Pro PE AGP 2x");
                //mDevices.Add(0x10025046,@"5046");
                //mDevices.Add(0x10025047,@"215R3BUA22");
                //mDevices.Add(0x10025048,@"Rage 128 Pro PH AGP");
                //mDevices.Add(0x10025049,@"bk2.0.2.vr001.001.002.002.004.025.prt3.ty.t");
                //mDevices.Add(0x1002504A,@"Rage 128 Pro PJ PCI");
                //mDevices.Add(0x1002504B,@"Rage 128 Pro PK AGP");
                //mDevices.Add(0x1002504C,@"Rage 128 Pro PL AGP");
                //mDevices.Add(0x1002504D,@"Rage 128 Pro PM PCI");
                //mDevices.Add(0x1002504E,@"Rage 128 Pro PN AGP");
                //mDevices.Add(0x1002504F,@"Rage 128 Pro PO AGP");
                //mDevices.Add(0x10025050,@"Scheda Grafica Standard PCI(VGA)");
                //mDevices.Add(0x10025051,@"Rage 128 Pro PQ AGP");
                //mDevices.Add(0x10025052,@"Rage 128 Pro PR AGP");
                //mDevices.Add(0x10025053,@"Rage 128 Pro PS PCI");
                //mDevices.Add(0x10025054,@"Rage 128 Pro PT AGP");
                //mDevices.Add(0x10025055,@"rage 128 pro agp 4x tmds");
                //mDevices.Add(0x10025056,@"Rage 128 Pro PV PCI");
                //mDevices.Add(0x10025057,@"Rage 128 Pro PW AGP");
                //mDevices.Add(0x10025058,@"Rage 128 Pro");
                //mDevices.Add(0x10025144,@"Radeon 7200 QD SDR/DDR");
                //mDevices.Add(0x10025145,@"Radeon QE");
                //mDevices.Add(0x10025146,@"Radeon QF");
                //mDevices.Add(0x10025147,@"Radeon QG");
                //mDevices.Add(0x10025148,@"R200");
                //mDevices.Add(0x10025149,@"Radeon R200 QI");
                //mDevices.Add(0x1002514A,@"Radeon R200 QJ");
                //mDevices.Add(0x1002514B,@"Radeon R200 QK");
                //mDevices.Add(0x1002514C,@"R200");
                //mDevices.Add(0x1002514D,@"R200");
                //mDevices.Add(0x1002514E,@"Radeon R200 QM");
                //mDevices.Add(0x1002514F,@"Radeon R200 QN");
                //mDevices.Add(0x10025157,@"RV200");
                //mDevices.Add(0x10025158,@"radeon 9700 or 9200");
                //mDevices.Add(0x10025159,@"RV100");
                //mDevices.Add(0x1002515A,@"Radeon VE QZ");
                //mDevices.Add(0x1002515E,@"Radeon ES1000");
                //mDevices.Add(0x10025168,@"Radeon R200 Qh");
                //mDevices.Add(0x10025169,@"Radeon R200 Qi");
                //mDevices.Add(0x1002516A,@"Radeon R200 Qj");
                //mDevices.Add(0x1002516B,@"Radeon R200 Qk");
                //mDevices.Add(0x1002516C,@"E7505");
                //mDevices.Add(0x1002516D,@"R200");
                //mDevices.Add(0x10025245,@"215R46ASA22");
                //mDevices.Add(0x10025246,@"Rage 128 PRO");
                //mDevices.Add(0x10025247,@"Rage 128 RG");
                //mDevices.Add(0x1002524B,@"g01080-108");
                //mDevices.Add(0x1002524C,@"Rage 128 VR RL AGP 2x");
                //mDevices.Add(0x10025345,@"Rage 128 4x SE PCI");
                //mDevices.Add(0x10025346,@"Rage 128 SF 4x AGP 2x");
                //mDevices.Add(0x10025347,@"Rage 128 SG 4x AGP 4x");
                //mDevices.Add(0x10025348,@"Rage 128 4x SH");
                //mDevices.Add(0x1002534B,@"Rage 128 SK PCI");
                //mDevices.Add(0x1002534C,@"Rage 128 SL AGP 2x");
                //mDevices.Add(0x1002534D,@"Rage 128 SM AGP 4x");
                //mDevices.Add(0x1002534E,@"Rage 128 4x");
                //mDevices.Add(0x10025354,@"Mach 64 ST");
                //mDevices.Add(0x10025446,@"Rage 128 PRO ULTRA");
                //mDevices.Add(0x1002544C,@"Rage 128 Pro TL");
                //mDevices.Add(0x10025452,@"Rage 128 Pro TR");
                //mDevices.Add(0x10025455,@"Rage 128 Pro Ultra TU");
                //mDevices.Add(0x10025460,@"RV370");
                //mDevices.Add(0x1002554D,@"R430");
                //mDevices.Add(0x10025551,@"R423GL-SE");
                //mDevices.Add(0x1002556D,@"R430");
                //mDevices.Add(0x10025571,@"R423GL-SE");
                //mDevices.Add(0x10025653,@"R410");
                //mDevices.Add(0x10025654,@"215VT2CA42");
                //mDevices.Add(0x10025655,@"Mach 64 VT3");
                //mDevices.Add(0x10025656,@"Mach 64 VT4 PCI");
                //mDevices.Add(0x10025830,@"RS300");
                //mDevices.Add(0x10025831,@"RS300");
                //mDevices.Add(0x10025832,@"RS300");
                //mDevices.Add(0x10025833,@"Radeon 9100 IGP");
                //mDevices.Add(0x10025834,@"RS300");
                //mDevices.Add(0x10025835,@"RS300");
                //mDevices.Add(0x10025838,@"Radeon 9100 IGP");
                //mDevices.Add(0x10025940,@"RV280");
                //mDevices.Add(0x10025941,@"RV280");
                //mDevices.Add(0x10025954,@"na");
                //mDevices.Add(0x10025955,@"???");
                //mDevices.Add(0x10025960,@"RV280");
                //mDevices.Add(0x10025961,@"RV280");
                //mDevices.Add(0x10025964,@"Radeon 9200");
                //mDevices.Add(0x10025965,@"Nvidia");
                //mDevices.Add(0x10025975,@"Radeon Xpress 1100");
                //mDevices.Add(0x10025A33,@"RC410");
                //mDevices.Add(0x10025A42,@"X200M");
                //mDevices.Add(0x10025A61,@"Radeon XPress 200 (RC410)");
                //mDevices.Add(0x10025A62,@"RC410M");
                //mDevices.Add(0x10025B63,@"Unknown");
                //mDevices.Add(0x10025B64,@"RV370GL");
                //mDevices.Add(0x10025B70,@"RADEON X300");
                //mDevices.Add(0x10025C61,@"bk-ati ver008.016m.085.006");
                //mDevices.Add(0x10025C63,@"RV280 (M9+)");
                //mDevices.Add(0x10025D44,@"RV280");
                //mDevices.Add(0x10025D45,@"RV280");
                //mDevices.Add(0x1002700F,@"A3/U1");
                //mDevices.Add(0x10027010,@"RS200");
                //mDevices.Add(0x10027109,@"R520");
                //mDevices.Add(0x10027142,@"N/A");
                //mDevices.Add(0x10027145,@"x1400");
                //mDevices.Add(0x10027146,@"N/A");
                //mDevices.Add(0x10027149,@"216CZJAKA12FAG");
                //mDevices.Add(0x10027152,@"Fire GL v3300");
                //mDevices.Add(0x10027183,@"RV505");
                //mDevices.Add(0x10027186,@"Mobility Radeon X1450");
                //mDevices.Add(0x10027187,@"Radeon 1300");
                //mDevices.Add(0x10027188,@"Mobility X2300");
                //mDevices.Add(0x100271A1,@"ati graphics");
                //mDevices.Add(0x100271A3,@"X1300");
                //mDevices.Add(0x100271A7,@"Radeon 1300");
                //mDevices.Add(0x10027249,@"ati 1900");
                //mDevices.Add(0x1002724B,@"180636911721");
                //mDevices.Add(0x10027280,@"Radeon X1950 Pro");
                //mDevices.Add(0x100272A0,@"Radeon X1950 Pro Secondary");
                //mDevices.Add(0x10027833,@"RS350");
                //mDevices.Add(0x1002791F,@"RS690");
                //mDevices.Add(0x10027942,@"ATI XPress 1250M");
                //mDevices.Add(0x10029400,@"R600");
                //mDevices.Add(0x100294C1,@"REV_00");
                //mDevices.Add(0x100294C3,@"RV610");
                //mDevices.Add(0x100294C4,@"RV610");
                //mDevices.Add(0x10029501,@"RV670");
                //mDevices.Add(0x10029581,@"634866");
                //mDevices.Add(0x10029587,@" Radeon hd 2600 pro (agp)");
                //mDevices.Add(0x10029588,@"RV630");
                //mDevices.Add(0x10029589,@"RV630");
                //mDevices.Add(0x10029610,@"780G");
                //mDevices.Add(0x10029876,@"ATI GTC (GT-C2U2)");
                //mDevices.Add(0x1002AA01,@"Ati Function driver for high definition audio - AT");
                //mDevices.Add(0x1002AA20,@"RV630");
                //mDevices.Add(0x1002CAB0,@"A3/U1");
                //mDevices.Add(0x1002CAB1,@"A3/U1");
                //mDevices.Add(0x1002CBB2,@"RS200");
                //mDevices.Add(0x10030201,@"0x0201");
                //mDevices.Add(0x10040005,@"82C591/2-FC1");
                //mDevices.Add(0x10040006,@"82C593");
                //mDevices.Add(0x10040007,@"82C594");
                //mDevices.Add(0x10040008,@"82C596/597");
                //mDevices.Add(0x10040009,@"82C597-AFC2");
                //mDevices.Add(0x1004000C,@"82C541");
                //mDevices.Add(0x1004000D,@"82C543");
                //mDevices.Add(0x10040100,@"dtdftdfz");
                //mDevices.Add(0x10040101,@"82C532");
                //mDevices.Add(0x10040102,@"82C534");
                //mDevices.Add(0x10040103,@"82C538");
                //mDevices.Add(0x10040104,@"82C535");
                //mDevices.Add(0x10040105,@"82C147");
                //mDevices.Add(0x10040200,@"82C975");
                //mDevices.Add(0x10040280,@"82C925");
                //mDevices.Add(0x10040304,@"SAA7785");
                //mDevices.Add(0x10040305,@"SAA7785");
                //mDevices.Add(0x10040306,@"SAA7785");
                //mDevices.Add(0x10040307,@"KTX0208X");
                //mDevices.Add(0x10040308,@"Philips PSC705 GamePort Enumerator");
                //mDevices.Add(0x10040702,@"VAS96011");
                //mDevices.Add(0x100B0001,@"DP83810");
                //mDevices.Add(0x100B0002,@"PC87415");
                //mDevices.Add(0x100B000E,@"PC87560");
                //mDevices.Add(0x100B000F,@"CS4210");
                //mDevices.Add(0x100B0011,@"PC87560");
                //mDevices.Add(0x100B0012,@"USB Controller");
                //mDevices.Add(0x100B001B,@"LM4560");
                //mDevices.Add(0x100B0020,@"DP83815/16");
                //mDevices.Add(0x100B0021,@"PC82440");
                //mDevices.Add(0x100B0022,@"DP83820/1");
                //mDevices.Add(0x100B0028,@"Geode GX2");
                //mDevices.Add(0x100B002A,@"CS5535");
                //mDevices.Add(0x100B002D,@"5535");
                //mDevices.Add(0x100B002E,@"Castle ()");
                //mDevices.Add(0x100B002F,@"Geode");
                //mDevices.Add(0x100B0030,@"NSC");
                //mDevices.Add(0x100B0500,@"SCx200");
                //mDevices.Add(0x100B0501,@"SCx200");
                //mDevices.Add(0x100B0502,@"SC1100/SCx200");
                //mDevices.Add(0x100B0503,@"SC1100/SCx200");
                //mDevices.Add(0x100B0504,@"SCx200");
                //mDevices.Add(0x100B0505,@"SCx200");
                //mDevices.Add(0x100B0510,@"SC1100");
                //mDevices.Add(0x100B0511,@"SC1100");
                //mDevices.Add(0x100B0515,@"SC1100");
                //mDevices.Add(0x100BD001,@"M2764AFI");
                //mDevices.Add(0x100C3202,@"ET4000W32P-A");
                //mDevices.Add(0x100C3205,@"ET4000W32P-B");
                //mDevices.Add(0x100C3206,@"ET4000W32P-C");
                //mDevices.Add(0x100C3207,@"ET4000W32P-D");
                //mDevices.Add(0x100C3208,@"ET6000");
                //mDevices.Add(0x100C4702,@"ET6300");
                //mDevices.Add(0x100E0564,@"STPC Client");
                //mDevices.Add(0x100E55CC,@"STPC Client");
                //mDevices.Add(0x100E9000,@"P9000");
                //mDevices.Add(0x100E9001,@"P9000");
                //mDevices.Add(0x100E9100,@"P9100");
                //mDevices.Add(0x10110001,@"DC21050");
                //mDevices.Add(0x10110002,@"DC21040");
                //mDevices.Add(0x10110004,@"DC21030");
                //mDevices.Add(0x10110007,@"Zephyr");
                //mDevices.Add(0x10110008,@"KZPSA");
                //mDevices.Add(0x10110009,@"DC21140");
                //mDevices.Add(0x1011000A,@"DC21230");
                //mDevices.Add(0x1011000C,@"1011");
                //mDevices.Add(0x1011000D,@"TGA2");
                //mDevices.Add(0x1011000F,@"DEFPA");
                //mDevices.Add(0x10110014,@"DC21041");
                //mDevices.Add(0x10110016,@"DGLPB");
                //mDevices.Add(0x10110019,@"DC21142/3");
                //mDevices.Add(0x10110021,@"21052[-AB]");
                //mDevices.Add(0x10110022,@"DC21150-AA");
                //mDevices.Add(0x10110023,@"DC21150");
                //mDevices.Add(0x10110024,@"DC21151/2");
                //mDevices.Add(0x10110025,@"21153");
                //mDevices.Add(0x10110026,@"21154");
                //mDevices.Add(0x10110034,@"Modem56");
                //mDevices.Add(0x10110045,@"DC21553");
                //mDevices.Add(0x10110046,@"21554");
                //mDevices.Add(0x10111065,@"21285");
                //mDevices.Add(0x10112000,@"3X-KPCON-AA");
                //mDevices.Add(0x10130038,@"FW82371EB");
                //mDevices.Add(0x10130040,@"CL-GD7555");
                //mDevices.Add(0x1013004C,@"CL-GD7556");
                //mDevices.Add(0x101300A0,@"CL-GD5440");
                //mDevices.Add(0x101300A2,@"CL-GD5432");
                //mDevices.Add(0x101300A4,@"CL-GD5434");
                //mDevices.Add(0x101300A8,@"CL-GD5434");
                //mDevices.Add(0x101300AC,@"CL-GD5426");
                //mDevices.Add(0x101300B8,@"CL-GD5446");
                //mDevices.Add(0x101300BC,@"CL-GD5480");
                //mDevices.Add(0x101300D0,@"CL-GD5462");
                //mDevices.Add(0x101300D4,@"CL-GD5455");
                //mDevices.Add(0x101300D5,@"CL-GD5464");
                //mDevices.Add(0x101300D6,@"CL-GD5465");
                //mDevices.Add(0x101300E8,@"CL-GD5436U");
                //mDevices.Add(0x10131013,@"cs4630cm");
                //mDevices.Add(0x10131100,@"CL-PD6729");
                //mDevices.Add(0x10131110,@"CL-PD6832");
                //mDevices.Add(0x10131112,@"CL-PD6834");
                //mDevices.Add(0x10131113,@"CL-PD6833");
                //mDevices.Add(0x10131200,@"CL-GD7542");
                //mDevices.Add(0x10131202,@"CL-GD7543");
                //mDevices.Add(0x10131204,@"CL-GD7541");
                //mDevices.Add(0x10134000,@"1085");
                //mDevices.Add(0x10134400,@"CL-CD4400");
                //mDevices.Add(0x10136001,@"CS4610/4611");
                //mDevices.Add(0x10136003,@"CS4614/22/24/");
                //mDevices.Add(0x10136004,@"CS464");
                //mDevices.Add(0x10136005,@"CS4280/4297x");
                //mDevices.Add(0x10139876,@"cirrus logic crystal CS 4614");
                //mDevices.Add(0x10140002,@"MCA Bridge");
                //mDevices.Add(0x10140005,@"Alta Lite");
                //mDevices.Add(0x10140007,@"Alta MP");
                //mDevices.Add(0x1014000A,@"Fire Coral");
                //mDevices.Add(0x10140017,@"CPU to PCI Bridge");
                //mDevices.Add(0x10140018,@"Auto LANStreamer");
                //mDevices.Add(0x1014001B,@"GXT-150P");
                //mDevices.Add(0x1014001D,@"82G2675");
                //mDevices.Add(0x10140020,@"MCA Bridge");
                //mDevices.Add(0x10140022,@"82351/2");
                //mDevices.Add(0x1014002D,@"Python");
                //mDevices.Add(0x1014002E,@"ServeRAID I/II/3x/4H");
                //mDevices.Add(0x10140036,@"Miami/PCI");
                //mDevices.Add(0x10140037,@"IBM27-82660");
                //mDevices.Add(0x1014003A,@"CPU to PCI Bridge");
                //mDevices.Add(0x1014003E,@"85H9533");
                //mDevices.Add(0x10140045,@"SSA Adapter");
                //mDevices.Add(0x10140046,@"MPIC");
                //mDevices.Add(0x10140047,@"PCI to PCI Bridge");
                //mDevices.Add(0x10140048,@"PCI to PCI Bridge");
                //mDevices.Add(0x10140049,@"Warhead SCSI Controller");
                //mDevices.Add(0x1014004D,@"IBM 3780IDSP");
                //mDevices.Add(0x1014004E,@"ATM Controller");
                //mDevices.Add(0x1014004F,@"ATM Controller");
                //mDevices.Add(0x10140050,@"ATM Controller");
                //mDevices.Add(0x10140053,@"85h9533");
                //mDevices.Add(0x10140057,@"85g1897");
                //mDevices.Add(0x1014005C,@"i82557B");
                //mDevices.Add(0x1014005D,@"05J3506");
                //mDevices.Add(0x1014007C,@"ATM Controller");
                //mDevices.Add(0x1014007D,@"3780IDSP");
                //mDevices.Add(0x10140090,@"GXT-3000P");
                //mDevices.Add(0x10140095,@"20H2999");
                //mDevices.Add(0x10140096,@"Chukar chipset SCSI Controller");
                //mDevices.Add(0x101400A1,@"PowerNP NPr2.7");
                //mDevices.Add(0x101400A5,@"ATM Controller");
                //mDevices.Add(0x101400A6,@"ATM 155Mbps MM Controller");
                //mDevices.Add(0x101400B7,@"GXT2000");
                //mDevices.Add(0x101400BE,@"ATM 622Mbps Controller");
                //mDevices.Add(0x101400CE,@"02li537");
                //mDevices.Add(0x101400F9,@"CPC700");
                //mDevices.Add(0x101400FC,@"CPC710");
                //mDevices.Add(0x10140105,@"CPC710");
                //mDevices.Add(0x1014010F,@"Remote Supervisor+Serial Port+Mouse/Keyb");
                //mDevices.Add(0x1014011B,@"Raid controller");
                //mDevices.Add(0x10140142,@"Yotta");
                //mDevices.Add(0x10140144,@"Yotta");
                //mDevices.Add(0x10140156,@"405GP");
                //mDevices.Add(0x10140170,@"RC1000");
                //mDevices.Add(0x10140188,@"PCI Bridge");
                //mDevices.Add(0x101401A7,@"IBM 133");
                //mDevices.Add(0x101401BD,@"ServeRAID 4/5");
                //mDevices.Add(0x1014027F,@"440GX");
                //mDevices.Add(0x10140295,@"NECSCE 11508082");
                //mDevices.Add(0x10140297,@"NECSCE 11508082");
                //mDevices.Add(0x10140302,@"PCI-X Host Bridge");
                //mDevices.Add(0x1014FFFF,@"MPIC 2");
                //mDevices.Add(0x10175343,@"v7-mpeg modul");
                //mDevices.Add(0x10191B10,@"sis-648D");
                //mDevices.Add(0x101A0005,@"8156");
                //mDevices.Add(0x101A0009,@"Altera FLEX");
                //mDevices.Add(0x101E9010,@"MegaRAID 428");
                //mDevices.Add(0x101E9030,@"EIDE Controller");
                //mDevices.Add(0x101E9031,@"EIDE Controller");
                //mDevices.Add(0x101E9032,@"IDE and SCSI Cntrlr");
                //mDevices.Add(0x101E9033,@"I960");
                //mDevices.Add(0x101E9040,@"Multimedia card");
                //mDevices.Add(0x101E9060,@"MegaRAID 434");
                //mDevices.Add(0x101E9063,@"MegaRAC");
                //mDevices.Add(0x10221100,@"Athlon 64 / Opteron");
                //mDevices.Add(0x10221101,@"Athlon 64 / Opteron");
                //mDevices.Add(0x10221102,@"Athlon 64 / Opteron");
                //mDevices.Add(0x10221103,@"Athlon 64 / Opteron");
                //mDevices.Add(0x10222000,@"Am79C970/1/2/3/5/6");
                //mDevices.Add(0x10222001,@"Am79C978");
                //mDevices.Add(0x10222003,@"Am1772");
                //mDevices.Add(0x10222020,@"Am53C974");
                //mDevices.Add(0x10222040,@"Am79C974");
                //mDevices.Add(0x10222081,@"Unknown");
                //mDevices.Add(0x10222082,@"GX3");
                //mDevices.Add(0x1022208F,@"CS5536");
                //mDevices.Add(0x10222093,@"CS5536");
                //mDevices.Add(0x10222094,@"CS5536");
                //mDevices.Add(0x10222095,@"CS5536");
                //mDevices.Add(0x10222096,@"Number: CS5536");
                //mDevices.Add(0x10222097,@"CS5536");
                //mDevices.Add(0x1022209A,@"CS5536");
                //mDevices.Add(0x10223000,@"SC520");
                //mDevices.Add(0x10227004,@"AMD-751");
                //mDevices.Add(0x10227006,@"AMD-751");
                //mDevices.Add(0x10227007,@"AMD-751");
                //mDevices.Add(0x1022700A,@"AMD-IGR4");
                //mDevices.Add(0x1022700B,@"AMD-IGR4");
                //mDevices.Add(0x1022700C,@"AMD-762");
                //mDevices.Add(0x1022700D,@"AMD-762");
                //mDevices.Add(0x1022700E,@"AMD-761");
                //mDevices.Add(0x1022700F,@"AMD-761");
                //mDevices.Add(0x10227400,@"AMD-755");
                //mDevices.Add(0x10227401,@"AMD-755");
                //mDevices.Add(0x10227403,@"AMD-755");
                //mDevices.Add(0x10227404,@"AMD-755");
                //mDevices.Add(0x10227408,@"AMD-756");
                //mDevices.Add(0x10227409,@"AMD-756");
                //mDevices.Add(0x1022740B,@"AMD-756");
                //mDevices.Add(0x1022740C,@"AMD-756");
                //mDevices.Add(0x10227410,@"AMD-766");
                //mDevices.Add(0x10227411,@"AMD-766");
                //mDevices.Add(0x10227412,@"AMD-766");
                //mDevices.Add(0x10227413,@"AMD-766");
                //mDevices.Add(0x10227414,@"AMD-766");
                //mDevices.Add(0x10227440,@"AMD-768");
                //mDevices.Add(0x10227441,@"AMD-768");
                //mDevices.Add(0x10227443,@"AMD-768");
                //mDevices.Add(0x10227445,@"AMD-768");
                //mDevices.Add(0x10227446,@"AMD-768");
                //mDevices.Add(0x10227448,@"AMD-768");
                //mDevices.Add(0x10227449,@"AMD-768");
                //mDevices.Add(0x10227450,@"AMD-8131");
                //mDevices.Add(0x10227451,@"AMD-8131");
                //mDevices.Add(0x10227454,@"AMD-8151");
                //mDevices.Add(0x10227455,@"AMD-8151");
                //mDevices.Add(0x10227458,@"AMD-8132");
                //mDevices.Add(0x10227459,@"AMD-8132");
                //mDevices.Add(0x10227460,@"AMD-8111");
                //mDevices.Add(0x10227461,@"AMD-8111");
                //mDevices.Add(0x10227462,@"AMD-8111");
                //mDevices.Add(0x10227463,@"AMD-8111");
                //mDevices.Add(0x10227464,@"AMD-8111");
                //mDevices.Add(0x10227468,@"AMD-8111");
                //mDevices.Add(0x10227469,@"AMD-8111");
                //mDevices.Add(0x1022746A,@"AMD-8111");
                //mDevices.Add(0x1022746B,@"AMD-8111");
                //mDevices.Add(0x1022746D,@"c-media cmi8738");
                //mDevices.Add(0x1022746E,@"AMD-8111");
                //mDevices.Add(0x1022756B,@"AMD-8111");
                //mDevices.Add(0x10230194,@"82C194");
                //mDevices.Add(0x10232000,@"4DWAVE-DX");
                //mDevices.Add(0x10232001,@"4D WAVE DX SOUND ACCELERATOR");
                //mDevices.Add(0x10232100,@"Cyber-XP4");
                //mDevices.Add(0x10232200,@"Volari V3");
                //mDevices.Add(0x10238400,@"CyberBlade i7");
                //mDevices.Add(0x10238420,@"51331071");
                //mDevices.Add(0x10238500,@"Cyber");
                //mDevices.Add(0x10238520,@"Trident Video Accelerator CyberBlade i1");
                //mDevices.Add(0x10238620,@"CyberBlade-i1");
                //mDevices.Add(0x10238820,@"CyberBlade XP");
                //mDevices.Add(0x10239320,@"TGUI9320");
                //mDevices.Add(0x10239350,@"TGUI9350");
                //mDevices.Add(0x10239360,@"Flat panel Cntrlr");
                //mDevices.Add(0x10239382,@"Cyber9382");
                //mDevices.Add(0x10239383,@"Cyber9383");
                //mDevices.Add(0x10239385,@"Cyber9385");
                //mDevices.Add(0x10239386,@"Cyber9386");
                //mDevices.Add(0x10239388,@"Cyber9388");
                //mDevices.Add(0x10239397,@"Cyber9397");
                //mDevices.Add(0x1023939A,@"Cyber9397DVD");
                //mDevices.Add(0x10239420,@"TGUI9420");
                //mDevices.Add(0x10239430,@"TGUI9430");
                //mDevices.Add(0x10239440,@"TGUI9682");
                //mDevices.Add(0x10239460,@"TGUI9460");
                //mDevices.Add(0x10239470,@"TGUI9470");
                //mDevices.Add(0x10239520,@"Cyber9520");
                //mDevices.Add(0x10239525,@"Cyber9525");
                //mDevices.Add(0x10239540,@"Cyber9540");
                //mDevices.Add(0x10239660,@"TGUI9660XGi");
                //mDevices.Add(0x10239680,@"TGUI9680");
                //mDevices.Add(0x10239682,@"TGUI9682");
                //mDevices.Add(0x10239683,@"TGUI9683");
                //mDevices.Add(0x10239685,@"ProVIDIA 9685");
                //mDevices.Add(0x10239750,@"3DImage 9750 PCI/AGP");
                //mDevices.Add(0x10239753,@"TGUI9753");
                //mDevices.Add(0x10239754,@"TGUI9753");
                //mDevices.Add(0x10239759,@"TGUI975?");
                //mDevices.Add(0x10239783,@"TGUI9783");
                //mDevices.Add(0x10239785,@"TGUI9785");
                //mDevices.Add(0x10239850,@"3D Image 9850 AGP");
                //mDevices.Add(0x10239880,@"Blade 3D 9880");
                //mDevices.Add(0x10239910,@"00011178");
                //mDevices.Add(0x10239930,@"CyberBlade XPm");
                //mDevices.Add(0x10239960,@"9960");
                //mDevices.Add(0x10250028,@"AC97 ID:SIL REV:0x27, 08");
                //mDevices.Add(0x10251435,@"M1435");
                //mDevices.Add(0x10251445,@"M1445");
                //mDevices.Add(0x10251449,@"M1449");
                //mDevices.Add(0x10251451,@"M1451");
                //mDevices.Add(0x10251461,@"M1461");
                //mDevices.Add(0x10251489,@"M1489");
                //mDevices.Add(0x10251511,@"M1511");
                //mDevices.Add(0x10251512,@"M1512");
                //mDevices.Add(0x10251513,@"M1513");
                //mDevices.Add(0x10251521,@"M1521");
                //mDevices.Add(0x10251523,@"M1523");
                //mDevices.Add(0x10251531,@"M1531");
                //mDevices.Add(0x10251533,@"M1533");
                //mDevices.Add(0x10251535,@"M1535");
                //mDevices.Add(0x10251541,@"M1541");
                //mDevices.Add(0x10251542,@"M1542");
                //mDevices.Add(0x10251543,@"M1543C");
                //mDevices.Add(0x10251561,@"M1561");
                //mDevices.Add(0x10251621,@"M1621");
                //mDevices.Add(0x10251631,@"M1631");
                //mDevices.Add(0x10251641,@"M1641");
                //mDevices.Add(0x10253141,@"M3141");
                //mDevices.Add(0x10253143,@"M3143");
                //mDevices.Add(0x10253145,@"M3145");
                //mDevices.Add(0x10253147,@"M3147");
                //mDevices.Add(0x10253149,@"M3149");
                //mDevices.Add(0x10253151,@"M3151");
                //mDevices.Add(0x10253307,@"M3307");
                //mDevices.Add(0x10253309,@"M3309");
                //mDevices.Add(0x10255212,@"M4803");
                //mDevices.Add(0x10255215,@"M5217");
                //mDevices.Add(0x10255217,@"M5217");
                //mDevices.Add(0x10255219,@"M5219");
                //mDevices.Add(0x10255225,@"M5225");
                //mDevices.Add(0x10255229,@"M5229");
                //mDevices.Add(0x10255235,@"1025");
                //mDevices.Add(0x10255237,@"M5237");
                //mDevices.Add(0x10255240,@"EIDE Controller");
                //mDevices.Add(0x10255241,@"PCMCIA Bridge");
                //mDevices.Add(0x10255242,@"General Purpose Controller");
                //mDevices.Add(0x10255243,@"PCI to PCI Bridge");
                //mDevices.Add(0x10255244,@"Floppy Disk Controller");
                //mDevices.Add(0x10255247,@"M1541");
                //mDevices.Add(0x10255427,@"PCI to AGP Bridge");
                //mDevices.Add(0x10255451,@"M5451");
                //mDevices.Add(0x10255453,@"M5453");
                //mDevices.Add(0x10257101,@"M7101");
                //mDevices.Add(0x10280001,@"PowerEdge 2 /Si");
                //mDevices.Add(0x10280002,@"PowerEdge 3/Di");
                //mDevices.Add(0x10280003,@"PowerEdge 3/Si");
                //mDevices.Add(0x10280004,@"PowerEdge 3/Si");
                //mDevices.Add(0x10280005,@"PowerEdge 3/Di");
                //mDevices.Add(0x10280006,@"PowerEdge 3/Di");
                //mDevices.Add(0x10280007,@"Remote Assistant Card");
                //mDevices.Add(0x10280008,@"PowerEdge 3/Di");
                //mDevices.Add(0x1028000A,@"PowerEdge 3/Di");
                //mDevices.Add(0x1028000C,@"Embedded Systems Management Device 4");
                //mDevices.Add(0x1028000E,@"000E");
                //mDevices.Add(0x10280011,@"DRAC4");
                //mDevices.Add(0x10280012,@"DRAC4-UART");
                //mDevices.Add(0x10280013,@"PERC 4e/Si and PERC 4e/Di");
                //mDevices.Add(0x10280014,@"DRAC4-0");
                //mDevices.Add(0x10280015,@"PERC 5/i RAID Controller");
                //mDevices.Add(0x10280287,@"DELL PERC 320/DC");
                //mDevices.Add(0x10281000,@"FA82537EP");
                //mDevices.Add(0x102A0000,@"HYDRA");
                //mDevices.Add(0x102A0010,@"ASPEN");
                //mDevices.Add(0x102B0010,@"MGA-I");
                //mDevices.Add(0x102B0518,@"MGA-PX2085");
                //mDevices.Add(0x102B0519,@"MGA-2064W");
                //mDevices.Add(0x102B051A,@"mga-1164sg-a");
                //mDevices.Add(0x102B051B,@"MGA-2164W");
                //mDevices.Add(0x102B051E,@"MGA-1164SG");
                //mDevices.Add(0x102B051F,@"MGA-200B");
                //mDevices.Add(0x102B0520,@"Matrox lnc MGA-G200B");
                //mDevices.Add(0x102B0521,@"Matrox lnc MGA-G200B");
                //mDevices.Add(0x102B0525,@"MGA G450 Dual Head");
                //mDevices.Add(0x102B0527,@"Parhelia AGP");
                //mDevices.Add(0x102B0528,@"Parhelia");
                //mDevices.Add(0x102B0D10,@"MGA-I");
                //mDevices.Add(0x102B1000,@"MGA-G100");
                //mDevices.Add(0x102B1001,@"MGA-G100");
                //mDevices.Add(0x102B1525,@"Fusion G450 AGP");
                //mDevices.Add(0x102B1527,@"Fusion Plus G800 AGP");
                //mDevices.Add(0x102B2007,@"Mistral");
                //mDevices.Add(0x102B2527,@"MGA-G550");
                //mDevices.Add(0x102B2537,@"Parhelia-LX");
                //mDevices.Add(0x102B2538,@"Matrox Millennium P650 LP PCIe 64");
                //mDevices.Add(0x102B4536,@"Meteor 2 STD/MC/Dig");
                //mDevices.Add(0x102B6573,@"Shark");
                //mDevices.Add(0x102B80A0,@"RT.x10");
                //mDevices.Add(0x102B9876,@"RT 2000");
                //mDevices.Add(0x102C00B8,@"64310");
                //mDevices.Add(0x102C00C0,@"69000");
                //mDevices.Add(0x102C00D0,@"65545");
                //mDevices.Add(0x102C00D8,@"65540");
                //mDevices.Add(0x102C00DC,@"65548");
                //mDevices.Add(0x102C00E0,@"65550");
                //mDevices.Add(0x102C00E4,@"65554");
                //mDevices.Add(0x102C00E5,@"65555");
                //mDevices.Add(0x102C00F0,@"68554");
                //mDevices.Add(0x102C00F4,@"68554");
                //mDevices.Add(0x102C00F5,@"68555");
                //mDevices.Add(0x102C01E0,@"65560");
                //mDevices.Add(0x102C0C30,@"69030");
                //mDevices.Add(0x102D50DC,@"3328");
                //mDevices.Add(0x102F0009,@"r4x00");
                //mDevices.Add(0x102F000A,@"TX3927");
                //mDevices.Add(0x102F0020,@"Meteor 155");
                //mDevices.Add(0x102F0030,@"TC35815CF");
                //mDevices.Add(0x102F0031,@"TX4938");
                //mDevices.Add(0x102F0105,@"TC86C001FG");
                //mDevices.Add(0x102F0106,@"TC86C001FG");
                //mDevices.Add(0x102F0107,@"TC86C001FG");
                //mDevices.Add(0x102F0108,@"TC86C001FG");
                //mDevices.Add(0x102F0180,@"TX4927");
                //mDevices.Add(0x102F0181,@"TX4925");
                //mDevices.Add(0x102F0182,@"TX4937");
                //mDevices.Add(0x10315601,@"MiroVIDEO DC20");
                //mDevices.Add(0x10315607,@"2IVDC-PCX1 1250400 033893932");
                //mDevices.Add(0x10315631,@"Media 3D");
                //mDevices.Add(0x10316057,@"MiroVIDEO DC10/DC30");
                //mDevices.Add(0x10330001,@"upD98409");
                //mDevices.Add(0x10330002,@"PCI to VL98 Bridge");
                //mDevices.Add(0x10330003,@"ATM Controller");
                //mDevices.Add(0x10330004,@"R4000");
                //mDevices.Add(0x10330005,@"PCI to 486 like peripheral bus Bridge");
                //mDevices.Add(0x10330006,@"GUI Accelerator");
                //mDevices.Add(0x10330007,@"unknown from Creative.com");
                //mDevices.Add(0x10330008,@"GUI Accelerator (vga equivalent)");
                //mDevices.Add(0x10330009,@"graphic Cntrlr for 98");
                //mDevices.Add(0x1033001A,@"Nile II");
                //mDevices.Add(0x1033001D,@"uPD98405");
                //mDevices.Add(0x10330021,@"Vrc4373");
                //mDevices.Add(0x10330029,@"PoverVR PCX1");
                //mDevices.Add(0x1033002A,@"PoverVR");
                //mDevices.Add(0x10330035,@"udp9210");
                //mDevices.Add(0x10330036,@"uPD98409");
                //mDevices.Add(0x1033003E,@"uPD66369");
                //mDevices.Add(0x10330046,@"PoverVR PCX2");
                //mDevices.Add(0x1033005A,@"Vrc5074");
                //mDevices.Add(0x10330063,@"uPD72862");
                //mDevices.Add(0x10330067,@"PowerVR Neon 250");
                //mDevices.Add(0x10330074,@"56k Voice Modem");
                //mDevices.Add(0x1033009B,@"Vrc5476");
                //mDevices.Add(0x103300BE,@"VR4122");
                //mDevices.Add(0x103300CD,@"uPD72870");
                //mDevices.Add(0x103300CE,@"uPD72871/2");
                //mDevices.Add(0x103300E0,@"uPD720100A");
                //mDevices.Add(0x103300E7,@"uPD72874");
                //mDevices.Add(0x103300F2,@"D72874GC");
                //mDevices.Add(0x10330520,@"NEC D72874GC");
                //mDevices.Add(0x10360000,@"TMC-18C30");
                //mDevices.Add(0x10390001,@"SiS730");
                //mDevices.Add(0x10390002,@"520");
                //mDevices.Add(0x10390003,@"SiS760");
                //mDevices.Add(0x10390005,@"Pentium chipset");
                //mDevices.Add(0x10390006,@"SiS 85C501");
                //mDevices.Add(0x10390008,@"SiS 85C503");
                //mDevices.Add(0x10390009,@"5595");
                //mDevices.Add(0x10390016,@"SiS961/962");
                //mDevices.Add(0x10390018,@"SiS950");
                //mDevices.Add(0x10390180,@"SiS180");
                //mDevices.Add(0x10390181,@"Raid Controller(?Mode Raid1)");
                //mDevices.Add(0x10390182,@"Raid Controller(?Mode Raid0+1)");
                //mDevices.Add(0x10390183,@"?SIS965");
                //mDevices.Add(0x10390190,@" SiS966L, SiS965L, SiS966, SiS965, SiS968");
                //mDevices.Add(0x10390191,@"SIS191");
                //mDevices.Add(0x10390200,@"SiS5597/98");
                //mDevices.Add(0x10390204,@"SiS 6215");
                //mDevices.Add(0x10390205,@"SiS 6205");
                //mDevices.Add(0x10390300,@"SiS315");
                //mDevices.Add(0x10390305,@"SiS305");
                //mDevices.Add(0x10390315,@"SiS 315");
                //mDevices.Add(0x10390325,@"SiS6330");
                //mDevices.Add(0x10390330,@"SiS330");
                //mDevices.Add(0x10390406,@"85C501");
                //mDevices.Add(0x10390496,@"85C496");
                //mDevices.Add(0x10390530,@"SiS530");
                //mDevices.Add(0x10390540,@"SiS540");
                //mDevices.Add(0x10390550,@"SiS550/1/2");
                //mDevices.Add(0x10390596,@"SiS596");
                //mDevices.Add(0x10390597,@"SiS5513");
                //mDevices.Add(0x10390601,@"SiS83C601");
                //mDevices.Add(0x10390620,@"SiS620");
                //mDevices.Add(0x10390630,@"SiS630");
                //mDevices.Add(0x10390635,@"SiS 635");
                //mDevices.Add(0x10390640,@"SiS 640");
                //mDevices.Add(0x10390645,@"SiS 645");
                //mDevices.Add(0x10390646,@"SiS645DX");
                //mDevices.Add(0x10390648,@"SiS648MX");
                //mDevices.Add(0x10390649,@"sis649");
                //mDevices.Add(0x10390650,@"SiS961");
                //mDevices.Add(0x10390651,@"SiS651");
                //mDevices.Add(0x10390655,@"SiS655");
                //mDevices.Add(0x10390656,@"???");
                //mDevices.Add(0x10390658,@"SiS R658");
                //mDevices.Add(0x10390659,@"SiS R659");
                //mDevices.Add(0x10390660,@"Host-to-PCI Bridge");
                //mDevices.Add(0x10390661,@"SiS 661FX");
                //mDevices.Add(0x10390662,@"???");
                //mDevices.Add(0x10390663,@"???");
                //mDevices.Add(0x10390730,@"SiS 730");
                //mDevices.Add(0x10390735,@"SiS 735");
                //mDevices.Add(0x10390740,@"SiS 740");
                //mDevices.Add(0x10390741,@"SiS741");
                //mDevices.Add(0x10390745,@"SiS745");
                //mDevices.Add(0x10390746,@"SiS746");
                //mDevices.Add(0x10390748,@"SiS748");
                //mDevices.Add(0x10390755,@"SiSM650?");
                //mDevices.Add(0x10390756,@"SiS755FX");
                //mDevices.Add(0x10390760,@"SiS760");
                //mDevices.Add(0x10390761,@"???");
                //mDevices.Add(0x10390762,@"???");
                //mDevices.Add(0x10390900,@"SiS900");
                //mDevices.Add(0x10390901,@"Device ID = 0900 SiS900");
                //mDevices.Add(0x10390962,@"SiS962");
                //mDevices.Add(0x10390963,@"SiS963");
                //mDevices.Add(0x10390964,@"SiS964");
                //mDevices.Add(0x10390999,@"sis950");
                //mDevices.Add(0x10391039,@"SiS5597");
                //mDevices.Add(0x10391182,@"Raid Controller(?Mode Raid5)");
                //mDevices.Add(0x10391183,@"?SIS966/968 ");
                //mDevices.Add(0x10391185,@"SIS968");
                //mDevices.Add(0x10393602,@"SiS83C602");
                //mDevices.Add(0x10395107,@"SiS5107");
                //mDevices.Add(0x10395300,@"SiS540");
                //mDevices.Add(0x10395315,@"SiS530/1/2");
                //mDevices.Add(0x10395401,@"SiS5401");
                //mDevices.Add(0x10395511,@"SiS5511/5512");
                //mDevices.Add(0x10395513,@"All SIS SouthBridge");
                //mDevices.Add(0x10395517,@"SiS5517");
                //mDevices.Add(0x10395518,@"SiS5518");
                //mDevices.Add(0x10395571,@"SiS5571");
                //mDevices.Add(0x10395581,@"SiS 5581");
                //mDevices.Add(0x10395582,@"SiS5582");
                //mDevices.Add(0x10395591,@"MTC0001");
                //mDevices.Add(0x10395596,@"SiS5596");
                //mDevices.Add(0x10395597,@"SiS5597");
                //mDevices.Add(0x10395600,@"SiS600");
                //mDevices.Add(0x10395630,@"SiS630");
                //mDevices.Add(0x10396204,@"SiS6204");
                //mDevices.Add(0x10396205,@"SiS6206");
                //mDevices.Add(0x10396225,@"SiS 950 m2284dxs");
                //mDevices.Add(0x10396236,@"SiS6236");
                //mDevices.Add(0x10396300,@"SiS630/730");
                //mDevices.Add(0x10396306,@"SiS530/620");
                //mDevices.Add(0x10396325,@"SiS650");
                //mDevices.Add(0x10396326,@"1039");
                //mDevices.Add(0x10396330,@"SiS661FX/M661FX/760/741/M760/M741");
                //mDevices.Add(0x10397001,@"SiS7001");
                //mDevices.Add(0x10397002,@"SiS7001");
                //mDevices.Add(0x10397005,@"SiS551/2");
                //mDevices.Add(0x10397007,@"OHCI Compliant FireWire Controller");
                //mDevices.Add(0x10397012,@"SiS7012");
                //mDevices.Add(0x10397013,@"SiS7013");
                //mDevices.Add(0x10397015,@"SiS550 dd");
                //mDevices.Add(0x10397016,@"SiS7016");
                //mDevices.Add(0x10397018,@"SiS7018");
                //mDevices.Add(0x10397019,@"SiS550/1/2");
                //mDevices.Add(0x10397300,@"SiS7013");
                //mDevices.Add(0x10399876,@"sis6215");
                //mDevices.Add(0x103C0024,@"Standard Vista USB Keyboard");
                //mDevices.Add(0x103C1005,@"A4977A");
                //mDevices.Add(0x103C1008,@"Donner GFX");
                //mDevices.Add(0x103C100A,@"hpVisualizeFX");
                //mDevices.Add(0x103C1028,@"Tachyon TL Fibre Channel Adapter");
                //mDevices.Add(0x103C1029,@"HPFC-5200B");
                //mDevices.Add(0x103C102A,@"Tach TS");
                //mDevices.Add(0x103C1030,@"J2585A");
                //mDevices.Add(0x103C1031,@"J2585B222");
                //mDevices.Add(0x103C1040,@"J2973A");
                //mDevices.Add(0x103C1041,@"J2585B");
                //mDevices.Add(0x103C1042,@"J2970A");
                //mDevices.Add(0x103C1048,@"SAS");
                //mDevices.Add(0x103C1049,@"DIVA1");
                //mDevices.Add(0x103C104A,@"DIVA2");
                //mDevices.Add(0x103C104B,@"SP2");
                //mDevices.Add(0x103C104D,@"J3242A");
                //mDevices.Add(0x103C1064,@"79C970");
                //mDevices.Add(0x103C10C1,@"NetServer Smart IRQ Router");
                //mDevices.Add(0x103C10ED,@"TopTools");
                //mDevices.Add(0x103C1200,@"82557B");
                //mDevices.Add(0x103C1219,@"NetServer PCI Hot-Plug Controller");
                //mDevices.Add(0x103C121A,@"NetServer SMIC Controller");
                //mDevices.Add(0x103C121B,@"NetServer Legacy COM Port Decoder");
                //mDevices.Add(0x103C121C,@"NetServer PCI COM Port Decoder");
                //mDevices.Add(0x103C1229,@"zx1");
                //mDevices.Add(0x103C122A,@"zx1");
                //mDevices.Add(0x103C122B,@"zx1");
                //mDevices.Add(0x103C12FA,@"Broadcom BCM4306");
                //mDevices.Add(0x103C2910,@"E2910A");
                //mDevices.Add(0x103C2920,@"Fast Host Interface");
                //mDevices.Add(0x103C2924,@"E2924A");
                //mDevices.Add(0x103C2925,@"E2925A");
                //mDevices.Add(0x103C2926,@"E2926A");
                //mDevices.Add(0x103C2927,@"E2927A");
                //mDevices.Add(0x103C2940,@"E2940A");
                //mDevices.Add(0x103C3206,@"unknown");
                //mDevices.Add(0x103C3220,@"P600");
                //mDevices.Add(0x103C3230,@"Smart Array P400 Controller");
                //mDevices.Add(0x103C3302,@"3305103C");
                //mDevices.Add(0x103C5461,@"unknown");
                //mDevices.Add(0x10430675,@"6600");
                //mDevices.Add(0x10441012,@"Domino");
                //mDevices.Add(0x1044A400,@"2124A/9X");
                //mDevices.Add(0x1044A500,@"PCI Bridge");
                //mDevices.Add(0x1044A501,@"I2O SmartRAID V Controller");
                //mDevices.Add(0x1044A511,@"Raptor");
                //mDevices.Add(0x1045A0F8,@"82C750");
                //mDevices.Add(0x1045C101,@"82C264");
                //mDevices.Add(0x1045C178,@"82C178");
                //mDevices.Add(0x1045C556,@"82C556");
                //mDevices.Add(0x1045C557,@"82C557");
                //mDevices.Add(0x1045C558,@"82C558");
                //mDevices.Add(0x1045C567,@"82C750");
                //mDevices.Add(0x1045C568,@"82C750");
                //mDevices.Add(0x1045C569,@"82C579");
                //mDevices.Add(0x1045C621,@"82C621");
                //mDevices.Add(0x1045C700,@"82C700");
                //mDevices.Add(0x1045C701,@"82C701");
                //mDevices.Add(0x1045C814,@"82C814");
                //mDevices.Add(0x1045C822,@"82C822");
                //mDevices.Add(0x1045C824,@"82C824");
                //mDevices.Add(0x1045C825,@"82C825 function 0");
                //mDevices.Add(0x1045C832,@"82C832");
                //mDevices.Add(0x1045C861,@"82C861");
                //mDevices.Add(0x1045C881,@"82C881");
                //mDevices.Add(0x1045C895,@"82C895");
                //mDevices.Add(0x1045C931,@"82C931");
                //mDevices.Add(0x1045C935,@"82C935");
                //mDevices.Add(0x1045D568,@"82C825");
                //mDevices.Add(0x1045D768,@"82C750");
                //mDevices.Add(0x10480253,@"0x0c92");
                //mDevices.Add(0x10480C60,@"Elsa Gladiac MX");
                //mDevices.Add(0x10480C71,@"Elsa Gladiac 721");
                //mDevices.Add(0x10481000,@"Quick Step 1000");
                //mDevices.Add(0x10483000,@"QuickStep 3000");
                //mDevices.Add(0x10488901,@"ELSA GLoria XL");
                //mDevices.Add(0x104A0008,@"STG 2000X");
                //mDevices.Add(0x104A0009,@"STG 1764X");
                //mDevices.Add(0x104A0010,@"STG4000");
                //mDevices.Add(0x104A0209,@"STPC Consmr/Indstrl");
                //mDevices.Add(0x104A020A,@"STPC Atlas/Elite");
                //mDevices.Add(0x104A0210,@"STPC Atlas");
                //mDevices.Add(0x104A021A,@"STPC Consmr-S/Elite");
                //mDevices.Add(0x104A021B,@"STPC Consumer-II");
                //mDevices.Add(0x104A0228,@"STPC Atlas");
                //mDevices.Add(0x104A0230,@"STPC Atlas");
                //mDevices.Add(0x104A0500,@"ST70137");
                //mDevices.Add(0x104A0981,@"10/100 Ethernet Adapter");
                //mDevices.Add(0x104A1746,@"STG 1746X");
                //mDevices.Add(0x104A2774,@"STE10/100A");
                //mDevices.Add(0x104A3520,@"MPEG-II Video Decoder");
                //mDevices.Add(0x104B1040,@"BT958 SCSI Host Adaptor");
                //mDevices.Add(0x104B8130,@"Flashpoint LT");
                //mDevices.Add(0x104C0500,@"TNETE100A/110A/211");
                //mDevices.Add(0x104C0508,@"TI380PCI");
                //mDevices.Add(0x104C1000,@"TI PCI Eagle i/f AS");
                //mDevices.Add(0x104C104C,@"PCI104c");
                //mDevices.Add(0x104C3D04,@"TVP4010");
                //mDevices.Add(0x104C3D07,@"TVP4020");
                //mDevices.Add(0x104C8000,@"TSB12LV21");
                //mDevices.Add(0x104C8009,@"TSB12LV22");
                //mDevices.Add(0x104C8010,@"TSB12LV26");
                //mDevices.Add(0x104C8011,@"PCI4450");
                //mDevices.Add(0x104C8017,@"PCI4410");
                //mDevices.Add(0x104C8019,@"TSB12LV23");
                //mDevices.Add(0x104C8020,@"TSB12LV26");
                //mDevices.Add(0x104C8021,@"TSB43AA22xxx");
                //mDevices.Add(0x104C8023,@"TSB43AB21/A");
                //mDevices.Add(0x104C8024,@"TSB43AB23");
                //mDevices.Add(0x104C8025,@"TSB82AA2");
                //mDevices.Add(0x104C8026,@"TSB43AB21");
                //mDevices.Add(0x104C8027,@"PCI4451");
                //mDevices.Add(0x104C8029,@"040803-2158");
                //mDevices.Add(0x104C8033,@"PCIxx21");
                //mDevices.Add(0x104C8034,@"10981734");
                //mDevices.Add(0x104C8035,@"N/A");
                //mDevices.Add(0x104C8036,@"PCI6515");
                //mDevices.Add(0x104C8038,@"ff101179");
                //mDevices.Add(0x104C803B,@"PCIxx12 VEN_104C&DEV_803B&SUBSYS_0753152D&REV_00\4");
                //mDevices.Add(0x104C803D,@"N/A");
                //mDevices.Add(0x104C8119,@"na");
                //mDevices.Add(0x104C8201,@"006D103C");
                //mDevices.Add(0x104C8204,@"4610, 4515, 4610FM");
                //mDevices.Add(0x104C8400,@"PCI-2422");
                //mDevices.Add(0x104C9065,@"TMX320C6412");
                //mDevices.Add(0x104C9066,@"Hawking HWP54G 802.11b/g Wireless PCI Adapter (Rev T2)");
                //mDevices.Add(0x104CA001,@"TDC1570");
                //mDevices.Add(0x104CA100,@"TDC1561");
                //mDevices.Add(0x104CA102,@"TNETA1575");
                //mDevices.Add(0x104CA186,@"TMS320C6416T");
                //mDevices.Add(0x104CAC10,@"PCI1050");
                //mDevices.Add(0x104CAC11,@"PCI1030/1053");
                //mDevices.Add(0x104CAC12,@"PCI1130");
                //mDevices.Add(0x104CAC13,@"PCI9440");
                //mDevices.Add(0x104CAC15,@"PCI-1131");
                //mDevices.Add(0x104CAC16,@"PCI1250");
                //mDevices.Add(0x104CAC17,@"PCI1220");
                //mDevices.Add(0x104CAC18,@"PCI1260");
                //mDevices.Add(0x104CAC19,@"PCI1221");
                //mDevices.Add(0x104CAC1A,@"PCI1210");
                //mDevices.Add(0x104CAC1B,@"PCI1450");
                //mDevices.Add(0x104CAC1F,@"PCI1251B");
                //mDevices.Add(0x104CAC20,@"PCI2030");
                //mDevices.Add(0x104CAC21,@"PCI2031");
                //mDevices.Add(0x104CAC22,@"PCI2032");
                //mDevices.Add(0x104CAC23,@"PCI2250");
                //mDevices.Add(0x104CAC28,@"PCI2050/2050I");
                //mDevices.Add(0x104CAC30,@"PCI1260");
                //mDevices.Add(0x104CAC40,@"PCI4450");
                //mDevices.Add(0x104CAC41,@"PCI4410");
                //mDevices.Add(0x104CAC42,@"PCI4451");
                //mDevices.Add(0x104CAC43,@"PCI4550");
                //mDevices.Add(0x104CAC44,@"PCI4510SDFSDFSD");
                //mDevices.Add(0x104CAC46,@"PCI4520");
                //mDevices.Add(0x104CAC50,@"PCI1410");
                //mDevices.Add(0x104CAC51,@"PCI-1420");
                //mDevices.Add(0x104CAC52,@"PCI1451");
                //mDevices.Add(0x104CAC53,@"PCI1421");
                //mDevices.Add(0x104CAC54,@"PCI-1510");
                //mDevices.Add(0x104CAC55,@"PCI1520");
                //mDevices.Add(0x104CAC56,@"PCI1510");
                //mDevices.Add(0x104CAC57,@"PCI1530");
                //mDevices.Add(0x104CAC58,@"PCI1515");
                //mDevices.Add(0x104CAC59,@"PCI1620");
                //mDevices.Add(0x104CAC5A,@"PCI1610");
                //mDevices.Add(0x104CAC8F,@"7420");
                //mDevices.Add(0x104CFE00,@"tsb12lv26");
                //mDevices.Add(0x104CFE03,@"12C01A");
                //mDevices.Add(0x104D8009,@"CXD1947");
                //mDevices.Add(0x104D8039,@"CXD3222");
                //mDevices.Add(0x104D8056,@"0x127a");
                //mDevices.Add(0x104D808A,@"Qc pass");
                //mDevices.Add(0x104E0017,@"OTI-64017");
                //mDevices.Add(0x104E0107,@"OTI107");
                //mDevices.Add(0x104E0109,@"Video Adapter");
                //mDevices.Add(0x104E0217,@"OTI-64217");
                //mDevices.Add(0x104E0317,@"OTI-64317");
                //mDevices.Add(0x104E0611,@"OTI-610");
                //mDevices.Add(0x104F104F,@"iatca8262");
                //mDevices.Add(0x10500000,@"004005-34c8c8");
                //mDevices.Add(0x10500001,@"W83769F");
                //mDevices.Add(0x10500033,@"W89C33");
                //mDevices.Add(0x10500105,@"W82C105");
                //mDevices.Add(0x10500628,@"W83628F/629D");
                //mDevices.Add(0x10500840,@"W89C840F");
                //mDevices.Add(0x10500940,@"w89c940f");
                //mDevices.Add(0x10501050,@"W99200aF");
                //mDevices.Add(0x10505A5A,@"W89C940F");
                //mDevices.Add(0x10506692,@"W6692CF");
                //mDevices.Add(0x10509921,@"W99200AF");
                //mDevices.Add(0x10509922,@"W9922PF");
                //mDevices.Add(0x10509960,@"W9960CF");
                //mDevices.Add(0x10509961,@"W9961CF");
                //mDevices.Add(0x10509970,@"W9970CF");
                //mDevices.Add(0x10509971,@"W9971CF");
                //mDevices.Add(0x10510100,@"Motorola MC145575");
                //mDevices.Add(0x10540001,@"PCI Bridge");
                //mDevices.Add(0x10540002,@"PCI bus Cntrlr");
                //mDevices.Add(0x10543505,@"SH7751");
                //mDevices.Add(0x10550810,@"EFAR 486 host Bridge");
                //mDevices.Add(0x10550922,@"Pentium/p54c host Bridge");
                //mDevices.Add(0x10550926,@"I don�t know pentium 133 intel   everex setpnot mx");
                //mDevices.Add(0x10559130,@"SLC90E66");
                //mDevices.Add(0x10559460,@"SLC90E66");
                //mDevices.Add(0x10559461,@"SLC90E66");
                //mDevices.Add(0x10559462,@"SLC90E66");
                //mDevices.Add(0x10559463,@"SLC90E66");
                //mDevices.Add(0x10570001,@"MPC105");
                //mDevices.Add(0x10570002,@"MPC106");
                //mDevices.Add(0x10570003,@"MPC8240");
                //mDevices.Add(0x10570004,@"MPC107");
                //mDevices.Add(0x10570006,@"MPC8245");
                //mDevices.Add(0x10570100,@"MC145575");
                //mDevices.Add(0x10570431,@"KTI829c");
                //mDevices.Add(0x10571801,@"DSP56301");
                //mDevices.Add(0x10571802,@"DSP56305");
                //mDevices.Add(0x105718C0,@"MPC8265A/66");
                //mDevices.Add(0x10573052,@"SM56");
                //mDevices.Add(0x10573055,@"sm56");
                //mDevices.Add(0x10573421,@"56IVMR/Phoenix 56ISM");
                //mDevices.Add(0x10574801,@"Raven");
                //mDevices.Add(0x10574802,@"Falcon");
                //mDevices.Add(0x10574803,@"Hawk");
                //mDevices.Add(0x10574806,@"CPX8216");
                //mDevices.Add(0x10574809,@"CPX8216T");
                //mDevices.Add(0x10575600,@"WDM MOT8888");
                //mDevices.Add(0x10575602,@"SM56");
                //mDevices.Add(0x10575608,@"52-6116-2A 21-864-4");
                //mDevices.Add(0x10575803,@"MPC5200");
                //mDevices.Add(0x10576400,@"MPC190");
                //mDevices.Add(0x105A0D30,@"PDC20265R");
                //mDevices.Add(0x105A0D38,@"PDC20263");
                //mDevices.Add(0x105A1275,@"PDC20275");
                //mDevices.Add(0x105A3318,@"PDC20318?");
                //mDevices.Add(0x105A3319,@"PDC20319?");
                //mDevices.Add(0x105A3371,@"PDC20371?");
                //mDevices.Add(0x105A3373,@"PDC20378");
                //mDevices.Add(0x105A3375,@"PDC20375?");
                //mDevices.Add(0x105A3376,@"PDC20376");
                //mDevices.Add(0x105A3570,@"PDC20771");
                //mDevices.Add(0x105A3571,@"0x3571");
                //mDevices.Add(0x105A3574,@"PDC20579");
                //mDevices.Add(0x105A3D18,@"PDC20518");
                //mDevices.Add(0x105A3D73,@"PDC20775");
                //mDevices.Add(0x105A4D30,@"PDC20267");
                //mDevices.Add(0x105A4D33,@"PDC20246");
                //mDevices.Add(0x105A4D38,@"PDC20262");
                //mDevices.Add(0x105A4D68,@"PDC20268");
                //mDevices.Add(0x105A4D69,@"PDC20269");
                //mDevices.Add(0x105A5275,@"PDC20276");
                //mDevices.Add(0x105A5300,@"DC5300");
                //mDevices.Add(0x105A6268,@"PDC20268R");
                //mDevices.Add(0x105A6269,@"PDC20271");
                //mDevices.Add(0x105A6621,@"PDC 20621(?)");
                //mDevices.Add(0x105A6622,@"PDC 20622 (?)");
                //mDevices.Add(0x105A6629,@"FastTrak TX4000 Controller");
                //mDevices.Add(0x105A7275,@"PDC 20277");
                //mDevices.Add(0x105D2309,@"Imagine 128");
                //mDevices.Add(0x105D2339,@"I128s2");
                //mDevices.Add(0x105D493D,@"T2R");
                //mDevices.Add(0x105D5348,@"Revolution IV");
                //mDevices.Add(0x10600001,@"UM82C881");
                //mDevices.Add(0x10600002,@"UM82C886");
                //mDevices.Add(0x10600101,@"UM8673F");
                //mDevices.Add(0x10600881,@"UM8881");
                //mDevices.Add(0x10600886,@"UM8886F");
                //mDevices.Add(0x10600891,@"UM82C891");
                //mDevices.Add(0x10601001,@"UM886A");
                //mDevices.Add(0x1060673A,@"UM8886BF");
                //mDevices.Add(0x1060673B,@"EIDE Master/DMA");
                //mDevices.Add(0x10608710,@"UM8710");
                //mDevices.Add(0x10608821,@"CPU/PCI Bridge");
                //mDevices.Add(0x10608822,@"PCI/ISA Bridge");
                //mDevices.Add(0x10608851,@"Pentium CPU/PCI Bridge");
                //mDevices.Add(0x10608852,@"Pentium CPU/ISA Bridge");
                //mDevices.Add(0x1060886A,@"UM8886A");
                //mDevices.Add(0x10608881,@"UM8881F");
                //mDevices.Add(0x10608886,@"UM8886");
                //mDevices.Add(0x1060888A,@"UM8886A");
                //mDevices.Add(0x10608891,@"UM8891");
                //mDevices.Add(0x10609017,@"UM9017F");
                //mDevices.Add(0x10609018,@"UM9018");
                //mDevices.Add(0x10609026,@"UM9026");
                //mDevices.Add(0x1060E881,@"UM8881");
                //mDevices.Add(0x1060E886,@"UM8886N");
                //mDevices.Add(0x1060E88A,@"UM8886N");
                //mDevices.Add(0x1060E891,@"UM8891N");
                //mDevices.Add(0x10610001,@"AGX013/016");
                //mDevices.Add(0x10610002,@"IIT3104");
                //mDevices.Add(0x10658139,@"Realtek 8139C Network Card");
                //mDevices.Add(0x10660000,@"PT80C826");
                //mDevices.Add(0x10660001,@"PT86C521");
                //mDevices.Add(0x10660002,@"PT86C523");
                //mDevices.Add(0x10660003,@"PT80C524");
                //mDevices.Add(0x10660004,@"PT80C525");
                //mDevices.Add(0x10660005,@"PC87550");
                //mDevices.Add(0x10668002,@"PT86C523");
                //mDevices.Add(0x10671002,@"VG500");
                //mDevices.Add(0x106B0001,@"Bandit");
                //mDevices.Add(0x106B0002,@"Grand Central");
                //mDevices.Add(0x106B0003,@"Control Video");
                //mDevices.Add(0x106B0004,@"PlanB");
                //mDevices.Add(0x106B0007,@"OHare");
                //mDevices.Add(0x106B000E,@"Hydra");
                //mDevices.Add(0x106B0010,@"Heathrow");
                //mDevices.Add(0x106B0017,@"Paddington");
                //mDevices.Add(0x106B0018,@"UniNorth");
                //mDevices.Add(0x106B001F,@"UniNorth");
                //mDevices.Add(0x106B0020,@"UniNorth");
                //mDevices.Add(0x106B0026,@"Pangea");
                //mDevices.Add(0x106B0027,@"Pangea");
                //mDevices.Add(0x106B002D,@"UniNorth 1.5");
                //mDevices.Add(0x106B002E,@"UniNorth 1.5");
                //mDevices.Add(0x106B002F,@"UniNorth 1.5");
                //mDevices.Add(0x106B0030,@"UniNorth/Pangea");
                //mDevices.Add(0x106B003B,@"Intrepid");
                //mDevices.Add(0x106B0050,@"Shasta");
                //mDevices.Add(0x106B0051,@"Shasta");
                //mDevices.Add(0x106B0052,@"Shasta");
                //mDevices.Add(0x106B0053,@"Shasta");
                //mDevices.Add(0x106B0054,@"Shasta");
                //mDevices.Add(0x106B0055,@"Shasta");
                //mDevices.Add(0x106B0058,@"U3L");
                //mDevices.Add(0x106C8801,@"Dual Pentium ISA/PCI Motherboard");
                //mDevices.Add(0x106C8802,@"P54C Tr8");
                //mDevices.Add(0x106C8803,@"Dual Window Graphics Accelerator");
                //mDevices.Add(0x106C8804,@"ht019a");
                //mDevices.Add(0x106C8805,@"100-BaseT LAN Controller");
                //mDevices.Add(0x106E4362,@"88E8053");
                //mDevices.Add(0x10730001,@"3D graphics Cntrlr");
                //mDevices.Add(0x10730002,@"YGV615");
                //mDevices.Add(0x10730003,@"00011179");
                //mDevices.Add(0x10730004,@"YMF754B");
                //mDevices.Add(0x10730005,@"DS1");
                //mDevices.Add(0x10730006,@"DS1");
                //mDevices.Add(0x10730008,@"DS1");
                //mDevices.Add(0x1073000A,@"YMF740");
                //mDevices.Add(0x1073000C,@"YMF740C");
                //mDevices.Add(0x1073000D,@"YMF724F");
                //mDevices.Add(0x10730010,@"YMF744B-V");
                //mDevices.Add(0x10730012,@"YMF754B");
                //mDevices.Add(0x10730020,@"744");
                //mDevices.Add(0x10731000,@"SW1000XG");
                //mDevices.Add(0x10732000,@"DS2416");
                //mDevices.Add(0x10744E78,@"82C500/1");
                //mDevices.Add(0x10771016,@"ISP10160");
                //mDevices.Add(0x10771020,@"ISP1040B/1020A");
                //mDevices.Add(0x10771022,@"ISP1022A");
                //mDevices.Add(0x10771080,@"ISP1080");
                //mDevices.Add(0x10771216,@"ISP12160");
                //mDevices.Add(0x10771240,@"ISP1240");
                //mDevices.Add(0x10771280,@"ISP1280");
                //mDevices.Add(0x10772020,@"ISP2020A");
                //mDevices.Add(0x10772100,@"ISP2100");
                //mDevices.Add(0x10772200,@"ISP2200");
                //mDevices.Add(0x10772300,@"ISP 2300");
                //mDevices.Add(0x10772312,@"ISP 2312");
                //mDevices.Add(0x10772422,@"ISP2422");
                //mDevices.Add(0x10772432,@"ISP2432");
                //mDevices.Add(0x10773010,@"n/a");
                //mDevices.Add(0x10776422,@"EP2422");
                //mDevices.Add(0x10776432,@"EP2432");
                //mDevices.Add(0x10780000,@"Cx5520");
                //mDevices.Add(0x10780001,@"MediaGXm");
                //mDevices.Add(0x10780002,@"Cx5520");
                //mDevices.Add(0x10780100,@"Cx5530");
                //mDevices.Add(0x10780101,@"Cx5530");
                //mDevices.Add(0x10780102,@"Cx5530");
                //mDevices.Add(0x10780103,@"Cx5530");
                //mDevices.Add(0x10780104,@"Cx5530");
                //mDevices.Add(0x10780400,@"ZFMicro");
                //mDevices.Add(0x10780401,@"ZFMicro");
                //mDevices.Add(0x10780402,@"ZFMicro");
                //mDevices.Add(0x10780403,@"ZFMicro");
                //mDevices.Add(0x107D0000,@"P86C850");
                //mDevices.Add(0x107E0001,@"FLIPPER");
                //mDevices.Add(0x107E0002,@"100 vg anylan Cntrlr");
                //mDevices.Add(0x107E0004,@"5526");
                //mDevices.Add(0x107E0005,@"x526");
                //mDevices.Add(0x107E0008,@"4575/5525/5575/6575");
                //mDevices.Add(0x107E9003,@"5535-4P-BRI-ST");
                //mDevices.Add(0x107E9007,@"5535-4P-BRI-U");
                //mDevices.Add(0x107E9008,@"5535-1P-SR");
                //mDevices.Add(0x107E900C,@"5535-1P-SR-ST");
                //mDevices.Add(0x107E900E,@"5535-1P-SR-U");
                //mDevices.Add(0x107E9011,@"5535-1P-PRI");
                //mDevices.Add(0x107E9013,@"5535-2P-PRI");
                //mDevices.Add(0x107E9023,@"5535-4P-BRI-ST");
                //mDevices.Add(0x107E9027,@"5536-4P-BRI-U");
                //mDevices.Add(0x107E9031,@"5535-1P-PRI");
                //mDevices.Add(0x107E9033,@"5536-2P-PRI");
                //mDevices.Add(0x107E9060,@"6535");
                //mDevices.Add(0x107E9070,@"4538");
                //mDevices.Add(0x107E9080,@"4532-002/005");
                //mDevices.Add(0x107E9081,@"4532-001/004");
                //mDevices.Add(0x107E9082,@"4532-000/003");
                //mDevices.Add(0x107E9090,@"107");
                //mDevices.Add(0x107E90A0,@"4539");
                //mDevices.Add(0x107F0802,@"SL82C105");
                //mDevices.Add(0x107F0803,@"EIDE Bus Master Controller");
                //mDevices.Add(0x107F0806,@"EIDE Controller");
                //mDevices.Add(0x107F2015,@"EIDE Controller");
                //mDevices.Add(0x10800600,@"82C596/9");
                //mDevices.Add(0x1080C691,@"Cypress CY7c68001");
                //mDevices.Add(0x1080C693,@"82C693");
                //mDevices.Add(0x10810D47,@"2330");
                //mDevices.Add(0x10830001,@"FR710");
                //mDevices.Add(0x10830613,@"Host Bridge");
                //mDevices.Add(0x10850001,@"UsbDgn");
                //mDevices.Add(0x108A0001,@"Model 617");
                //mDevices.Add(0x108A0010,@"Model 618");
                //mDevices.Add(0x108A0040,@"dataBLIZZARD");
                //mDevices.Add(0x108A3000,@"Model 2106");
                //mDevices.Add(0x108D0001,@"OC-3136/37");
                //mDevices.Add(0x108D0002,@"OC-3139f");
                //mDevices.Add(0x108D0004,@"OC-3139/40");
                //mDevices.Add(0x108D0005,@"OC-3250");
                //mDevices.Add(0x108D0006,@"OC-3530");
                //mDevices.Add(0x108D0007,@"OC-3141");
                //mDevices.Add(0x108D0008,@"OC-3540");
                //mDevices.Add(0x108D000A,@"OC-3150");
                //mDevices.Add(0x108D0011,@"OC-2805");
                //mDevices.Add(0x108D0012,@"OC-2325");
                //mDevices.Add(0x108D0013,@"OC-2183/85");
                //mDevices.Add(0x108D0014,@"OC-2326");
                //mDevices.Add(0x108D0019,@"OC-2327/50");
                //mDevices.Add(0x108D0021,@"OC-6151/52");
                //mDevices.Add(0x108D0022,@"ATM Adapter");
                //mDevices.Add(0x108E0001,@"SPARC EBUS");
                //mDevices.Add(0x108E1000,@"PCIO");
                //mDevices.Add(0x108E1001,@"PCIO");
                //mDevices.Add(0x108E1100,@"RIO EBUS");
                //mDevices.Add(0x108E1101,@"RIO GEM");
                //mDevices.Add(0x108E1102,@"RIO 1394");
                //mDevices.Add(0x108E1103,@"RIO USB");
                //mDevices.Add(0x108E2BAD,@"GEM");
                //mDevices.Add(0x108E5000,@"SME2411");
                //mDevices.Add(0x108E5043,@"SunPCI");
                //mDevices.Add(0x108E7063,@"SunPCi");
                //mDevices.Add(0x108E8000,@"STP2223BGA");
                //mDevices.Add(0x108E8001,@"Schizo");
                //mDevices.Add(0x108EA000,@"UltraSPARC IIi");
                //mDevices.Add(0x108EA001,@"UltraSPARC IIe");
                //mDevices.Add(0x10910020,@"3D Graphics Processor");
                //mDevices.Add(0x10910021,@"3D graphics processor w/texturing");
                //mDevices.Add(0x10910040,@"3D graphics frame buffer");
                //mDevices.Add(0x10910041,@"3D graphics frame buffer");
                //mDevices.Add(0x10910060,@"Proprietary bus Bridge");
                //mDevices.Add(0x109100E4,@"Powerstorm 4D50T");
                //mDevices.Add(0x10910720,@"Motion JPEG Codec");
                //mDevices.Add(0x109200A0,@"SpeedStar Pro SE");
                //mDevices.Add(0x109200A8,@"SpeedStar 64");
                //mDevices.Add(0x10920550,@"Viper V550");
                //mDevices.Add(0x109208D4,@"Supra 2260");
                //mDevices.Add(0x1092094C,@"SupraExpress 56i Pro");
                //mDevices.Add(0x109209C8,@"SUP2761");
                //mDevices.Add(0x10921002,@"R6793-12");
                //mDevices.Add(0x10921092,@"Viper V330");
                //mDevices.Add(0x10926120,@"Maximum");
                //mDevices.Add(0x10928810,@"Stealth SE");
                //mDevices.Add(0x10928811,@"Stealth 64/SE");
                //mDevices.Add(0x10928880,@"Stealth Video");
                //mDevices.Add(0x10928881,@"Stealth Video");
                //mDevices.Add(0x109288B0,@"Stealth 64 Video");
                //mDevices.Add(0x109288B1,@"Stealth 64 Video");
                //mDevices.Add(0x109288C0,@"Stealth 64");
                //mDevices.Add(0x109288C1,@"Stealth 64");
                //mDevices.Add(0x109288D0,@"Stealth 64");
                //mDevices.Add(0x109288D1,@"Stealth 64");
                //mDevices.Add(0x109288F0,@"Stealth 64 Video");
                //mDevices.Add(0x109288F1,@"Stealth 64 Video");
                //mDevices.Add(0x10929876,@"Supra Express 56i Pro CW #2");
                //mDevices.Add(0x10929999,@"Monster Sound");
                //mDevices.Add(0x10930160,@"PCI-DIO-96");
                //mDevices.Add(0x10930161,@"PCI-1200");
                //mDevices.Add(0x10930162,@"PCI-MIO-16XE-50");
                //mDevices.Add(0x10931150,@"PCI-DIO-32HS");
                //mDevices.Add(0x10931170,@"PCI-MIO-16XE-10");
                //mDevices.Add(0x10931180,@"PCI-MIO-16E-1");
                //mDevices.Add(0x10931190,@"PCI-MIO-16E-4");
                //mDevices.Add(0x10931270,@"PCI-6032E");
                //mDevices.Add(0x10931310,@"PCI-6602");
                //mDevices.Add(0x10931330,@"PCI-6031E");
                //mDevices.Add(0x10931340,@"PCI-6033E");
                //mDevices.Add(0x10931350,@"PCI-6071E");
                //mDevices.Add(0x109317D0,@"PCI-6503");
                //mDevices.Add(0x10932A60,@"PCI-6023E");
                //mDevices.Add(0x10932A70,@"PCI-6024E");
                //mDevices.Add(0x10932A80,@"PCI-6025E");
                //mDevices.Add(0x10932C80,@"PCI-6035E");
                //mDevices.Add(0x1093B001,@"IMAQ-PCI-1408");
                //mDevices.Add(0x1093B011,@"IMAQ-PXI-1408");
                //mDevices.Add(0x1093B021,@"IMAQ-PCI-1424");
                //mDevices.Add(0x1093B031,@"IMAQ-PCI-1413");
                //mDevices.Add(0x1093B041,@"IMAQ-PCI-1407");
                //mDevices.Add(0x1093B051,@"IMAQ-PXI-1407");
                //mDevices.Add(0x1093B061,@"IMAQ-PCI-1411");
                //mDevices.Add(0x1093B071,@"IMAQ-PCI-1422");
                //mDevices.Add(0x1093B081,@"IMAQ-PXI-1422");
                //mDevices.Add(0x1093B091,@"IMAQ-PXI-1411");
                //mDevices.Add(0x1093C801,@"PCI-GPIB");
                //mDevices.Add(0x10950640,@"PCI0640A/B");
                //mDevices.Add(0x10950641,@"PCI0640");
                //mDevices.Add(0x10950642,@"PCI0642");
                //mDevices.Add(0x10950643,@"PCI0643");
                //mDevices.Add(0x10950646,@"PCI0646");
                //mDevices.Add(0x10950647,@"PCI0647");
                //mDevices.Add(0x10950648,@"PCI-648");
                //mDevices.Add(0x10950649,@"PCI-649");
                //mDevices.Add(0x10950650,@"PBC0650A");
                //mDevices.Add(0x10950670,@"USB0670");
                //mDevices.Add(0x10950673,@"USB0673");
                //mDevices.Add(0x10950680,@"SiI 0680");
                //mDevices.Add(0x10952455,@"SI3124");
                //mDevices.Add(0x10953112,@"SiI 3112");
                //mDevices.Add(0x10953114,@"Sil 3114");
                //mDevices.Add(0x10953124,@"SiI 3124");
                //mDevices.Add(0x10953132,@"SiI 3132");
                //mDevices.Add(0x10953512,@"Sil 3512");
                //mDevices.Add(0x10953531,@"3531");
                //mDevices.Add(0x10961106,@"0x3059");
                //mDevices.Add(0x10970038,@"EIDE Controller (single FIFO)");
                //mDevices.Add(0x10980001,@"QD8500");
                //mDevices.Add(0x10980002,@"QD8580");
                //mDevices.Add(0x109A8280,@"0x8280");
                //mDevices.Add(0x109E0350,@"BT848KPF");
                //mDevices.Add(0x109E0351,@"B t878 khf");
                //mDevices.Add(0x109E0369,@"Bt878");
                //mDevices.Add(0x109E036E,@"878Asad");
                //mDevices.Add(0x109E036F,@"Bt878");
                //mDevices.Add(0x109E0370,@"Bt880");
                //mDevices.Add(0x109E0878,@"7610144D&REV_02\4&1F7DBC9F&0&09F0");
                //mDevices.Add(0x109E0879,@"Bt879khf");
                //mDevices.Add(0x109E0880,@"Bt880");
                //mDevices.Add(0x109E109E,@"BT848akpf");
                //mDevices.Add(0x109E2115,@"BtV 2115 Mera Lun");
                //mDevices.Add(0x109E2125,@"BtV 2125");
                //mDevices.Add(0x109E2164,@"BtV 2164");
                //mDevices.Add(0x109E2165,@"BtV 2165");
                //mDevices.Add(0x109E8230,@"BtV 8230");
                //mDevices.Add(0x109E8472,@"Bt8471/72");
                //mDevices.Add(0x109E8474,@"Bt8474");
                //mDevices.Add(0x10A80000,@"64-bit GUI Accelerator");
                //mDevices.Add(0x10A90004,@"O2 MACE");
                //mDevices.Add(0x10A90005,@"RAD Audio");
                //mDevices.Add(0x10A90006,@"HPCEX");
                //mDevices.Add(0x10A90007,@"RPCEX");
                //mDevices.Add(0x10A90008,@"DiVO VIP");
                //mDevices.Add(0x10A90009,@"Alteon");
                //mDevices.Add(0x10A90010,@"AMP");
                //mDevices.Add(0x10A90011,@"GRIP");
                //mDevices.Add(0x10A90012,@"SGH PSHAC GSN");
                //mDevices.Add(0x10A91001,@"Magic Carpet");
                //mDevices.Add(0x10A91002,@"Lithium");
                //mDevices.Add(0x10A91003,@"Dual JPEG 1");
                //mDevices.Add(0x10A91004,@"Dual JPEG 2");
                //mDevices.Add(0x10A91005,@"Dual JPEG 3");
                //mDevices.Add(0x10A91006,@"Dual JPEG 4");
                //mDevices.Add(0x10A91007,@"Dual JPEG 5");
                //mDevices.Add(0x10A91008,@"Cesium");
                //mDevices.Add(0x10A92001,@"Fibre Channel");
                //mDevices.Add(0x10A92002,@"ASDE");
                //mDevices.Add(0x10A98001,@"O2 1394");
                //mDevices.Add(0x10A98002,@"G-net NT");
                //mDevices.Add(0x10AB1005,@"USB\VID_0000&PID_0000\5&5657949&0&8");
                //mDevices.Add(0x10AD0001,@"W83769F");
                //mDevices.Add(0x10AD0103,@"sl82c103");
                //mDevices.Add(0x10AD0105,@"W83789F");
                //mDevices.Add(0x10AD0565,@"W83C553F");
                //mDevices.Add(0x10B50480,@"IOP 480");
                //mDevices.Add(0x10B50960,@"PCI 9080RDK-960");
                //mDevices.Add(0x10B51030,@"Gazel R685");
                //mDevices.Add(0x10B51054,@"Gazel R697");
                //mDevices.Add(0x10B51078,@"PCI 9050");
                //mDevices.Add(0x10B51103,@"PCI 9050");
                //mDevices.Add(0x10B51146,@"PCI 9050");
                //mDevices.Add(0x10B51147,@"PCI 9050");
                //mDevices.Add(0x10B51151,@"Gazel R753");
                //mDevices.Add(0x10B51152,@"Gazel R753");
                //mDevices.Add(0x10B52724,@"Thales PCSM Security Card");
                //mDevices.Add(0x10B53001,@"plx9030");
                //mDevices.Add(0x10B55406,@"PCI RDK9054-LITE");
                //mDevices.Add(0x10B55601,@"PCI 9056");
                //mDevices.Add(0x10B56520,@"PCI6520");
                //mDevices.Add(0x10B58111,@"PEX 8311");
                //mDevices.Add(0x10B58509,@"PEX8509");
                //mDevices.Add(0x10B58516,@"PEX 8516");
                //mDevices.Add(0x10B58518,@"PEX8518-AB25BI");
                //mDevices.Add(0x10B58532,@"PEX 8532");
                //mDevices.Add(0x10B59030,@"PCI 9030");
                //mDevices.Add(0x10B59036,@"PCI9036");
                //mDevices.Add(0x10B59050,@"PCI 9050");
                //mDevices.Add(0x10B59052,@"PCI 9052");
                //mDevices.Add(0x10B59054,@"PCI 9054");
                //mDevices.Add(0x10B59056,@"PCI9056");
                //mDevices.Add(0x10B59060,@"PCI9060");
                //mDevices.Add(0x10B5906D,@"PCI 9060SD");
                //mDevices.Add(0x10B5906E,@"PCI 9060ES");
                //mDevices.Add(0x10B59080,@"PCI 9080");
                //mDevices.Add(0x10B60001,@"Smart 16/4");
                //mDevices.Add(0x10B60002,@"Smart 16/4");
                //mDevices.Add(0x10B60003,@"Smart 16/4");
                //mDevices.Add(0x10B60004,@"Smart 16/4 Ringnode Mk1 (PCIBM1)");
                //mDevices.Add(0x10B60006,@"16/4 CardBus Adapter (Eric 2)");
                //mDevices.Add(0x10B60007,@"Presto PCI");
                //mDevices.Add(0x10B60009,@"Smart 100/16/4 PCi-HS Ringnode");
                //mDevices.Add(0x10B6000A,@"Smart 100/16/4 PCI Ringnode");
                //mDevices.Add(0x10B6000B,@"16/4 CardBus  Adapter Mk2");
                //mDevices.Add(0x10B61000,@"Horizon");
                //mDevices.Add(0x10B61001,@"Ambassador");
                //mDevices.Add(0x10B61002,@"Ambassador");
                //mDevices.Add(0x10B70001,@"3C985");
                //mDevices.Add(0x10B70013,@"3com p/n: 3CRDAG675");
                //mDevices.Add(0x10B71000,@"3C905CX-TXNM");
                //mDevices.Add(0x10B71006,@"14e4:1645");
                //mDevices.Add(0x10B71007,@"3C556");
                //mDevices.Add(0x10B71700,@"3C940");
                //mDevices.Add(0x10B71F1F,@"3CRWE777A");
                //mDevices.Add(0x10B73390,@"3C339");
                //mDevices.Add(0x10B73590,@"3C359");
                //mDevices.Add(0x10B74500,@"3C450");
                //mDevices.Add(0x10B75055,@"3C555");
                //mDevices.Add(0x10B75057,@"3C575");
                //mDevices.Add(0x10B75157,@"3C575B");
                //mDevices.Add(0x10B75257,@"3CCFE575CT");
                //mDevices.Add(0x10B75900,@"3C590");
                //mDevices.Add(0x10B75920,@"3C592");
                //mDevices.Add(0x10B75950,@"3C595");
                //mDevices.Add(0x10B75951,@"3C595");
                //mDevices.Add(0x10B75952,@"3C595");
                //mDevices.Add(0x10B75970,@"3C597");
                //mDevices.Add(0x10B75B57,@"3C595");
                //mDevices.Add(0x10B76055,@"3C556");
                //mDevices.Add(0x10B76056,@"3CN3AC1556B");
                //mDevices.Add(0x10B76560,@"3CCFE656");
                //mDevices.Add(0x10B76561,@"FEM656");
                //mDevices.Add(0x10B76562,@"3CCFEM656");
                //mDevices.Add(0x10B76563,@"FEM656B");
                //mDevices.Add(0x10B76564,@"3CCFEM656");
                //mDevices.Add(0x10B76565,@"3CCFEM656C");
                //mDevices.Add(0x10B77646,@"3CSOHO100B-TX");
                //mDevices.Add(0x10B77770,@"???");
                //mDevices.Add(0x10B78811,@"Token Ring");
                //mDevices.Add(0x10B79000,@"3C900-TPO");
                //mDevices.Add(0x10B79001,@"3C900-COMBO");
                //mDevices.Add(0x10B79004,@"3C900-TPO");
                //mDevices.Add(0x10B79005,@"3C900B-COMBO");
                //mDevices.Add(0x10B79006,@"3C900B-TPC");
                //mDevices.Add(0x10B7900A,@"3C900B-FL");
                //mDevices.Add(0x10B79050,@"3C905B - Combo");
                //mDevices.Add(0x10B79051,@"3C905-T4");
                //mDevices.Add(0x10B79055,@"3C905-TX");
                //mDevices.Add(0x10B79056,@"3C905B-T4");
                //mDevices.Add(0x10B79058,@"3C905B-COMBO");
                //mDevices.Add(0x10B7905A,@"3C905B-FX");
                //mDevices.Add(0x10B79200,@"3C905 CX-TX-M");
                //mDevices.Add(0x10B79201,@"3C920B-EMB");
                //mDevices.Add(0x10B79202,@"3C920B-EMB");
                //mDevices.Add(0x10B79210,@"3C920B-EMB-WNM");
                //mDevices.Add(0x10B79300,@"3csoho100b-tx");
                //mDevices.Add(0x10B79800,@"3C980-TX");
                //mDevices.Add(0x10B79805,@"3C980-TX");
                //mDevices.Add(0x10B79902,@"3CR990-TX-95");
                //mDevices.Add(0x10B79903,@"3CR990-TX-97");
                //mDevices.Add(0x10B79905,@"3C990B-FX");
                //mDevices.Add(0x10B79908,@"3CR990SVR95");
                //mDevices.Add(0x10B79909,@"3CR990SVR97");
                //mDevices.Add(0x10B7D004,@"3C900B-TPO");
                //mDevices.Add(0x10B80005,@"LAN83C170QF/171");
                //mDevices.Add(0x10B80006,@"LAN83C175");
                //mDevices.Add(0x10B81000,@"37C665");
                //mDevices.Add(0x10B81001,@"37C922");
                //mDevices.Add(0x10B8A011,@"83C170QF");
                //mDevices.Add(0x10B8B106,@"SMC34C90");
                //mDevices.Add(0x10B90101,@"CMI8338/C3DX");
                //mDevices.Add(0x10B90102,@"CMI8338/C3DX");
                //mDevices.Add(0x10B90111,@"CMI8738/C3DX");
                //mDevices.Add(0x10B90780,@"???");
                //mDevices.Add(0x10B90782,@"???");
                //mDevices.Add(0x10B91435,@"M1435");
                //mDevices.Add(0x10B91445,@"M1445");
                //mDevices.Add(0x10B91449,@"M1449");
                //mDevices.Add(0x10B91451,@"M1451");
                //mDevices.Add(0x10B91461,@"M1461");
                //mDevices.Add(0x10B91489,@"M1489");
                //mDevices.Add(0x10B91511,@"M1511");
                //mDevices.Add(0x10B91513,@"M1513");
                //mDevices.Add(0x10B91521,@"M1521");
                //mDevices.Add(0x10B91523,@"M1523");
                //mDevices.Add(0x10B91533,@"M1535+");
                //mDevices.Add(0x10B91535,@"M1535x");
                //mDevices.Add(0x10B91541,@"M1541");
                //mDevices.Add(0x10B91543,@"M1543 a1");
                //mDevices.Add(0x10B91561,@"M1561");
                //mDevices.Add(0x10B91563,@"M1563");
                //mDevices.Add(0x10B91632,@"M1632");
                //mDevices.Add(0x10B91641,@"M1641");
                //mDevices.Add(0x10B91644,@"M1644");
                //mDevices.Add(0x10B91646,@"M1646");
                //mDevices.Add(0x10B91647,@"M1647");
                //mDevices.Add(0x10B91651,@"M1651");
                //mDevices.Add(0x10B91661,@"M1661");
                //mDevices.Add(0x10B91667,@"M1667");
                //mDevices.Add(0x10B91671,@"M1671");
                //mDevices.Add(0x10B91672,@"M1672");
                //mDevices.Add(0x10B91681,@"M1681");
                //mDevices.Add(0x10B91687,@"M1687");
                //mDevices.Add(0x10B93141,@"M3141");
                //mDevices.Add(0x10B93143,@"M3143");
                //mDevices.Add(0x10B93145,@"M3145");
                //mDevices.Add(0x10B93147,@"M3147");
                //mDevices.Add(0x10B93149,@"M3149");
                //mDevices.Add(0x10B93151,@"M3151");
                //mDevices.Add(0x10B93307,@"M3307");
                //mDevices.Add(0x10B93309,@"M3309");
                //mDevices.Add(0x10B95212,@"M4803");
                //mDevices.Add(0x10B95215,@"MS4803");
                //mDevices.Add(0x10B95217,@"m5217h");
                //mDevices.Add(0x10B95219,@"Ali M5219 PCI BUS MASTER IDE Controller");
                //mDevices.Add(0x10B95225,@"M5225");
                //mDevices.Add(0x10B95228,@"M1563");
                //mDevices.Add(0x10B95229,@"M5229 Southbridge");
                //mDevices.Add(0x10B95235,@"M1621");
                //mDevices.Add(0x10B95236,@"M5273");
                //mDevices.Add(0x10B95237,@"M5273 A1 for windows 98");
                //mDevices.Add(0x10B95239,@"527210B9");
                //mDevices.Add(0x10B95249,@"M5249");
                //mDevices.Add(0x10B95251,@"M5251");
                //mDevices.Add(0x10B95253,@"M5253");
                //mDevices.Add(0x10B95261,@"M5261");
                //mDevices.Add(0x10B95263,@"Albatron K8ULTRA-U Pro");
                //mDevices.Add(0x10B95281,@"M1565/1566");
                //mDevices.Add(0x10B95287,@"ULI M1573");
                //mDevices.Add(0x10B95288,@"ULI M1575/M1697 ");
                //mDevices.Add(0x10B95289,@"ULI M1567/M1689 ");
                //mDevices.Add(0x10B95450,@"Agere Systems AC97 Modem");
                //mDevices.Add(0x10B95451,@"M5451");
                //mDevices.Add(0x10B95455,@"M1563M Southbridge");
                //mDevices.Add(0x10B95457,@"M1563M");
                //mDevices.Add(0x10B95461,@"SW\{CD171DE3-69E5-11D2-B56D-0000F8754380}\{9B36589");
                //mDevices.Add(0x10B95471,@"M1563M Southbridge");
                //mDevices.Add(0x10B95473,@"M1563M Southbridge");
                //mDevices.Add(0x10B97101,@"M7101");
                //mDevices.Add(0x10B97471,@"M1563M Southbridge");
                //mDevices.Add(0x10B99876,@"mdv92xp");
                //mDevices.Add(0x10BA0304,@"GUI Accelerator");
                //mDevices.Add(0x10BD0803,@" MYSON Technology Inc SURECOM EP-320X-S 100/10M Et");
                //mDevices.Add(0x10BD0E34,@"NE34");
                //mDevices.Add(0x10BD5240,@"0055");
                //mDevices.Add(0x10BD5241,@"PCMCIA Bridge");
                //mDevices.Add(0x10BD5242,@"General Purpose Cntrlr");
                //mDevices.Add(0x10BD5243,@"00000000");
                //mDevices.Add(0x10BD5244,@"FCD Cntrlr");
                //mDevices.Add(0x10BD8139,@"surecom EP-320X-R adapter with realtek 8139c chip");
                //mDevices.Add(0x10C80000,@"Graphics Cntrlr");
                //mDevices.Add(0x10C80003,@"NM2093");
                //mDevices.Add(0x10C80004,@"NM2160");
                //mDevices.Add(0x10C80005,@"NM2200");
                //mDevices.Add(0x10C80006,@"NM2360");
                //mDevices.Add(0x10C80016,@"NM2380");
                //mDevices.Add(0x10C80025,@"NM2230");
                //mDevices.Add(0x10C80083,@"NM2097");
                //mDevices.Add(0x10C88005,@"NM2200");
                //mDevices.Add(0x10C88006,@"NM2360");
                //mDevices.Add(0x10C88016,@"NM2380");
                //mDevices.Add(0x10CD1100,@"ASC1100");
                //mDevices.Add(0x10CD1200,@"ASC1200");
                //mDevices.Add(0x10CD1300,@"ASC-3550b");
                //mDevices.Add(0x10CD2300,@"F914C536");
                //mDevices.Add(0x10CD2500,@"ASC38C0800/1600");
                //mDevices.Add(0x10CD4000,@"ASC30C0400");
                //mDevices.Add(0x10CF10C5,@"FMV-103");
                //mDevices.Add(0x10CF2001,@"MB86605");
                //mDevices.Add(0x10CF2002,@"MB86606");
                //mDevices.Add(0x10CF2005,@"MB86974");
                //mDevices.Add(0x10CF200C,@"MB86974");
                //mDevices.Add(0x10CF2010,@"OHCI FireWire Controller");
                //mDevices.Add(0x10CF2011,@"MPEG2 R-Engine (MPEG2 Hardware Encoder)");
                //mDevices.Add(0x10CF2019,@"MB86295");
                //mDevices.Add(0x10CF201E,@"MB86296");
                //mDevices.Add(0x10D6FF51,@"0x0100");
                //mDevices.Add(0x10D90066,@"MX86101P");
                //mDevices.Add(0x10D90512,@"MX98713");
                //mDevices.Add(0x10D90531,@"MX98715/25");
                //mDevices.Add(0x10D90532,@"MX98723/727");
                //mDevices.Add(0x10D90553,@"MX987x5");
                //mDevices.Add(0x10D98625,@"MX86250");
                //mDevices.Add(0x10D98626,@"MX86251");
                //mDevices.Add(0x10D98627,@"MX86251");
                //mDevices.Add(0x10D98888,@"MX86250");
                //mDevices.Add(0x10D9C115,@"lc82c115");
                //mDevices.Add(0x10DC0001,@"STAR/RD24");
                //mDevices.Add(0x10DC0002,@"ATT 2C15-3 (FPGA)");
                //mDevices.Add(0x10DC0004,@"EP20S780");
                //mDevices.Add(0x10DC0010,@"680-1110-150/400");
                //mDevices.Add(0x10DC0011,@"680-1110-200/450");
                //mDevices.Add(0x10DC0012,@"S32PCI64");
                //mDevices.Add(0x10DC0021,@"HIPPI destination");
                //mDevices.Add(0x10DC0022,@"HIPPI source");
                //mDevices.Add(0x10DC0033,@"EP20KE (APEX-FPGA)");
                //mDevices.Add(0x10DC0101,@"SL651 7057 C200");
                //mDevices.Add(0x10DC016A,@"XC4VFX100");
                //mDevices.Add(0x10DC10DC,@"ATT 2C15-3 (FPGA)");
                //mDevices.Add(0x10DD0001,@"3D graphics processor");
                //mDevices.Add(0x10DE0001,@"Lucent 0x00da");
                //mDevices.Add(0x10DE0008,@"NV1");
                //mDevices.Add(0x10DE0009,@"NV1");
                //mDevices.Add(0x10DE0010,@"NV2");
                //mDevices.Add(0x10DE0018,@"NV3");
                //mDevices.Add(0x10DE0019,@"NV3");
                //mDevices.Add(0x10DE0020,@"NV4");
                //mDevices.Add(0x10DE0028,@"NV5");
                //mDevices.Add(0x10DE0029,@"NV5");
                //mDevices.Add(0x10DE002A,@"NV5");
                //mDevices.Add(0x10DE002B,@"NV5");
                //mDevices.Add(0x10DE002C,@"8.0");
                //mDevices.Add(0x10DE002D,@"NV5");
                //mDevices.Add(0x10DE002E,@"NV6");
                //mDevices.Add(0x10DE002F,@"NV6");
                //mDevices.Add(0x10DE0035,@"MCP04");
                //mDevices.Add(0x10DE0036,@"MCP04");
                //mDevices.Add(0x10DE003E,@"MCP04");
                //mDevices.Add(0x10DE0040,@"NV40.0");
                //mDevices.Add(0x10DE0041,@"NV40.1");
                //mDevices.Add(0x10DE0042,@"NV40.2");
                //mDevices.Add(0x10DE0043,@"NV40.3");
                //mDevices.Add(0x10DE0045,@"NV40.5");
                //mDevices.Add(0x10DE0049,@"NV40GL");
                //mDevices.Add(0x10DE004E,@"NV40GL");
                //mDevices.Add(0x10DE0053,@"CK804");
                //mDevices.Add(0x10DE0054,@"CK804");
                //mDevices.Add(0x10DE0055,@"CK804");
                //mDevices.Add(0x10DE0057,@"nForce4 Ultra");
                //mDevices.Add(0x10DE0059,@"Realtek ALC850");
                //mDevices.Add(0x10DE0060,@"nForce MCP2");
                //mDevices.Add(0x10DE0064,@"nForce MCP-T");
                //mDevices.Add(0x10DE0065,@"nForce MCP2/MCP2-T/MCP2-U");
                //mDevices.Add(0x10DE0066,@"nForce MCP-T");
                //mDevices.Add(0x10DE0067,@"nForce MCP2");
                //mDevices.Add(0x10DE0068,@"nForce MCP2");
                //mDevices.Add(0x10DE006A,@"nForce MCP2");
                //mDevices.Add(0x10DE006B,@"nForce MCP-T?");
                //mDevices.Add(0x10DE006C,@"nForce");
                //mDevices.Add(0x10DE006D,@"nForce MCP-T");
                //mDevices.Add(0x10DE006E,@"nForce MCP2");
                //mDevices.Add(0x10DE0085,@"MCP2S");
                //mDevices.Add(0x10DE008C,@"RLT8201BL");
                //mDevices.Add(0x10DE008E,@"MCP2S");
                //mDevices.Add(0x10DE0091,@"G70");
                //mDevices.Add(0x10DE0092,@"NV47 (20x1,7vp)");
                //mDevices.Add(0x10DE009C,@"G70.1");
                //mDevices.Add(0x10DE009D,@"NVIDIA GPU Quadro FX 4500");
                //mDevices.Add(0x10DE00A0,@"NV5");
                //mDevices.Add(0x10DE00C0,@"NV41.0");
                //mDevices.Add(0x10DE00C1,@"NV41.1");
                //mDevices.Add(0x10DE00C2,@"NV41.2");
                //mDevices.Add(0x10DE00C8,@"NV41");
                //mDevices.Add(0x10DE00C9,@"NV41");
                //mDevices.Add(0x10DE00CC,@"NV41");
                //mDevices.Add(0x10DE00CD,@"NV41GL");
                //mDevices.Add(0x10DE00CE,@"NV41GL");
                //mDevices.Add(0x10DE00D0,@"nForce 3");
                //mDevices.Add(0x10DE00D1,@"nForce 3");
                //mDevices.Add(0x10DE00D2,@"nForce 3?");
                //mDevices.Add(0x10DE00D4,@"nForce MCp2");
                //mDevices.Add(0x10DE00D5,@"nForce 150");
                //mDevices.Add(0x10DE00D6,@"nForce MCP3?");
                //mDevices.Add(0x10DE00D7,@"nForce MCP3?");
                //mDevices.Add(0x10DE00D8,@"nForce MCP3?");
                //mDevices.Add(0x10DE00D9,@"nForce 3");
                //mDevices.Add(0x10DE00DA,@"Lucent");
                //mDevices.Add(0x10DE00DD,@"nForce MCP3?");
                //mDevices.Add(0x10DE00DF,@"Marvell 88E1111");
                //mDevices.Add(0x10DE00E0,@"nForce3 250");
                //mDevices.Add(0x10DE00E1,@"nForce3 250");
                //mDevices.Add(0x10DE00E2,@"nForce3 250");
                //mDevices.Add(0x10DE00E3,@"nForce 250");
                //mDevices.Add(0x10DE00E4,@"nForce3 250");
                //mDevices.Add(0x10DE00E5,@"nForce3 250");
                //mDevices.Add(0x10DE00E7,@"nForce3 250");
                //mDevices.Add(0x10DE00E8,@"nForce3 250");
                //mDevices.Add(0x10DE00EA,@"nForce3 250");
                //mDevices.Add(0x10DE00ED,@"nForce3 250");
                //mDevices.Add(0x10DE00EE,@"nForce 250");
                //mDevices.Add(0x10DE00F0,@"NVBR02");
                //mDevices.Add(0x10DE00F1,@"NV43+BR02");
                //mDevices.Add(0x10DE00F2,@"NV43");
                //mDevices.Add(0x10DE00F5,@"G70");
                //mDevices.Add(0x10DE00F6,@"NV41");
                //mDevices.Add(0x10DE00F8,@"NVBR02");
                //mDevices.Add(0x10DE00F9,@"NVBR02");
                //mDevices.Add(0x10DE00FA,@"NVBR02.2");
                //mDevices.Add(0x10DE00FB,@"NVBR02.3");
                //mDevices.Add(0x10DE00FC,@"NVBR02.4");
                //mDevices.Add(0x10DE00FD,@"NVBR02GL");
                //mDevices.Add(0x10DE00FE,@"NVBR02GL");
                //mDevices.Add(0x10DE00FF,@"NVBR02.7");
                //mDevices.Add(0x10DE0100,@"NV10");
                //mDevices.Add(0x10DE0101,@"NV10");
                //mDevices.Add(0x10DE0102,@"NV10");
                //mDevices.Add(0x10DE0103,@"NV10GL");
                //mDevices.Add(0x10DE0110,@"NV11");
                //mDevices.Add(0x10DE0111,@"NV11DDR");
                //mDevices.Add(0x10DE0112,@"NV11");
                //mDevices.Add(0x10DE0113,@"NV11GL");
                //mDevices.Add(0x10DE0141,@"NV43");
                //mDevices.Add(0x10DE0145,@"NV43");
                //mDevices.Add(0x10DE014D,@"unknown");
                //mDevices.Add(0x10DE014E,@"NV43");
                //mDevices.Add(0x10DE014F,@"NV43");
                //mDevices.Add(0x10DE0150,@"NV15");
                //mDevices.Add(0x10DE0151,@"NV15 DDR");
                //mDevices.Add(0x10DE0152,@"NV15BR");
                //mDevices.Add(0x10DE0153,@"NV15GL");
                //mDevices.Add(0x10DE0162,@"NV44?");
                //mDevices.Add(0x10DE0164,@"NV44?");
                //mDevices.Add(0x10DE0165,@"NVS285");
                //mDevices.Add(0x10DE0170,@"NV17.2");
                //mDevices.Add(0x10DE0171,@"NV17.2");
                //mDevices.Add(0x10DE0172,@"NV17.3");
                //mDevices.Add(0x10DE0173,@"NV17.4");
                //mDevices.Add(0x10DE0174,@"NV17M");
                //mDevices.Add(0x10DE0175,@"NV17M");
                //mDevices.Add(0x10DE0176,@"NV17M");
                //mDevices.Add(0x10DE0177,@"NV17M");
                //mDevices.Add(0x10DE0178,@"NV17GL.1");
                //mDevices.Add(0x10DE0179,@"NV17M");
                //mDevices.Add(0x10DE017A,@"NV17GL.2");
                //mDevices.Add(0x10DE017B,@"NV17GL.3");
                //mDevices.Add(0x10DE017C,@"NV17M");
                //mDevices.Add(0x10DE0181,@"NV18B");
                //mDevices.Add(0x10DE0182,@"NV18.2");
                //mDevices.Add(0x10DE0183,@"NV18.4");
                //mDevices.Add(0x10DE0185,@"NV18.6");
                //mDevices.Add(0x10DE0186,@"NV18M");
                //mDevices.Add(0x10DE0187,@"nv18m");
                //mDevices.Add(0x10DE018A,@"Quadro NVS");
                //mDevices.Add(0x10DE018D,@"NV18M");
                //mDevices.Add(0x10DE0191,@"G80");
                //mDevices.Add(0x10DE0193,@"G80");
                //mDevices.Add(0x10DE019D,@"G80");
                //mDevices.Add(0x10DE019E,@"G80");
                //mDevices.Add(0x10DE01A0,@"NVCrush11");
                //mDevices.Add(0x10DE01A4,@"nForce");
                //mDevices.Add(0x10DE01A5,@"nForce");
                //mDevices.Add(0x10DE01A6,@"nForce");
                //mDevices.Add(0x10DE01A8,@"nForce 220");
                //mDevices.Add(0x10DE01A9,@"nForce 420");
                //mDevices.Add(0x10DE01AA,@"nForce 220/230");
                //mDevices.Add(0x10DE01AB,@"nForce 415/420/430");
                //mDevices.Add(0x10DE01AC,@"nForce 2x0/415/4x0");
                //mDevices.Add(0x10DE01AD,@"nForce 2x0/415/4x0");
                //mDevices.Add(0x10DE01B0,@"nForce MCP");
                //mDevices.Add(0x10DE01B1,@"nForce MCP");
                //mDevices.Add(0x10DE01B2,@"nForce");
                //mDevices.Add(0x10DE01B4,@"nForce MCP");
                //mDevices.Add(0x10DE01B7,@"nForce");
                //mDevices.Add(0x10DE01B8,@"nForce");
                //mDevices.Add(0x10DE01BC,@"nForce MCP");
                //mDevices.Add(0x10DE01C1,@"nForce MCP");
                //mDevices.Add(0x10DE01C2,@"nForce MCP");
                //mDevices.Add(0x10DE01C3,@"nForce MCP");
                //mDevices.Add(0x10DE01D1,@"unknown");
                //mDevices.Add(0x10DE01D7,@"unknown");
                //mDevices.Add(0x10DE01DC,@"nvidia mobile graphics");
                //mDevices.Add(0x10DE01DD,@"7500LE");
                //mDevices.Add(0x10DE01E0,@"nForce2");
                //mDevices.Add(0x10DE01E1,@"nForce2");
                //mDevices.Add(0x10DE01E8,@"nForce2");
                //mDevices.Add(0x10DE01EA,@"nForce2");
                //mDevices.Add(0x10DE01EB,@"nForce2");
                //mDevices.Add(0x10DE01EC,@"nForce2");
                //mDevices.Add(0x10DE01ED,@"nForce2");
                //mDevices.Add(0x10DE01EE,@"nForce2");
                //mDevices.Add(0x10DE01EF,@"nForce2");
                //mDevices.Add(0x10DE0200,@"NV20");
                //mDevices.Add(0x10DE0201,@"NV2");
                //mDevices.Add(0x10DE0202,@"NV20BR");
                //mDevices.Add(0x10DE0203,@"NV20DCC");
                //mDevices.Add(0x10DE0221,@"unknown");
                //mDevices.Add(0x10DE0241,@"NVS 210S");
                //mDevices.Add(0x10DE0242,@"unknown");
                //mDevices.Add(0x10DE0244,@"Geforce Go 6150");
                //mDevices.Add(0x10DE0250,@"NV25");
                //mDevices.Add(0x10DE0251,@"NV25.2");
                //mDevices.Add(0x10DE0253,@"NVIDIA Corporation");
                //mDevices.Add(0x10DE0258,@"NV25GL.1");
                //mDevices.Add(0x10DE0259,@"NV25GL.2");
                //mDevices.Add(0x10DE025B,@"NV25GL.4");
                //mDevices.Add(0x10DE0264,@"NVIDIA SMB Bus Controller");
                //mDevices.Add(0x10DE0265,@"MCP51");
                //mDevices.Add(0x10DE0266,@"MCP51S");
                //mDevices.Add(0x10DE0267,@"MCP51S");
                //mDevices.Add(0x10DE0268,@"430");
                //mDevices.Add(0x10DE026C,@"6150");
                //mDevices.Add(0x10DE0271,@"unknown");
                //mDevices.Add(0x10DE0280,@"NV28.1");
                //mDevices.Add(0x10DE0281,@"NV28.2");
                //mDevices.Add(0x10DE0282,@"NV28.3");
                //mDevices.Add(0x10DE0286,@"???");
                //mDevices.Add(0x10DE0288,@"NV28GL.1");
                //mDevices.Add(0x10DE0289,@"NV28GL.2");
                //mDevices.Add(0x10DE028C,@"NV28GL");
                //mDevices.Add(0x10DE0291,@"0x0401");
                //mDevices.Add(0x10DE029C,@"G71");
                //mDevices.Add(0x10DE029D,@"G71");
                //mDevices.Add(0x10DE02A0,@"NV2A");
                //mDevices.Add(0x10DE02E2,@"Video Card GPU BFG 7300 512MB GT");
                //mDevices.Add(0x10DE0300,@"NV30.1");
                //mDevices.Add(0x10DE0301,@"NV30.2");
                //mDevices.Add(0x10DE0302,@"NV30.3");
                //mDevices.Add(0x10DE0308,@"NV30GL.1");
                //mDevices.Add(0x10DE0309,@"NV30GL.2");
                //mDevices.Add(0x10DE030A,@"NV30GL");
                //mDevices.Add(0x10DE0311,@"NV31.1");
                //mDevices.Add(0x10DE0312,@"NV31.2");
                //mDevices.Add(0x10DE0313,@"NV31?");
                //mDevices.Add(0x10DE0314,@"NV31");
                //mDevices.Add(0x10DE0318,@"NV31GL.1");
                //mDevices.Add(0x10DE0319,@"NV31GL.2");
                //mDevices.Add(0x10DE031A,@"NV31GL");
                //mDevices.Add(0x10DE031B,@"NV31B");
                //mDevices.Add(0x10DE031C,@"NV31C");
                //mDevices.Add(0x10DE0321,@"NV34.2");
                //mDevices.Add(0x10DE0322,@"NV34.3");
                //mDevices.Add(0x10DE0323,@"NV34.4");
                //mDevices.Add(0x10DE0324,@"NV");
                //mDevices.Add(0x10DE0325,@"nv36m");
                //mDevices.Add(0x10DE0326,@"NV34.6");
                //mDevices.Add(0x10DE0327,@"NV34.7");
                //mDevices.Add(0x10DE0328,@"NV34GL");
                //mDevices.Add(0x10DE0329,@"G73");
                //mDevices.Add(0x10DE032A,@"NV34GL.3");
                //mDevices.Add(0x10DE032B,@"NV34GL.4");
                //mDevices.Add(0x10DE032C,@"NV34GL");
                //mDevices.Add(0x10DE032D,@"NV34M");
                //mDevices.Add(0x10DE032F,@"NV34GL");
                //mDevices.Add(0x10DE0330,@"???");
                //mDevices.Add(0x10DE0331,@"???");
                //mDevices.Add(0x10DE0332,@"???");
                //mDevices.Add(0x10DE0333,@"NV38");
                //mDevices.Add(0x10DE0334,@"NV35.4");
                //mDevices.Add(0x10DE0338,@"NV35GL");
                //mDevices.Add(0x10DE0341,@"NV36.1");
                //mDevices.Add(0x10DE0342,@"NV36.2");
                //mDevices.Add(0x10DE0343,@"NV36.3");
                //mDevices.Add(0x10DE0344,@"NV36.4");
                //mDevices.Add(0x10DE0345,@"NV36.5");
                //mDevices.Add(0x10DE0348,@"NV36M");
                //mDevices.Add(0x10DE034E,@"nv36GL");
                //mDevices.Add(0x10DE034F,@"NV36GL?");
                //mDevices.Add(0x10DE036C,@"81FB1043");
                //mDevices.Add(0x10DE036E,@"MCP55");
                //mDevices.Add(0x10DE036F,@"MCP55");
                //mDevices.Add(0x10DE0371,@"ADI 1988");
                //mDevices.Add(0x10DE037E,@"MCP55");
                //mDevices.Add(0x10DE037F,@"MCP55S");
                //mDevices.Add(0x10DE0391,@"G73 B1");
                //mDevices.Add(0x10DE0393,@"UNKOWN");
                //mDevices.Add(0x10DE0398,@"G73");
                //mDevices.Add(0x10DE03D0,@"unknown");
                //mDevices.Add(0x10DE03E0,@"nForce 430");
                //mDevices.Add(0x10DE03E7,@"MCP61");
                //mDevices.Add(0x10DE03EC,@"MCP61");
                //mDevices.Add(0x10DE03EF,@"nForce 405");
                //mDevices.Add(0x10DE03F0,@"MCP61");
                //mDevices.Add(0x10DE03F6,@"MCP61");
                //mDevices.Add(0x10DE03F7,@"MCP61");
                //mDevices.Add(0x10DE0402,@"0402");
                //mDevices.Add(0x10DE0407,@"NVIDIA GeForce 8600M GT");
                //mDevices.Add(0x10DE040C,@"NVIDIA Quadro NVS 570M");
                //mDevices.Add(0x10DE0421,@"G86");
                //mDevices.Add(0x10DE0427,@"unknown");
                //mDevices.Add(0x10DE0428,@"NVIDIA� GeForce 8400M G");
                //mDevices.Add(0x10DE0429,@"Unknown");
                //mDevices.Add(0x10DE0448,@"MCP65");
                //mDevices.Add(0x10DE044C,@"MCP65");
                //mDevices.Add(0x10DE044D,@"MCP65");
                //mDevices.Add(0x10DE044E,@"MCP67D");
                //mDevices.Add(0x10DE044F,@"MCP65");
                //mDevices.Add(0x10DE045D,@"MCP65");
                //mDevices.Add(0x10DE0550,@"MCP67");
                //mDevices.Add(0x10DE0554,@"MCP67");
                //mDevices.Add(0x10DE0555,@"MCP67");
                //mDevices.Add(0x10DE0556,@"MCP67");
                //mDevices.Add(0x10DE0558,@"MCP67");
                //mDevices.Add(0x10DE0559,@"MCP67");
                //mDevices.Add(0x10DE055A,@"MCP67");
                //mDevices.Add(0x10DE0560,@"MCP67");
                //mDevices.Add(0x10DE056C,@"MCP73");
                //mDevices.Add(0x10DE0768,@"MCP ?");
                //mDevices.Add(0x10DE07B5,@"MCP72");
                //mDevices.Add(0x10DE07B9,@"MCP72");
                //mDevices.Add(0x10DE07D8,@"nForce 7100-630i (MCP73PV)");
                //mDevices.Add(0x10DE07DC,@"nForce 7100-630i (MCP73PV)");
                //mDevices.Add(0x10DE07F0,@"MCP73");
                //mDevices.Add(0x10DE07F4,@"MCP73");
                //mDevices.Add(0x10DE07F5,@"MCP73");
                //mDevices.Add(0x10DE07F6,@"MCP73");
                //mDevices.Add(0x10DE07F7,@"MCP73");
                //mDevices.Add(0x10DE07F8,@"MCP73");
                //mDevices.Add(0x10DE07F9,@"MCP73");
                //mDevices.Add(0x10DE07FA,@"MCP73");
                //mDevices.Add(0x10DE07FB,@"MCP73");
                //mDevices.Add(0x10DE0AB8,@"MCP79");
                //mDevices.Add(0x10DE0AB9,@"MCP79");
                //mDevices.Add(0x10DE0ABC,@"MCP79");
                //mDevices.Add(0x10DE0ABD,@"MCP79");
                //mDevices.Add(0x10DE10DE,@"NV3");
                //mDevices.Add(0x10DF10DF,@"Light Pulse");
                //mDevices.Add(0x10DF1AE5,@"LP6000");
                //mDevices.Add(0x10DFF0A5,@"PL1050Ex");
                //mDevices.Add(0x10DFF0E5,@"LPe1150");
                //mDevices.Add(0x10DFF700,@"LP7000");
                //mDevices.Add(0x10DFF800,@"LP8000");
                //mDevices.Add(0x10DFF900,@"????");
                //mDevices.Add(0x10DFF980,@"LP9802 / DC");
                //mDevices.Add(0x10DFFA00,@"LP10000");
                //mDevices.Add(0x10E10391,@"TRM-S1040");
                //mDevices.Add(0x10E1690C,@"DC-690c");
                //mDevices.Add(0x10E1DC20,@"DC-290");
                //mDevices.Add(0x10E30000,@"CA91C042/142");
                //mDevices.Add(0x10E30148,@"Tundra Tsi148");
                //mDevices.Add(0x10E30513,@"Tsi320");
                //mDevices.Add(0x10E30850,@"Tsi850");
                //mDevices.Add(0x10E30854,@"Tsi850");
                //mDevices.Add(0x10E30860,@"CA91C860");
                //mDevices.Add(0x10E30862,@"CA91L862A");
                //mDevices.Add(0x10E38260,@"CA91L8200/8260");
                //mDevices.Add(0x10E38261,@"CA91L8200/8260");
                //mDevices.Add(0x10E82011,@"Q-Motion pci 264");
                //mDevices.Add(0x10E84750,@"S5920Q 0048 A023");
                //mDevices.Add(0x10E85920,@"S5920q");
                //mDevices.Add(0x10E88033,@"BBK-PCI light");
                //mDevices.Add(0x10E88043,@"LANai4.x");
                //mDevices.Add(0x10E88062,@"S5933");
                //mDevices.Add(0x10E8807D,@"S5933");
                //mDevices.Add(0x10E88088,@"FS");
                //mDevices.Add(0x10E88089,@"SOB");
                //mDevices.Add(0x10E8809C,@"S5933");
                //mDevices.Add(0x10E880D7,@"PCI-9112");
                //mDevices.Add(0x10E880D8,@"PCI-7200");
                //mDevices.Add(0x10E880D9,@"PCI-9118");
                //mDevices.Add(0x10E8811A,@"PCI-DSlink");
                //mDevices.Add(0x10E88170,@"S5933");
                //mDevices.Add(0x10EA1680,@"IGA-1680");
                //mDevices.Add(0x10EA1682,@"IGA-1682");
                //mDevices.Add(0x10EA1683,@"IGA-1683");
                //mDevices.Add(0x10EA2000,@"CyberPro 2010");
                //mDevices.Add(0x10EA2010,@"CyberPro 20xx/2000A");
                //mDevices.Add(0x10EA5000,@"CyberPro 5000");
                //mDevices.Add(0x10EA5050,@"CyberPro 5050");
                //mDevices.Add(0x10EB0101,@"3GA");
                //mDevices.Add(0x10EB8111,@"Twist3");
                //mDevices.Add(0x10EC0139,@"rtl8139B");
                //mDevices.Add(0x10EC0260,@"RTL260");
                //mDevices.Add(0x10EC0262,@"ALC882");
                //mDevices.Add(0x10EC0268,@"Realtek ALC268");
                //mDevices.Add(0x10EC0880,@"al880");
                //mDevices.Add(0x10EC0883,@"alc883");
                //mDevices.Add(0x10EC0885,@"ALC885");
                //mDevices.Add(0x10EC0888,@"realtek high definition audio");
                //mDevices.Add(0x10EC8021,@"RTL8029AS");
                //mDevices.Add(0x10EC8029,@"RTL8029(as)pci ethernet nic");
                //mDevices.Add(0x10EC8119,@"10");
                //mDevices.Add(0x10EC8129,@"RTL8129");
                //mDevices.Add(0x10EC8131,@"LFE8139ATX");
                //mDevices.Add(0x10EC8138,@"RT8139B/C");
                //mDevices.Add(0x10EC8139,@"RTL-8139/8139C/8139C+");
                //mDevices.Add(0x10EC8167,@"-8168");
                //mDevices.Add(0x10EC8168,@"RTL8111B");
                //mDevices.Add(0x10EC8169,@"RTL8119");
                //mDevices.Add(0x10EC8180,@"RTL8180");
                //mDevices.Add(0x10EC8185,@"RTL-8185");
                //mDevices.Add(0x10EC8186,@"Gigabit");
                //mDevices.Add(0x10ED7310,@"V7310");
                //mDevices.Add(0x10EE0314,@"Spartan XC2S50E");
                //mDevices.Add(0x10EE3FC0,@"RME Digi96");
                //mDevices.Add(0x10EE3FC1,@"RME Digi96/8");
                //mDevices.Add(0x10EE3FC2,@"RME Digi 96/8 Pro");
                //mDevices.Add(0x10EE3FC3,@"RME Digi96/8 Pad");
                //mDevices.Add(0x10EE3FC4,@"Digi9652");
                //mDevices.Add(0x10EE3FC5,@"HDSP 9632");
                //mDevices.Add(0x10EE5343,@"Seamont SC100");
                //mDevices.Add(0x10EE8130,@"Durango PMC");
                //mDevices.Add(0x10EE8381,@"Santos");
                //mDevices.Add(0x10EF8154,@"M815x");
                //mDevices.Add(0x10F0A800,@"VCL-P");
                //mDevices.Add(0x10F0B300,@"VCL-M");
                //mDevices.Add(0x10F11566,@"IDE/SCSI");
                //mDevices.Add(0x10F11677,@"Multimedia");
                //mDevices.Add(0x10F12013,@"RS-56 sp-pci");
                //mDevices.Add(0x10F41300,@"rev1.1");
                //mDevices.Add(0x10F5A001,@"NDR4000");
                //mDevices.Add(0x10F60111,@"CMI8738/C3DX");
                //mDevices.Add(0x10FA0000,@"BT848KPF");
                //mDevices.Add(0x10FA0001,@"GUI Accelerator");
                //mDevices.Add(0x10FA0002,@"GUI Accelerator");
                //mDevices.Add(0x10FA0003,@"GUI Accelerator");
                //mDevices.Add(0x10FA0004,@"GUI Accelerator");
                //mDevices.Add(0x10FA0005,@"GUI Accelerator");
                //mDevices.Add(0x10FA0006,@"GUI Accelerator");
                //mDevices.Add(0x10FA0007,@"GUI Accelerator");
                //mDevices.Add(0x10FA0008,@"GUI Accelerator");
                //mDevices.Add(0x10FA0009,@"GUI Accelerator");
                //mDevices.Add(0x10FA000A,@"GUI Accelerator");
                //mDevices.Add(0x10FA000B,@"GUI Accelerator");
                //mDevices.Add(0x10FA000C,@"Targa 1000");
                //mDevices.Add(0x10FA000D,@"GUI Accelerator");
                //mDevices.Add(0x10FA000E,@"GUI Accelerator");
                //mDevices.Add(0x10FA000F,@"GUI Accelerator");
                //mDevices.Add(0x10FA0010,@"GUI Accelerator");
                //mDevices.Add(0x10FA0011,@"GUI Accelerator");
                //mDevices.Add(0x10FA0012,@"GUI Accelerator");
                //mDevices.Add(0x10FA0013,@"GUI Accelerator");
                //mDevices.Add(0x10FA0014,@"GUI Accelerator");
                //mDevices.Add(0x10FA0015,@"GUI Accelerator");
                //mDevices.Add(0x10FC8139,@"4020011B");
                //mDevices.Add(0x11003044,@"VT6306L");
                //mDevices.Add(0x11010002,@"INI-920");
                //mDevices.Add(0x11011060,@"INI-A100U2W");
                //mDevices.Add(0x1101134A,@"Ultra SCSI Adapter");
                //mDevices.Add(0x11019100,@"INI-9010/9010W");
                //mDevices.Add(0x11019400,@"INIC-940");
                //mDevices.Add(0x11019401,@"INIC-935");
                //mDevices.Add(0x11019500,@"1101");
                //mDevices.Add(0x11019502,@"INIC-950P");
                //mDevices.Add(0x11019700,@"Fast Wide SCSI");
                //mDevices.Add(0x11020002,@"t4780010004541");
                //mDevices.Add(0x11020003,@"EMU8008");
                //mDevices.Add(0x11020004,@"Creative SB Audigy 2 ZS (WDM)");
                //mDevices.Add(0x11020005,@"SB0460");
                //mDevices.Add(0x11020006,@"emu10k1x");
                //mDevices.Add(0x11020007,@"C6SB0410515017656A");
                //mDevices.Add(0x11020008,@"ca0108");
                //mDevices.Add(0x11021017,@"CT6760");
                //mDevices.Add(0x11021020,@"NV5");
                //mDevices.Add(0x11021047,@"EV1938");
                //mDevices.Add(0x11021371,@"ES1371-3");
                //mDevices.Add(0x11022898,@"es2898");
                //mDevices.Add(0x11024001,@"EMU10K1");
                //mDevices.Add(0x11027002,@"EMU10000");
                //mDevices.Add(0x11027003,@"EMU10K2");
                //mDevices.Add(0x11027004,@"Game port for SB Live! Series");
                //mDevices.Add(0x11027005,@"???");
                //mDevices.Add(0x11027802,@"EMU1OK1-NGF");
                //mDevices.Add(0x11028938,@"EV1938");
                //mDevices.Add(0x11030004,@"HPT366/368");
                //mDevices.Add(0x11030005,@"HPT370/372");
                //mDevices.Add(0x11030006,@"HPT302");
                //mDevices.Add(0x11030007,@"HPT371");
                //mDevices.Add(0x11030008,@"HPT-374");
                //mDevices.Add(0x11031720,@"RR172x");
                //mDevices.Add(0x11031740,@"RR174x");
                //mDevices.Add(0x11031742,@"RR174x");
                //mDevices.Add(0x11032210,@"RR2210");
                //mDevices.Add(0x11032300,@"RR2300");
                //mDevices.Add(0x11032310,@"RR231x");
                //mDevices.Add(0x11032340,@"RR2340");
                //mDevices.Add(0x11032522,@"RR252x");
                //mDevices.Add(0x11033120,@"RR312x");
                //mDevices.Add(0x11033220,@"RR322x");
                //mDevices.Add(0x11033320,@"RR332x");
                //mDevices.Add(0x11033410,@"RR341x");
                //mDevices.Add(0x11033510,@"RR35xx");
                //mDevices.Add(0x11033511,@"RR35xx");
                //mDevices.Add(0x11033520,@"RR35xx");
                //mDevices.Add(0x11033521,@"RR35xx");
                //mDevices.Add(0x11033522,@"RR35xx");
                //mDevices.Add(0x11033540,@"RR35xx");
                //mDevices.Add(0x11034320,@"RR432x");
                //mDevices.Add(0x11035081,@"RR18xx");
                //mDevices.Add(0x11036081,@"RR222x/224x");
                //mDevices.Add(0x11037042,@"RR231x");
                //mDevices.Add(0x11055000,@"Multimedia");
                //mDevices.Add(0x11058300,@"EM8400");
                //mDevices.Add(0x11058400,@"EM8400");
                //mDevices.Add(0x11058475,@"EM8475");
                //mDevices.Add(0x11060130,@"VT6305");
                //mDevices.Add(0x11060198,@"CPU to PCI Bridge");
                //mDevices.Add(0x11060238,@"K8T890");
                //mDevices.Add(0x11060259,@"unknown");
                //mDevices.Add(0x11060269,@"KT880");
                //mDevices.Add(0x11060282,@"K8T880Pro");
                //mDevices.Add(0x11060305,@"VT8363A/8365");
                //mDevices.Add(0x11060391,@"VT8363/71");
                //mDevices.Add(0x11060501,@"VT8501");
                //mDevices.Add(0x11060505,@"82C505");
                //mDevices.Add(0x11060561,@"82C570 MV");
                //mDevices.Add(0x11060571,@"VT82C586A/B/VT82C686/A/B/VT823x/A/C");
                //mDevices.Add(0x11060576,@"82C576");
                //mDevices.Add(0x11060581,@"CX700");
                //mDevices.Add(0x11060585,@"VT82C585VP/VPX");
                //mDevices.Add(0x11060586,@"VT82C586VP");
                //mDevices.Add(0x11060591,@"VT8237S");
                //mDevices.Add(0x11060595,@"VT82C595");
                //mDevices.Add(0x11060596,@"VT82C596");
                //mDevices.Add(0x11060597,@"VT82C597");
                //mDevices.Add(0x11060598,@"VT82C598");
                //mDevices.Add(0x11060601,@"VIA8601");
                //mDevices.Add(0x11060605,@"VT82c686");
                //mDevices.Add(0x11060680,@"VT82C680");
                //mDevices.Add(0x11060686,@"VT82C686/686A/686B");
                //mDevices.Add(0x11060691,@"VIA VT KN133");
                //mDevices.Add(0x11060692,@"North Bridge");
                //mDevices.Add(0x11060693,@"VT82C693");
                //mDevices.Add(0x11060926,@"VT86C926");
                //mDevices.Add(0x11061000,@"82C570MV");
                //mDevices.Add(0x11061106,@"060000");
                //mDevices.Add(0x11061204,@"???");
                //mDevices.Add(0x11061238,@"K8T890");
                //mDevices.Add(0x11061259,@"unknown");
                //mDevices.Add(0x11061269,@"KT880");
                //mDevices.Add(0x11061282,@"K8T880Pro");
                //mDevices.Add(0x11061571,@"VT82C416");
                //mDevices.Add(0x11061595,@"VT82C595/97");
                //mDevices.Add(0x11061708,@"0x1708");
                //mDevices.Add(0x11062006,@"VT6105M");
                //mDevices.Add(0x11062204,@"???");
                //mDevices.Add(0x11062238,@"K8T890");
                //mDevices.Add(0x11062259,@"unknown");
                //mDevices.Add(0x11062269,@"KT880");
                //mDevices.Add(0x11062282,@"K8T880Pro");
                //mDevices.Add(0x11063038,@"VT83C572/6202/82C586A/B/82C596/A/B/82C686A/B etc");
                //mDevices.Add(0x11063040,@"VT82C586A/B");
                //mDevices.Add(0x11063041,@"82C570MV");
                //mDevices.Add(0x11063043,@"VT86C100A");
                //mDevices.Add(0x11063044,@"VT8375");
                //mDevices.Add(0x11063050,@"VT82C596/596A/596");
                //mDevices.Add(0x11063051,@"Power Management Controller");
                //mDevices.Add(0x11063053,@"VT6105M");
                //mDevices.Add(0x11063057,@"VT82C686A/B");
                //mDevices.Add(0x11063058,@"VT8237");
                //mDevices.Add(0x11063059,@"VT8237");
                //mDevices.Add(0x11063065,@"VT6102");
                //mDevices.Add(0x11063068,@"VT82C686A/B&VT8231");
                //mDevices.Add(0x11063074,@"VT8233");
                //mDevices.Add(0x11063086,@"VT82C686");
                //mDevices.Add(0x11063091,@"VT8633");
                //mDevices.Add(0x11063099,@"VT8366/66A/67");
                //mDevices.Add(0x11063101,@"VT8653");
                //mDevices.Add(0x11063102,@"VT8362");
                //mDevices.Add(0x11063103,@"VT8615");
                //mDevices.Add(0x11063104,@"VT6202");
                //mDevices.Add(0x11063106,@"VT6105M/LOM");
                //mDevices.Add(0x11063107,@"VT8233/A AC97' Enhance Audio Controller");
                //mDevices.Add(0x11063108,@"Unknown");
                //mDevices.Add(0x11063109,@"VT8233/A AC97' Enhance Audio Controller");
                //mDevices.Add(0x11063112,@"VT8361");
                //mDevices.Add(0x11063113,@"PCI to PCI Bridge");
                //mDevices.Add(0x11063116,@"VT8375");
                //mDevices.Add(0x11063118,@"CN400");
                //mDevices.Add(0x11063119,@"VT6120/VT6121/VT6122");
                //mDevices.Add(0x11063122,@"VT8623");
                //mDevices.Add(0x11063123,@"VT8623");
                //mDevices.Add(0x11063128,@"vt8753");
                //mDevices.Add(0x11063133,@"VT3133");
                //mDevices.Add(0x11063147,@"VT8233");
                //mDevices.Add(0x11063148,@"VT8751");
                //mDevices.Add(0x11063149,@"VT8237 Family/ VT6420");
                //mDevices.Add(0x11063156,@"VT8372");
                //mDevices.Add(0x11063158,@"CPU-to-PCI Bridge");
                //mDevices.Add(0x11063164,@"VT6410");
                //mDevices.Add(0x11063168,@"VT8374");
                //mDevices.Add(0x11063177,@"VT8235");
                //mDevices.Add(0x11063178,@"CPU to PCI Bridge");
                //mDevices.Add(0x11063188,@"K8HTB-8237");
                //mDevices.Add(0x11063189,@"VT8377");
                //mDevices.Add(0x11063198,@"VEN_1106&DEV_B198&SUBSYS_00000000&REV_00");
                //mDevices.Add(0x11063202,@"CPU to PCI Bridge");
                //mDevices.Add(0x11063204,@"1394 i2c");
                //mDevices.Add(0x11063205,@"CPU to PCI Bridge");
                //mDevices.Add(0x11063208,@"PT890");
                //mDevices.Add(0x11063209,@"CPU to PCI Bridge");
                //mDevices.Add(0x11063213,@"PCI to PCI Bridge");
                //mDevices.Add(0x11063227,@"VT8237");
                //mDevices.Add(0x11063230,@"K8N890CE Display Driver");
                //mDevices.Add(0x11063238,@"K8T890");
                //mDevices.Add(0x11063249,@"VT6421");
                //mDevices.Add(0x11063258,@"PT880");
                //mDevices.Add(0x11063259,@"???");
                //mDevices.Add(0x11063269,@"KT880");
                //mDevices.Add(0x11063282,@"K8T880Pro");
                //mDevices.Add(0x11063288,@"040300");
                //mDevices.Add(0x11063343,@"81CE1043");
                //mDevices.Add(0x11063344,@"P4M800PRO&8237R");
                //mDevices.Add(0x11063349,@"VT8251");
                //mDevices.Add(0x11063371,@"S3G700");
                //mDevices.Add(0x11064149,@"VT6420");
                //mDevices.Add(0x11064204,@"???");
                //mDevices.Add(0x11064238,@"???");
                //mDevices.Add(0x11064258,@"???");
                //mDevices.Add(0x11064259,@"???");
                //mDevices.Add(0x11064269,@"KT880");
                //mDevices.Add(0x11064282,@"K8T880Pro");
                //mDevices.Add(0x11065030,@"VT82C596");
                //mDevices.Add(0x11066100,@"VIA VT86C100A");
                //mDevices.Add(0x11067204,@"K8M400");
                //mDevices.Add(0x11067205,@"KM400");
                //mDevices.Add(0x11067238,@"K8T890");
                //mDevices.Add(0x11067258,@"PT880");
                //mDevices.Add(0x11067259,@"PM880");
                //mDevices.Add(0x11067269,@"KT880");
                //mDevices.Add(0x11067282,@"K8T880Pro");
                //mDevices.Add(0x11067353,@"CX700");
                //mDevices.Add(0x11067372,@"VT8237S");
                //mDevices.Add(0x11068208,@"PT890?");
                //mDevices.Add(0x11068231,@"VT8231");
                //mDevices.Add(0x11068235,@"VT8751");
                //mDevices.Add(0x11068237,@"VT8237");
                //mDevices.Add(0x11068305,@"VT8363A/65");
                //mDevices.Add(0x11068391,@"VT8363/71");
                //mDevices.Add(0x11068501,@"VT8501");
                //mDevices.Add(0x11068596,@"VT82C596");
                //mDevices.Add(0x11068597,@"VT82C597");
                //mDevices.Add(0x11068598,@"VT82C598");
                //mDevices.Add(0x11068601,@"VT82C601");
                //mDevices.Add(0x11068602,@"CPU to AGP Bridge");
                //mDevices.Add(0x11068605,@"VT8605");
                //mDevices.Add(0x11068691,@"VT82C691/693A/694X");
                //mDevices.Add(0x11068693,@"VT82C693");
                //mDevices.Add(0x11069238,@"K8T890");
                //mDevices.Add(0x11069398,@"VT8601");
                //mDevices.Add(0x11069876,@"VT8233/A AC97' Enhance Audio Controller");
                //mDevices.Add(0x1106A208,@"PT890");
                //mDevices.Add(0x1106A238,@"K8T890");
                //mDevices.Add(0x1106B091,@"VT8633");
                //mDevices.Add(0x1106B099,@"VT8366/A");
                //mDevices.Add(0x1106B101,@"VT8653");
                //mDevices.Add(0x1106B102,@"VT8362");
                //mDevices.Add(0x1106B103,@"VT8615");
                //mDevices.Add(0x1106B112,@"VT8361");
                //mDevices.Add(0x1106B113,@"I/O APIC");
                //mDevices.Add(0x1106B115,@"VT8363/65");
                //mDevices.Add(0x1106B116,@"VT8375");
                //mDevices.Add(0x1106B133,@"vt686b");
                //mDevices.Add(0x1106B148,@"VT8751 Apollo");
                //mDevices.Add(0x1106B156,@"VT8372");
                //mDevices.Add(0x1106B158,@"PCI-to-PCI Bridge (AGP)");
                //mDevices.Add(0x1106B168,@"VT8235");
                //mDevices.Add(0x1106B188,@"PCI-to-PCI Bridge (AGP 2.0/3.0)");
                //mDevices.Add(0x1106B198,@"546546");
                //mDevices.Add(0x1106B213,@"I/O APIC");
                //mDevices.Add(0x1106C208,@"PT890");
                //mDevices.Add(0x1106C238,@"K8T890");
                //mDevices.Add(0x1106D208,@"PT890");
                //mDevices.Add(0x1106D213,@"PCI to PCI Bridge");
                //mDevices.Add(0x1106D238,@"K8T890");
                //mDevices.Add(0x1106E208,@"PT890");
                //mDevices.Add(0x1106E238,@"K8T890");
                //mDevices.Add(0x1106F208,@"PT890");
                //mDevices.Add(0x1106F238,@"K8T890");
                //mDevices.Add(0x11078576,@"PCI Host Bridge");
                //mDevices.Add(0x11080100,@"p1690plus-AA");
                //mDevices.Add(0x11080101,@"p1690plus-AB");
                //mDevices.Add(0x11080105,@"P1690Plus");
                //mDevices.Add(0x11080108,@"P1690Plus");
                //mDevices.Add(0x11080138,@"P1690Plus");
                //mDevices.Add(0x11080139,@"P1690Plus");
                //mDevices.Add(0x1108013C,@"P1690Plus");
                //mDevices.Add(0x1108013D,@"P1690Plus");
                //mDevices.Add(0x11091400,@"EM110TX");
                //mDevices.Add(0x110A2101,@"PEB 20321");
                //mDevices.Add(0x110A2102,@"PEB 20534");
                //mDevices.Add(0x110A3141,@"01");
                //mDevices.Add(0x110A4033,@"ERTEC400");
                //mDevices.Add(0x110B0001,@"Mpact");
                //mDevices.Add(0x110B0002,@"GM90C7110VX");
                //mDevices.Add(0x11122200,@"2200");
                //mDevices.Add(0x11122300,@"2300");
                //mDevices.Add(0x11122400,@"2400");
                //mDevices.Add(0x11131211,@"mpx en5038a1");
                //mDevices.Add(0x11131216,@"EN5251BE");
                //mDevices.Add(0x11131217,@"EN-2242");
                //mDevices.Add(0x11135105,@"EN-1660");
                //mDevices.Add(0x11139211,@"EN-1207D");
                //mDevices.Add(0x11139511,@"SMC en5251be");
                //mDevices.Add(0x11139876,@"EN5251BE");
                //mDevices.Add(0x11140506,@"AT76C506");
                //mDevices.Add(0x11143202,@"AT97SC3202");
                //mDevices.Add(0x11160022,@"DT3001");
                //mDevices.Add(0x11160023,@"DT3002");
                //mDevices.Add(0x11160024,@"DT3003");
                //mDevices.Add(0x11160025,@"DT3004");
                //mDevices.Add(0x11160026,@"Dt3005");
                //mDevices.Add(0x11160027,@"DT3001-PGL");
                //mDevices.Add(0x11160028,@"DT3003-PGL");
                //mDevices.Add(0x11179500,@"max-lc SVGA card");
                //mDevices.Add(0x11179501,@"MaxPCI image processing board");
                //mDevices.Add(0x11190000,@"GDT6000/6020/6050");
                //mDevices.Add(0x11190001,@"GDT6000/6010");
                //mDevices.Add(0x11190002,@"GDT6110/6510");
                //mDevices.Add(0x11190003,@"GDT6120/6520");
                //mDevices.Add(0x11190004,@"GDT6530");
                //mDevices.Add(0x11190005,@"GDT6550");
                //mDevices.Add(0x11190006,@"GDT6117/6517");
                //mDevices.Add(0x11190007,@"GDT6127/6527");
                //mDevices.Add(0x11190008,@"GDT6537");
                //mDevices.Add(0x11190009,@"GDT6557/6557-ECC");
                //mDevices.Add(0x1119000A,@"GDT6115/6515");
                //mDevices.Add(0x1119000B,@"GDT6125/6525");
                //mDevices.Add(0x1119000C,@"GDT6535");
                //mDevices.Add(0x1119000D,@"GDT6555/6555-ECC");
                //mDevices.Add(0x11190100,@"GDT6117RP/6517RP");
                //mDevices.Add(0x11190101,@"GDT6127RP/6527RP");
                //mDevices.Add(0x11190102,@"GDT6537RP");
                //mDevices.Add(0x11190103,@"GDT6557RP");
                //mDevices.Add(0x11190104,@"GDT6111RP/6511RP");
                //mDevices.Add(0x11190105,@"GDT6121RP/6521RP");
                //mDevices.Add(0x11190110,@"GDT6117RD/6517RD");
                //mDevices.Add(0x11190111,@"GDT6127RD/6527RD");
                //mDevices.Add(0x11190112,@"GDT6537RD");
                //mDevices.Add(0x11190113,@"GDT6557RD");
                //mDevices.Add(0x11190114,@"GDT6111RD/6511RD");
                //mDevices.Add(0x11190115,@"GDT6127RD/6527RD");
                //mDevices.Add(0x11190118,@"GDT6x18RD");
                //mDevices.Add(0x11190119,@"GDT6x28RD");
                //mDevices.Add(0x1119011A,@"GDT6538RD/6638RD");
                //mDevices.Add(0x1119011B,@"GDT6558RD/6658RD");
                //mDevices.Add(0x11190120,@"GDT6117RP2/6517RP2");
                //mDevices.Add(0x11190121,@"GDT6127RP2/6527RP2");
                //mDevices.Add(0x11190122,@"GDT6537RP2");
                //mDevices.Add(0x11190123,@"GDT6557RP2");
                //mDevices.Add(0x11190124,@"GDT6111RP2/6511RP2");
                //mDevices.Add(0x11190125,@"GDT6127RP2/6527RP2");
                //mDevices.Add(0x11190136,@"GDT 6x13RS");
                //mDevices.Add(0x11190137,@"GDT 6x23RS");
                //mDevices.Add(0x11190138,@"GDT 6x18RS");
                //mDevices.Add(0x11190139,@"GDT 6x28RS");
                //mDevices.Add(0x1119013A,@"GDT 6x38RS");
                //mDevices.Add(0x1119013B,@"GDT 6x58RS");
                //mDevices.Add(0x1119013C,@"GDT 6x33RS");
                //mDevices.Add(0x1119013D,@"GDT 6x43RS");
                //mDevices.Add(0x1119013E,@"GDT 6x53RS");
                //mDevices.Add(0x1119013F,@"GDT 6x63RS");
                //mDevices.Add(0x11190166,@"GDT 7x13RN");
                //mDevices.Add(0x11190167,@"GDT 7x23RN");
                //mDevices.Add(0x11190168,@"GDT7x18RN");
                //mDevices.Add(0x11190169,@"GDT7x28RN");
                //mDevices.Add(0x1119016A,@"GDT7538RN/7638RN");
                //mDevices.Add(0x1119016B,@"GDT7558RN/7658RN");
                //mDevices.Add(0x1119016C,@"GDT 7x33RN");
                //mDevices.Add(0x1119016D,@"GDT 7x43RN");
                //mDevices.Add(0x1119016E,@"GDT 7x53RN");
                //mDevices.Add(0x1119016F,@"GDT 7x63RN");
                //mDevices.Add(0x111901D6,@"GDT 4x13RZ");
                //mDevices.Add(0x111901D7,@"GDT 4x23RZ");
                //mDevices.Add(0x111901F6,@"GDT 8x13RZ");
                //mDevices.Add(0x111901F7,@"GDT 8x23RZ");
                //mDevices.Add(0x111901FC,@"GDT 8x33RZ");
                //mDevices.Add(0x111901FD,@"GDT 8x43RZ");
                //mDevices.Add(0x111901FE,@"GDT 8x53RZ");
                //mDevices.Add(0x111901FF,@"GDT 8x63RZ");
                //mDevices.Add(0x11190210,@"GDT6519RD/6619RD");
                //mDevices.Add(0x11190211,@"GDT6529RD/6629RD");
                //mDevices.Add(0x11190260,@"GDT7519RN/7619RN");
                //mDevices.Add(0x11190261,@"GDT7529RN/7629RN");
                //mDevices.Add(0x11190300,@"GDT Rx");
                //mDevices.Add(0x111A0000,@"155P-MF1");
                //mDevices.Add(0x111A0002,@"166P-MF1");
                //mDevices.Add(0x111A0003,@"ENI-25P");
                //mDevices.Add(0x111C0001,@"Powerbus Bridge");
                //mDevices.Add(0x111D0001,@"IDT77201/211");
                //mDevices.Add(0x111D0003,@"IDT77222/252");
                //mDevices.Add(0x111D0004,@"IDT77V252");
                //mDevices.Add(0x111F4A47,@"Precision MX");
                //mDevices.Add(0x111F5243,@"Frame Capture Bus Interface");
                //mDevices.Add(0x11270200,@"FireRunner PCA-200");
                //mDevices.Add(0x11270210,@"PCA-200PC");
                //mDevices.Add(0x11270250,@"ATM");
                //mDevices.Add(0x11270300,@"PCA-200E");
                //mDevices.Add(0x11270310,@"ATM");
                //mDevices.Add(0x11270400,@"ForeRunner HE");
                //mDevices.Add(0x112E0000,@"EIDE/hdd and IDE/cd-rom Ctrlr");
                //mDevices.Add(0x112E000B,@"EIDE/hdd and IDE/cd-rom Ctrlr");
                //mDevices.Add(0x1130F211,@"0x010");
                //mDevices.Add(0x11311131,@"7130");
                //mDevices.Add(0x11311201,@"PTD3000");
                //mDevices.Add(0x11311234,@"EHCI USB 2.0 Controller");
                //mDevices.Add(0x11311301,@"PTD3210");
                //mDevices.Add(0x11311562,@"ISP1561");
                //mDevices.Add(0x11312780,@"TDA2780AQ");
                //mDevices.Add(0x11313400,@"UCB1500");
                //mDevices.Add(0x11313401,@"UCB1500");
                //mDevices.Add(0x11315400,@"TriMedia TM1000/1100");
                //mDevices.Add(0x11315402,@"TriMedia TM-1300EH");
                //mDevices.Add(0x11315406,@"TriMedia PNX1700");
                //mDevices.Add(0x11317130,@"73c0a1434628");
                //mDevices.Add(0x11317133,@"Pinnacle PCTV 110i");
                //mDevices.Add(0x11317134,@"SAA7135");
                //mDevices.Add(0x11317145,@"d145ah");
                //mDevices.Add(0x11317146,@"saa7135HL");
                //mDevices.Add(0x11319730,@"SAA9730");
                //mDevices.Add(0x11337711,@"EiconCard C91");
                //mDevices.Add(0x11337901,@"EiconCard S90");
                //mDevices.Add(0x11337943,@"EiconCard S94");
                //mDevices.Add(0x11337944,@"EiconCard S94");
                //mDevices.Add(0x1133B923,@"EiconCard P92");
                //mDevices.Add(0x1133E001,@"DIVA Pro 2.0 S/T");
                //mDevices.Add(0x1133E002,@"DIVA 2.0 S/T");
                //mDevices.Add(0x1133E003,@"DIVA Pro 2.0 U");
                //mDevices.Add(0x1133E004,@"DIVA 2.0 U");
                //mDevices.Add(0x1133E005,@"DIVA 2.01 S/T");
                //mDevices.Add(0x1133E00B,@"DIVA ISDN 2.02 PCI");
                //mDevices.Add(0x1133E010,@"Maestra");
                //mDevices.Add(0x1133E012,@"MaestraQ");
                //mDevices.Add(0x1133E013,@"MaestraQ-U");
                //mDevices.Add(0x1133E014,@"MaestraP");
                //mDevices.Add(0x1133E015,@"Diva Server PRI-30M PCI v.2");
                //mDevices.Add(0x1133E018,@"DIVA Server BRI-2M/-2F");
                //mDevices.Add(0x11340001,@"Raceway Bridge");
                //mDevices.Add(0x11340002,@"DPRB");
                //mDevices.Add(0x11350001,@"Printer Cntrlr");
                //mDevices.Add(0x11388905,@"8905");
                //mDevices.Add(0x113C0000,@"PCI9060");
                //mDevices.Add(0x113C0001,@"PCI9060");
                //mDevices.Add(0x113C0911,@"PCI911");
                //mDevices.Add(0x113C0912,@"PCI912");
                //mDevices.Add(0x113C0913,@"PCI913");
                //mDevices.Add(0x113C0914,@"PCI914");
                //mDevices.Add(0x113F0808,@"SST-64P");
                //mDevices.Add(0x113F1010,@"SST-128P");
                //mDevices.Add(0x11410001,@"EIDE/ATAPI super adapter");
                //mDevices.Add(0x11423210,@"ProMotion 3210");
                //mDevices.Add(0x11426410,@"6410 6422");
                //mDevices.Add(0x11426412,@"GUI Accelerator");
                //mDevices.Add(0x11426420,@"GUI Accelerator");
                //mDevices.Add(0x11426422,@"ProMotion-6422");
                //mDevices.Add(0x11426424,@"ProVideo 6424");
                //mDevices.Add(0x11426425,@"ProMotion AT25");
                //mDevices.Add(0x11426426,@"GUI Accelerator");
                //mDevices.Add(0x1142643D,@"AT25");
                //mDevices.Add(0x11429876,@"ProMotion AT25");
                //mDevices.Add(0x11440001,@"Noservo Cntrlr");
                //mDevices.Add(0x1145F020,@"CardBus ATAPI Host Adapter");
                //mDevices.Add(0x1145F021,@"NPATA32");
                //mDevices.Add(0x11471123,@"123");
                //mDevices.Add(0x11484000,@"SK-NET");
                //mDevices.Add(0x11484200,@"Token Ring Adapter");
                //mDevices.Add(0x11484300,@"SysKonnect Genesis");
                //mDevices.Add(0x11484320,@"Marvell Yukon");
                //mDevices.Add(0x11489000,@"Marvell Yukon II PCI-X");
                //mDevices.Add(0x11489E00,@"Marvell Yukon EC");
                //mDevices.Add(0x114A5565,@"VMIPCI-5565");
                //mDevices.Add(0x114A5579,@"VMIPCI-5579");
                //mDevices.Add(0x114A5588,@"VMICPCI5588");
                //mDevices.Add(0x114A6504,@"Timer/SRAM FPGA");
                //mDevices.Add(0x114A7587,@"VMIVME-7587");
                //mDevices.Add(0x114D2189,@"0x1002114D");
                //mDevices.Add(0x114F0002,@"AccelePort EPC");
                //mDevices.Add(0x114F0003,@"RightSwitch SE-6");
                //mDevices.Add(0x114F0004,@"AccelePort Xem");
                //mDevices.Add(0x114F0005,@"AccelePort Xr");
                //mDevices.Add(0x114F0006,@"AccelePort C/X");
                //mDevices.Add(0x114F0007,@"DataFire PCI 1 S/T");
                //mDevices.Add(0x114F0009,@"AccelePort Xr/J");
                //mDevices.Add(0x114F000A,@"AccelePort EPC/J");
                //mDevices.Add(0x114F000C,@"DataFirePRIme T1");
                //mDevices.Add(0x114F000D,@"SyncPort");
                //mDevices.Add(0x114F0011,@"AccelePort8r EIA-232");
                //mDevices.Add(0x114F0012,@"AccelePort8r EIA-422");
                //mDevices.Add(0x114F0013,@"AccelePort Xr");
                //mDevices.Add(0x114F0014,@"AccelePort8r EIA-422");
                //mDevices.Add(0x114F0015,@"AccelePort Xem");
                //mDevices.Add(0x114F0016,@"AccelePort EPC/X");
                //mDevices.Add(0x114F0017,@"AccelePort C/X");
                //mDevices.Add(0x114F0019,@"DataFire PCI 1 U");
                //mDevices.Add(0x114F001A,@"DataFirePRIme E1");
                //mDevices.Add(0x114F001B,@"AccelePort C/X (IBM)");
                //mDevices.Add(0x114F001D,@"DataFire RAS");
                //mDevices.Add(0x114F001F,@"ClydeNonCsu6034");
                //mDevices.Add(0x114F0020,@"ClydeNonCsu6032");
                //mDevices.Add(0x114F0021,@"ClydeNonCsu4");
                //mDevices.Add(0x114F0022,@"ClydeNonCsu2");
                //mDevices.Add(0x114F0023,@"AccelePort RAS");
                //mDevices.Add(0x114F0024,@"DataFire RAS B4 ST/U");
                //mDevices.Add(0x114F0026,@"AccelePort 4r 920");
                //mDevices.Add(0x114F0027,@"AccelePort 8r 920");
                //mDevices.Add(0x114F0029,@"DigiClassic PCI");
                //mDevices.Add(0x114F0034,@"AccelePort 2r 920");
                //mDevices.Add(0x114F0035,@"DataFire DSP");
                //mDevices.Add(0x114F0040,@"AccelePort Xp");
                //mDevices.Add(0x114F0042,@"AccelePort 2p PCI");
                //mDevices.Add(0x114F0070,@"DataFire Micro V");
                //mDevices.Add(0x114F0071,@"DataFire Micro V");
                //mDevices.Add(0x114F0072,@"DataFire Micro V");
                //mDevices.Add(0x114F0073,@"DataFire Micro V");
                //mDevices.Add(0x114F6001,@"Avanstar");
                //mDevices.Add(0x11550810,@"486 CPU/PCI Bridge");
                //mDevices.Add(0x11550922,@"2838");
                //mDevices.Add(0x11550926,@"PCI/ISA Bridge");
                //mDevices.Add(0x11583011,@"Tokenet/vg 1001/10m anylan");
                //mDevices.Add(0x11589050,@"Lanfleet/Truevalue");
                //mDevices.Add(0x11589051,@"Lanfleet/Truevalue");
                //mDevices.Add(0x11590001,@"MV1000");
                //mDevices.Add(0x11590002,@"MV-1500");
                //mDevices.Add(0x115D0003,@"RBEM56G-100");
                //mDevices.Add(0x115D0005,@"CardBus Ethernet 10/100");
                //mDevices.Add(0x115D0007,@"CardBus Ethernet 10/100");
                //mDevices.Add(0x115D000B,@"CardBus Ethernet 10/100");
                //mDevices.Add(0x115D000C,@"MPCI 3A56GSP-100 PA");
                //mDevices.Add(0x115D000F,@"CardBus Ethernet 10/100");
                //mDevices.Add(0x115D0076,@"Xircom MPCI3B-56G (Lucent SCORPIO) Soft");
                //mDevices.Add(0x115D00D4,@"MPCI");
                //mDevices.Add(0x115D0101,@"CardBus 56k Modem");
                //mDevices.Add(0x115D0103,@"CardBus Ehternet + 56k Modem");
                //mDevices.Add(0x11610001,@"Host Bridge");
                //mDevices.Add(0x11630001,@"Verite 1000");
                //mDevices.Add(0x11632000,@"Verite 2x00");
                //mDevices.Add(0x11650001,@"Motion JPEG rec/play w/audio");
                //mDevices.Add(0x11660005,@"NB6536 (CNB20-LE)");
                //mDevices.Add(0x11660006,@"NB6536 (CNB20-HE)");
                //mDevices.Add(0x11660007,@"NB6635 (CNB20-LE/HE)");
                //mDevices.Add(0x11660008,@"NB6536 (CNB20-HE)");
                //mDevices.Add(0x11660009,@"NB6536 (CNB20-LE)");
                //mDevices.Add(0x11660010,@"CIOB30");
                //mDevices.Add(0x11660011,@"CMIC-HE");
                //mDevices.Add(0x11660012,@"CMIC-LE");
                //mDevices.Add(0x11660013,@"CNB20-HE");
                //mDevices.Add(0x11660014,@"CNB20-HE");
                //mDevices.Add(0x11660015,@"CMIC-GC");
                //mDevices.Add(0x11660016,@"CMIC-GC");
                //mDevices.Add(0x11660017,@"CMIC-SL");
                //mDevices.Add(0x11660101,@"CIOB-X2");
                //mDevices.Add(0x11660103,@"BCM5715");
                //mDevices.Add(0x11660110,@"CIOB-E");
                //mDevices.Add(0x11660200,@"OSB4");
                //mDevices.Add(0x11660201,@"CSB5");
                //mDevices.Add(0x11660203,@"CSB6");
                //mDevices.Add(0x11660211,@"OSB4");
                //mDevices.Add(0x11660212,@"CSB5");
                //mDevices.Add(0x11660213,@"OSB6");
                //mDevices.Add(0x11660217,@"OSB6");
                //mDevices.Add(0x11660220,@"OSB4");
                //mDevices.Add(0x11660221,@"OSB6");
                //mDevices.Add(0x11660223,@"0x0223");
                //mDevices.Add(0x11660225,@"CSB5");
                //mDevices.Add(0x11660227,@"CSB6");
                //mDevices.Add(0x11660230,@"???");
                //mDevices.Add(0x11660241,@"BC4852");
                //mDevices.Add(0x11690102,@"QL5032");
                //mDevices.Add(0x11690202,@"QL5032");
                //mDevices.Add(0x11690302,@"QL5032");
                //mDevices.Add(0x11690402,@"QL5032");
                //mDevices.Add(0x11690502,@"QL5232");
                //mDevices.Add(0x11690902,@"QL5232");
                //mDevices.Add(0x11692001,@"Ql5032-33APQ208C");
                //mDevices.Add(0x116A6100,@"Bus/Tag Channel");
                //mDevices.Add(0x116A6800,@"Escon Channel");
                //mDevices.Add(0x116A7100,@"Bus/Tag Channel");
                //mDevices.Add(0x116A7800,@"Escon Channel");
                //mDevices.Add(0x116E0500,@"Vx500");
                //mDevices.Add(0x11720001,@"EPF6016ATC144-2");
                //mDevices.Add(0x11720004,@"Multi-serial card");
                //mDevices.Add(0x11768474,@"42000133");
                //mDevices.Add(0x1178AFA1,@"Fast Ethernet");
                //mDevices.Add(0x11790102,@"toshiba america info systems");
                //mDevices.Add(0x11790103,@"Extended PCI IDE Controller Type-B");
                //mDevices.Add(0x11790406,@"Tecra a2");
                //mDevices.Add(0x11790601,@"Toshiba CPU to PCI bridge");
                //mDevices.Add(0x11790602,@"PCI to ISA Bridge for Notebooks");
                //mDevices.Add(0x11790603,@"ToPIC95");
                //mDevices.Add(0x11790604,@"PCI to PCI Bridge for Notebooks");
                //mDevices.Add(0x11790605,@"PCI to ISA Bridge for Notebooks");
                //mDevices.Add(0x11790606,@"PCI to ISA Bridge for Notebooks");
                //mDevices.Add(0x11790609,@"PCI to PCI Bridge for Notebooks");
                //mDevices.Add(0x1179060A,@"ToPIC95B");
                //mDevices.Add(0x1179060F,@"ToPIC97");
                //mDevices.Add(0x11790611,@"PCI to ISA Bridge");
                //mDevices.Add(0x11790617,@"ToPIC100");
                //mDevices.Add(0x11790618,@"CPU to PCI and PCI to ISA Bridge");
                //mDevices.Add(0x11790701,@"vt82c693");
                //mDevices.Add(0x11790804,@"0x0804");
                //mDevices.Add(0x11790805,@"PCI\VEN_1180&DEV_0592&SUBSYS_828D1033&REV_06\4&5A9");
                //mDevices.Add(0x11790D01,@"0x0D01");
                //mDevices.Add(0x117913A8,@"XR17C158/154/152");
                //mDevices.Add(0x117B8320,@"GCE-8320B");
                //mDevices.Add(0x117E0001,@"Printer Host");
                //mDevices.Add(0x11800475,@"RL5c475");
                //mDevices.Add(0x11800476,@"unknown");
                //mDevices.Add(0x11800478,@"RB5c478");
                //mDevices.Add(0x11800552,@"R5C552");
                //mDevices.Add(0x11800592,@"13871043");
                //mDevices.Add(0x11800822,@"R5C832, R5C843");
                //mDevices.Add(0x11800832,@"unknown");
                //mDevices.Add(0x11800843,@"R5C853");
                //mDevices.Add(0x11800852,@"01cf1028");
                //mDevices.Add(0x11805551,@"Unknown");
                //mDevices.Add(0x11858929,@"EIDE Controller");
                //mDevices.Add(0x11860100,@"dl10038d");
                //mDevices.Add(0x11861002,@"DFE-550TX/580TX/DFE-550FX");
                //mDevices.Add(0x11861100,@"Fast Ethernet Adapter");
                //mDevices.Add(0x11861300,@"DL 10038C or 10038D (Remark of Realtek RTL-8139)");
                //mDevices.Add(0x11861301,@"DFE-530TX");
                //mDevices.Add(0x11861340,@"DFE-690TXD");
                //mDevices.Add(0x11861561,@"DRP-32TXD");
                //mDevices.Add(0x11863065,@"14001186");
                //mDevices.Add(0x11863106,@"DFE 530TX+ rev E1");
                //mDevices.Add(0x11863300,@"D-Link Air Wireless Network (DWL-G510)");
                //mDevices.Add(0x11864000,@"DL2000");
                //mDevices.Add(0x11864001,@"DFE-650TX");
                //mDevices.Add(0x11864300,@"dlg10028");
                //mDevices.Add(0x11864B01,@"DGE-530T");
                //mDevices.Add(0x11864C00,@"DGE-530T");
                //mDevices.Add(0x11891592,@"VL/PCI Bridge");
                //mDevices.Add(0x118C0014,@"PCIB");
                //mDevices.Add(0x118C1117,@"MAC-94C201B3");
                //mDevices.Add(0x118D0001,@"n/a");
                //mDevices.Add(0x118D0012,@"Model 12");
                //mDevices.Add(0x118D0014,@"Model 14");
                //mDevices.Add(0x118D0024,@"Model 24");
                //mDevices.Add(0x118D0044,@"Model 44");
                //mDevices.Add(0x118D0112,@"Model 12");
                //mDevices.Add(0x118D0114,@"Model 14");
                //mDevices.Add(0x118D0124,@"Model 24");
                //mDevices.Add(0x118D0144,@"Model 44");
                //mDevices.Add(0x118D0212,@"Model 12");
                //mDevices.Add(0x118D0214,@"Model 14");
                //mDevices.Add(0x118D0224,@"Model 24");
                //mDevices.Add(0x118D0244,@"Model 44");
                //mDevices.Add(0x118D0312,@"Model 12");
                //mDevices.Add(0x118D0314,@"Model 14");
                //mDevices.Add(0x118D0324,@"Model 24");
                //mDevices.Add(0x118D0344,@"Model 44");
                //mDevices.Add(0x11902550,@"TC-2550");
                //mDevices.Add(0x1190C721,@"EIDE");
                //mDevices.Add(0x1190C731,@"TP-910/920/940");
                //mDevices.Add(0x11910001,@"IDE Ctrlr");
                //mDevices.Add(0x11910002,@"ATP850UF");
                //mDevices.Add(0x11910003,@"SCSI-2 cache Cntrlr");
                //mDevices.Add(0x11910004,@"ATP8400");
                //mDevices.Add(0x11910005,@"ATP850UF");
                //mDevices.Add(0x11910006,@"ATP860A");
                //mDevices.Add(0x11910007,@"ATP860R");
                //mDevices.Add(0x11910008,@"ATP865");
                //mDevices.Add(0x11910009,@"ATP865");
                //mDevices.Add(0x1191000B,@"AEC6897");
                //mDevices.Add(0x1191000D,@"ATP8620");
                //mDevices.Add(0x11918001,@"ATP8600");
                //mDevices.Add(0x11918002,@"ATP850S");
                //mDevices.Add(0x11918010,@"ATP870");
                //mDevices.Add(0x11918020,@"ATP870");
                //mDevices.Add(0x11918030,@"ATP870");
                //mDevices.Add(0x11918040,@"ATP870");
                //mDevices.Add(0x11918050,@"Ultra Wide SCSI Controller");
                //mDevices.Add(0x11918060,@"AEC671x");
                //mDevices.Add(0x11918081,@"AEC-67160");
                //mDevices.Add(0x1197010C,@"CompuScope 82G");
                //mDevices.Add(0x11990001,@"IRMA 3270 PCI Adapter");
                //mDevices.Add(0x11990002,@"Advanced ISCA PCI Adapter");
                //mDevices.Add(0x11990201,@"SDLC PCI Adapter");
                //mDevices.Add(0x119B1221,@"82C092G");
                //mDevices.Add(0x119E0001,@"MB86697");
                //mDevices.Add(0x119E0003,@"MB86695");
                //mDevices.Add(0x11A87302,@"NTX-8023-PCI 2MB Long Card");
                //mDevices.Add(0x11A87308,@"NTX-8023-PCI 8MB Long Card");
                //mDevices.Add(0x11A87402,@"NTX-8023-PCI 2MB Short Card");
                //mDevices.Add(0x11A87408,@"NTX-8023-PCI 8MB Short Card");
                //mDevices.Add(0x11A94240,@"AMCC S5933Q");
                //mDevices.Add(0x11AB0146,@"GT-64010/A");
                //mDevices.Add(0x11AB11AB,@"88E8055 PCI-E");
                //mDevices.Add(0x11AB13F8,@"W8300");
                //mDevices.Add(0x11AB1FAA,@"8335");
                //mDevices.Add(0x11AB4320,@"88E8055");
                //mDevices.Add(0x11AB4350,@"88E8036");
                //mDevices.Add(0x11AB4351,@"88E8039");
                //mDevices.Add(0x11AB4353,@"88E8039");
                //mDevices.Add(0x11AB4354,@"88E8040");
                //mDevices.Add(0x11AB4360,@"88E8052");
                //mDevices.Add(0x11AB4361,@"88E8036");
                //mDevices.Add(0x11AB4362,@"88E8053");
                //mDevices.Add(0x11AB4363,@"88E8053");
                //mDevices.Add(0x11AB4364,@"88E8056");
                //mDevices.Add(0x11AB4365,@"88E8070");
                //mDevices.Add(0x11AB4611,@"GT-64115");
                //mDevices.Add(0x11AB4620,@"GT-64120");
                //mDevices.Add(0x11AB4801,@"GT-48001");
                //mDevices.Add(0x11AB4809,@"EV-48300");
                //mDevices.Add(0x11AB5005,@"F5D5005");
                //mDevices.Add(0x11AB5040,@"88SX5040");
                //mDevices.Add(0x11AB5041,@"88SX504");
                //mDevices.Add(0x11AB5080,@"RocketRAID 182x");
                //mDevices.Add(0x11AB5081,@"RocketRAID 182x");
                //mDevices.Add(0x11AB6041,@"MV88SX6041");
                //mDevices.Add(0x11AB6145,@"Add-on IC to provide 4x SATA Ports, attached to ICH7 (SthBridge?) via PCI-Express.");
                //mDevices.Add(0x11AB6320,@"GT-64130/131");
                //mDevices.Add(0x11AB6480,@"MV6446x");
                //mDevices.Add(0x11AB6485,@"MV6446x");
                //mDevices.Add(0x11AB9653,@"GT-96100A");
                //mDevices.Add(0x11ABF003,@"GT-64010");
                //mDevices.Add(0x11ABF004,@"GT64120");
                //mDevices.Add(0x11ABF006,@"GT-64120A");
                //mDevices.Add(0x11AD0001,@"LC82C168?");
                //mDevices.Add(0x11AD0002,@"LC82C169C");
                //mDevices.Add(0x11ADC115,@"LC82C115");
                //mDevices.Add(0x11AE4153,@"Bridge Controller");
                //mDevices.Add(0x11AE5842,@"Bridge Controller");
                //mDevices.Add(0x11AF0001,@"9704");
                //mDevices.Add(0x11AF000A,@"Nitris");
                //mDevices.Add(0x11AF000B,@"Nitris DX / Mojo DX");
                //mDevices.Add(0x11B00001,@"V960PBC/PSC");
                //mDevices.Add(0x11B00002,@"V961PBC/PSC");
                //mDevices.Add(0x11B00004,@"V962PBC/PSC");
                //mDevices.Add(0x11B00010,@"V292PBC/PSC");
                //mDevices.Add(0x11B00021,@"V363EPC");
                //mDevices.Add(0x11B00022,@"V363EPC");
                //mDevices.Add(0x11B00024,@"V363EPC");
                //mDevices.Add(0x11B00030,@"V363EPC");
                //mDevices.Add(0x11B00100,@"V320USC");
                //mDevices.Add(0x11B00101,@"V320USC");
                //mDevices.Add(0x11B00102,@"V320USC");
                //mDevices.Add(0x11B00103,@"V320USC");
                //mDevices.Add(0x11B00200,@"V370PDC");
                //mDevices.Add(0x11B00292,@"V292PBC");
                //mDevices.Add(0x11B00500,@"V340HPC");
                //mDevices.Add(0x11B00960,@"V96xPBC");
                //mDevices.Add(0x11B0C960,@"V96DPC");
                //mDevices.Add(0x11B50001,@"PMC/PMX1553");
                //mDevices.Add(0x11B50002,@"PMCF1");
                //mDevices.Add(0x11B50003,@"PMCMMA");
                //mDevices.Add(0x11B50004,@"PMCVGO");
                //mDevices.Add(0x11B50005,@"PMCPCIS");
                //mDevices.Add(0x11B50006,@"PMCPCIM");
                //mDevices.Add(0x11B50007,@"PMCQ1");
                //mDevices.Add(0x11B50008,@"EPMCQ2");
                //mDevices.Add(0x11B50009,@"PMCPIO1");
                //mDevices.Add(0x11B50012,@"PMCF1");
                //mDevices.Add(0x11B50013,@"PMC1553EX");
                //mDevices.Add(0x11B50014,@"PMC1553E");
                //mDevices.Add(0x11B52200,@"PMCFA2C");
                //mDevices.Add(0x11B80001,@"Quad PeerMaster");
                //mDevices.Add(0x11B9C0ED,@"SSA Ctrlr");
                //mDevices.Add(0x11BC0001,@"NPI NuCard");
                //mDevices.Add(0x11BD0015,@"660806-2.0");
                //mDevices.Add(0x11BDBEBE,@"51015777");
                //mDevices.Add(0x11BDBEDE,@"51015777");
                //mDevices.Add(0x11C10440,@"LT Winmodem 56k");
                //mDevices.Add(0x11C10441,@"4390143");
                //mDevices.Add(0x11C10442,@"1648T00");
                //mDevices.Add(0x11C10443,@"LT Winmodem");
                //mDevices.Add(0x11C10444,@"LT Winmodem");
                //mDevices.Add(0x11C10445,@"LT Winmodem");
                //mDevices.Add(0x11C10446,@"LT Winmodem");
                //mDevices.Add(0x11C10447,@"LT Winmodem");
                //mDevices.Add(0x11C10448,@"LT Winmodem 56k");
                //mDevices.Add(0x11C10449,@"LT Winmodem 56k");
                //mDevices.Add(0x11C1044A,@"LT Winmodem 56k");
                //mDevices.Add(0x11C1044B,@"LT Winmodem");
                //mDevices.Add(0x11C1044C,@"LT Winmodem");
                //mDevices.Add(0x11C1044D,@"LT Winmodem");
                //mDevices.Add(0x11C1044E,@"12232");
                //mDevices.Add(0x11C1044F,@"90094-1");
                //mDevices.Add(0x11C10450,@"1456VQH19R-1(INT)");
                //mDevices.Add(0x11C10451,@"LT Winmodem");
                //mDevices.Add(0x11C10452,@"LT Winmodem");
                //mDevices.Add(0x11C10453,@"LT Winmodem");
                //mDevices.Add(0x11C10454,@"LT Winmodem");
                //mDevices.Add(0x11C10455,@"LT Winmodem");
                //mDevices.Add(0x11C10456,@"LT Winmodem");
                //mDevices.Add(0x11C10457,@"LT Winmodem");
                //mDevices.Add(0x11C10458,@"1648C");
                //mDevices.Add(0x11C10459,@"LT Winmodem");
                //mDevices.Add(0x11C1045A,@"LT Winmodem");
                //mDevices.Add(0x11C1045D,@"LT WinModem");
                //mDevices.Add(0x11C10461,@"V90 Wildfire Modem");
                //mDevices.Add(0x11C10462,@"1690");
                //mDevices.Add(0x11C10464,@"This is NOT a Riptide! (as previously stated)");
                //mDevices.Add(0x11C10480,@"Venus Winmodem");
                //mDevices.Add(0x11C1048C,@"1648c-tv5");
                //mDevices.Add(0x11C1048E,@"svp92pl-t00");
                //mDevices.Add(0x11C1048F,@"SV92P-T00");
                //mDevices.Add(0x11C10600,@"sv92p2");
                //mDevices.Add(0x11C10620,@"SV92PP");
                //mDevices.Add(0x11C11040,@"Prespa");
                //mDevices.Add(0x11C13026,@"0x11c13026");
                //mDevices.Add(0x11C15400,@"OR3TP12");
                //mDevices.Add(0x11C15801,@"USB Open Host Controller");
                //mDevices.Add(0x11C15802,@"USS-312MC");
                //mDevices.Add(0x11C15803,@"USS-344");
                //mDevices.Add(0x11C15805,@"uss344");
                //mDevices.Add(0x11C15811,@"FW322");
                //mDevices.Add(0x11C19876,@"lucent 1646T00");
                //mDevices.Add(0x11C1AB20,@"WaveLAN");
                //mDevices.Add(0x11C1AB30,@"Hermes2?");
                //mDevices.Add(0x11C1ED00,@"ET-131x");
                //mDevices.Add(0x11C63001,@"VM-1200 Opto Unit Controller");
                //mDevices.Add(0x11C80658,@"PSB 32");
                //mDevices.Add(0x11C8D665,@"PSB64");
                //mDevices.Add(0x11C8D667,@"PSB66");
                //mDevices.Add(0x11C90010,@"16-line serial port w/DMA");
                //mDevices.Add(0x11C90011,@"4-line serial port w/DMA");
                //mDevices.Add(0x11CB2000,@"PCI-9050-1100083-11");
                //mDevices.Add(0x11CB4000,@"SUPI-1");
                //mDevices.Add(0x11CB8000,@"T225");
                //mDevices.Add(0x11CE102B,@"1001");
                //mDevices.Add(0x11D101F7,@"VxP524");
                //mDevices.Add(0x11D101F8,@"VxP524");
                //mDevices.Add(0x11D41535,@"ADSP-21535");
                //mDevices.Add(0x11D41805,@"62412-51");
                //mDevices.Add(0x11D41884,@"AD1884HD");
                //mDevices.Add(0x11D41889,@"AD1980");
                //mDevices.Add(0x11D41981,@"AD1981HD");
                //mDevices.Add(0x11D41983,@"AD1983HD");
                //mDevices.Add(0x11D41986,@"AD1986A");
                //mDevices.Add(0x11D4198B,@"AD1988B");
                //mDevices.Add(0x11D42192,@"ADSP-2192");
                //mDevices.Add(0x11D4219A,@"ADSP-2192");
                //mDevices.Add(0x11D4219E,@"ADSP-2192");
                //mDevices.Add(0x11D42F44,@"ADSP-2141");
                //mDevices.Add(0x11D50115,@"10115");
                //mDevices.Add(0x11D50116,@"10118");
                //mDevices.Add(0x11D50117,@"10117");
                //mDevices.Add(0x11D50118,@"10118");
                //mDevices.Add(0x11DB1234,@"Dreamcast Broadband Adapter");
                //mDevices.Add(0x11DE6057,@"ZR36057/36067");
                //mDevices.Add(0x11DE6120,@"ZR36120PQC");
                //mDevices.Add(0x11F04232,@"FASTline UTP Quattro");
                //mDevices.Add(0x11F04233,@"FASTline FO");
                //mDevices.Add(0x11F04234,@"FASTline UTP");
                //mDevices.Add(0x11F04235,@"FASTline-II UTP");
                //mDevices.Add(0x11F04236,@"FASTline-II FO");
                //mDevices.Add(0x11F04731,@"GIGAline");
                //mDevices.Add(0x11F42915,@"2915");
                //mDevices.Add(0x11F60112,@"ATT2MD11");
                //mDevices.Add(0x11F60113,@"FreedomLine 100");
                //mDevices.Add(0x11F61401,@"832AE28030680");
                //mDevices.Add(0x11F62011,@"TXA9882");
                //mDevices.Add(0x11F62201,@"TXA9883");
                //mDevices.Add(0x11F69881,@"TXA9881?");
                //mDevices.Add(0x11F87364,@"PM7364");
                //mDevices.Add(0x11F87366,@"PM7364");
                //mDevices.Add(0x11F87367,@"PM7367");
                //mDevices.Add(0x11F87375,@"PM7375");
                //mDevices.Add(0x11F87380,@"PM7380");
                //mDevices.Add(0x11F87382,@"PM7382");
                //mDevices.Add(0x11F87384,@"PM7384");
                //mDevices.Add(0x11F88000,@"PM8000 SPC");
                //mDevices.Add(0x11F88010,@"PM8010 SRC");
                //mDevices.Add(0x11FE0001,@"RocketPort");
                //mDevices.Add(0x11FE0002,@"RocketPort");
                //mDevices.Add(0x11FE0003,@"RocketPort");
                //mDevices.Add(0x11FE0004,@"RocketPort");
                //mDevices.Add(0x11FE0005,@"RocketPort");
                //mDevices.Add(0x11FE0006,@"RocketPort");
                //mDevices.Add(0x11FE0007,@"RocketPort");
                //mDevices.Add(0x11FE0008,@"RocketPort");
                //mDevices.Add(0x11FE0009,@"RocketPort");
                //mDevices.Add(0x11FE000A,@"RocketPort");
                //mDevices.Add(0x11FE000B,@"RocketPort");
                //mDevices.Add(0x11FE000C,@"RocketPort");
                //mDevices.Add(0x11FE000D,@"RocketPort");
                //mDevices.Add(0x11FE8015,@"RocketPort");
                //mDevices.Add(0x12020001,@"NAIATMPCI");
                //mDevices.Add(0x12084853,@"HS-Link Device");
                //mDevices.Add(0x12090100,@"PCI 9054");
                //mDevices.Add(0x120E0100,@"Cyclom-Y");
                //mDevices.Add(0x120E0101,@"Cyclom-Y");
                //mDevices.Add(0x120E0102,@"Cyclom-4Y");
                //mDevices.Add(0x120E0103,@"Cyclom-4Y");
                //mDevices.Add(0x120E0104,@"Cyclom-8Y");
                //mDevices.Add(0x120E0105,@"Cyclom-8Y");
                //mDevices.Add(0x120E0200,@"Cyclom-Z");
                //mDevices.Add(0x120E0201,@"Cyclom-Z");
                //mDevices.Add(0x120E0300,@"PC300 RX 2");
                //mDevices.Add(0x120E0301,@"PC300 RX 1");
                //mDevices.Add(0x120E0302,@"PC300 TE 2");
                //mDevices.Add(0x120E0303,@"PC300 TE 1");
                //mDevices.Add(0x120F0001,@"Roadrunner");
                //mDevices.Add(0x12160003,@"PTM400 PCI Taxi Module");
                //mDevices.Add(0x12176729,@"OZ6729");
                //mDevices.Add(0x1217673A,@"OZ6730");
                //mDevices.Add(0x12176832,@"OZ6832/3");
                //mDevices.Add(0x12176836,@"OZ6836/6860");
                //mDevices.Add(0x12176872,@"OZ6812");
                //mDevices.Add(0x12176925,@"OZ6922");
                //mDevices.Add(0x12176933,@"OZ6933");
                //mDevices.Add(0x12176972,@"OZ6912");
                //mDevices.Add(0x12177110,@"OZ711Mx");
                //mDevices.Add(0x12177113,@"0Z711EC1");
                //mDevices.Add(0x12177114,@"OZ711M1");
                //mDevices.Add(0x12177120,@"Unknown device");
                //mDevices.Add(0x12177130,@"OZ711M3");
                //mDevices.Add(0x12177134,@"OZ711MP1/MS1");
                //mDevices.Add(0x121A0001,@"Voodoo");
                //mDevices.Add(0x121A0002,@"Voodoo2");
                //mDevices.Add(0x121A0003,@"Voodoo Banshee");
                //mDevices.Add(0x121A0005,@"Voodoo3");
                //mDevices.Add(0x121A0007,@"Voodoo4");
                //mDevices.Add(0x121A0009,@"Voodoo5");
                //mDevices.Add(0x121A0010,@"Rampage");
                //mDevices.Add(0x121A0057,@"Voodoo 3/3000");
                //mDevices.Add(0x12201220,@"9622qac");
                //mDevices.Add(0x12230001,@"KatanaQp");
                //mDevices.Add(0x12230002,@"KosaiPM");
                //mDevices.Add(0x12230044,@"Memory controller");
                //mDevices.Add(0x12241000,@"CL560?");
                //mDevices.Add(0x122D1206,@"ICH2");
                //mDevices.Add(0x122D4201,@"MR2800W");
                //mDevices.Add(0x122D50DC,@"PCI168/3328");
                //mDevices.Add(0x122D80DA,@"3328");
                //mDevices.Add(0x122F37AF,@"0x9030");
                //mDevices.Add(0x12360000,@"RealMagic64/GX");
                //mDevices.Add(0x12360531,@"MX98715/25");
                //mDevices.Add(0x12366401,@"REALmagic64/GX");
                //mDevices.Add(0x12369708,@"realmagic64/gx");
                //mDevices.Add(0x123D0010,@"PCI-DV");
                //mDevices.Add(0x123F00E4,@"cl 480");
                //mDevices.Add(0x123F6120,@"12.03");
                //mDevices.Add(0x123F8120,@"176");
                //mDevices.Add(0x123F8888,@"12.03");
                //mDevices.Add(0x12411603,@"keyboard");
                //mDevices.Add(0x12421460,@"JNIC-1460");
                //mDevices.Add(0x12421560,@"JNIC-1560");
                //mDevices.Add(0x12424643,@"JNI PCI 64-bit Fibrechannel");
                //mDevices.Add(0x12440700,@"B1");
                //mDevices.Add(0x12440800,@"C4");
                //mDevices.Add(0x12440A00,@"A1");
                //mDevices.Add(0x12440E00,@"Fritz!PCI 2.0 ISDN Controller");
                //mDevices.Add(0x12441100,@"C2");
                //mDevices.Add(0x12441200,@"T1");
                //mDevices.Add(0x12442700,@"E13 32A5KYW / 2CAKRCT");
                //mDevices.Add(0x124D0000,@"EasyConnect 8/32");
                //mDevices.Add(0x124D0002,@"EasyConnect 8/64");
                //mDevices.Add(0x124D0003,@"EasyIO PCI");
                //mDevices.Add(0x124F0041,@"IFT-2000");
                //mDevices.Add(0x12551110,@"MPEG Forge");
                //mDevices.Add(0x12551210,@"MPEG Fusion");
                //mDevices.Add(0x12552110,@"VideoPlex-pci");
                //mDevices.Add(0x12552120,@"VideoPlex plus");
                //mDevices.Add(0x12552130,@"VideoQuest");
                //mDevices.Add(0x12564201,@"PCI-2240i");
                //mDevices.Add(0x12564401,@"PCI-2220i");
                //mDevices.Add(0x12565201,@"PCI-2000");
                //mDevices.Add(0x12592503,@"Realtek 8139b");
                //mDevices.Add(0x12592560,@"AT-2560 Fast Ethernet Adapter (i82557B)");
                //mDevices.Add(0x125B0B95,@"AX88772");
                //mDevices.Add(0x125B1400,@"AX88140A");
                //mDevices.Add(0x125B1720,@"AX88172");
                //mDevices.Add(0x125D0000,@"ESS336H");
                //mDevices.Add(0x125D1961,@"Multimedia Device");
                //mDevices.Add(0x125D1968,@"ES2839");
                //mDevices.Add(0x125D1969,@"ES72222");
                //mDevices.Add(0x125D1978,@"ES1978");
                //mDevices.Add(0x125D1988,@"ES1988");
                //mDevices.Add(0x125D1989,@"ES1989");
                //mDevices.Add(0x125D1990,@"ES2898S");
                //mDevices.Add(0x125D1998,@"Maestro 3i");
                //mDevices.Add(0x125D199B,@"ES1938/41/46 solo audio");
                //mDevices.Add(0x125D2808,@"ES336H");
                //mDevices.Add(0x125D2828,@"ES2828S");
                //mDevices.Add(0x125D2838,@"ES56H-PI");
                //mDevices.Add(0x125D2839,@"2838");
                //mDevices.Add(0x125D2898,@"2898");
                //mDevices.Add(0x125F2084,@"AMCC S5933");
                //mDevices.Add(0x12603872,@"PRISM 3");
                //mDevices.Add(0x12603873,@"ISL3874A");
                //mDevices.Add(0x12603890,@"ISL3890");
                //mDevices.Add(0x12608130,@"HMP8130");
                //mDevices.Add(0x12608131,@"HMP8131");
                //mDevices.Add(0x12660001,@"NE10/100 Adapter (i82557B)");
                //mDevices.Add(0x12661910,@"NE2000Plus (RT8029) Ethernet Adapter");
                //mDevices.Add(0x12671016,@"NICCY PCI card");
                //mDevices.Add(0x12674243,@"Satellite receiver board / MPEG2 decoder");
                //mDevices.Add(0x12675352,@"PCR2101");
                //mDevices.Add(0x12675A4B,@"telsatturbo");
                //mDevices.Add(0x12680204,@"Unknown");
                //mDevices.Add(0x126C1F1F,@"e-mobility 802.11b Wireless LAN PCI Card");
                //mDevices.Add(0x126F0710,@"SM710");
                //mDevices.Add(0x126F0712,@"SM712");
                //mDevices.Add(0x126F0720,@"SM720 / SM722");
                //mDevices.Add(0x126F0810,@"SM810");
                //mDevices.Add(0x126F0811,@"SM811");
                //mDevices.Add(0x126F0820,@"SM820");
                //mDevices.Add(0x126F0910,@"SM910");
                //mDevices.Add(0x12730002,@"DirecPC");
                //mDevices.Add(0x12741274,@"5880x");
                //mDevices.Add(0x12741371,@"ES 1373");
                //mDevices.Add(0x12741373,@"ES1373");
                //mDevices.Add(0x12745000,@"ES1373");
                //mDevices.Add(0x12745880,@"5880x");
                //mDevices.Add(0x12780701,@"TPE3/TM3");
                //mDevices.Add(0x12790060,@"TM8000");
                //mDevices.Add(0x12790061,@"TMTM8000");
                //mDevices.Add(0x12790295,@"Virtual Northbridge");
                //mDevices.Add(0x12790395,@"LongRun");
                //mDevices.Add(0x12790396,@"SDRAM Controller");
                //mDevices.Add(0x12790397,@"BIOS scratchpad");
                //mDevices.Add(0x12829009,@"DM9009");
                //mDevices.Add(0x12829102,@"DM9102/A/AF");
                //mDevices.Add(0x12830801,@"IT8152F/G");
                //mDevices.Add(0x1283673A,@"IT8330G");
                //mDevices.Add(0x12838152,@"IT8152F/G");
                //mDevices.Add(0x12838172,@"IT8172G");
                //mDevices.Add(0x12838211,@"IT8211F");
                //mDevices.Add(0x12838212,@"IT8212F");
                //mDevices.Add(0x12838330,@"IT8330G");
                //mDevices.Add(0x12838872,@"IT8871/72");
                //mDevices.Add(0x12838875,@"IT8875F");
                //mDevices.Add(0x12838888,@"IT8888F");
                //mDevices.Add(0x12838889,@"IT8889F");
                //mDevices.Add(0x12839876,@"IT8875F");
                //mDevices.Add(0x1283E886,@"IT8330G");
                //mDevices.Add(0x12850100,@"ES1849");
                //mDevices.Add(0x1287001E,@"LS220D");
                //mDevices.Add(0x1287001F,@"LS220C");
                //mDevices.Add(0x12870020,@"LS242");
                //mDevices.Add(0x128AF001,@"Ethernet 10/100");
                //mDevices.Add(0x128D0021,@"ATM Adapter");
                //mDevices.Add(0x129A0415,@"PBT-415");
                //mDevices.Add(0x129A0515,@"PBT-515");
                //mDevices.Add(0x129A0615,@"PBT-615");
                //mDevices.Add(0x129A0715,@"Vanguard PCI/PMC/cPCI");
                //mDevices.Add(0x129ADD10,@"DPIO");
                //mDevices.Add(0x129ADD11,@"DPIO2");
                //mDevices.Add(0x129ADD12,@"DPIO2-66");
                //mDevices.Add(0x12A3ECB8,@"1646T00");
                //mDevices.Add(0x12AA5568,@"Wan405");
                //mDevices.Add(0x12AA556C,@"NAI HSSI Sniffer PCI Adapter");
                //mDevices.Add(0x12AB3000,@"JRS-3DS100");
                //mDevices.Add(0x12AE0001,@"3C986");
                //mDevices.Add(0x12AE0002,@"3C986-T");
                //mDevices.Add(0x12B91006,@"5610");
                //mDevices.Add(0x12B91007,@"AD1807JS");
                //mDevices.Add(0x12B91008,@"USR5610B");
                //mDevices.Add(0x12BA0032,@"Hammerhead-Lite-PCI");
                //mDevices.Add(0x12C19080,@"Sync4hs/CCP/PCI/MP");
                //mDevices.Add(0x12C30058,@"HT80232");
                //mDevices.Add(0x12C35598,@"HT80229");
                //mDevices.Add(0x12C40005,@"Blue Heat-8 RS232");
                //mDevices.Add(0x12C5007F,@"ISE");
                //mDevices.Add(0x12C50081,@"PCIVST");
                //mDevices.Add(0x12C50085,@"Video Simulator/Sender");
                //mDevices.Add(0x12C50086,@"THR2");
                //mDevices.Add(0x12C70546,@"kssjsjj");
                //mDevices.Add(0x12C70561,@"BRI/2 Type Card (Voice Driver)");
                //mDevices.Add(0x12C70647,@"D/240JCT-T1 Card");
                //mDevices.Add(0x12C70648,@"D/300JCT-E1 Card");
                //mDevices.Add(0x12C70649,@"D/300JCT-E1 Card");
                //mDevices.Add(0x12C70651,@"MSI PCI Card");
                //mDevices.Add(0x12C70673,@"BRI/160-PCI Card");
                //mDevices.Add(0x12C70674,@"BRI/120-PCI Card");
                //mDevices.Add(0x12C70675,@"BRI/80-PCI Card");
                //mDevices.Add(0x12C70676,@"D/41JCT Card");
                //mDevices.Add(0x12C70685,@"D/480JCT-2T1 Card");
                //mDevices.Add(0x12C70687,@"D/600JCT-2E1 (75 Ohm) Card");
                //mDevices.Add(0x12C70689,@"D/600JCT-2E1");
                //mDevices.Add(0x12C70707,@"D/320JCT (Resource Only) Card");
                //mDevices.Add(0x12C70708,@"D/160JCT (Resource Only) Card");
                //mDevices.Add(0x12CB0027,@"StudioCard");
                //mDevices.Add(0x12CB002D,@"BX-12");
                //mDevices.Add(0x12CB002E,@"SC-2000");
                //mDevices.Add(0x12CB002F,@"LX-44");
                //mDevices.Add(0x12CB0030,@"SC-22");
                //mDevices.Add(0x12CB0031,@"BX-44");
                //mDevices.Add(0x12CB0032,@"LX-24M");
                //mDevices.Add(0x12CB0033,@"LX-22M");
                //mDevices.Add(0x12CB0034,@"BX-8");
                //mDevices.Add(0x12CB0035,@"BX-12e");
                //mDevices.Add(0x12D40301,@"EP1S");
                //mDevices.Add(0x12D51000,@"MAP-CA");
                //mDevices.Add(0x12D51002,@"MAP-1000");
                //mDevices.Add(0x12D871E2,@"PI7C7300");
                //mDevices.Add(0x12D88150,@"PI7C8150");
                //mDevices.Add(0x12D88152,@"PI7C8152");
                //mDevices.Add(0x12DB0003,@"FoxFire II");
                //mDevices.Add(0x12DE0200,@"Cryptoswift 200");
                //mDevices.Add(0x12DF2102,@"Infineon PEB 20534");
                //mDevices.Add(0x12DF8236,@"Unknown");
                //mDevices.Add(0x12E00010,@"ST16C654");
                //mDevices.Add(0x12E00020,@"ST16C654");
                //mDevices.Add(0x12E00030,@"ST16C654");
                //mDevices.Add(0x12E41000,@"TR1000 PRI");
                //mDevices.Add(0x12E41140,@"ISDN Controller");
                //mDevices.Add(0x12E4B005,@"TR1000 BRI");
                //mDevices.Add(0x12E4B006,@"TR1000 BRI");
                //mDevices.Add(0x12EB0001,@"AU8820");
                //mDevices.Add(0x12EB0002,@"AU8830A2");
                //mDevices.Add(0x12EB0003,@"312EB&REV2");
                //mDevices.Add(0x12EC8139,@"0xxxx");
                //mDevices.Add(0x12F21002,@"215RAAGCGA11F");
                //mDevices.Add(0x12F23059,@"VT8237");
                //mDevices.Add(0x12F80002,@"VideoMaker");
                //mDevices.Add(0x13030001,@"0239");
                //mDevices.Add(0x13030002,@"M44/cM44 DSP board");
                //mDevices.Add(0x13030003,@"Quattro6x DSP board");
                //mDevices.Add(0x13030004,@"Chico/ChicoPlus Data Acquisition Board");
                //mDevices.Add(0x13030005,@"Code Hammer Jtag Debugger board");
                //mDevices.Add(0x13030006,@"Matador DSP board");
                //mDevices.Add(0x13030007,@"Quixote DSP board");
                //mDevices.Add(0x13030008,@"Quadia C64x DSP");
                //mDevices.Add(0x13030009,@"Quadia DSP Baseboard");
                //mDevices.Add(0x13070001,@"PCI-DAS1602/16");
                //mDevices.Add(0x13070006,@"PCI-GPIB");
                //mDevices.Add(0x1307000B,@"PCI-DIO48H");
                //mDevices.Add(0x1307000C,@"PCI-PDISO8");
                //mDevices.Add(0x1307000D,@"PCI-PDISO16");
                //mDevices.Add(0x1307000F,@"PCI-DAS1200");
                //mDevices.Add(0x13070010,@"PCI-DAS1602/12");
                //mDevices.Add(0x13070014,@"PCI-DIO24H");
                //mDevices.Add(0x13070015,@"PCI-DIO24H/CTR3");
                //mDevices.Add(0x13070016,@"PCI-DIO24H/CTR16");
                //mDevices.Add(0x13070017,@"PCI-DIO96H");
                //mDevices.Add(0x13070018,@"PCI-CTR05");
                //mDevices.Add(0x13070019,@"PCI-DAS1200/JR");
                //mDevices.Add(0x1307001A,@"PCI-DAS1001");
                //mDevices.Add(0x1307001B,@"PCI-DAS1002");
                //mDevices.Add(0x1307001C,@"PCI-DAS1602JR/16");
                //mDevices.Add(0x1307001D,@"PCI-DAS6402/16");
                //mDevices.Add(0x1307001E,@"PCI-DAS6402/12");
                //mDevices.Add(0x1307001F,@"PCI-DAS16/M1");
                //mDevices.Add(0x13070020,@"PCI-DDA02/12");
                //mDevices.Add(0x13070021,@"PCI-DDA04/12");
                //mDevices.Add(0x13070022,@"PCI-DDA08/12");
                //mDevices.Add(0x13070023,@"PCI-DDA02/16");
                //mDevices.Add(0x13070024,@"PCI-DDA04/16");
                //mDevices.Add(0x13070025,@"PCI-DDA08/16");
                //mDevices.Add(0x13070026,@"PCI-DAC04/12-HS");
                //mDevices.Add(0x13070027,@"PCI-DAC04/16-HS");
                //mDevices.Add(0x13070028,@"PCI-DIO24");
                //mDevices.Add(0x13070029,@"PCI-DAS08");
                //mDevices.Add(0x1307002C,@"PCI-INT32");
                //mDevices.Add(0x13070033,@"PCI-DUAL-AC5");
                //mDevices.Add(0x13070034,@"PCI-DAS-TC");
                //mDevices.Add(0x13070035,@"PCI-DAS64/M1/16");
                //mDevices.Add(0x13070036,@"PCI-DAS64/M2/16");
                //mDevices.Add(0x13070037,@"PCI-DAS64/M3/16");
                //mDevices.Add(0x1307004C,@"PCI-DAS1000");
                //mDevices.Add(0x1307004D,@"PCI-QUAD04");
                //mDevices.Add(0x13070064,@"PCI-CTR10");
                //mDevices.Add(0x13080001,@"NetCelerator Adapter");
                //mDevices.Add(0x13100003,@"9060");
                //mDevices.Add(0x1310000D,@"FPGA PCI Bridge");
                //mDevices.Add(0x13170531,@"nm0403A03461H121S122P");
                //mDevices.Add(0x13170981,@"AN981");
                //mDevices.Add(0x13170985,@"an983b");
                //mDevices.Add(0x13171985,@"AN985");
                //mDevices.Add(0x13172850,@"016");
                //mDevices.Add(0x13175120,@"ADM5120");
                //mDevices.Add(0x13177892,@"PCT 789T-A");
                //mDevices.Add(0x13178201,@"ADM8211");
                //mDevices.Add(0x13179511,@"ADM9511");
                //mDevices.Add(0x13179513,@"ADM9513");
                //mDevices.Add(0x13180911,@"G-NIC II");
                //mDevices.Add(0x13190801,@"FM0801");
                //mDevices.Add(0x13190802,@"FM801-as");
                //mDevices.Add(0x13191000,@"FM801-as");
                //mDevices.Add(0x13191001,@"FM801");
                //mDevices.Add(0x13191319,@"FM0801-as");
                //mDevices.Add(0x13194901,@"FM801-AU");
                //mDevices.Add(0x131F2002,@"CyberSerial 16850");
                //mDevices.Add(0x131F2011,@"Duet 1S(16650)+1P");
                //mDevices.Add(0x131F2012,@"Duet 1S(16850)+1P");
                //mDevices.Add(0x131F2020,@"SIIG cyber8000");
                //mDevices.Add(0x131F2042,@"Trio 1S(16850)+2P");
                //mDevices.Add(0x131F2050,@"Siig Inc CyberSerial (4-port) 16550");
                //mDevices.Add(0x131F2051,@"CyberSerial 16650");
                //mDevices.Add(0x131F2052,@"CyberSerial 16850");
                //mDevices.Add(0x131F2060,@"Trio 2S(16550)+1P");
                //mDevices.Add(0x131F2061,@"Trio 2S(16650)+1P");
                //mDevices.Add(0x131F2062,@"Trio 2S(16850)+1P");
                //mDevices.Add(0x13288888,@"rev 1.5");
                //mDevices.Add(0x13325410,@"MM-5410D");
                //mDevices.Add(0x13325415,@"MM-5415CN");
                //mDevices.Add(0x13325425,@"MM-5425CN");
                //mDevices.Add(0x13326140,@"MM-6140D");
                //mDevices.Add(0x133D1000,@"SST-5136-PFB-PCI");
                //mDevices.Add(0x13443240,@"CopperHead CopperTail SC1 AMC AC97");
                //mDevices.Add(0x13443320,@"MT8LLN21PADF");
                //mDevices.Add(0x13443321,@"21PAD");
                //mDevices.Add(0x13443470,@"MT7LLN22NCNE");
                //mDevices.Add(0x13444020,@"CopperHead CopperTail SC1 IDE Controller");
                //mDevices.Add(0x13444030,@"CopperHead CopperTail SC1 USB Controller");
                //mDevices.Add(0x134A0001,@"F01 2ASV17184.1");
                //mDevices.Add(0x134A0002,@"Domex DMX3192U/3194UP SCSI Adapter");
                //mDevices.Add(0x134D2188,@"fa82537ep");
                //mDevices.Add(0x134D2189,@"HSP688T");
                //mDevices.Add(0x134D2486,@"82537");
                //mDevices.Add(0x134D7890,@"fa82537ep");
                //mDevices.Add(0x134D7891,@"PCT 789T");
                //mDevices.Add(0x134D7892,@"PCT 789T-A");
                //mDevices.Add(0x134D7893,@"S911 K017");
                //mDevices.Add(0x134D7894,@"688w");
                //mDevices.Add(0x134D7895,@"PCT789T");
                //mDevices.Add(0x134D7896,@"pct789t-c");
                //mDevices.Add(0x134D7897,@"97860963");
                //mDevices.Add(0x134D9714,@"PCT 288-1A");
                //mDevices.Add(0x134DD800,@"pct388p-a");
                //mDevices.Add(0x135A0224,@"PLX9050");
                //mDevices.Add(0x135E5101,@"5101");
                //mDevices.Add(0x135E5102,@"PCI-ACB");
                //mDevices.Add(0x135E7101,@"Single Port RS-232/422/485/520");
                //mDevices.Add(0x135E7201,@"Dual Port RS-232/422/485 Interface");
                //mDevices.Add(0x135E7202,@"Dual Port RS-232 Interface");
                //mDevices.Add(0x135E7401,@"Four Port RS-232 Interface");
                //mDevices.Add(0x135E7402,@"Four Port RS-422/485 Interface");
                //mDevices.Add(0x135E7801,@"Eight Port RS-232 Interface");
                //mDevices.Add(0x135E8001,@"8001");
                //mDevices.Add(0x13600101,@"PCI32");
                //mDevices.Add(0x13600102,@"PCI509");
                //mDevices.Add(0x13600103,@"PCI510");
                //mDevices.Add(0x13600104,@"PCI511");
                //mDevices.Add(0x13600105,@"PEX511");
                //mDevices.Add(0x13600201,@"GPS167PCI");
                //mDevices.Add(0x13600202,@"GPS168PCI");
                //mDevices.Add(0x13600203,@"GPS169PCI");
                //mDevices.Add(0x13600204,@"GPS170PCI");
                //mDevices.Add(0x13600205,@"GPS170PEX");
                //mDevices.Add(0x13600301,@"TCR510PCI");
                //mDevices.Add(0x13600302,@"TCR167PCI");
                //mDevices.Add(0x13600303,@"TCR511PCI");
                //mDevices.Add(0x13600304,@"TCR511PEX");
                //mDevices.Add(0x13659050,@"HYSDN");
                //mDevices.Add(0x13752571,@"NA");
                //mDevices.Add(0x13822009,@"PLX TECHNOLOGY \n PCI9052 \n 0435 \n BX9497.1 MALA");
                //mDevices.Add(0x13822048,@"Prodif Plus sound card");
                //mDevices.Add(0x13822088,@"Marc 8 Midi");
                //mDevices.Add(0x13854100,@"MA301");
                //mDevices.Add(0x1385620A,@"GA620");
                //mDevices.Add(0x1385622A,@"GA622");
                //mDevices.Add(0x1385630A,@"GA630");
                //mDevices.Add(0x13858169,@"GA311");
                //mDevices.Add(0x1385F311,@"FA311");
                //mDevices.Add(0x13875310,@"N/A");
                //mDevices.Add(0x13890001,@"PCI1500PFB");
                //mDevices.Add(0x13890104,@"PCI-CANIO");
                //mDevices.Add(0x13931040,@"C104H/PCI");
                //mDevices.Add(0x13931320,@"CP-132");
                //mDevices.Add(0x13931340,@"CP--134U-I-DB9 V2.3 UPCI Board");
                //mDevices.Add(0x13931680,@"C168H/PCI");
                //mDevices.Add(0x13931681,@"CP-168U");
                //mDevices.Add(0x13932040,@"CP-204J");
                //mDevices.Add(0x13932180,@"C218");
                //mDevices.Add(0x13932210,@"---");
                //mDevices.Add(0x13932240,@"---");
                //mDevices.Add(0x13933200,@"C320");
                //mDevices.Add(0x13936800,@"0x0000");
                //mDevices.Add(0x13940001,@"LXT1001");
                //mDevices.Add(0x13970B4D,@"HFC-8S 16B8D8S0");
                //mDevices.Add(0x13972BD0,@"HFC-PCI A ISDN 2BDS0");
                //mDevices.Add(0x13978B4D,@"HFC-4S ISDN 8B4D4S0");
                //mDevices.Add(0x1397B000,@"B000");
                //mDevices.Add(0x1397B006,@"B006");
                //mDevices.Add(0x1397B007,@"B007");
                //mDevices.Add(0x1397B008,@"B008");
                //mDevices.Add(0x1397B009,@"B009");
                //mDevices.Add(0x1397B00A,@"B00A");
                //mDevices.Add(0x1397B00B,@"B00B");
                //mDevices.Add(0x1397B00C,@"B00C");
                //mDevices.Add(0x1397B100,@"B100");
                //mDevices.Add(0x139A0007,@"SES2000");
                //mDevices.Add(0x13A30005,@"7751");
                //mDevices.Add(0x13A30006,@"6500");
                //mDevices.Add(0x13A30007,@"7811");
                //mDevices.Add(0x13A30012,@"7951");
                //mDevices.Add(0x13A30014,@"7814/7851/7854");
                //mDevices.Add(0x13A30015,@"8065");
                //mDevices.Add(0x13A30017,@"8165");
                //mDevices.Add(0x13A30018,@"8154");
                //mDevices.Add(0x13A30020,@"7954/7955");
                //mDevices.Add(0x13A76240,@"Network Controller");
                //mDevices.Add(0x13A80152,@"XR17C152");
                //mDevices.Add(0x13A80154,@"XR17C154");
                //mDevices.Add(0x13A80158,@"XR17C158");
                //mDevices.Add(0x13C00010,@"SyncLink Adapter");
                //mDevices.Add(0x13C00020,@"SyncLink SCC");
                //mDevices.Add(0x13C00030,@"SyncLink 4 Port Adapter");
                //mDevices.Add(0x13C00070,@"SyncLink GT Adapter");
                //mDevices.Add(0x13C00080,@"SyncLink GT4 Adapter");
                //mDevices.Add(0x13C00090,@"SyncLink AC Adapter");
                //mDevices.Add(0x13C00210,@"SyncLink Serial Adapter version 2");
                //mDevices.Add(0x13C11000,@"9548qad");
                //mDevices.Add(0x13C11001,@"7000/8000 series");
                //mDevices.Add(0x13C11002,@"9000 series");
                //mDevices.Add(0x13C11003,@"9550SX/9590SE Series");
                //mDevices.Add(0x13C11004,@"9650SE Series");
                //mDevices.Add(0x13C70ADC,@"Multi-Function Analogue/Digital IO card");
                //mDevices.Add(0x13C70B10,@"Parallel I/O Card");
                //mDevices.Add(0x13C70D10,@"Digital I/O Card");
                //mDevices.Add(0x13C75744,@"Watchdog Card");
                //mDevices.Add(0x13D02103,@"T228502");
                //mDevices.Add(0x13D1AB06,@"FE2000VX");
                //mDevices.Add(0x13D1AB08,@"SMC8035TX");
                //mDevices.Add(0x13D78086,@"ac97");
                //mDevices.Add(0x13D81000,@"XQ11800FP");
                //mDevices.Add(0x13DF0001,@"PCI56RVP");
                //mDevices.Add(0x13EA3131,@"DS3131");
                //mDevices.Add(0x13EA3134,@"DS3134");
                //mDevices.Add(0x13F00200,@"FFK 8508");
                //mDevices.Add(0x13F00201,@"ST201");
                //mDevices.Add(0x13F00300,@"NX1001");
                //mDevices.Add(0x13F01023,@"NX1101");
                //mDevices.Add(0x13F013F0,@"-PCI\VEN_13F0&DEV_1023&SUBSYS_81801043&REV_41\4&BC");
                //mDevices.Add(0x13F60100,@"CMI8338/PCI C3DX");
                //mDevices.Add(0x13F60101,@"CMI8338-031");
                //mDevices.Add(0x13F60111,@"CMI8738/PCI C3DX");
                //mDevices.Add(0x13F60112,@"CMI-8378B/PCI-6CH");
                //mDevices.Add(0x13F60211,@"CMI8738/PCI-SX");
                //mDevices.Add(0x13F60300,@"0x4005");
                //mDevices.Add(0x13FE1240,@"PCI-1710");
                //mDevices.Add(0x13FE1680,@"PCI-1680U-A");
                //mDevices.Add(0x13FE1713,@"PCI-1713");
                //mDevices.Add(0x13FE1724,@"PCI-1724");
                //mDevices.Add(0x14000004,@"m583mlr");
                //mDevices.Add(0x14001401,@"9432 TX");
                //mDevices.Add(0x14022E00,@"ME-2000/2600");
                //mDevices.Add(0x14024610,@"ME-4610");
                //mDevices.Add(0x14024650,@"ME-4650");
                //mDevices.Add(0x14070100,@"0439");
                //mDevices.Add(0x14070101,@"Lava Quattro PCI A/B");
                //mDevices.Add(0x14070102,@"Lava Quattro PCI C/D");
                //mDevices.Add(0x14070110,@"Lava DSerial PCI Port A");
                //mDevices.Add(0x14070111,@"Lava DSerial PCI Port B");
                //mDevices.Add(0x14070180,@"Lava Octopus PCI Ports 1-4");
                //mDevices.Add(0x14070181,@"Lava Octopus PCI Ports 5-8");
                //mDevices.Add(0x14070200,@"9923");
                //mDevices.Add(0x14070201,@"LavaPort Quad-650 PCI A/B");
                //mDevices.Add(0x14070202,@"LavaPort Quad-650 PCI C/D");
                //mDevices.Add(0x14070220,@"LavaPort Quad-650 PCI A/B");
                //mDevices.Add(0x14070221,@"LavaPort Quad-650 PCI C/D");
                //mDevices.Add(0x14070400,@"Lava 8255 PIO PCI");
                //mDevices.Add(0x14070500,@"Lava Single Serial 550 PCI");
                //mDevices.Add(0x14070510,@"Lava SP Serial 550 PCI");
                //mDevices.Add(0x14070511,@"Lava SP BIDIR Parallel PCI");
                //mDevices.Add(0x14070520,@"Lava RS422 SS-PCI");
                //mDevices.Add(0x14070600,@"4-0 9253");
                //mDevices.Add(0x14070A00,@"LavaPort PCI");
                //mDevices.Add(0x14078000,@"Lava Parallel");
                //mDevices.Add(0x14078001,@"Lava Dual Parallel port A");
                //mDevices.Add(0x14078002,@"9923");
                //mDevices.Add(0x14078003,@"Lava Dual Parallel port B");
                //mDevices.Add(0x14078800,@"BOCA Research IOPPAR");
                //mDevices.Add(0x14097168,@"40371409");
                //mDevices.Add(0x14121712,@"ICE1712");
                //mDevices.Add(0x14121724,@"VT1720/24");
                //mDevices.Add(0x14158401,@"OX9162");
                //mDevices.Add(0x14158403,@"OX9162");
                //mDevices.Add(0x14159500,@"OX16PCI954");
                //mDevices.Add(0x14159501,@"OX16PCI954");
                //mDevices.Add(0x14159505,@"OXm16PCI952");
                //mDevices.Add(0x1415950A,@"OX16PCI954");
                //mDevices.Add(0x1415950B,@"OXCB950");
                //mDevices.Add(0x14159510,@"OX16PCI954");
                //mDevices.Add(0x14159511,@"OX9160");
                //mDevices.Add(0x14159512,@"OX16PCI954");
                //mDevices.Add(0x14159513,@"OX16PCI954");
                //mDevices.Add(0x14159521,@"OX16PCI952");
                //mDevices.Add(0x14159523,@"OX16PCI952");
                //mDevices.Add(0x141F6181,@"KFIR");
                //mDevices.Add(0x14350531,@"DELETE");
                //mDevices.Add(0x14356020,@"SPM6020");
                //mDevices.Add(0x14356030,@"SPM6030");
                //mDevices.Add(0x14356420,@"SPM186420");
                //mDevices.Add(0x14356430,@"SPM176430");
                //mDevices.Add(0x14357520,@"DM7520");
                //mDevices.Add(0x14357540,@"SDM7540");
                //mDevices.Add(0x14480001,@"ADAT/EDIT");
                //mDevices.Add(0x144A7248,@"PCI-9052");
                //mDevices.Add(0x144A7250,@"PCI-7250");
                //mDevices.Add(0x144A7256,@"PCI-7256");
                //mDevices.Add(0x144A7296,@"PCI-7296 (PLX PCI-9052)");
                //mDevices.Add(0x144A7432,@"PCI-7432");
                //mDevices.Add(0x144A7433,@"PCI-7433");
                //mDevices.Add(0x144A7434,@"PCI-7434");
                //mDevices.Add(0x144A7841,@"PCI-7841");
                //mDevices.Add(0x144A8133,@"PCI-8133");
                //mDevices.Add(0x144A8554,@"PCI-8554");
                //mDevices.Add(0x144A9111,@"PCI-9111");
                //mDevices.Add(0x144A9113,@"PCI-9113");
                //mDevices.Add(0x144A9114,@"PCI-9114");
                //mDevices.Add(0x145F0001,@"NextMove PCI");
                //mDevices.Add(0x145F0002,@"NextMove PCI-2");
                //mDevices.Add(0x146200C1,@"NV41.1");
                //mDevices.Add(0x14624720,@"883");
                //mDevices.Add(0x14625071,@"883");
                //mDevices.Add(0x14627960,@"MCP2T");
                //mDevices.Add(0x14710188,@"RoadRunner 10");
                //mDevices.Add(0x148C4011,@"RV250");
                //mDevices.Add(0x148C4152,@"0x1002");
                //mDevices.Add(0x148D1003,@"Rockwell HCF chipset");
                //mDevices.Add(0x14B11033,@"R6795-12");
                //mDevices.Add(0x14B30000,@"9ff-usa-a99-99-0970");
                //mDevices.Add(0x14B50200,@"Scope");
                //mDevices.Add(0x14B50300,@"Pulsar");
                //mDevices.Add(0x14B50400,@"Pulsar SRB");
                //mDevices.Add(0x14B50600,@"Pulsar 2");
                //mDevices.Add(0x14B50800,@"DSP-Board");
                //mDevices.Add(0x14B50900,@"DSP-Board");
                //mDevices.Add(0x14B50A00,@"DSP-Board");
                //mDevices.Add(0x14B50B00,@"DSP-Board");
                //mDevices.Add(0x14B70001,@"Symphony 4110");
                //mDevices.Add(0x14B90001,@"PC4800");
                //mDevices.Add(0x14B90340,@"Cisco Systems 340 PCI Wireless LAN Adptr");
                //mDevices.Add(0x14B92500,@"PC2500 DS");
                //mDevices.Add(0x14B93100,@"PC3100 FH");
                //mDevices.Add(0x14B93101,@"PC3100 FH");
                //mDevices.Add(0x14B93500,@"PC3500 FH");
                //mDevices.Add(0x14B94500,@"PC4500 DS");
                //mDevices.Add(0x14B94800,@"PC4800 DS");
                //mDevices.Add(0x14B9A504,@"A504 Cisco MAC");
                //mDevices.Add(0x14B9A506,@"AIR-MP21G");
                //mDevices.Add(0x14C18043,@"LANai 9.2 0129");
                //mDevices.Add(0x14CF2920,@"FPMC-FIO1-F100-1");
                //mDevices.Add(0x14D40400,@"Panacom 7");
                //mDevices.Add(0x14D90010,@"AP1011/SP1011");
                //mDevices.Add(0x14DB2100,@"PCI IO 1S");
                //mDevices.Add(0x14DB2101,@"PCI IO 1S-650");
                //mDevices.Add(0x14DB2102,@"PCI IO 1S-850");
                //mDevices.Add(0x14DB2110,@"PCI IO 1S1P");
                //mDevices.Add(0x14DB2111,@"PCI IO 1S1P-650");
                //mDevices.Add(0x14DB2112,@"PCI IO 1S1P-850");
                //mDevices.Add(0x14DB2120,@"TK9902");
                //mDevices.Add(0x14DB2121,@"PCI IO 2P");
                //mDevices.Add(0x14DB2130,@"PCI IO 2S");
                //mDevices.Add(0x14DB2131,@"PCI IO 2S-650");
                //mDevices.Add(0x14DB2132,@"PCI IO 2S-850");
                //mDevices.Add(0x14DB2140,@"PCI IO 2P1S");
                //mDevices.Add(0x14DB2141,@"PCI IO 2P1S-650");
                //mDevices.Add(0x14DB2142,@"PCI IO 2P1S-850");
                //mDevices.Add(0x14DB2144,@"PCI IO 2P2S");
                //mDevices.Add(0x14DB2145,@"PCI IO 2P2S-650");
                //mDevices.Add(0x14DB2146,@"PCI IO 2P2S-850");
                //mDevices.Add(0x14DB2150,@"PCI IO 4S");
                //mDevices.Add(0x14DB2151,@"PCI IO 4S-654");
                //mDevices.Add(0x14DB2152,@"PCI IO 4S-850");
                //mDevices.Add(0x14DB2160,@"PCI IO 2S1P");
                //mDevices.Add(0x14DB2161,@"PCI IO 2S1P-650");
                //mDevices.Add(0x14DB2162,@"PCI IO 2S1P-850");
                //mDevices.Add(0x14DB2180,@"PCI IO 8S");
                //mDevices.Add(0x14DB2181,@"PCI IO 8S-654");
                //mDevices.Add(0x14DB2182,@"PCI IO 8S-850");
                //mDevices.Add(0x14DC0000,@"PCI 230");
                //mDevices.Add(0x14DC0001,@"PCI242H");
                //mDevices.Add(0x14DC0002,@"PCI244H");
                //mDevices.Add(0x14DC0003,@"PCI247H");
                //mDevices.Add(0x14DC0004,@"PCI248H");
                //mDevices.Add(0x14DC0005,@"PCI249H");
                //mDevices.Add(0x14DC0006,@"PCI260");
                //mDevices.Add(0x14DC0007,@"PCI224");
                //mDevices.Add(0x14DC0008,@"PCI234");
                //mDevices.Add(0x14DC0009,@"PCI236");
                //mDevices.Add(0x14DC000A,@"PCI272");
                //mDevices.Add(0x14DC000B,@"PCI215");
                //mDevices.Add(0x14DC000C,@"PCI263");
                //mDevices.Add(0x14E4034F,@"NV36GL?");
                //mDevices.Add(0x14E40800,@"BCM33xx/47xx");
                //mDevices.Add(0x14E40804,@"BCM33xx/47xx");
                //mDevices.Add(0x14E40805,@"BCM33xx/47xx");
                //mDevices.Add(0x14E40806,@"BCM33xx/47xx");
                //mDevices.Add(0x14E4080B,@"BCM33xx/47xx");
                //mDevices.Add(0x14E4080F,@"BCM33xx/47xx");
                //mDevices.Add(0x14E40811,@"BCM33xx/47xx");
                //mDevices.Add(0x14E40816,@"BCM3302");
                //mDevices.Add(0x14E414E4,@"BCM5787M");
                //mDevices.Add(0x14E41600,@"BCM5752");
                //mDevices.Add(0x14E41644,@" BCM5751F");
                //mDevices.Add(0x14E41645,@"BCM5701");
                //mDevices.Add(0x14E41646,@"BCM5702x1");
                //mDevices.Add(0x14E41647,@"BCM5703");
                //mDevices.Add(0x14E41648,@"BCM5704");
                //mDevices.Add(0x14E4164C,@"BCM5708");
                //mDevices.Add(0x14E4164D,@"BCM5702FE");
                //mDevices.Add(0x14E41653,@"BCM5788");
                //mDevices.Add(0x14E41654,@"BCM5705-");
                //mDevices.Add(0x14E41658,@"BCM5750");
                //mDevices.Add(0x14E41659,@"BCM5721");
                //mDevices.Add(0x14E4165A,@"5722");
                //mDevices.Add(0x14E4165D,@"BCM5705M");
                //mDevices.Add(0x14E4165E,@"BCM5705M");
                //mDevices.Add(0x14E4166B,@"BCM5780S");
                //mDevices.Add(0x14E4166D,@"BCM5705MFE");
                //mDevices.Add(0x14E4166E,@"BCM5705");
                //mDevices.Add(0x14E41672,@"BCM5754M");
                //mDevices.Add(0x14E41673,@"B5755M");
                //mDevices.Add(0x14E41676,@"BCM5750");
                //mDevices.Add(0x14E41677,@"BCM5750A1");
                //mDevices.Add(0x14E4167A,@"BCM5754");
                //mDevices.Add(0x14E4167B,@"BCM5755/5780");
                //mDevices.Add(0x14E4167C,@"BCM5750");
                //mDevices.Add(0x14E4167E,@"BCM5751FKFB");
                //mDevices.Add(0x14E41693,@"BCM5787A");
                //mDevices.Add(0x14E41696,@"BCM5782");
                //mDevices.Add(0x14E4169A,@"BCM5786");
                //mDevices.Add(0x14E4169B,@"BCM5787");
                //mDevices.Add(0x14E4169C,@"BCM5788");
                //mDevices.Add(0x14E4169D,@"BCM5782k FB");
                //mDevices.Add(0x14E416A6,@"BCM5702X");
                //mDevices.Add(0x14E416A7,@"BCM5703X");
                //mDevices.Add(0x14E416A8,@"BCM5704");
                //mDevices.Add(0x14E416AA,@"B06BDRV");
                //mDevices.Add(0x14E416C6,@"BCM5702A3");
                //mDevices.Add(0x14E416C7,@"BCM 5703CKHB");
                //mDevices.Add(0x14E416DD,@"BCM5781");
                //mDevices.Add(0x14E416FD,@"BCM5753M");
                //mDevices.Add(0x14E416FE,@"BCM5753F");
                //mDevices.Add(0x14E4170C,@"BCM4401");
                //mDevices.Add(0x14E4170D,@"BCM5901");
                //mDevices.Add(0x14E4170E,@"BCM5901");
                //mDevices.Add(0x14E41713,@"BCM5906m");
                //mDevices.Add(0x14E43352,@"BCM3352");
                //mDevices.Add(0x14E43360,@"BCM3360");
                //mDevices.Add(0x14E44211,@"BCM HPNA");
                //mDevices.Add(0x14E44212,@"BCM V.90");
                //mDevices.Add(0x14E44301,@"BCM4301 802.11b");
                //mDevices.Add(0x14E44303,@"BCM4303");
                //mDevices.Add(0x14E44305,@"BCM4307");
                //mDevices.Add(0x14E44306,@"BCM4306");
                //mDevices.Add(0x14E44307,@"BCM4307");
                //mDevices.Add(0x14E44310,@"BCM4310");
                //mDevices.Add(0x14E44311,@"BCM4311");
                //mDevices.Add(0x14E44312,@"BCM4310");
                //mDevices.Add(0x14E44313,@"BCM4310");
                //mDevices.Add(0x14E44315,@"BCM4310");
                //mDevices.Add(0x14E44318,@"BCM4301");
                //mDevices.Add(0x14E44320,@"BCM94309");
                //mDevices.Add(0x14E44321,@"BCM4306");
                //mDevices.Add(0x14E44322,@"BCM4306");
                //mDevices.Add(0x14E44323,@"BCM4306");
                //mDevices.Add(0x14E44324,@"BCM4309");
                //mDevices.Add(0x14E44325,@"BCM4306");
                //mDevices.Add(0x14E44326,@"BCM4306");
                //mDevices.Add(0x14E44328,@"BCM94321KFBG");
                //mDevices.Add(0x14E44401,@"BCM4401");
                //mDevices.Add(0x14E44402,@"BCM4402");
                //mDevices.Add(0x14E44403,@"BCM4402");
                //mDevices.Add(0x14E44410,@"BCM4413");
                //mDevices.Add(0x14E44411,@"BCM4212");
                //mDevices.Add(0x14E44412,@"BCM4412");
                //mDevices.Add(0x14E44430,@"BCM44xx");
                //mDevices.Add(0x14E44432,@"BCM44xx");
                //mDevices.Add(0x14E44610,@"BCM4610");
                //mDevices.Add(0x14E44611,@"BCM4610");
                //mDevices.Add(0x14E44612,@"BCM4610");
                //mDevices.Add(0x14E44613,@"BCM4610");
                //mDevices.Add(0x14E44614,@"BCM4610");
                //mDevices.Add(0x14E44615,@"BCM4610");
                //mDevices.Add(0x14E44704,@"BCM4704");
                //mDevices.Add(0x14E44708,@"BCM4704");
                //mDevices.Add(0x14E44710,@"BCM4710");
                //mDevices.Add(0x14E44711,@"BCM47xx");
                //mDevices.Add(0x14E44712,@"BCM47xx");
                //mDevices.Add(0x14E44713,@"BCM47xx");
                //mDevices.Add(0x14E44714,@"BCM47xx");
                //mDevices.Add(0x14E44715,@"BCM47xx");
                //mDevices.Add(0x14E44716,@"BCM47xx");
                //mDevices.Add(0x14E44717,@"BCM47xx");
                //mDevices.Add(0x14E44718,@"BCM47xx");
                //mDevices.Add(0x14E44720,@"BCM4712");
                //mDevices.Add(0x14E45365,@"BCM5365P");
                //mDevices.Add(0x14E45600,@"BCM5600");
                //mDevices.Add(0x14E45605,@"BCM5605");
                //mDevices.Add(0x14E45615,@"BCM5615");
                //mDevices.Add(0x14E45625,@"BCM5625");
                //mDevices.Add(0x14E45645,@"BCM5645");
                //mDevices.Add(0x14E45670,@"BCM5670");
                //mDevices.Add(0x14E45680,@"BCM5680");
                //mDevices.Add(0x14E45690,@"BCM5690");
                //mDevices.Add(0x14E45691,@"BCM5691");
                //mDevices.Add(0x14E45802,@"BCM5802");
                //mDevices.Add(0x14E45805,@"BCM5805");
                //mDevices.Add(0x14E45820,@"BCM5820");
                //mDevices.Add(0x14E45821,@"BCM5821");
                //mDevices.Add(0x14E45822,@"BCM5822");
                //mDevices.Add(0x14E45823,@"BCM5823");
                //mDevices.Add(0x14E45824,@"BCM5824");
                //mDevices.Add(0x14E45840,@"BCM5840");
                //mDevices.Add(0x14E45841,@"BCM5841");
                //mDevices.Add(0x14E45850,@"BCM5850");
                //mDevices.Add(0x14E47321,@"BCM5751");
                //mDevices.Add(0x14E47411,@"BCM7411");
                //mDevices.Add(0x14EAAB06,@"FNW-3603-TX");
                //mDevices.Add(0x14EB0020,@"BEMx.x");
                //mDevices.Add(0x14EB0C01,@"S1D13808");
                //mDevices.Add(0x14F11035,@"unknown");
                //mDevices.Add(0x14F11059,@"DI15630-5, DI5631, DI5633");
                //mDevices.Add(0x14F11456,@"1456");
                //mDevices.Add(0x14F11611,@"AccessRunner ADSL Modem");
                //mDevices.Add(0x14F12400,@"unknown");
                //mDevices.Add(0x14F12702,@"cx11252-15");
                //mDevices.Add(0x14F12BFA,@"Unknown");
                //mDevices.Add(0x14F12C06,@"10250093");
                //mDevices.Add(0x14F12F00,@"00101767");
                //mDevices.Add(0x14F12F20,@"CX11252-11");
                //mDevices.Add(0x14F12F30,@"01");
                //mDevices.Add(0x14F12F40,@"71030277");
                //mDevices.Add(0x14F15045,@"4.0.3.1");
                //mDevices.Add(0x14F15051,@"4.0.1.6");
                //mDevices.Add(0x14F15B7A,@"Belived to be a CX23418");
                //mDevices.Add(0x14F18800,@"2003");
                //mDevices.Add(0x14F18852,@"0x7717");
                //mDevices.Add(0x14F20001,@"Moselle Split Bridge");
                //mDevices.Add(0x14F20002,@"Capilano Split Bridge");
                //mDevices.Add(0x14F20120,@"Merlin Split Bridge");
                //mDevices.Add(0x14F20121,@"PCI Parallel Port");
                //mDevices.Add(0x14F20122,@"PCI Serial Port");
                //mDevices.Add(0x14F20123,@"PCI PS/2 Keyboard Port");
                //mDevices.Add(0x14F20124,@"3103");
                //mDevices.Add(0x14FD0001,@"H260u");
                //mDevices.Add(0x15070001,@"MPC105 Eagle");
                //mDevices.Add(0x15070002,@"MPC106 Grackle");
                //mDevices.Add(0x15070003,@"MPC8240 Kahlua");
                //mDevices.Add(0x15070100,@"MPC145575 HFC-PCI");
                //mDevices.Add(0x15070431,@"KTI829c 100VG");
                //mDevices.Add(0x15074801,@"Raven");
                //mDevices.Add(0x15074802,@"Falcon");
                //mDevices.Add(0x15074803,@"Hawk");
                //mDevices.Add(0x15074806,@"CPX8216");
                //mDevices.Add(0x15160800,@"MTD800");
                //mDevices.Add(0x15160803,@"Myson MTD803/TAMARACK TC6020");
                //mDevices.Add(0x15160891,@"MTD891");
                //mDevices.Add(0x15192004,@"0x1");
                //mDevices.Add(0x151A1002,@"PCI-1002");
                //mDevices.Add(0x151A1004,@"PCI-1004");
                //mDevices.Add(0x151A1008,@"PCI-1008");
                //mDevices.Add(0x151B9080,@"me594v02100 combox");
                //mDevices.Add(0x151F0568,@"1.0.1.8");
                //mDevices.Add(0x15220100,@"PBridge+");
                //mDevices.Add(0x15240510,@"1.4.5.0");
                //mDevices.Add(0x15240530,@"CB-712/714/810");
                //mDevices.Add(0x15240550,@"CB-712/714/810");
                //mDevices.Add(0x15240610,@"???");
                //mDevices.Add(0x15241211,@"CB-1211");
                //mDevices.Add(0x15241225,@"CB-1225");
                //mDevices.Add(0x15241410,@"CB-1420");
                //mDevices.Add(0x15241411,@"CB-710/2/4/810");
                //mDevices.Add(0x15241412,@"CB-712/4");
                //mDevices.Add(0x15241420,@"CB-1420");
                //mDevices.Add(0x15241421,@"CB-720/2/4");
                //mDevices.Add(0x15241422,@"CB-722/4");
                //mDevices.Add(0x153B1115,@"ICE1712 Envy24");
                //mDevices.Add(0x153B1143,@"SAA7134HL");
                //mDevices.Add(0x153B6003,@"CS4614/22/24");
                //mDevices.Add(0x15409524,@"saa7134");
                //mDevices.Add(0x15433052,@"30201543");
                //mDevices.Add(0x15433155,@"Unknown");
                //mDevices.Add(0x154980FF,@"PCI-ISA-001");
                //mDevices.Add(0x15550002,@"PLX PCI 9050");
                //mDevices.Add(0x155E0020,@"MFC3");
                //mDevices.Add(0x15620001,@"LA-41x3");
                //mDevices.Add(0x15620002,@"LA-5030");
                //mDevices.Add(0x15620003,@"LA-5033");
                //mDevices.Add(0x156A5000,@"NA");
                //mDevices.Add(0x156A5100,@"NA");
                //mDevices.Add(0x1571A001,@"CCSI PCI20");
                //mDevices.Add(0x1571A002,@"CCSI PCI20-485D");
                //mDevices.Add(0x1571A003,@"CCSI PCI20-485X");
                //mDevices.Add(0x1571A004,@"CCSI PCI20-CXB");
                //mDevices.Add(0x1571A005,@"CCSI PCI20-CXS");
                //mDevices.Add(0x1571A006,@"CCSI PCI20-FOG-SMA");
                //mDevices.Add(0x1571A007,@"CCSI PCI20-FOG-ST");
                //mDevices.Add(0x1571A008,@"CCSI PCI20-TB5");
                //mDevices.Add(0x1571A009,@"CCSI PCI20-5-485");
                //mDevices.Add(0x1571A00A,@"CCSI PCI20-5-485D");
                //mDevices.Add(0x1571A00B,@"CCSI PCI20-5-485X");
                //mDevices.Add(0x1571A00C,@"CCSI PIC20-5-FOG-ST");
                //mDevices.Add(0x1571A00D,@"CCSI PCI20-5-FOG-SMA");
                //mDevices.Add(0x1571A00E,@"COM200C22");
                //mDevices.Add(0x1571A201,@"CCSI PCI22-485");
                //mDevices.Add(0x1571A202,@"CCSI PCI22-485D");
                //mDevices.Add(0x1571A203,@"CCSI PCI22-485X");
                //mDevices.Add(0x1571A204,@"CCSI PCI22-CHB");
                //mDevices.Add(0x1571A205,@"CCSI PCI22-FOG-ST");
                //mDevices.Add(0x1571A206,@"CCSI PCI22-THB");
                //mDevices.Add(0x157C8001,@"Fix2000");
                //mDevices.Add(0x15881100,@"PAX.port 1100");
                //mDevices.Add(0x15882000,@"AMD '971");
                //mDevices.Add(0x15888023,@"SNP8023");
                //mDevices.Add(0x158B0015,@"HLF-VMPEU560-C");
                //mDevices.Add(0x15920781,@"W83787IF");
                //mDevices.Add(0x15920782,@"W837871F");
                //mDevices.Add(0x15920783,@"TC32001 PCI I/O");
                //mDevices.Add(0x15920785,@"907ab184740802");
                //mDevices.Add(0x15920786,@"Winbond w83787if");
                //mDevices.Add(0x15920787,@"winbond w83787if");
                //mDevices.Add(0x15920788,@"w837871f");
                //mDevices.Add(0x1592078A,@"007AB192367002ND");
                //mDevices.Add(0x15A20001,@"TA700");
                //mDevices.Add(0x15AD0405,@"VGA 4.0.5");
                //mDevices.Add(0x15AD0710,@"Virtual SVGA");
                //mDevices.Add(0x15AD0720,@"VMXNET");
                //mDevices.Add(0x15AD0770,@"n/a");
                //mDevices.Add(0x15AD0801,@"n/a");
                //mDevices.Add(0x15B00001,@"FM-1789");
                //mDevices.Add(0x15B02BD0,@"2BD0");
                //mDevices.Add(0x15B35274,@"MT21108");
                //mDevices.Add(0x15B36278,@"MT25208A0-FCC");
                //mDevices.Add(0x15B83009,@"xPCI-3504");
                //mDevices.Add(0x15BC0101,@"n2530a");
                //mDevices.Add(0x15BC0103,@"QX4");
                //mDevices.Add(0x15BC2530,@"???");
                //mDevices.Add(0x15BC2531,@"???");
                //mDevices.Add(0x15BC2532,@"???");
                //mDevices.Add(0x15BC2929,@"E2929A");
                //mDevices.Add(0x15D10001,@"TC11IB");
                //mDevices.Add(0x15D10003,@"PEB 20544 E v1.1");
                //mDevices.Add(0x15D10004,@"PEB 3454 E v1.1");
                //mDevices.Add(0x15DC0001,@"Argus 300");
                //mDevices.Add(0x15E20500,@"Internet PhoneJack PCI Card");
                //mDevices.Add(0x15E80130,@"NCP130");
                //mDevices.Add(0x15E80131,@"Prism II");
                //mDevices.Add(0x15E91841,@"NetStaQ ADMA-100");
                //mDevices.Add(0x15F12F30,@"CX11252-15");
                //mDevices.Add(0x15F20001,@"Spot RT");
                //mDevices.Add(0x15F20002,@"Spot RT #2");
                //mDevices.Add(0x15F20003,@"Spot Insight");
                //mDevices.Add(0x16190400,@"FarSync T2P");
                //mDevices.Add(0x16190440,@"FarSync T4P");
                //mDevices.Add(0x16190610,@"FarSync T1U");
                //mDevices.Add(0x16190620,@"FarSync T2U");
                //mDevices.Add(0x16190640,@"FarSync T4U");
                //mDevices.Add(0x16191610,@"FarSync TE1");
                //mDevices.Add(0x16191612,@"FarSync TE1C");
                //mDevices.Add(0x16192610,@"FarSync DSL-S1");
                //mDevices.Add(0x16193640,@"FarSync T4E");
                //mDevices.Add(0x16194640,@"FarSync T4Ue (PCI Express)");
                //mDevices.Add(0x16210020,@"LynxTWO-A");
                //mDevices.Add(0x16210021,@"LynxTWO-B");
                //mDevices.Add(0x16210022,@"LynxTWO-C");
                //mDevices.Add(0x16210023,@"Lynx L22");
                //mDevices.Add(0x16210024,@"Lynx AES16");
                //mDevices.Add(0x16210025,@"Lynx AES16-SRC");
                //mDevices.Add(0x16291003,@"Format Synchronizer v3.0");
                //mDevices.Add(0x16292002,@"Fast Universal Data Output");
                //mDevices.Add(0x162D0100,@"Repeographics controller");
                //mDevices.Add(0x162D0101,@"Reprographics Controller");
                //mDevices.Add(0x162D0102,@"Reprographics Controller");
                //mDevices.Add(0x162D0103,@"Reprographics Controller");
                //mDevices.Add(0x162F1111,@"TS-PRL1");
                //mDevices.Add(0x162F1112,@"TS-PMA");
                //mDevices.Add(0x16381100,@"WL11000P");
                //mDevices.Add(0x163C3052,@"R6793-11");
                //mDevices.Add(0x163CFF02,@"SL2800 PCI");
                //mDevices.Add(0x164F0001,@"PLX 9054");
                //mDevices.Add(0x164F0002,@"PLX 9054");
                //mDevices.Add(0x16570646,@"BRE040");
                //mDevices.Add(0x16580704,@"905410B5");
                //mDevices.Add(0x165AC100,@"PIXCI CL1");
                //mDevices.Add(0x165AD200,@"PIXCI D2X");
                //mDevices.Add(0x165AD300,@"PIXCI D3X");
                //mDevices.Add(0x165C0002,@"FT232BL");
                //mDevices.Add(0x165F2000,@"XILINX SPARTAN");
                //mDevices.Add(0x16680100,@"0304");
                //mDevices.Add(0x166D0001,@"BCM1125/BCM1250 PCI Configuration Header");
                //mDevices.Add(0x166D0002,@"BCM1125/BCM1250 HyperTransport Configuration Heade");
                //mDevices.Add(0x167F4634,@"FOB-IO Card");
                //mDevices.Add(0x167F4C32,@"L2B PCI Board");
                //mDevices.Add(0x167F5344,@"FOB-SD Card");
                //mDevices.Add(0x167F5443,@"FOB-TDC Card");
                //mDevices.Add(0x16810050,@"HWGPCI-54");
                //mDevices.Add(0x168C0007,@"AR5005");
                //mDevices.Add(0x168C0011,@"AR5bmb5");
                //mDevices.Add(0x168C0012,@"AR5211");
                //mDevices.Add(0x168C0013,@"AR5212, AR5213");
                //mDevices.Add(0x168C001A,@"Atheros AR5005G");
                //mDevices.Add(0x168C001B,@"AR5006X");
                //mDevices.Add(0x168C0024,@"AR5008");
                //mDevices.Add(0x168C1014,@"AR5212");
                //mDevices.Add(0x168CFF96,@"AR5212");
                //mDevices.Add(0x16930212,@"PLX PCI9054");
                //mDevices.Add(0x16930213,@"Motorola MPC8245");
                //mDevices.Add(0x16AE1141,@"SafeXcel-1141");
                //mDevices.Add(0x16CA0001,@"Rocket Drive");
                //mDevices.Add(0x16EC0116,@"RTL8169S");
                //mDevices.Add(0x16EC1007,@"0637");
                //mDevices.Add(0x16EC2013,@"11323A");
                //mDevices.Add(0x16EC2F00,@"USRobotics 5660A - Internal Soft Modem");
                //mDevices.Add(0x16EC3685,@"???");
                //mDevices.Add(0x16EC5685,@"E129336-1");
                //mDevices.Add(0x170B0100,@"NSP2000-SSL");
                //mDevices.Add(0x17257174,@"VSC7174");
                //mDevices.Add(0x17341011,@"AIC-7902W");
                //mDevices.Add(0x17341012,@"CSB6");
                //mDevices.Add(0x17341013,@"BCM5703");
                //mDevices.Add(0x173B03E8,@"AC1000");
                //mDevices.Add(0x173B03EA,@"AC1002");
                //mDevices.Add(0x17438139,@"ROL/F-100");
                //mDevices.Add(0x174B0260,@"RV280");
                //mDevices.Add(0x174B0261,@"RV280");
                //mDevices.Add(0x174B7176,@"RV250");
                //mDevices.Add(0x174B7177,@"RV280");
                //mDevices.Add(0x174B7C12,@"RV280");
                //mDevices.Add(0x174B7C13,@"RV280");
                //mDevices.Add(0x17531001,@"VP500");
                //mDevices.Add(0x17531004,@"VP1000");
                //mDevices.Add(0x17550000,@"Au1500");
                //mDevices.Add(0x17AF4150,@"200");
                //mDevices.Add(0x17CC2280,@"Net 2280");
                //mDevices.Add(0x17D55831,@"X1");
                //mDevices.Add(0x17D55832,@"X2");
                //mDevices.Add(0x17D55833,@"X3");
                //mDevices.Add(0x17EE4153,@"RV350");
                //mDevices.Add(0x18133059,@"VT8237");
                //mDevices.Add(0x18134000,@"MD5628D-L-A");
                //mDevices.Add(0x18134100,@"Ambient MD8820");
                //mDevices.Add(0x18140101,@"2460  802.11b");
                //mDevices.Add(0x18140201,@"0x03011814");
                //mDevices.Add(0x18140301,@"b8341462");
                //mDevices.Add(0x18140302,@"RT2525 2.4GHz transceiver + RT2560 MAC/BBP");
                //mDevices.Add(0x18882503,@"Bt881");
                //mDevices.Add(0x18882504,@"Bt878");
                //mDevices.Add(0x18883503,@"nVidia NV28");
                //mDevices.Add(0x18883505,@"nVidia NV28");
                //mDevices.Add(0x18C91011,@"Leonardo CL");
                //mDevices.Add(0x18C91012,@"Leonardo CL-P");
                //mDevices.Add(0x18C91013,@"Leonardo CL-DB");
                //mDevices.Add(0x18C91014,@"Leonardo CL-P-DB");
                //mDevices.Add(0x18C91015,@"Leonardo CL-DVR");
                //mDevices.Add(0x18C91016,@"Leonardo CL-DVR-DB");
                //mDevices.Add(0x18C92011,@"Picasso 2SQ");
                //mDevices.Add(0x18C92012,@"Picasso 3C/3Cpro");
                //mDevices.Add(0x18C92013,@"Picasso LS");
                //mDevices.Add(0x18C92014,@"Picasso CL");
                //mDevices.Add(0x18C92015,@"Picasso FI");
                //mDevices.Add(0x18C92016,@"Picasso SDI");
                //mDevices.Add(0x18C92017,@"Picasso DUO");
                //mDevices.Add(0x18C92021,@"Colory");
                //mDevices.Add(0x18C93011,@"Valentino");
                //mDevices.Add(0x18CA0040,@"8085");
                //mDevices.Add(0x18F70001,@"ESCC-PCI-335");
                //mDevices.Add(0x18F70002,@"422/4-PCI-335");
                //mDevices.Add(0x18F70004,@"422/2-PCI-335");
                //mDevices.Add(0x19100001,@"SW5000-NCA");
                //mDevices.Add(0x19710001,@"AGEIA PhysX 100 Series PCI Express Card");
                //mDevices.Add(0x19711011,@"AGEIA PhysX 100 Series PCI Card");
                //mDevices.Add(0x19711021,@"AGEIA PhysX 200 Series PCI Express Card");
                //mDevices.Add(0x19AC0001,@"ACA2400");
                //mDevices.Add(0x1B130001,@"GeForce4 MX 440");
                //mDevices.Add(0x1DE10391,@"TRM-S1040");
                //mDevices.Add(0x1DE12020,@"DC-390");
                //mDevices.Add(0x1DE1690C,@"DC-690C");
                //mDevices.Add(0x1DE1DC29,@"DC290M");
                //mDevices.Add(0x33880020,@"HB2/HB6");
                //mDevices.Add(0x33880021,@"HB1/1-SE/HB6");
                //mDevices.Add(0x33880022,@"HB4");
                //mDevices.Add(0x33880028,@"HB8");
                //mDevices.Add(0x33880029,@"HB8");
                //mDevices.Add(0x33880030,@"HB7");
                //mDevices.Add(0x33880031,@"PCI 6156");
                //mDevices.Add(0x33888011,@"II");
                //mDevices.Add(0x33888012,@"cmi 8330");
                //mDevices.Add(0x33888013,@"PLX Registration Record Management");
                //mDevices.Add(0x3D3D0001,@"GLint 300SX");
                //mDevices.Add(0x3D3D0002,@"GLint 500TX");
                //mDevices.Add(0x3D3D0003,@"GLint");
                //mDevices.Add(0x3D3D0004,@"3C0SX");
                //mDevices.Add(0x3D3D0005,@"Permedia");
                //mDevices.Add(0x3D3D0006,@"GLint MX");
                //mDevices.Add(0x3D3D0007,@"3D Extreme");
                //mDevices.Add(0x3D3D0008,@"GLint Gamma G1");
                //mDevices.Add(0x3D3D0009,@"Permedia2v");
                //mDevices.Add(0x3D3D000A,@"Permedia 3");
                //mDevices.Add(0x3D3D000C,@"Permedia 3");
                //mDevices.Add(0x3D3D000D,@"GLINT R4");
                //mDevices.Add(0x3D3D000E,@"GLINT Gamma G2");
                //mDevices.Add(0x3D3D0030,@"0x030000");
                //mDevices.Add(0x3D3D0100,@"Permedia II");
                //mDevices.Add(0x3D3D1004,@"Permedia");
                //mDevices.Add(0x3D3D3D04,@"Permedia");
                //mDevices.Add(0x3D3DFFFF,@"GLint VGA");
                //mDevices.Add(0x40050300,@"3220");
                //mDevices.Add(0x40050308,@"3220");
                //mDevices.Add(0x40050309,@"ALS300+");
                //mDevices.Add(0x40051064,@"ALG2064");
                //mDevices.Add(0x40052064,@"<DELETE>");
                //mDevices.Add(0x40052128,@"ALG2364A");
                //mDevices.Add(0x40052301,@"ALG2301");
                //mDevices.Add(0x40052302,@"ALG2302");
                //mDevices.Add(0x40052303,@"AVG2302");
                //mDevices.Add(0x40052364,@"ALG2364");
                //mDevices.Add(0x40052464,@"ALG2464");
                //mDevices.Add(0x40052501,@"ALG2564A/25128A");
                //mDevices.Add(0x40054000,@"ALS4000");
                //mDevices.Add(0x41440040,@"ADM-XRC");
                //mDevices.Add(0x41440041,@"ADM-XRC-II Lite");
                //mDevices.Add(0x41440042,@"ADM-XRC-II");
                //mDevices.Add(0x41440043,@"ADM-XPL");
                //mDevices.Add(0x41440044,@"ADM-XP");
                //mDevices.Add(0x41440045,@"ADP-WRC-II");
                //mDevices.Add(0x41440046,@"ADP-DRC-II");
                //mDevices.Add(0x41440049,@"ADP-XPI");
                //mDevices.Add(0x4144004A,@"ADP-XPI (PCI-X)");
                //mDevices.Add(0x4144004F,@"ADM-XRC-4FX");
                //mDevices.Add(0x41440050,@"ADM-XRC-5LX");
                //mDevices.Add(0x41440051,@"ADM-XRC-5T1");
                //mDevices.Add(0x416C0100,@"AlladinCARD");
                //mDevices.Add(0x416C0200,@"CPC");
                //mDevices.Add(0x4C530000,@"PLUSTEST");
                //mDevices.Add(0x4D510200,@"MQ-200");
                //mDevices.Add(0x50532010,@"Daytona Audio Adapter");
                //mDevices.Add(0x53330551,@"86C551");
                //mDevices.Add(0x53335333,@"S3 86c765");
                //mDevices.Add(0x53335631,@"86C325");
                //mDevices.Add(0x53338800,@"86C866");
                //mDevices.Add(0x53338801,@"86C964");
                //mDevices.Add(0x53338810,@"86C732");
                //mDevices.Add(0x53338811,@"8622mcq04");
                //mDevices.Add(0x53338812,@"86CM65?");
                //mDevices.Add(0x53338813,@"86C764");
                //mDevices.Add(0x53338814,@"86C767");
                //mDevices.Add(0x53338815,@"86CM66");
                //mDevices.Add(0x5333883D,@"86C988");
                //mDevices.Add(0x53338870,@"Fire GL");
                //mDevices.Add(0x53338880,@"86C868");
                //mDevices.Add(0x53338881,@"86C868");
                //mDevices.Add(0x53338882,@"86C868");
                //mDevices.Add(0x53338883,@"86C868");
                //mDevices.Add(0x533388B0,@"86C928");
                //mDevices.Add(0x533388B1,@"86C928");
                //mDevices.Add(0x533388B2,@"86C928");
                //mDevices.Add(0x533388B3,@"86C928");
                //mDevices.Add(0x533388C0,@"86C864");
                //mDevices.Add(0x533388C1,@"86C864");
                //mDevices.Add(0x533388C2,@"86C864");
                //mDevices.Add(0x533388C3,@"86C864");
                //mDevices.Add(0x533388D0,@"86C964");
                //mDevices.Add(0x533388D1,@"86C964");
                //mDevices.Add(0x533388D2,@"86C964");
                //mDevices.Add(0x533388D3,@"86C964");
                //mDevices.Add(0x533388F0,@"86C968");
                //mDevices.Add(0x533388F1,@"86C968");
                //mDevices.Add(0x533388F2,@"86C968");
                //mDevices.Add(0x533388F3,@"86C968");
                //mDevices.Add(0x53338900,@"86C775");
                //mDevices.Add(0x53338901,@"pci\ven_5333dev_8C2E&SUBSYS_00011179&REV_05\4&74C6");
                //mDevices.Add(0x53338902,@"86C551");
                //mDevices.Add(0x53338903,@"TrioV Family");
                //mDevices.Add(0x53338904,@"86C365/366");
                //mDevices.Add(0x53338905,@"86c765");
                //mDevices.Add(0x53338906,@"86c765");
                //mDevices.Add(0x53338907,@"86c765");
                //mDevices.Add(0x53338908,@"9711 MCN74");
                //mDevices.Add(0x53338909,@"7699688");
                //mDevices.Add(0x5333890A,@"0x00091011");
                //mDevices.Add(0x5333890B,@"9726 c19394.00");
                //mDevices.Add(0x5333890C,@"86C765");
                //mDevices.Add(0x5333890D,@"86C765 Trio64V+ compatible");
                //mDevices.Add(0x5333890E,@"9711 MCN74");
                //mDevices.Add(0x5333890F,@"86c765");
                //mDevices.Add(0x53338A01,@"86C375/86C385");
                //mDevices.Add(0x53338A10,@"86C357");
                //mDevices.Add(0x53338A11,@"86C359");
                //mDevices.Add(0x53338A12,@"86C359");
                //mDevices.Add(0x53338A13,@"86C368");
                //mDevices.Add(0x53338A20,@"86C391");
                //mDevices.Add(0x53338A21,@"86C390");
                //mDevices.Add(0x53338A22,@"86c398");
                //mDevices.Add(0x53338A23,@"86C394-397");
                //mDevices.Add(0x53338A25,@"86C370");
                //mDevices.Add(0x53338A26,@"86C395B");
                //mDevices.Add(0x53338C00,@"85C260");
                //mDevices.Add(0x53338C01,@"86C260");
                //mDevices.Add(0x53338C02,@"86C240");
                //mDevices.Add(0x53338C03,@"86C280 db");
                //mDevices.Add(0x53338C10,@"86C270/274/290/294");
                //mDevices.Add(0x53338C12,@"86C270/274/290/294");
                //mDevices.Add(0x53338C13,@"82C294");
                //mDevices.Add(0x53338C22,@"86C508");
                //mDevices.Add(0x53338C2A,@"86C544");
                //mDevices.Add(0x53338C2B,@"86C553");
                //mDevices.Add(0x53338C2C,@"86C564");
                //mDevices.Add(0x53338C2D,@"86C573");
                //mDevices.Add(0x53338C2E,@"86C584");
                //mDevices.Add(0x53338C2F,@"86C594");
                //mDevices.Add(0x53338D01,@"86C380/381");
                //mDevices.Add(0x53338D02,@"86c387");
                //mDevices.Add(0x53338D04,@"86C410");
                //mDevices.Add(0x53338E00,@"86C777/787");
                //mDevices.Add(0x53338E01,@"86C732");
                //mDevices.Add(0x53339102,@"86c410");
                //mDevices.Add(0x5333CA00,@"86C617");
                //mDevices.Add(0x55550003,@"VPI General Digital Video Component");
                //mDevices.Add(0x66660001,@"815");
                //mDevices.Add(0x66660002,@"PCCOM8");
                //mDevices.Add(0x80860008,@"Extended Express System Support Ctrlr");
                //mDevices.Add(0x80860309,@"80303");
                //mDevices.Add(0x8086030D,@"80312");
                //mDevices.Add(0x80860318,@"80219");
                //mDevices.Add(0x80860319,@"80219");
                //mDevices.Add(0x80860326,@"670xPXH");
                //mDevices.Add(0x80860327,@"6700PXH");
                //mDevices.Add(0x80860329,@"6700PXH");
                //mDevices.Add(0x8086032A,@"6700PXH");
                //mDevices.Add(0x8086032C,@"6702PXH");
                //mDevices.Add(0x80860330,@"80332");
                //mDevices.Add(0x80860331,@"80332");
                //mDevices.Add(0x80860332,@"80332");
                //mDevices.Add(0x80860333,@"80332");
                //mDevices.Add(0x80860334,@"80332");
                //mDevices.Add(0x80860335,@"80331");
                //mDevices.Add(0x80860336,@"80331");
                //mDevices.Add(0x80860340,@"41210");
                //mDevices.Add(0x80860341,@"41210");
                //mDevices.Add(0x80860370,@"80333");
                //mDevices.Add(0x80860371,@"80333");
                //mDevices.Add(0x80860372,@"80333");
                //mDevices.Add(0x80860373,@"80333");
                //mDevices.Add(0x80860374,@"80333");
                //mDevices.Add(0x80860482,@"82375MB");
                //mDevices.Add(0x80860483,@"82424TX/ZX");
                //mDevices.Add(0x80860484,@"82378ZB/IB");
                //mDevices.Add(0x80860486,@"82425EX");
                //mDevices.Add(0x808604A3,@"82434bX/zX");
                //mDevices.Add(0x80860500,@"E8870");
                //mDevices.Add(0x80860501,@"E8870");
                //mDevices.Add(0x80860502,@"E8870");
                //mDevices.Add(0x80860503,@"E8870");
                //mDevices.Add(0x80860510,@"E8870IO");
                //mDevices.Add(0x80860511,@"E8870IO");
                //mDevices.Add(0x80860512,@"E8870IO");
                //mDevices.Add(0x80860513,@"E8870IO");
                //mDevices.Add(0x80860514,@"E8870IO");
                //mDevices.Add(0x80860515,@"E8870IO");
                //mDevices.Add(0x80860516,@"E8870IO");
                //mDevices.Add(0x80860530,@"E8870SP");
                //mDevices.Add(0x80860531,@"E8870SP");
                //mDevices.Add(0x80860532,@"E8870SP");
                //mDevices.Add(0x80860533,@"E8870SP");
                //mDevices.Add(0x80860534,@"E8870SP");
                //mDevices.Add(0x80860535,@"E8870SP");
                //mDevices.Add(0x80860536,@"E8870SP");
                //mDevices.Add(0x80860537,@"E8870SP");
                //mDevices.Add(0x80860600,@"01af8086");
                //mDevices.Add(0x80860960,@"80960RP");
                //mDevices.Add(0x80860962,@"80960RM/RN");
                //mDevices.Add(0x80860964,@"80960RP");
                //mDevices.Add(0x80861000,@"82542");
                //mDevices.Add(0x80861001,@"2572");
                //mDevices.Add(0x80861002,@"Pro 100 LAN+Modem 56 CardBus II");
                //mDevices.Add(0x80861004,@"82543GC");
                //mDevices.Add(0x80861008,@"82544EI/GC");
                //mDevices.Add(0x80861009,@"82544EI");
                //mDevices.Add(0x8086100C,@"82543EI/GC");
                //mDevices.Add(0x8086100D,@"82544GC");
                //mDevices.Add(0x8086100E,@"82540EM");
                //mDevices.Add(0x8086100F,@"82545EM");
                //mDevices.Add(0x80861010,@"82546EB");
                //mDevices.Add(0x80861011,@"82545EM");
                //mDevices.Add(0x80861012,@"82546EB");
                //mDevices.Add(0x80861013,@"82541EI");
                //mDevices.Add(0x80861014,@"82541ER");
                //mDevices.Add(0x80861015,@"82540EM");
                //mDevices.Add(0x80861016,@"82540EP");
                //mDevices.Add(0x80861017,@"82540EP");
                //mDevices.Add(0x80861018,@"82541EI");
                //mDevices.Add(0x80861019,@"82547EI");
                //mDevices.Add(0x8086101A,@"82547EI");
                //mDevices.Add(0x8086101E,@"82540EP");
                //mDevices.Add(0x80861026,@"82545ep");
                //mDevices.Add(0x80861027,@"82545GM");
                //mDevices.Add(0x80861028,@"82545GM");
                //mDevices.Add(0x80861029,@"82559");
                //mDevices.Add(0x80861030,@"825593");
                //mDevices.Add(0x80861031,@"82801CAM");
                //mDevices.Add(0x80861032,@"82801CAM");
                //mDevices.Add(0x80861033,@"82801CAM");
                //mDevices.Add(0x80861034,@"82801CAM");
                //mDevices.Add(0x80861035,@"82562EH");
                //mDevices.Add(0x80861036,@"82562EH");
                //mDevices.Add(0x80861037,@"82801CAM");
                //mDevices.Add(0x80861038,@"82801CAM");
                //mDevices.Add(0x80861039,@"10011734");
                //mDevices.Add(0x8086103A,@"82801DB");
                //mDevices.Add(0x8086103B,@"82801DB");
                //mDevices.Add(0x8086103C,@"82801DB");
                //mDevices.Add(0x8086103D,@"82801DB");
                //mDevices.Add(0x8086103E,@"82801DB");
                //mDevices.Add(0x80861040,@"536EP");
                //mDevices.Add(0x80861042,@"PRO/Wireless 2011 LAN PCI Card");
                //mDevices.Add(0x80861043,@"82801");
                //mDevices.Add(0x80861048,@"82597EX");
                //mDevices.Add(0x80861049,@"82566MM NIC");
                //mDevices.Add(0x8086104A,@"82566DM");
                //mDevices.Add(0x80861050,@"82801EB/ER");
                //mDevices.Add(0x80861051,@"82801EB/ER");
                //mDevices.Add(0x80861052,@"82801EB/ER");
                //mDevices.Add(0x80861053,@"82801EB/ER");
                //mDevices.Add(0x80861054,@"82801EB/ER");
                //mDevices.Add(0x80861055,@"82801EB/ER");
                //mDevices.Add(0x80861059,@"82551QM");
                //mDevices.Add(0x80861064,@"Intel Pro VE");
                //mDevices.Add(0x80861065,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x80861075,@"82547EI");
                //mDevices.Add(0x80861076,@"82541EI");
                //mDevices.Add(0x80861077,@"82547EI");
                //mDevices.Add(0x80861078,@"82541ER");
                //mDevices.Add(0x80861079,@"82546EB");
                //mDevices.Add(0x8086107A,@"82546EB");
                //mDevices.Add(0x8086107B,@"82546EB");
                //mDevices.Add(0x8086107C,@"82541PI");
                //mDevices.Add(0x80861080,@"0321CGEA04");
                //mDevices.Add(0x8086108B,@"PC82573V");
                //mDevices.Add(0x80861092,@"27DA");
                //mDevices.Add(0x8086109A,@"82573L");
                //mDevices.Add(0x808610BD,@"82566DM");
                //mDevices.Add(0x80861100,@"82815/EP/P");
                //mDevices.Add(0x80861101,@"82815/EP/P");
                //mDevices.Add(0x80861102,@"82815");
                //mDevices.Add(0x80861110,@"82815");
                //mDevices.Add(0x80861112,@"82815");
                //mDevices.Add(0x80861120,@"82815");
                //mDevices.Add(0x80861121,@"82815");
                //mDevices.Add(0x80861130,@"82815/82815EM/EP");
                //mDevices.Add(0x80861131,@"82815/82815EM/EP");
                //mDevices.Add(0x80861132,@"65416");
                //mDevices.Add(0x80861161,@"82806AA");
                //mDevices.Add(0x80861162,@"BECC");
                //mDevices.Add(0x80861179,@"82546EB");
                //mDevices.Add(0x80861200,@"IXP1200");
                //mDevices.Add(0x80861209,@"8255xER/IT");
                //mDevices.Add(0x80861221,@"82092AA");
                //mDevices.Add(0x80861222,@"82092AA");
                //mDevices.Add(0x80861223,@"SAA 7116 H");
                //mDevices.Add(0x80861225,@"82452KX/GX");
                //mDevices.Add(0x80861226,@"82596");
                //mDevices.Add(0x80861227,@"82865g");
                //mDevices.Add(0x80861228,@"EE PRO/100 Smart");
                //mDevices.Add(0x80861229,@"82557/8/9/0/1");
                //mDevices.Add(0x8086122D,@"82437FX");
                //mDevices.Add(0x8086122E,@"82371FB");
                //mDevices.Add(0x80861230,@"FW82371AB");
                //mDevices.Add(0x80861231,@"16345");
                //mDevices.Add(0x80861234,@"82371MX");
                //mDevices.Add(0x80861235,@"82439TX");
                //mDevices.Add(0x80861237,@"82440LX/EX");
                //mDevices.Add(0x80861239,@"82371FB");
                //mDevices.Add(0x8086123B,@"82380PB");
                //mDevices.Add(0x8086123C,@"82380AB");
                //mDevices.Add(0x8086123D,@"683053");
                //mDevices.Add(0x8086123E,@"82466GX");
                //mDevices.Add(0x8086123F,@"82466GX");
                //mDevices.Add(0x80861240,@"82752");
                //mDevices.Add(0x8086124B,@"4229");
                //mDevices.Add(0x80861250,@"82430HX");
                //mDevices.Add(0x808612D8,@"92XX");
                //mDevices.Add(0x80861360,@"82806AA");
                //mDevices.Add(0x80861361,@"82806AA");
                //mDevices.Add(0x80861460,@"82870P2");
                //mDevices.Add(0x80861461,@"14611014");
                //mDevices.Add(0x80861462,@"82870P2");
                //mDevices.Add(0x80861960,@"80960RP");
                //mDevices.Add(0x80861962,@"???");
                //mDevices.Add(0x80861A12,@"???");
                //mDevices.Add(0x80861A13,@"???");
                //mDevices.Add(0x80861A20,@"82840");
                //mDevices.Add(0x80861A21,@"82840");
                //mDevices.Add(0x80861A22,@"82840");
                //mDevices.Add(0x80861A23,@"82840");
                //mDevices.Add(0x80861A24,@"82840");
                //mDevices.Add(0x80861A30,@"82845G[GL/GV/GE/PE]");
                //mDevices.Add(0x80861A31,@"82845[MP/MZ]");
                //mDevices.Add(0x80861A38,@"5000P");
                //mDevices.Add(0x80862125,@"82801AA");
                //mDevices.Add(0x80862410,@"82801AA");
                //mDevices.Add(0x80862411,@"82801AA");
                //mDevices.Add(0x80862412,@"82801AA");
                //mDevices.Add(0x80862413,@"82801AA");
                //mDevices.Add(0x80862415,@"Intel 82801DBM SM/BUS Controller 24C3");
                //mDevices.Add(0x80862416,@"82801fb");
                //mDevices.Add(0x80862418,@"82801AA");
                //mDevices.Add(0x80862420,@"82801AB");
                //mDevices.Add(0x80862421,@"82801AB");
                //mDevices.Add(0x80862422,@"82801AB");
                //mDevices.Add(0x80862423,@"82801AB");
                //mDevices.Add(0x80862425,@"82845G");
                //mDevices.Add(0x80862426,@"82801AB");
                //mDevices.Add(0x80862428,@"82801AB");
                //mDevices.Add(0x80862431,@"82810");
                //mDevices.Add(0x80862440,@"82801BA");
                //mDevices.Add(0x80862441,@"82801BA");
                //mDevices.Add(0x80862442,@"82801BA/BAM");
                //mDevices.Add(0x80862443,@"0055110A");
                //mDevices.Add(0x80862444,@"82801BA/BAM");
                //mDevices.Add(0x80862445,@"82801E");
                //mDevices.Add(0x80862446,@"82801BA/BAM");
                //mDevices.Add(0x80862448,@"82801BAM/CAM/DBM");
                //mDevices.Add(0x80862449,@"82559ER");
                //mDevices.Add(0x8086244A,@"82801BAM");
                //mDevices.Add(0x8086244B,@"82801BA");
                //mDevices.Add(0x8086244C,@"82801BAM");
                //mDevices.Add(0x8086244E,@"82801DB");
                //mDevices.Add(0x80862450,@"82801E");
                //mDevices.Add(0x80862452,@"82801E");
                //mDevices.Add(0x80862453,@"82801E");
                //mDevices.Add(0x80862459,@"82801E");
                //mDevices.Add(0x8086245B,@"82801E");
                //mDevices.Add(0x8086245D,@"82801E");
                //mDevices.Add(0x8086245E,@"82801E");
                //mDevices.Add(0x80862480,@"82801CA");
                //mDevices.Add(0x80862481,@"82801CA");
                //mDevices.Add(0x80862482,@"82801CA/CAM");
                //mDevices.Add(0x80862483,@"82801CA/CAM");
                //mDevices.Add(0x80862484,@"82801CA/CAM");
                //mDevices.Add(0x80862485,@"82801CA/CAM");
                //mDevices.Add(0x80862486,@"82801CA/CAM");
                //mDevices.Add(0x80862487,@"82801CA/CAM");
                //mDevices.Add(0x8086248A,@"82801CAM");
                //mDevices.Add(0x8086248B,@"82801CA");
                //mDevices.Add(0x8086248C,@"82801CAM");
                //mDevices.Add(0x8086248D,@"82801??");
                //mDevices.Add(0x808624C0,@"82801DB/DBL");
                //mDevices.Add(0x808624C2,@"82801DB/DBL/DBM");
                //mDevices.Add(0x808624C3,@"82801DB/DBL/DBM");
                //mDevices.Add(0x808624C4,@"82801DB/DBL/DBM");
                //mDevices.Add(0x808624C5,@"Realtek ALC268 ");
                //mDevices.Add(0x808624C6,@"82801");
                //mDevices.Add(0x808624C7,@"82801DB/DBL/DBM");
                //mDevices.Add(0x808624CA,@"82801DBM");
                //mDevices.Add(0x808624CB,@"82801DB/DBL");
                //mDevices.Add(0x808624CC,@"82801DBM");
                //mDevices.Add(0x808624CD,@"82801DB/DBL/DBM");
                //mDevices.Add(0x808624D0,@"82801EB/ER");
                //mDevices.Add(0x808624D1,@"82801EB/ER");
                //mDevices.Add(0x808624D2,@"82801EB/ER");
                //mDevices.Add(0x808624D3,@"82801EB/ER");
                //mDevices.Add(0x808624D4,@"82801EB/ER");
                //mDevices.Add(0x808624D5,@"815B104D");
                //mDevices.Add(0x808624D6,@"82801EB/ER");
                //mDevices.Add(0x808624D7,@"82801EB/ER");
                //mDevices.Add(0x808624DB,@"82801EB/ER");
                //mDevices.Add(0x808624DC,@"82801EB");
                //mDevices.Add(0x808624DD,@"82801EB/ER");
                //mDevices.Add(0x808624DE,@"82801EB/ER");
                //mDevices.Add(0x808624DF,@"82801ER");
                //mDevices.Add(0x80862500,@"82820");
                //mDevices.Add(0x80862501,@"82820");
                //mDevices.Add(0x80862502,@"82820");
                //mDevices.Add(0x80862503,@"82820");
                //mDevices.Add(0x80862504,@"82820");
                //mDevices.Add(0x8086250B,@"82820");
                //mDevices.Add(0x8086250F,@"82820");
                //mDevices.Add(0x80862520,@"82805AA");
                //mDevices.Add(0x80862521,@"82804AA");
                //mDevices.Add(0x80862530,@"82850/E");
                //mDevices.Add(0x80862531,@"82860");
                //mDevices.Add(0x80862532,@"82850/850E/860");
                //mDevices.Add(0x80862533,@"82860");
                //mDevices.Add(0x80862534,@"82860");
                //mDevices.Add(0x80862535,@"82860");
                //mDevices.Add(0x80862536,@"82860");
                //mDevices.Add(0x80862539,@"82860");
                //mDevices.Add(0x80862540,@"E7500");
                //mDevices.Add(0x80862541,@"E7500/E7501");
                //mDevices.Add(0x80862543,@"E7500/E7501");
                //mDevices.Add(0x80862544,@"E7500/E7501");
                //mDevices.Add(0x80862545,@"E7500/E7501");
                //mDevices.Add(0x80862546,@"E7500/E7501");
                //mDevices.Add(0x80862547,@"E7500/E7501");
                //mDevices.Add(0x80862548,@"E7500/E7501");
                //mDevices.Add(0x8086254C,@"E7501");
                //mDevices.Add(0x80862550,@"E7505");
                //mDevices.Add(0x80862551,@"25511014");
                //mDevices.Add(0x80862552,@"E7205/E7505");
                //mDevices.Add(0x80862553,@"E7505");
                //mDevices.Add(0x80862554,@"E7505");
                //mDevices.Add(0x80862560,@"82845G/GL/GV/GE/PE");
                //mDevices.Add(0x80862561,@"82845G/GL/GV/GE/PE");
                //mDevices.Add(0x80862562,@"82845G");
                //mDevices.Add(0x80862570,@"82865G/PE/P, 82848P");
                //mDevices.Add(0x80862571,@"82865G/PE/P, 82848P");
                //mDevices.Add(0x80862572,@"82865G");
                //mDevices.Add(0x80862573,@"82865G/PE/P, 82848P");
                //mDevices.Add(0x80862576,@"82865G/PE/P, 82848P");
                //mDevices.Add(0x80862578,@"82875P/E7210");
                //mDevices.Add(0x80862579,@"82875P");
                //mDevices.Add(0x8086257B,@"82875P/E7210");
                //mDevices.Add(0x8086257E,@"82875P/E7210");
                //mDevices.Add(0x80862580,@"915G/P/GV");
                //mDevices.Add(0x80862581,@"915G/P/GV, 925X/XE?");
                //mDevices.Add(0x80862582,@"0181102");
                //mDevices.Add(0x80862584,@"82925X/XE");
                //mDevices.Add(0x80862588,@"E7221");
                //mDevices.Add(0x80862589,@"E7221");
                //mDevices.Add(0x8086258A,@"E7221");
                //mDevices.Add(0x80862590,@"915GM");
                //mDevices.Add(0x80862592,@"Intel 82852/82855 Graphic controller family");
                //mDevices.Add(0x808625A1,@"6300ESB");
                //mDevices.Add(0x808625A2,@"6300ESB");
                //mDevices.Add(0x808625A3,@"6300ESB");
                //mDevices.Add(0x808625A4,@"6300ESB");
                //mDevices.Add(0x808625A6,@"6300ESB");
                //mDevices.Add(0x808625A7,@"6300ESB");
                //mDevices.Add(0x808625A9,@"6300ESB");
                //mDevices.Add(0x808625AA,@"6300ESB");
                //mDevices.Add(0x808625AB,@"6300ESB");
                //mDevices.Add(0x808625AC,@"6300ESB");
                //mDevices.Add(0x808625AD,@"6300ESB");
                //mDevices.Add(0x808625AE,@"6300ESB");
                //mDevices.Add(0x808625B0,@"6300ESB");
                //mDevices.Add(0x80862600,@"E8500");
                //mDevices.Add(0x80862601,@"E8500");
                //mDevices.Add(0x80862602,@"E8500");
                //mDevices.Add(0x80862603,@"E8500");
                //mDevices.Add(0x80862604,@"E8500");
                //mDevices.Add(0x80862605,@"E8500");
                //mDevices.Add(0x80862606,@"E8500");
                //mDevices.Add(0x80862607,@"E8500");
                //mDevices.Add(0x80862640,@"82801FB/FR");
                //mDevices.Add(0x80862641,@"82801FBM");
                //mDevices.Add(0x80862651,@"82801Fb");
                //mDevices.Add(0x80862652,@"82801FR");
                //mDevices.Add(0x80862653,@"82801FBM");
                //mDevices.Add(0x80862658,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x80862659,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x8086265A,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x8086265B,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x8086265C,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x80862660,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x80862662,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x80862664,@"82801FB");
                //mDevices.Add(0x80862666,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x80862668,@"11583659");
                //mDevices.Add(0x80862669,@"2028026");
                //mDevices.Add(0x8086266A,@"82801BA/CA");
                //mDevices.Add(0x8086266C,@"82801FB/FR/FW/FRW");
                //mDevices.Add(0x8086266D,@"82801EB");
                //mDevices.Add(0x8086266E,@"Intel Corporation  82830M/MG SDRAM Controller / Ho");
                //mDevices.Add(0x8086266F,@"82801FB/FBM/FW/FR/FRW");
                //mDevices.Add(0x80862681,@"62089A2");
                //mDevices.Add(0x80862770,@"82945G/GZ/P/PL");
                //mDevices.Add(0x80862771,@"82945G/GZ/P/PL");
                //mDevices.Add(0x80862772,@"82945G/GZ");
                //mDevices.Add(0x8086277C,@"82975X");
                //mDevices.Add(0x80862780,@"82915G");
                //mDevices.Add(0x80862782,@"82915G");
                //mDevices.Add(0x80862792,@"Mobile 82915GLx/x/x");
                //mDevices.Add(0x808627A0,@"874079");
                //mDevices.Add(0x808627A6,@"945GM");
                //mDevices.Add(0x808627B8,@"945GL");
                //mDevices.Add(0x808627C1,@"82801GB/GR/GH");
                //mDevices.Add(0x808627C5,@"82801GBM/GHM");
                //mDevices.Add(0x808627C6,@"82801GHM");
                //mDevices.Add(0x808627CA,@"USB UHCI Controller");
                //mDevices.Add(0x808627CB,@"USB UHCI Controller");
                //mDevices.Add(0x808627DA,@"1B761019");
                //mDevices.Add(0x808627DC,@"Intel PRO/100");
                //mDevices.Add(0x80862820,@"82801HB/HR/HH/HO");
                //mDevices.Add(0x80862821,@"82821HR/HH/HO");
                //mDevices.Add(0x80862822,@"82801HR/HH/HO&82801IR/IH/IO(AIE=0)/ICH10R");
                //mDevices.Add(0x80862824,@"82810HB");
                //mDevices.Add(0x80862825,@"82801HB/HR/HH/HO");
                //mDevices.Add(0x80862828,@"82801HBM/HEM");
                //mDevices.Add(0x80862829,@"82801HBM/HEM");
                //mDevices.Add(0x8086282A,@"ICH8ME/9ME");
                //mDevices.Add(0x8086283A,@"81EC1043 (?)");
                //mDevices.Add(0x8086284B,@"82801H");
                //mDevices.Add(0x80862850,@"82801HBM/HEM");
                //mDevices.Add(0x80862920,@"82801(IB)/IR/IH/IO");
                //mDevices.Add(0x80862921,@"82801IR/IH/IO");
                //mDevices.Add(0x80862922,@"82801IR/IH/IO");
                //mDevices.Add(0x80862923,@"82801IB");
                //mDevices.Add(0x80862925,@"82801IR/IH/IO(AIE=1)");
                //mDevices.Add(0x80862926,@"82801IR/IH/IO");
                //mDevices.Add(0x80862928,@"?(ICH9M Family)");
                //mDevices.Add(0x80862929,@"ICH9M/ME");
                //mDevices.Add(0x8086292D,@"?(ICH9M Family)");
                //mDevices.Add(0x8086292E,@"?(ICH9M Family)");
                //mDevices.Add(0x80862930,@"ICH9 Family SMBus Controller");
                //mDevices.Add(0x8086293E,@"486486");
                //mDevices.Add(0x8086294C,@"82566DC-2");
                //mDevices.Add(0x80862987,@"Q965/Q963");
                //mDevices.Add(0x80862992,@"Intel(R) Express Chipset video");
                //mDevices.Add(0x80862993,@"Intel(R) Express Chipset (Dell Version)");
                //mDevices.Add(0x80862994,@"0x8086");
                //mDevices.Add(0x80862997,@"Q965/Q963");
                //mDevices.Add(0x808629A4,@"n/a");
                //mDevices.Add(0x808629B2,@"Q35");
                //mDevices.Add(0x808629B3,@"Q35");
                //mDevices.Add(0x808629B4,@"Q35-Chipset");
                //mDevices.Add(0x808629B7,@"Q35-Chipset");
                //mDevices.Add(0x80862A02,@"Unknown");
                //mDevices.Add(0x80862A03,@"82Q965");
                //mDevices.Add(0x80862A04,@"Q965/Q963");
                //mDevices.Add(0x80862A07,@"Q965/Q963");
                //mDevices.Add(0x80863092,@"SRCU32");
                //mDevices.Add(0x80863200,@"31244");
                //mDevices.Add(0x80863340,@"82855PM");
                //mDevices.Add(0x80863341,@"82855PM");
                //mDevices.Add(0x80863342,@"82855PM");
                //mDevices.Add(0x8086348D,@"82541EI");
                //mDevices.Add(0x80863575,@"82830[MP]");
                //mDevices.Add(0x80863576,@"82830M/MP");
                //mDevices.Add(0x80863577,@"82830M/MG");
                //mDevices.Add(0x80863578,@"82830[MP]");
                //mDevices.Add(0x80863579,@"82835");
                //mDevices.Add(0x8086357B,@"82835");
                //mDevices.Add(0x80863580,@"852GM/GMV");
                //mDevices.Add(0x80863581,@"82852GME/PM");
                //mDevices.Add(0x80863582,@"852GM/GMV");
                //mDevices.Add(0x80863584,@"852GM/GMV");
                //mDevices.Add(0x80863585,@"852GM");
                //mDevices.Add(0x80863590,@"E7520");
                //mDevices.Add(0x80863591,@"E7520");
                //mDevices.Add(0x80863592,@"E7320");
                //mDevices.Add(0x80863593,@"E7525");
                //mDevices.Add(0x80863594,@"E7520");
                //mDevices.Add(0x80863595,@"E7525");
                //mDevices.Add(0x80863596,@"E7525");
                //mDevices.Add(0x80863597,@"E7525");
                //mDevices.Add(0x80863598,@"E7520");
                //mDevices.Add(0x80863599,@"E7520");
                //mDevices.Add(0x8086359A,@"E7520");
                //mDevices.Add(0x8086359B,@"E7525");
                //mDevices.Add(0x8086359E,@"E7525");
                //mDevices.Add(0x8086360B,@"PCI\VEN_8086&DEV_2448&SUBSYS_00000000&REV881\3&18d");
                //mDevices.Add(0x80863A00,@"ICH10 Family");
                //mDevices.Add(0x80863A02,@"ICH10R");
                //mDevices.Add(0x80863A03,@"ICH10");
                //mDevices.Add(0x80863A06,@"ICH10 Family");
                //mDevices.Add(0x80863A20,@"ICH10 Family");
                //mDevices.Add(0x80863A22,@"ICH10R");
                //mDevices.Add(0x80863A23,@"ICH10");
                //mDevices.Add(0x80863A26,@"ICH10 Family");
                //mDevices.Add(0x80864000,@"Creatix");
                //mDevices.Add(0x80864220,@"MPCI3B");
                //mDevices.Add(0x80864222,@"10418086");
                //mDevices.Add(0x80864223,@"2915ABG");
                //mDevices.Add(0x80864224,@"Intel Pro Wireless 2915ABG");
                //mDevices.Add(0x80864227,@"3945ABG");
                //mDevices.Add(0x80864229,@"Intel 4965AGN");
                //mDevices.Add(0x8086422D,@"Intel 4965AGN");
                //mDevices.Add(0x80864230,@"Intel 4965AGN");
                //mDevices.Add(0x80864233,@"Intel 4965AGN");
                //mDevices.Add(0x8086444E,@"TurboMemory");
                //mDevices.Add(0x80865001,@"PRO/DSL 2100");
                //mDevices.Add(0x80865005,@"PRO/DSL 2200");
                //mDevices.Add(0x80865029,@"?(EP80579)");
                //mDevices.Add(0x80865200,@"NH82801GR");
                //mDevices.Add(0x80865201,@"Network Controller");
                //mDevices.Add(0x80865309,@"80303");
                //mDevices.Add(0x8086530D,@"80312");
                //mDevices.Add(0x80866960,@"EHCI 960 emulator");
                //mDevices.Add(0x80867000,@"82371SB");
                //mDevices.Add(0x80867010,@"82371SB");
                //mDevices.Add(0x80867020,@"82371SB");
                //mDevices.Add(0x80867030,@"82437VX");
                //mDevices.Add(0x80867051,@"PB 642365-003");
                //mDevices.Add(0x80867100,@"82439TX");
                //mDevices.Add(0x80867110,@"82371AB/EB/MB");
                //mDevices.Add(0x80867111,@"82371AB/EB/MB");
                //mDevices.Add(0x80867112,@"82371AB/EB/MB");
                //mDevices.Add(0x80867113,@"82371AB/EB/MB");
                //mDevices.Add(0x80867120,@"82810");
                //mDevices.Add(0x80867121,@"82810");
                //mDevices.Add(0x80867122,@"82810-DC100");
                //mDevices.Add(0x80867123,@"82810-DC100");
                //mDevices.Add(0x80867124,@"82810E");
                //mDevices.Add(0x80867125,@"82810E");
                //mDevices.Add(0x80867126,@"82810-DC133");
                //mDevices.Add(0x80867127,@"82810-DC133");
                //mDevices.Add(0x80867128,@"82810-M DC-100");
                //mDevices.Add(0x8086712A,@"82810-M DC-133");
                //mDevices.Add(0x80867180,@"intel 82801 IB ICH7 - high definition audio");
                //mDevices.Add(0x80867181,@"82443 ex/lx");
                //mDevices.Add(0x80867182,@"440LX/EX");
                //mDevices.Add(0x80867190,@"82443BX/ZX");
                //mDevices.Add(0x80867191,@"82443BX/ZX");
                //mDevices.Add(0x80867192,@"82443BX/ZX");
                //mDevices.Add(0x80867194,@"82443MX");
                //mDevices.Add(0x80867195,@"82443MX");
                //mDevices.Add(0x80867196,@"82440 - 443MX");
                //mDevices.Add(0x80867198,@"82443MX");
                //mDevices.Add(0x80867199,@"82443MX");
                //mDevices.Add(0x8086719A,@"82443MX");
                //mDevices.Add(0x8086719B,@"82443MX");
                //mDevices.Add(0x808671A0,@"82443GX");
                //mDevices.Add(0x808671A1,@"intel 82801 IB ICH9 - high definition audio");
                //mDevices.Add(0x808671A2,@"82443GX");
                //mDevices.Add(0x80867221,@"82810");
                //mDevices.Add(0x80867600,@"82372FB/82468GX");
                //mDevices.Add(0x80867601,@"82372FB/82468GX");
                //mDevices.Add(0x80867602,@"82372FB/82468GX");
                //mDevices.Add(0x80867603,@"82372FB/82468GX");
                //mDevices.Add(0x80867605,@"82372FB");
                //mDevices.Add(0x80867800,@"82740");
                //mDevices.Add(0x80868086,@"park");
                //mDevices.Add(0x8086811A,@"Atom SCH");
                //mDevices.Add(0x808684C4,@"82454KX/GX");
                //mDevices.Add(0x808684C5,@"82453KX/GX");
                //mDevices.Add(0x808684CA,@"82451NX");
                //mDevices.Add(0x808684CB,@"82454NX/82467GX");
                //mDevices.Add(0x808684E0,@"82461GX");
                //mDevices.Add(0x808684E1,@"82462GX");
                //mDevices.Add(0x808684E2,@"82465GX");
                //mDevices.Add(0x808684E3,@"82463GX");
                //mDevices.Add(0x808684E4,@"82464GX");
                //mDevices.Add(0x808684E6,@"82466GX");
                //mDevices.Add(0x808684EA,@"82460GX");
                //mDevices.Add(0x808685A1,@"6300ESB");
                //mDevices.Add(0x808685A2,@"6300ESB");
                //mDevices.Add(0x808685A3,@"6300ESB");
                //mDevices.Add(0x808685A4,@"6300ESB");
                //mDevices.Add(0x808685A6,@"6300ESB");
                //mDevices.Add(0x808685A7,@"6300ESB");
                //mDevices.Add(0x808685A9,@"6300ESB");
                //mDevices.Add(0x808685AA,@"6300ESB");
                //mDevices.Add(0x80869620,@"I2O RAID PCI to PCI Bridge");
                //mDevices.Add(0x80869621,@"SRCU21");
                //mDevices.Add(0x80869622,@"SRCUxx");
                //mDevices.Add(0x80869641,@"SRCU31");
                //mDevices.Add(0x808696A1,@"SRCU31L");
                //mDevices.Add(0x80869874,@"AC97");
                //mDevices.Add(0x80869876,@"i845");
                //mDevices.Add(0x8086B152,@"S21152BB");
                //mDevices.Add(0x8086B154,@"S21154AE/BE");
                //mDevices.Add(0x8086B555,@"21555");
                //mDevices.Add(0x88661689,@"T2-MP3-001");
                //mDevices.Add(0x90040078,@"aic-7880p");
                //mDevices.Add(0x90041078,@"AIC-7810C");
                //mDevices.Add(0x90041135,@"0x0035");
                //mDevices.Add(0x90041160,@"AIC-1160");
                //mDevices.Add(0x90042178,@"AIC-7821");
                //mDevices.Add(0x90043860,@"AIC-2930U Ultra SCSI Ctrlr");
                //mDevices.Add(0x90043B78,@"AHA-4944W/4944UW");
                //mDevices.Add(0x90045075,@"AIC-755x");
                //mDevices.Add(0x90045078,@"AIC-7850P");
                //mDevices.Add(0x90045175,@"AIC-755x");
                //mDevices.Add(0x90045178,@"AIC-7850");
                //mDevices.Add(0x90045275,@"AIC-755x");
                //mDevices.Add(0x90045278,@"AIC-7850");
                //mDevices.Add(0x90045375,@"AIC-755x");
                //mDevices.Add(0x90045378,@"AIC-7850");
                //mDevices.Add(0x90045475,@"AIC-755x");
                //mDevices.Add(0x90045478,@"AIC-7850");
                //mDevices.Add(0x90045575,@"AVA-2930");
                //mDevices.Add(0x90045578,@"AIC-7855");
                //mDevices.Add(0x90045675,@"AIC-755x");
                //mDevices.Add(0x90045678,@"AIC-7856");
                //mDevices.Add(0x90045775,@"AIC-755x");
                //mDevices.Add(0x90045778,@"AIC-7850");
                //mDevices.Add(0x90045800,@"AIC-5800");
                //mDevices.Add(0x90045900,@"ANA-5910/30/40");
                //mDevices.Add(0x90045905,@"ANA-5910A/30A/40A");
                //mDevices.Add(0x90046038,@"AHA-2930C");
                //mDevices.Add(0x90046075,@"AIC-7560?");
                //mDevices.Add(0x90046078,@"AIC-7860");
                //mDevices.Add(0x90046178,@"AIC-7861");
                //mDevices.Add(0x90046278,@"AIC-7860");
                //mDevices.Add(0x90046378,@"AIC-7860");
                //mDevices.Add(0x90046478,@"AIC-786x");
                //mDevices.Add(0x90046578,@"AIC-786x");
                //mDevices.Add(0x90046678,@"AIC-786x");
                //mDevices.Add(0x90046778,@"AIC-786x");
                //mDevices.Add(0x90046915,@"ANA620xx/69011A");
                //mDevices.Add(0x90047078,@"AIC-7870");
                //mDevices.Add(0x90047178,@"AHA-2940/2940W");
                //mDevices.Add(0x90047278,@"AHA-3940/3940W");
                //mDevices.Add(0x90047378,@"AHA-3985");
                //mDevices.Add(0x90047478,@"AHA-2944");
                //mDevices.Add(0x90047578,@"AHA-3944/3944W");
                //mDevices.Add(0x90047678,@"AHA-4944W/4944UW");
                //mDevices.Add(0x90047778,@"AIC-787x");
                //mDevices.Add(0x90047810,@"aic 7810");
                //mDevices.Add(0x90047815,@"AIC-7515");
                //mDevices.Add(0x90047850,@"aic-7850");
                //mDevices.Add(0x90047855,@"AHA-2930");
                //mDevices.Add(0x90047860,@"AIC-7860");
                //mDevices.Add(0x90047870,@"AIC-7870");
                //mDevices.Add(0x90047871,@"aha 2940");
                //mDevices.Add(0x90047872,@"aha 3940");
                //mDevices.Add(0x90047873,@"aha 3985");
                //mDevices.Add(0x90047874,@"aha 2944");
                //mDevices.Add(0x90047880,@"aic7880p");
                //mDevices.Add(0x90047890,@"AIC-7890");
                //mDevices.Add(0x90047891,@"AIC-789x");
                //mDevices.Add(0x90047892,@"AIC-789x");
                //mDevices.Add(0x90047893,@"AIC-789x");
                //mDevices.Add(0x90047894,@"AIC-789x");
                //mDevices.Add(0x90047895,@"AIC-7895");
                //mDevices.Add(0x90047896,@"AIC-789x");
                //mDevices.Add(0x90047897,@"AIC-789x");
                //mDevices.Add(0x90048078,@"AIC-7880");
                //mDevices.Add(0x90048178,@"AHA-2940U/2940UW");
                //mDevices.Add(0x90048278,@"AHA-3940Uxx");
                //mDevices.Add(0x90048378,@"AIC-7883U");
                //mDevices.Add(0x90048478,@"ADAPTEC 2940UW CN SCSI");
                //mDevices.Add(0x90048578,@"AHA-3944U/3944UWD");
                //mDevices.Add(0x90048678,@"AHA-4944UW");
                //mDevices.Add(0x90048778,@"AIC-788x");
                //mDevices.Add(0x90048878,@"AIC-7888?");
                //mDevices.Add(0x90048B78,@"ABA-1030");
                //mDevices.Add(0x9004EC78,@"AHA-4944W/4944UW");
                //mDevices.Add(0x90050010,@"AIC-7890AB scsi controller");
                //mDevices.Add(0x90050011,@"11111");
                //mDevices.Add(0x90050013,@"AIC-7890/1");
                //mDevices.Add(0x9005001F,@"AIC-7890 AB for Windows  XP");
                //mDevices.Add(0x90050020,@"AIC-789x");
                //mDevices.Add(0x9005002F,@"AIC-789x");
                //mDevices.Add(0x90050030,@"AIC-789x");
                //mDevices.Add(0x9005003F,@"AIC-789x");
                //mDevices.Add(0x90050050,@"AHA-3940U2x/3950U2x Ultra2 SCSI Adapter");
                //mDevices.Add(0x90050051,@"AHA-3950U2x Ultra2 SCSI Adapter");
                //mDevices.Add(0x90050053,@"AIC-7896");
                //mDevices.Add(0x9005005F,@"AIC-7896/7");
                //mDevices.Add(0x90050080,@"AIC-7892Q");
                //mDevices.Add(0x90050081,@"AIC-7892B");
                //mDevices.Add(0x90050083,@"AIC-7892D");
                //mDevices.Add(0x9005008F,@"AIC-7892");
                //mDevices.Add(0x900500C0,@"AIC-7899A");
                //mDevices.Add(0x900500C1,@"AIC-7899B");
                //mDevices.Add(0x900500C3,@"AIC-7899D");
                //mDevices.Add(0x900500C5,@"RAID Subsystem HBA");
                //mDevices.Add(0x900500CF,@"AIC-7899G");
                //mDevices.Add(0x90050258,@"AAC-RAID");
                //mDevices.Add(0x9005041F,@"AIC 9410");
                //mDevices.Add(0x9005043E,@"AIC9450W");
                //mDevices.Add(0x90058000,@"ASC-29320A");
                //mDevices.Add(0x9005800F,@"AIC-7901");
                //mDevices.Add(0x90058010,@"ASC-39320");
                //mDevices.Add(0x90058011,@"ASC-39320D");
                //mDevices.Add(0x90058012,@"ASC-29320");
                //mDevices.Add(0x90058014,@"ASC-29320LP");
                //mDevices.Add(0x90058015,@"ASC-39320");
                //mDevices.Add(0x90058016,@"ASC-39320A");
                //mDevices.Add(0x90058017,@"ASC-29320ALP");
                //mDevices.Add(0x9005801C,@"AIC-?????");
                //mDevices.Add(0x9005801D,@"AIC-7902B");
                //mDevices.Add(0x9005801E,@"AIC-7901");
                //mDevices.Add(0x9005801F,@"AIC-7902");
                //mDevices.Add(0x90058080,@"ASC-29320A");
                //mDevices.Add(0x9005808F,@"AIC-7901");
                //mDevices.Add(0x90058090,@"ASC-39320");
                //mDevices.Add(0x90058091,@"ASC-39320D");
                //mDevices.Add(0x90058092,@"ASC-29320");
                //mDevices.Add(0x90058093,@"ASC-29320B");
                //mDevices.Add(0x90058094,@"ASC-29320LP");
                //mDevices.Add(0x90058095,@"ASC-39320");
                //mDevices.Add(0x90058096,@"ASC-39320A");
                //mDevices.Add(0x90058097,@"ASC-29320ALP");
                //mDevices.Add(0x9005809C,@"ASC-39320D");
                //mDevices.Add(0x9005809D,@"AIC-7902B");
                //mDevices.Add(0x9005809E,@"AIC-7901A");
                //mDevices.Add(0x9005809F,@"AIC-7902B");
                //mDevices.Add(0x94126565,@"HT6565");
                //mDevices.Add(0x99020001,@"SG2010");
                //mDevices.Add(0x99020002,@"SG2010");
                //mDevices.Add(0x99020003,@"SG1010");
                //mDevices.Add(0xAA4203A3,@"9400-0931");
                //mDevices.Add(0xC0DE5600,@"62802");
                //mDevices.Add(0xC0DEC0DE,@"62802-51");
                //mDevices.Add(0xD4D40601,@"PCI Mezzanine Card");
                //mDevices.Add(0xDEAF9050,@"PC Weasel PCI VGA Device");
                //mDevices.Add(0xDEAF9051,@"PC Weasel PCI Serial Comm. Device");
                //mDevices.Add(0xDEAF9052,@"PC Weasel PCI");
                //mDevices.Add(0xE1590001,@"Ambient MD3200 A");
                //mDevices.Add(0xE1590002,@"Sedlbauer Speed PCI");
                //mDevices.Add(0xE1590600,@"Tiger 600");
                //mDevices.Add(0xEACE3100,@"DAG 3.10");
                //mDevices.Add(0xEACE3200,@"DAG 3.2x");
                //mDevices.Add(0xEACE320E,@"DAG 3.2E");
                //mDevices.Add(0xEACE340E,@"DAG 3.4E");
                //mDevices.Add(0xEACE341E,@"DAG 3.41E");
                //mDevices.Add(0xEACE3500,@"DAG 3.5");
                //mDevices.Add(0xEACE351C,@"DAG 3.5ECM");
                //mDevices.Add(0xEACE4100,@"DAG 4.10");
                //mDevices.Add(0xEACE4110,@"DAG 4.11");
                //mDevices.Add(0xEACE4200,@"DAG 4.2");
                //mDevices.Add(0xEACE420E,@"DAG 4.2E");
                //mDevices.Add(0xEDD8A091,@"ARK1000PV");
                //mDevices.Add(0xEDD8A099,@"ARK2000PV");
                //mDevices.Add(0xEDD8A0A1,@"ARK2000MT");
                //mDevices.Add(0xEDD8A0A9,@"ARK2000MI");
                //mDevices.Add(0xEDD8A0B1,@"ARK2000MI+");
                //mDevices.Add(0xFA570001,@"PMC");




                #endregion Fill devices

                #endregion PCIHelper
            }

            public string FindVendor(UInt32 aVendorID) {
                return mVendors[aVendorID];
            }

            public string FindDevice(UInt32 aVendorID,UInt32 aDeviceID)
            {
                return mDevices[aVendorID*0x10000+aDeviceID];
            }
            /*protected List<DeviceID> mVendors = new List<DeviceID>();

            public DeviceIDs()
            {
                mVendors.Add(new DeviceID(0x8086, "Intel"));
            }

            public string FindVendor(UInt32 aVendorID)
            {
                for (int i = 0; i < mVendors.Count; i++)
                {

                    if (mVendors[i].Key == aVendorID)
                        return mVendors[i].Value;
                }
                return null;
            }*/
        }

        protected static PCIDevice[] mDevices;
        public static PCIDevice[] Devices {
            get { return mDevices; }
        }

        protected static bool mEnumerationCompleted = false;

        public static PCIDevice GetPCIDevice(byte bus, byte slot, byte function) {
            if (!mEnumerationCompleted) {
                Init();
            }

            foreach (PCIDevice dev in PCIBus.Devices) {
                if (dev.Bus == bus &&
                    dev.Slot == slot &&
                    dev.Function == function)
                    return dev;
            }
            
            return null;
        }

        public static void Init() {
            var xDevices = new List<PCIDevice>(16);
            EnumerateBus(0, ref xDevices);
            Console.WriteLine("Converting devices list to array.");
            Console.Write("Count: ");
            Console.WriteLine("");
            mDevices = xDevices.ToArray();
            Console.WriteLine("");
            mEnumerationCompleted = true;
        }

        /// <summary>
        /// Enumerates a given bus, adding devices to Devices
        /// </summary>
        /// <param name="aBus">The bus number to enumerate</param>
        /// <param name="rDevices">The list of Devices</param>
        private static void EnumerateBus(byte aBus, ref List<PCIDevice> rDevices) {
              for (byte xSlot = 0; xSlot < 32; xSlot++) {                
                byte xMaxFunctions = 1;
                for (byte xFunction = 0; xFunction < xMaxFunctions; xFunction++) {
                    PCIDevice xPCIDevice = new PCIDeviceNormal(aBus, xSlot, xFunction);
                    if (xPCIDevice.DeviceExists) {
                        if (xPCIDevice.HeaderType == 2) { /* PCIHeaderType.Cardbus */
                            xPCIDevice = new PCIDeviceCardBus(aBus, xSlot, xFunction);
                        }

                        if (xPCIDevice.HeaderType == 1) { /* PCIHeaderType.Bridge */
                            xPCIDevice = new PCIDeviceBridge(aBus, xSlot, xFunction);
                        }
                        if (xPCIDevice is PCIDeviceBridge)
                        {
                            EnumerateBus(((PCIDeviceBridge)xPCIDevice).SecondaryBus, ref rDevices);
                        }
                        rDevices.Add(xPCIDevice);
                        if (xPCIDevice.IsMultiFunction) {
                            xMaxFunctions = 8;
                        }
                    }
                }
            }
        }
    }

    [Flags]
    public enum PCICommand : short
    {
        IO = 0x1,     /* Enable response in I/O space */
        Memort = 0x2,     /* Enable response in Memory space */
        Master = 0x4,     /* Enable bus mastering */
        Special = 0x8,     /* Enable response to special cycles */
        Invalidate = 0x10,    /* Use memory write and invalidate */
        VGA_Pallete = 0x20,   /* Enable palette snooping */
        Parity = 0x40,    /* Enable parity checking */
        Wait = 0x80,    /* Enable address/data stepping */
        SERR = 0x100,   /* Enable SERR */
        Fast_Back = 0x200,   /* Enable back-to-back writes */
    }

    [Flags]
    public enum PCIStatus : int
    {
        CAP_LIST = 0x10,   /* Support Capability List */
        SUPPORT_66MHZ = 0x20,    /* Support 66 Mhz PCI 2.1 bus */
        UDF = 0x40,    /* Support User Definable Features [obsolete] */
        FAST_BACK = 0x80,    /* Accept fast-back to back */
        PARITY = 0x100,   /* Detected parity error */
        DEVSEL_MASK = 0x600,   /* DEVSEL timing */
        DEVSEL_FAST = 0x000,
        DEVSEL_MEDIUM = 0x200,
        DEVSEL_SLOW = 0x400,
        SIG_TARGET_ABORT = 0x800, /* Set on target abort */
        REC_TARGET_ABORT = 0x1000, /* Master ack of " */
        REC_MASTER_ABORT = 0x2000, /* Set on master abort */
        SIG_SYSTEM_ERROR = 0x4000, /* Set when we drive SERR */
        DETECTED_PARITY = 0x8000 /* Set on parity error */
    }


    public enum PCIHeaderType : byte
    {
        Normal = 0,
        Bridge = 1,
        Cardbus = 2
    }

    
    [Flags]
    public enum PCIBist : byte
    {
        CocdMask = 0x0f,   /* Return result */
        Start = 0x40,   /* 1 to start BIST, 2 secs or less */
        Capable = 0x80    /* 1 if BIST capable */
    }


    /// <summary>
    /// Class representing CardBus PCI Device device
    /// </summary>
    public class PCIDeviceCardBus : PCIDevice
    {
        public PCIDeviceCardBus(byte bus, byte slot, byte function)
            : base (bus,slot,function)
        {
        }

        public override int NumberOfBaseAddresses()
        {
            //get
            //{
                return 6;
            //}
        }
    }

    /// <summary>
    /// Class representing a PCI to PCI Bridge PCI Device
    /// </summary>
    public class PCIDeviceBridge : PCIDevice
    {
        public PCIDeviceBridge(byte bus, byte slot, byte function)
            : base (bus,slot,function)
        {
        }

        public override int NumberOfBaseAddresses()
        {
           //  get
           // {
                return 2;
            ////}
        }

        
        public byte PrimaryBus { get { return Read8(0x18); } set { Write8(0x18, value); } }
        public byte SecondaryBus { get { return Read8(0x19); } set { Write8(0x19, value); } }
        public byte SubordinateBus { get { return Read8(0x1a); } set { Write8(0x1a, value); } }
        public byte SecondaryLatencyTime { get { return Read8(0x1b); } set { Write8(0x1b, value); } }
    }


    /// <summary>
    /// Class representing a Normal PCI Device
    /// </summary>
    public class PCIDeviceNormal : PCIDevice
    {
        public PCIDeviceNormal(byte bus, byte slot, byte function)
            : base (bus,slot,function)
        {
        }

        public override int NumberOfBaseAddresses()
        {
            //get
            //{
                return 6;
            //}
        }

        [Obsolete("Use PciDevice.GetAddressSpace(2)")]
        public UInt32 BaseAddress2 { get { return Read32(0x18); } set { Write32(0x18, value); } }
        [Obsolete("Use PciDevice.GetAddressSpace(3)")]
        public UInt32 BaseAddress3 { get { return Read32(0x1a); } set { Write32(0x1a, value); } }
        [Obsolete("Use PciDevice.GetAddressSpace(4)")]
        public UInt32 BaseAddress4 { get { return Read32(0x20); } set { Write32(0x20, value); } }
        [Obsolete("Use PciDevice.GetAddressSpace(5)")]
        public UInt32 BaseAddress5 { get { return Read32(0x24); } set { Write32(0x24, value); } }
       
    }

    public abstract class PCIDevice
    {
        /// <summary>
        /// Gets the number of address spaces this device type has
        /// </summary>
        /// <returns></returns>
        public abstract int NumberOfBaseAddresses();
       //{
        //   get;
       //}
           

       private static string[] classtext = new string[]          
       {
        "pre pci 2.0",		// 00
        "disk",		// 01
        "network",		// 02
        "display",		// 03
        "multimedia",	// 04
        "memory",		// 05
        "bridge",		// 06
        "communication",	// 07
        "system peripheral",// 08
        "input",		// 09
        "docking station",	// 0A
        "CPU",		// 0B
        "serial bus",	// 0C
       };

        private static string[][] subclasstext = new string[][]
        { 
            new string[] { "VGA Device", "non VGA device"},
            new string[] { "SCSI" ,"IDE" , "floppy","IPI","RAID", "other" },
            new string[] { "Ethernet", "TokenRing", "FDDI" , "ATM" , "other" },
            new string[] { "VGA", "SuperVGA","XGA", "other"},
            new string[] { "video" ,"audio", "other"},
            new string[] { "RAM", "Flash memory" , "other"},
            new string[] { "CPU/PCI" ,"PCI/ISA" , "PCI/EISA" , "PCI/MCA","PCI/PCI" , "PCI/PCMCIA", "PCI/NuBus", "PCI/CardBus", "other"},
            new string[] { "serial", "parallel", "other"},
            new string[] { "PIC", "DMAC" , "timer" ,"RTC", "other"},
            new string[] { "keyboard","digitizer","mouse", "other" },
            new string[] { "generic" , "other" },
            new string[] { "386", "486","Pentium" , "P6" ,"Alpha","coproc","other" },
            new string[] { "Firewire", "ACCESS.bus" , "SSA", "USB" ,"Fiber Channel" , "other"},
        };


        public string GetClassInfo()
        {
            int cc = ClassCode;
            int sc = SubClass; 
            
            if (cc >= classtext.Length)            
                return "unknown class (" + cc.ToString() + ") / subclass (" + sc.ToString() + ")";
            
            
            if (sc >= subclasstext[cc].Length)            
                return String.Concat(classtext[cc], " / unknown subclass (", sc.ToString(), ")");
            
            return String.Concat( classtext[cc] , " / " , subclasstext[cc][sc]);
        }

        private const UInt32 PCI_BASE_ADDRESS_SPACE = 0x01;    /* 0 = memory, 1 = I/O */
        private const UInt32 PCI_BASE_ADDRESS_SPACE_IO = 0x01;
        private const UInt32 PCI_BASE_ADDRESS_SPACE_MEMORY = 0x00;
        private const UInt32 PCI_BASE_ADDRESS_MEM_TYPE_MASK = 0x06;
        private const UInt32 PCI_BASE_ADDRESS_MEM_TYPE_32 = 0x00;   /* 32 bit address */
        private const UInt32 PCI_BASE_ADDRESS_MEM_TYPE_1M = 0x02;   /* Below 1M [obsolete] */
        private const UInt32 PCI_BASE_ADDRESS_MEM_TYPE_64 = 0x04;   /* 64 bit address */
        private const UInt32 PCI_BASE_ADDRESS_MEM_PREFETCH = 0x08;  /* prefetchable? */
        private const UInt32 PCI_BASE_ADDRESS_MEM_MASK = 0xfffffff0;
        private const UInt32 PCI_BASE_ADDRESS_IO_MASK = 0xfffffffc;


        protected PCIDevice(byte bus, byte slot, byte function)
        {
            this.Bus = bus;
            this.Slot = slot;
            this.Function = function;
            this.Claimed = false;
        }

        private bool NeedsIO = false;
        private bool NeedsMemory = false;

        private bool _NeedsLayingout = true;

        /// <summary>
        /// Creates the AddressSpaces for this device.
        /// </summary>
        private void LayoutIO()
        {
            //Console.WriteLine("Checking AdressSpaces of PCI(" + Bus + ", " + Slot + ", " + Function + ")");
            IOMaps = new Kernel.AddressSpace[NumberOfBaseAddresses()];

            for (byte i = 0; i < NumberOfBaseAddresses(); i++)
            {
                UInt32 address = GetBaseAddressInternal(i);
                SetBaseAddressInternal(i, 0xffffffff);
                UInt32 flags = GetBaseAddressInternal(i);
                SetBaseAddressInternal(i, address);
                if (address == 0)
                {
                    //Console.WriteLine("register " + i + " - none " + PCIBus.ToHex(flags,8));

                    IOMaps[i] = null;
                }
                else if ((address & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_MEMORY)
                {
                    UInt32 size = (~(PCI_BASE_ADDRESS_MEM_MASK & flags))+1;
                    IOMaps[i] = new Kernel.MemoryAddressSpace(address, size);
                    //Console.WriteLine("register " + i + " - " + size + "b mem");

                    NeedsMemory = true;
                }
                else if ((address & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_IO)
                {
                    UInt32 size = (~(PCI_BASE_ADDRESS_IO_MASK & flags)) & 0xFF;
                    IOMaps[i] = new Kernel.IOAddressSpace(address-1, ((uint)size));
                    NeedsIO = true;
                }
            }

            _NeedsLayingout = false;
        }

        private Kernel.AddressSpace[] IOMaps;

        /// <summary>
        /// Gets the AddressSpace object assosiated with the device
        /// </summary>
        /// <param name="index">Address Space to return</param>
        /// <returns></returns>
        public Kernel.AddressSpace GetAddressSpace(byte index)
        {
            if (index < 0 || index >= NumberOfBaseAddresses())
                throw new ArgumentOutOfRangeException("index");
            
            if (_NeedsLayingout)
                LayoutIO();

            return IOMaps[index];
        }

        /// <summary>
        /// Has this device been claimed a driver
        /// </summary>
        public bool Claimed { get; set; }
        /// <summary>
        /// The bus the device is on
        /// </summary>
        public byte Bus { get; private set; }
        /// <summary>
        /// The slot the device is using
        /// </summary>
        public byte Slot { get; private set; }
        /// <summary>
        /// The function number of the device
        /// </summary>
        public byte Function { get; private set; }

        /// <summary>
        /// Is this an actual device?
        /// </summary>
        public bool DeviceExists { get { return VendorID != 0xFFFF && VendorID != 0x0; } }
        /// <summary>
        /// Is this a multifunction device?
        /// </summary>
        public bool IsMultiFunction { get { return (Read8(0x0e) & 0xF0) != 0; } }

        /// <summary>
        /// The Vendor ID
        /// </summary>
        public UInt16 VendorID { get { return Read16(0x0); } }
        /// <summary>
        /// The Device ID
        /// </summary>
        public UInt16 DeviceID { get { return Read16(0x2); } }

        /// <summary>
        /// the command register of this PCI Device
        /// </summary>
        public PCICommand Command { get { return (PCICommand)Read16(0x4); } set { Write16(0x4, (ushort)value); } }
        
        /// <summary>
        /// The Status Register of this PCI Device
        /// </summary>
        public PCIStatus Status { get { return (PCIStatus)Read16(0x6); } set { Write16(0x6, (ushort)value); } }

        /// <summary>
        /// The Revision ID of this PCI Device
        /// </summary>
        public UInt16 RevisionID { get { return Read16(0x8); } }
        /// <summary>
        /// The Programming Interface Number of this PCI Device
        /// </summary>
        public byte ProgIF { get { return Read8(0x9); } }
        /// <summary>
        /// The Sub class of this PCI Device
        /// </summary>
        public byte SubClass { get { return Read8(0xa); } }
        /// <summary>
        /// The Class of this PCI Device
        /// </summary>
        public byte ClassCode { get { return Read8(0xb); } }

        /// <summary>
        /// The Cache Line Size of this PCI Device
        /// </summary>
        public byte CacheLineSize { get { return Read8(0x0c); } set { Write8(0x0c, value); } }
        /// <summary>
        /// The Latency Timer of this PCI Device
        /// </summary>
        public byte LatencyTimer { get { return Read8(0x0d); } set { Write8(0x0d, value); } }
        /// <summary>
        /// The header type of this PCI Device
        /// </summary>
        public byte HeaderType { get { return (byte)(Read8(0x0e) & 0xf); } }
        /// <summary>
        /// The BIST Register of this PCI Device
        /// </summary>
        public PCIBist Bist { get { return (PCIBist)Read8(0x0f); } set { Write8(0x0f, (byte)value); } }

        [Obsolete("This function should be private. Use PciDevice.GetAddressSpace(index)")]
        public UInt32 GetBaseAddress(byte index)
        {
            return Read32((byte)(0x10 + index * 4));
        }

        public UInt32 GetBaseAddressInternal(byte index)
        {
            return Read32((byte)(0x10 + (index * 4)));
        }
        
        private void SetBaseAddressInternal(byte index, UInt32 value)
        {
            Write32((byte)(0x10 + (index * 4)), value);
        }

        [Obsolete("Use PciDevice.GetAddressSpace(0)")]
        public UInt32 BaseAddress0 { get { return Read32(0x10); } set { Write32(0x10, value); } }
        [Obsolete("Use PciDevice.GetAddressSpace(1)")]
        public UInt32 BaseAddress1 { get { return Read32(0x14); } set { Write32(0x14, value); } }
         
        public byte InterruptLine { get { return Read8(0x3c); } set { Write8(0x3c, value); } }
        public byte InterruptPin { get { return Read8(0x3d); } set { Write8(0x3d, value); } }
        public byte MinGNT { get { return Read8(0x3e); } set { Write8(0x3e, value); } }
        public byte MaxLAT { get { return Read8(0x3f); } set { Write8(0x3f, value); } }

        protected const ushort ConfigAddr = 0xCF8;
        protected const ushort ConfigData = 0xCFC;

        /// <summary>
        /// Disables the device
        /// </summary>
        public void DisableDevice()
        {
            Command = Command & (~PCICommand.IO & ~PCICommand.Master & PCICommand.Memort);
        }

        /// <summary>
        /// enables the device
        /// </summary>
        public void EnableDevice()
        {
            Command = Command | ((NeedsIO ? PCICommand.IO : 0) | PCICommand.Master | (NeedsMemory ? PCICommand.Memort : 0));
        }

        private UInt32 GetAddress(byte aRegister)
        {
            return (UInt32)(
                // Bits 23-16
                ((UInt32)Bus << 16)
                // Bits 15-11
                | (((UInt32)Slot & 0x1F) << 11)
                // Bits 10-8
                | (((UInt32)Function & 0x07) << 8)
                // Bits 7-0
                | ((UInt32)aRegister & 0xFC)
                // Enable bit - must be set
                | 0x80000000);
        }

        protected UInt32 Read32(byte aRegister)
        {
            Kernel.CPUBus.Write32(ConfigAddr, GetAddress(aRegister));
            return Kernel.CPUBus.Read32(ConfigData);
        }

        protected UInt16 Read16(byte aRegister)
        {
            Kernel.CPUBus.Write32(ConfigAddr, GetAddress(aRegister));
            return (UInt16)(Kernel.CPUBus.Read32(ConfigData) >> ((aRegister % 4) * 8) & 0xFFFF);
        }

        protected byte Read8(byte aRegister)
        {
            Kernel.CPUBus.Write32(ConfigAddr, GetAddress(aRegister));
            return (byte)(Kernel.CPUBus.Read32(ConfigData) >> ((aRegister % 4) * 8) & 0xFF);
        }

        protected void Write32(byte aRegister, UInt32 value)
        {
            Kernel.CPUBus.Write32(ConfigAddr, GetAddress(aRegister));
            Kernel.CPUBus.Write32(ConfigData, value);
        }

        protected void Write16(byte aRegister, UInt16 value)
        {
            Kernel.CPUBus.Write32(ConfigAddr, GetAddress(aRegister));
            Kernel.CPUBus.Write16(ConfigData, value);
        }

        protected void Write8(byte aRegister, byte value)
        {
            Kernel.CPUBus.Write32(ConfigAddr, GetAddress(aRegister));
            Kernel.CPUBus.Write8(ConfigData, value);
        } 
    }
}